#!/bin/sh
#@***h* GPCPcdr_ECPm_DiagPlots_AnomMap.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   GPCPcdr_ECPm_DiagPlots_AnomMap.sh
#
# PURPOSE
#   To produce rough maps of the Monthly GPCP input data
#
# DESCRIPTION
# This script runs matlab code that produces rough maps of the GPCP input data
# Note that output is stored in GPCP_Output or NRT/GPCP_Output
#
# Inputs:
#  yyyy, mm - 4 digit year and 2 digit month
#  filetype - either 'cdr', 'icdr' or 'interim' (default='cdr')
#        'cdr' - plot a field from a CDR file
#        'icdr' - plot a field form an ICDR/NRT file 
#
# GPCPcdr_ECPm_DiagPlots_AnomMap.sh 2013 09 icdr
#
# Written by MRP Sapiano
#
# may need to update the location of matlab - type "whereis matlab" at the unix command line
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 3/2016 Initial Version prepared for NCEI
#
#@*****

# get inputs 
yyyy=$1
mm=$2
filetype=$3

if [ $# -ne 3 ]; then
    echo "Incorrect number of inputs; need: yyyy mm filetype"
    exit 0
fi



/usr/bin/matlab -nosplash -nodisplay -nodesktop -r "path(path,'GPCPcdr_m5/GPCPcdr_m5_DiagPlots/'); GPCPcdr_m5_DiagPlots_AnomMaps($yyyy,$mm,'$filetype',16); exit;" 




