#!/bin/sh
#@***h* GPCPcdr_ECPm_DiagPlots_Map.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   GPCPcdr_ECPm_DiagPlots_Map.sh
#
# PURPOSE
#   To produce rough maps of the Monthly GPCP input data
#
# DESCRIPTION
# This script runs matlab code that produces rough maps of the GPCP input data
# Note that output is stored in GPCP_Output or NRT/GPCP_Output
#
# Inputs:
#  yyyy, mm - 4 digit year and 2 digit month
#  filetype - either 'cdr', 'icdr' or 'interim' (default='cdr')
#        'cdr' - plot a field from a CDR file
#        'icdr' - plot a field form an ICDR/NRT file 
#        'interim' - plot an interim field
#  field - name of field to plot (default='precip')
#  font size - size of basic font (device dependent; default=14)
#
# Values for field:
# If filetype is 'cdr' or 'icdr' then 'field' must be 'precip' or 
# 'precip_error'. If filetype os 'interim', then 'field' must be 
# one of the values from the following table.  There are three
# different data periods:
#   1  Jan 79 to Dec 85
#   2  Jan 86 to jul 87 (plus dec 87)
#   3  Aug 87-present (excl dec 87)
#
# Field  Data
# Name   Period Name 
#  pop       1  Outgoing Longwave Radiation (OLR) Precipitation 
#                  Index (OPI) precipitation estimate
#  pms     123  Multi-satellite precipitation
#  pga     123  Gauge precipitation
#  psg     123  Satellite/gauge precipitation
#  eop     1    Outgoing Longwave Radiation (OLR) Precipitation  
#                  Index (OPI) precipitation random error
#  ems     123  Multi-satellite precipitation random error
#  ega     123  Gauge precipitation random error
#  esg     123  Satellite/gauge precipitation random error
#  nga     123  Gauge precipitation number of gauges
#  pag      23  Adjusted GOES Precipitation Index (AGPI)  
#                  precipitation estimate
#  eag      23  Adjusted GOES Precipitation Index (AGPI)  
#                  precipitation random error
#  pop      2   Outgoing Longwave Radiation (OLR) Precipitation  
#                  Index (OPI) precipitation estimate
#  pgp      23  GOES Precipitation Index (GPI) precipitation  
#                  estimate
#  ngp      23  GOES Precipitation Index (GPI) image number
#  pse       3  SSM/I emission precipitation estimate
#  nse       3  SSM/I emission precipitation number of samples
#  pss       3  SSM/I scattering precipitation estimate
#  nss       3  SSM/I scattering precipitation number of samples
#  psc       3  SSM/I composite precipitation estimate
#  ssc       3  SSM/I composite precipitation source
#  nsc       3  SSM/I composite precipitation number of samples
#  pst       3  Merged SSMI/TOVS precipitation estimate
#  est       3  Merged SSMI/TOVS precipitation random error
#  sst       3  Merged SSMI/TOVS precipitation source
#  ptv       3  TOVS precipitation estimate
#
# GPCPcdr_ECPm_DiagPlots_Map.sh 2013 09 icdr precip
#
# may need to update the location of matlab - type "whereis matlab" at the unix command line
#
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 3/2016 Initial Version prepared for NCEI
#
#@*****

# get inputs 
yyyy=$1
mm=$2
filetype=$3

if [ $# -lt 3 ]; then
    echo "Incorrect number of inputs; need at least: yyyy mm filetype"
    exit 0
elif [ $# -eq 3 ]; then
    echo "producing plot of precip for $yyyy/$mm $filetype"
    variable='precip'
elif [ $# -eq 4 ]; then
   variable=$4
elif [ $# -gt 4 ]; then
    echo "Incorrect number of inputs; need: yyyy mm filetype variable"
    exit 0
fi



/usr/bin/matlab -nosplash -nodisplay -nodesktop -r "path(path,'GPCPcdr_m5/GPCPcdr_m5_DiagPlots/'); GPCPcdr_m5_DiagPlots_maps($yyyy,$mm,'$filetype','$variable',16); exit;" 


