#!/bin/sh
#@***h* GPCPcdr_ECPm_ProcessAll.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   GPCPcdr_ECPm_ProcessAll.sh
#
# PURPOSE
#   This routine produces the monthly GPCP NetCDF file.
#
# DESCRIPTION
# Executive Control Package - Monthly
# Code to process several years of data. This code
# should be used to reprocess monthly data for 
# the whole period. 
# This code calls GPCPcdr_ECPm_Process1month.sh
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 3/2016 Initial Version prepared for NCEI
#
#@*****




#############################################
# flags for processing

flag_m1_GetInputFiles=1
flag_GPCPcdr_m2_METH_SSMI=1
flag_GPCPcdr_m2_Adjust_Gauge=1
flag_GPCPcdr_m2_Grid_AIRS_1month=1
flag_GPCPcdr_m2_TOVS_AIRS=1
flag_GPCPcdr_m4_combMW=1
flag_GPCPcdr_m4_XOVER=1
flag_GPCPcdr_m4_corr2=1
flag_GPCPcdr_m4_fuse2=1
flag_copy_all_final=1



#############################################
# Establish months to be processed

# Start year and month 
yyyy1=2004

# End year and month
yyyy2=2015


# Scroll through all years and months
AllYears=`seq $yyyy1 $yyyy2`
AllMonths=`seq -f "%02g" 1 12`

for yyyy in $AllYears; do
    for mm in $AllMonths; do


	./GPCPcdr_ECPm_Process1month.sh $yyyy $mm $flag_m1_GetInputFiles $flag_GPCPcdr_m2_METH_SSMI $flag_GPCPcdr_m2_Adjust_Gauge $flag_GPCPcdr_m2_Grid_AIRS_1month $flag_GPCPcdr_m2_TOVS_AIRS $flag_GPCPcdr_m4_combMW $flag_GPCPcdr_m4_XOVER $flag_GPCPcdr_m4_corr2 $flag_GPCPcdr_m4_fuse2 $flag_copy_all_final

    done
done



