#!/bin/sh

# Code to test if data exists and email when it does
# currently just checks for IR and SSMI data

# get current year and month for testing
yyyy=`date -d 'last month' '+%Y'`
mm=`date -d 'last month' '+%m'`
ssmi='F17'


# read test results file to see if data already exists
testfile=`cat testresults.txt` 
testyyyy=$(echo $testfile | awk -F,  '{print $1}')
testmm=$(echo $testfile | awk -F,  '{print $2}')
testir=$(echo $testfile | awk -F,  '{print $3}')
testrss=$(echo $testfile | awk -F,  '{print $4}')

# if month or year are not current, then reset testyyyy and testmm
if [ $yyyy != $testyyyy ] || [ $mm != $testmm ]; then
  testir=0
  testrss=0
  echo "month/year different - resetting for new month"
fi

# test for IR data
if [ $testir -ne 1 ]; then
  rm -r index.html
  wget ftp.cpc.ncep.noaa.gov/precip/xie/pickup/gpcp/$yyyy/$yyyy$mm/
  lines=`cat index.html`
  if echo "$lines" | grep -q "hr.$yyyy$mm"; then
    ir_exists=1
  else
    ir_exists=0
  fi
else
  ir_exists=1
fi


# test for RSS data at RSS Cloud service
if [ $testrss -ne 1 ]; then
  rm -r index.html
#  wget ftp://noaa.ncdc:ocean7.glowingglobe@va.remss.com/F17/
# wget ftp://ftp2.remss.com/ssmis/tb/cdr/F17_bad37V/
  wget ftp://ftp2.remss.com/ssmis/tb/cdr/F17/
  lines=`cat index.html`
  if echo "$lines" | grep -q "RSS_SSMIS_FCDR_V07R00_${ssmi}_D${yyyy}${mm}.tar"; then
    rss_exists=1
  else
    rss_exists=0
  fi
else
  rss_exists=1
fi


# if new data is available, email jjwang@umd.edu
sendemail=0
if [ $testir -ne $ir_exists ]; then
  sendemail=1
fi
if [ $testrss -ne $rss_exists ]; then
  sendemail=1
fi
if [ `date '+%H'` -eq 16 ]; then
  sendemail=1
fi

# report status
echo "status: IR=$ir_exists (was $testir); RSS=$rss_exists (was $testrss)"
echo "sendemail flag=$sendemail"

# send email if above conditions are met
if [ $sendemail -eq 1 ]; then
  echo "IR data availability: $ir_exists (was $testir); RSS data availability: $rss_exists (was $testrss)" | mailx -s "GPCP ICDR data update" jjwang@umd.edu 
fi


# Update test results files for next time
echo $yyyy,$mm,$ir_exists,$rss_exists > testresults.txt




