#/bin/sh
# Code to check input data exist and print summaries
#
#  Run from CODEPATH:
# Chk_GPCPcdr_1month.sh 2013 02

yyyy=$1
mm=$2


# check dates are valid
if [ $yyyy -lt 1979 ]
then
    echo "Invalid year - GPCP starts in 1979/01. Exiting..."
    exit
fi
if [  $mm -lt 1 -o $mm -gt 12 ]
then
    echo "Month must be between 1 and 12. Exiting..."
    exit
fi


# print header
echo ""
echo "############################################"
echo " Printing summary of datasets for $yyyy/$mm"

# get file directories
InputPATH=$(echo `grep "InputPATH" OptionsM.h` | awk -F\"  '{print $2}')
InterimPATH=$(echo `grep "InterimPATH" OptionsM.h` | awk -F\"  '{print $2}')
GPCCPATH=$(echo `grep "GPCCPATH" OptionsM.h` | awk -F\"  '{print $2}')
GPCCnrtPATH=$(echo `grep "GPCCPATH" OptionsM_NRT.h` | awk -F\"  '{print $2}')

# get 2-digit year
yy=`echo $yyyy | awk '{printf "%2.2d",substr($yyyy,3,2)}'`


# get number of days in month
if [  $mm -eq 1 -o $mm -eq 3 -o $mm -eq 5 -o $mm -eq 7 -o $mm -eq 8 -o $mm -eq 10 -o $mm -eq 12 ]
then
  DaysInMon=31
elif [ $mm -eq 4 -o $mm -eq 6 -o $mm -eq 9 -o $mm -eq 11 ]
then
  DaysInMon=30
elif [ $mm -eq 2 ]
then
  if [ `expr $yyyy % 4` -eq 0 ]
  then
    DaysInMon=29
  else
    DaysInMon=28
  fi
fi



###########################                                                                               # Check RSS SSMI/SSMIS data
NssmiFiles=`expr $DaysInMon \* 14`

# F17
if [ $yyyy$mm -ge 200901 ]; then
    echo ""
    if [ ! -d "${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/F17/${yyyy}/${mm}" ]; then
        echo "RSS SSMIS:    Directory does not exist: ${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/F17/${yyyy}/${mm}/"
    else
        lines=`ls -1shSr ${InputPATH}/SSMIS_RSS_NCDCcdr_TbV7/F17/${yyyy}/${mm}/`
        nlines=`echo "$lines" | wc -l`
        nlines=`expr $nlines - 1`
        min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
        max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "RSS SSMIS:    $nlines (expect ~ $NssmiFiles files); min: $min, max: $max"
    fi

# F13
elif [ $yyyy$mm -ge 199506 ]; then
    echo ""
    if [ ! -d "${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F13/${yyyy}/${mm}" ]; then
        echo "RSS SSMI F13:  Directory does not exist: ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F13/${yyyy}/${mm}/"
    else
        lines=`ls -1shSr ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F13/${yyyy}/${mm}/`
        nlines=`echo "$lines" | wc -l`
        nlines=`expr $nlines - 1`
        min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
        max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "RSS SSMI F13: $nlines (expect ~ $NssmiFiles files); min: $min, max: $max"
    fi

# F11
elif [ $yyyy$mm -ge 199201 ]; then
    echo ""
    if [ ! -d "${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F11/${yyyy}/${mm}" ]; then
	echo "RSS SSMI F11:  Directory does not exist: ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F11/${yyyy}/${mm}/"
    else
        lines=`ls -1shSr ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F11/${yyyy}/${mm}/`
        nlines=`echo "$lines" | wc -l`
        nlines=`expr $nlines - 1`
min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
        max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
        echo "RSS SSMI F11: $nlines (expect ~ $NssmiFiles files); min: $min, max: $max"
    fi

# F08
elif [ $yyyy$mm -ge 198708 ]; then
    echo ""
    if [ ! -d "${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F08/${yyyy}/${mm}" ]; then
	echo "RSS SSMI F08:  Directory does not exist: ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F08/${yyyy}/${mm}/"
    else
        lines=`ls -1shSr ${InputPATH}/SSMI_RSS_NCDCcdr_TbV6/F08/${yyyy}/${mm}/`
        nlines=`echo "$lines" | wc -l`
        nlines=`expr $nlines - 1`
	min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
        max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
        echo "RSS SSMI F08: $nlines (expect ~ $NssmiFiles files); min: $min, max: $max"
    fi
    
fi


###########################
# Check Ferraro data exists for current month

if [ $yyyy -gt 1987 ]
then
    echo ""
    
    # confirm all files are present
    if [ ! -f "${InputPATH}/Ferraro/PR1.MON" ]
    then
	echo "File missing: ${InputPATH}/Ferraro/PR1.MON"
    fi
    if [ ! -f "${InputPATH}/Ferraro/PR2.MON" ]
    then
	echo "File missing: ${InputPATH}/Ferraro/PR2.MON"
    fi
    if [ ! -f "${InputPATH}/Ferraro/SSA.MON" ]
    then
	echo "File missing: ${InputPATH}/Ferraro/SSA.MON"
    fi
    if [ ! -f "${InputPATH}/Ferraro/PF1.MON" ]
    then
	echo "File missing: ${InputPATH}/Ferraro/PF1.MON"
    fi
    if [ ! -f "${InputPATH}/Ferraro/PF2.MON" ]
    then
	echo "File missing: ${InputPATH}/Ferraro/PF2.MON"
    fi
    if [ ! -f "${InputPATH}/Ferraro/SNW.MON" ]
    then
	echo "File missing: ${InputPATH}/Ferraro/SNW.MON"
    fi
    if [ ! -f "${InputPATH}/Ferraro/ICE.MON" ]
    then
	echo "File missing: ${InputPATH}/Ferraro/ICE.MON"
    fi
    
   # check that the rainfall is non-zero
    sum=`GPCPcdr_m1/GPCPcdr_m1_ChkL1files_Ferraro $yyyy $mm`
    if [ $sum -gt 0 ]
    then
	echo "Ferraro:      sum of data for $yyyy/$mm = $sum (data exists)"
    else
	echo "Ferraro:      sum of data for $yyyy/$mm = $sum (missing data)"
    fi
fi





###########################
# Check AIRS/TOVS data

# TOVS Data
if [ $yyyy$mm -ge 198708 ] && [ $yyyy$mm -le 200504 ] && [ $yyyy$mm -ne 200402 ]
then
    echo ""
    if [ -f "${InputPATH}/TOVS/monthlies.rain.${yyyy}.am.pm.avg" ]
    then
	fsize=`du -b ${InputPATH}/TOVS/monthlies.rain.${yyyy}.am.pm.avg | awk -F" "  '{print $1}'`
	echo "TOVS:         File size: $fsize / 3110496"
    else
	echo "TOVS:         File does not exist for $yyyy"
    fi
fi

# V4 Data
if [ $yyyy$mm -ge 200505 ] && [ $yyyy$mm -le 200708 ] || [ $yyyy$mm -eq 200402 ]
then
    echo ""
    if [ ! -d "${InputPATH}/AIRS_V4_1DD/${yyyy}/${mm}" ]; then
	echo "AIRS V4 1DD:  Directory does not exist: ${InputPATH}/AIRS_V4_1DD/${yyyy}/${mm}"
    else
	lines=`ls -1shSr ${InputPATH}/AIRS_V4_1DD/${yyyy}/${mm}/`
	nlines=`echo "$lines" | wc -l`
	nlines=`expr $nlines - 1`
	min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
	max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "AIRS V4 1DD:  $nlines / $DaysInMon files; min: $min, max: $max"
    fi
fi

# V5 Data
if [ $yyyy$mm -ge 200709 ] && [ $yyyy$mm -le 201302 ]
then
    echo ""
    if [ ! -d "${InputPATH}/AIRS_V5/${yyyy}/${mm}" ]; then
        echo "AIRS V5:      Directory does not exist: ${InputPATH}/AIRS_V5/${yyyy}/${mm}/"
    else
	lines=`ls -1shSr ${InputPATH}/AIRS_V5/${yyyy}/${mm}/`
	nlines=`echo "$lines" | wc -l`
	nlines=`expr $nlines - 1`
	min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
	max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "AIRS V5:      $nlines / $DaysInMon files; min: $min, max: $max"
    fi
fi

# V6 Data
if [ $yyyy$mm -ge 201303 ] 
then
    echo ""
    if [ ! -d "${InputPATH}/AIRS_V6/${yyyy}/${mm}" ]; then
        echo "AIRS V6:      Directory does not exist: ${InputPATH}/AIRS_V6/${yyyy}/${mm}/"
    else
	lines=`ls -1shSr ${InputPATH}/AIRS_V6/${yyyy}/${mm}/`
	nlines=`echo "$lines" | wc -l`
	nlines=`expr $nlines - 1`
	min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
	max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "AIRS V6:      $nlines / $DaysInMon files; min: $min, max: $max"
    fi
fi



################################
# Check IR data

# Annual data
if [ $yyyy$mm -ge 198601 -a $yyyy$mm -le 199612 ]
then
    echo ""

    # set fname based on period
    if [ $yyyy$mm -ge 198601 -a $yyyy$mm -le 198707 ] || [ $yyyy$mm -eq 198712 ]
    then
	fname=${InputPATH}/mergIR_annual_files/m_geo.${yy}
	tgtsize="12110116"
    elif [ $yyyy$mm -ge 198708 -a $yyyy$mm -le 198711 ] || [ $yyyy$mm -ge 199101 -a $yyyy$mm -le 199601 ]
    then
	fname=${InputPATH}/mergIR_annual_files/m_geo3-da.${yy}
	tgtsize="86787948"
    elif [ $yyyy$mm -ge 198801 -a $yyyy$mm -le 199012 ]
    then
	fname=${InputPATH}/mergIR_annual_files/m_geo3.${yy}
	tgtsize="86788532"
    fi
    
    if [ -f $fname ]
    then
        fsize=`du -b $fname | awk -F" "  '{print $1}'`
        echo "Annual IR:    File size: $fsize / $tgtsize"
    else
        echo "Annual IR:    File does not exist: $fname"
    fi
fi

# 3hrly data
if [ $yyyy -ge 1996 ]
then
    echo ""
    if [ ! -d "${InputPATH}/mergIR_3hr_1deg/${yyyy}/${mm}" ]; then
	echo "3hrly IR:     Directory does not exist: ${InputPATH}/mergIR_3hr_1deg/${yyyy}/${mm}"
    else
	lines=`ls -1shSr ${InputPATH}/mergIR_3hr_1deg/${yyyy}/${mm}/hr*`
	nlines=`echo "$lines" | wc -l`
	min=`echo "$lines" | head -n 2 | tail -n 1 | awk -F" "  '{print $1}'`
	max=`echo "$lines" | tail -n 1 | awk -F" "  '{print $1}'`
	echo "3hrly IR:     $nlines / $DaysInMon files; min: $min, max: $max"
    fi
fi



######################
# OPI data

if [ $yyyy -ge 1979 -a $yyyy -le 1987 ]
then
    echo ""
    if [ -f "${InputPATH}/OPI_le/opi.${yyyy}${mm}.rfix" ]
    then
        fsize=`du -b "${InputPATH}/OPI_le/opi.${yyyy}${mm}.rfix" | awk -F" "  '{print $1}'`
        echo "OPI:          File size: $fsize / 41472"
    else
        echo "OPI:          File does not exist: $fname"
    fi
fi


######################
# GPCC Gauge data
echo ""

# GPCC Guage:
if [ $yyyy -le 2014 ]
then
  if [ -f "${GPCCPATH}/full_data_v7_25.nc" ]
  then
    fsize=`du -b "${GPCCPATH}/full_data_v7_25.nc" | awk -F" "  '{print $1}'`
    echo "GPCC Full:    File size: $fsize / 112488088"
  fi
elif [ -f "${GPCCPATH}/monitoring_v5_25_${yyyy}_${mm}.nc" ]
then
    fsize=`du -b "${GPCCPATH}/monitoring_v5_25_${yyyy}_${mm}.nc" | awk -F" "  '{print $1}'`
    echo "GPCC Monit.:  File size: $fsize / 296544"
else
    echo "GPCC:         File does not exist for $yyyy/$mm"
fi

# NRT Gauge
if [ $yyyy -ge 2014 ]
then
    if [ -f "${GPCCnrtPATH}/first_guess_monthly_${yyyy}_${mm}.nc" ]
    then
	fsize=`du -b "${GPCCnrtPATH}/first_guess_monthly_${yyyy}_${mm}.nc" | awk -F" "  '{print $1}'`
	echo "GPCC First:   File size: $fsize / 526212"
    else
	echo "GPCC First:   No First Guess file for $yyyy/$mm"
    fi
fi


# Print footer
echo ""
echo " End of Summary"
echo "############################################"