      program GPCPcdr_m2_Adjust_Gauge
C     Code to apply wind loss correction to GPCC
C     Also reformats and creates interim files for m4 steps
C     based on Gengaugf.f
c
c     Change Log:
c     Sep 24, 2015; Sapiano: split read code into separate subroutine
c                            and added ability to read netCDF 
c                            V7/Monitoring V5 files
c
C     Input command-line arguments:
C
C     cmonth    CHAR    month of data to be processed
C     cyear     CHAR    year of data to be processed
C     cinfname  CHAR    input monthly gauge file name (full name)
C     binpath   CHAR    output root directory
C     cbflag    CHAR    flag specifying application of bias adj
C                       ('Y' = apply adjustment,
C                        'N' = no adjustment)
C     csflag    CHAR    flag specifying source of GPCC data
C                       ('F' = full product,
C                        'M' = monitoring product,
C                        'G' = First guesss)



      IMPLICIT NONE


      INTEGER*4 inunit, icount, nskip, i, j, length, ios, ibias,
     +     ibunit, imonth, iyear, itfact
      INTEGER*4 ioutunit, nin, iret

      INTEGER*4 IMRTODY(12)/31,28,31,30,31,30,31,31,30,31,30,31/
     
      REAL*4            gauge(144,72), gaugen(144,72), gaugeb(144,72),
     +     cgauge(144,72), tempgb(144,72)
     
      CHARACTER*1       cbflag, csflag, cdummy
      CHARACTER*2       cmonth
      CHARACTER*4       cyear
      CHARACTER*256     cinfname, ogfname, ognfname, biasfnam, binpath
     
      LOGICAL           opn
      integer*4     Ninputs, strlen

C     Set the location of the bias files
      CHARACTER*45  biaspath
      DATA biaspath /'GPCPcdr_m2/GPCPcdr_m2_Adjust_Gauge/Ancillary/'/



c     ******************************************************
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 6 ) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        ' Incorrect number of input arguments (should be 6)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, cyear)
      call getarg(2, cmonth)
      call getarg(3, cinfname)
      call getarg(4, binpath)
      call getarg(5, cbflag)
      call getarg(6, csflag)

c     Convert month and year to numbers
      read(cmonth, '(i10)') imonth
      read(cyear, '(i10)' ) iyear




C***********************************************************************
c     Read GPCC data
c     GPCC Full V6 and Mon V4 (used in V2.2) used ascii format
C      call Read_GPCCv6_ASCII(cinfname, csflag, gauge, gaugen)

c     GPCC Full V7 and Mon V5 (used in V2.3) use NetCDF
      if (csflag .EQ. 'M' .OR. csflag .EQ. 'F') then
         call Read_GPCCv7_NetCDF(cinfname, csflag, iyear, imonth, 
     +        gauge, gaugen)

c     Read 1deg GPCC First guess data from NetCDF file and convert to 2.5deg
      else if (csflag .EQ. 'G') then
         call Read_GPCC_FG_NetCDF(cinfname, iyear, imonth,
     +        gauge, gaugen)
      else
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'invalid data type flag (csflag)'
         STOP
      endif





C***********************************************************************
C     If specified, retrieve the rain gauge bias file.  If not 
C     specified, assign a bias value of 1.0.
C
      IF  (cbflag .EQ. 'Y') THEN
C     
         biasfnam = biaspath//'error02.a'//cmonth
c   
C        check for a free iounit
         ibunit = 0
         opn = .TRUE.
         DO WHILE (opn)
            ibunit = ibunit + 1
            INQUIRE(ibunit, OPENED = opn)
         END DO
         
         OPEN (ibunit, FILE = biasfnam, ACCESS = 'SEQUENTIAL', 
     +        FORM = 'FORMATTED', IOSTAT = ios)
         if ( ios .NE. 0 ) then
            WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &           'Failed to open input bias file ', biasfnam
            STOP
         end if
C     
         READ (ibunit, 200, IOSTAT = ios) cdummy
         if ( ios .NE. 0 ) then
            WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &           'Failed to read input bias file ', biasfnam
            STOP
         end if
C     
         DO j = 1, 72
            DO i = 1, 144
               READ (ibunit, 201, IOSTAT = ios) ibias
               if ( ios .NE. 0) then                  
                  WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &                 'Failed to read input bias file ', biasfnam
                  STOP
               end if
               
               IF  (ibias .GT. 0) THEN
                  gaugeb(i,j) = ibias * 0.001
                  IF  (gaugeb(i,j) .GT. 3.0) THEN
                     gaugeb(i,j) = 3.0
                  END IF
               ELSE
                  gaugeb(i,j) = 1.0
               END IF
            END DO
         END DO
c
         CLOSE (ibunit, IOSTAT = ios)
         if ( ios .NE. 0 ) then
            WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &           'Failed to close input bias file ', biasfnam
            STOP
         end if
C     
         DO j = 1, 72
            DO i = 1, 144
               IF  (i .LE. 72) THEN
                  tempgb(i,j) = gaugeb(i + 72, j)
               ELSE
                  tempgb(i,j) = gaugeb(i - 72, j)
               END IF
            END DO
         END DO
C     
         DO i = 1, 144
            DO j = 1, 72
               gaugeb(i,j) = tempgb(i,j)
            END DO
         END DO
C     
      ELSE
C     
         DO i = 1, 144
            DO j = 1, 72
               gaugeb(i,j) = 1.0
            END DO
         END DO
C     
      END IF
C     
C***********************************************************************
C     Apply the gauge bias.
C     
      DO i = 1, 144
         DO j = 1, 72
            cgauge(i,j) = gauge(i,j) * gaugeb(i,j)
         END DO
      END DO
      
      
     
      
C***********************************************************************
C     Compute the gauge quality as a surrogate for the gauge number.
C     
C     
      CALL ADJNG (imonth, iyear, gauge, cgauge, gaugen)
C     
C***********************************************************************
C     Convert the units of precipitation from mm/month to mm/day.
C     
      IF  (MOD ((iyear - 1996), 4) .NE. 0) THEN
         DO i = 1, 144
            DO j = 1, 72
               IF  (cgauge(i,j) .GT. -0.1) THEN
                  cgauge(i,j) = cgauge(i,j) / 
     +                 FLOAT (IMRTODY(imonth))
               END IF
            END DO
         END DO
      ELSE
         IF (imonth .EQ. 2) THEN
            itfact = 29
         ELSE
            itfact = IMRTODY(imonth)
         END IF
         DO i = 1, 144
            DO j = 1, 72
               IF  (cgauge(i,j) .GT. -0.1) THEN
                  cgauge(i,j) = cgauge(i,j) / FLOAT (itfact)
               END IF
            END DO
         END DO
      END IF


C***********************************************************************
C     Create the gauge analysis and gauge number output file names and
C     write the data to disk.
C     
      do strlen = 1,len_trim(binpath)
         if (binpath(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1

c     Save first output file: cgauge
      ogfname = binpath(1:strlen)//'oc.'//cyear//cmonth//'.corg'
      call Write_Binary_File(ogfname, cgauge, 144, 72)

c     Save second output file: gaugen
      ognfname = binpath(1:strlen)//'oc.'//cyear//cmonth//'.qga'
      call Write_Binary_File(ognfname, gaugen, 144, 72)



c     Format statements
 200  FORMAT (A1)
 201  FORMAT (9X,I5)


      end program GPCPcdr_m2_Adjust_Gauge

