      subroutine Read_GPCC_FG_NetCDF(cinfname, iyear, imonth,
     +      gauge, gaugen)
c     Code to read GPCC Fist Guess from NetCDF and degrade to 2.5 degree resolution
c
c     Inputs:
c       cinfname   Name of input file
c       gauge      GPCC gauge precip
c       gaugen     GPCC number of gauges

      implicit none
      include 'netcdf.inc'

      CHARACTER*256     cinfname
      integer iyear, imonth

      integer i, j, k1, k2, Nsum, NsumN
      REAL*4  gauge(144,72), gaugen(144,72)
      REAL*4 gpcc_0p5(720,360), gpccN_0p5(720,360)
      REAL*4 gpcc_1p0(360,180), gpccN_1p0(360,180)
      REAL*4 gpcc_2p5(144,72), gpccN_2p5(144,72)
      REAL*4 sum, sumN


      integer status, ncid, ncid_p, ncid_s
      integer start(3), count(3)

      data start /1, 1, 1/    ! start at first value
      data count /360, 180, 1/ ! orientation is lon, lat, time




c     Open file
      status=NF_OPEN(cinfname, 0, ncid)
      if (status .NE. NF_NOERR)  then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'Could not open NetCDF file'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif
      
      
c     Get handle for precip and read
      status=NF_INQ_VARID (ncid, 'p', ncid_p)
      if (status .NE. NF_NOERR) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'Could not read NetCDF handle for gauge'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif
      
      status=NF_GET_VARA_REAL (ncid, ncid_p, start, count, gpcc_1p0)
      if (status .NE. NF_NOERR)  then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'Could not read NetCDF gauge'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif
      
c     Get handle for number of stations and read
      status=NF_INQ_VARID (ncid, 's', ncid_s)
      if (status .NE. NF_NOERR) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'Could not read NetCDF handle for gaugen'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif
      
      status=NF_GET_VARA_REAL (ncid, ncid_s, start, count, gpccN_1p0)
      if (status .NE. NF_NOERR) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'Could not read NetCDF gaugen'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif
      
      
c     Close file
      status=NF_CLOSE(ncid)
      if (status .NE. NF_NOERR) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'Could not close NetCDF file'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif
      




c*****************************************************
c     re-grid 1 deg to 2.5 deg


c     First, sub sample to .5 degree
      do i=1,360
         do j=1,180
            gpcc_0p5(i*2-1,j*2-1)=gpcc_1p0(i,j)
            gpcc_0p5(i*2-1,j*2)=gpcc_1p0(i,j)
            gpcc_0p5(i*2,j*2-1)=gpcc_1p0(i,j)
            gpcc_0p5(i*2,j*2)=gpcc_1p0(i,j)
            gpccN_0p5(i*2-1,j*2-1)=gpccN_1p0(i,j)
            gpccN_0p5(i*2-1,j*2)=gpccN_1p0(i,j)
            gpccN_0p5(i*2,j*2-1)=gpccN_1p0(i,j)
            gpccN_0p5(i*2,j*2)=gpccN_1p0(i,j)
         end do
      end do


c     Now average the .5 to 2.5
      do i=1,144
         do j=1,72
            
            sum=0
            Nsum=0
            sumN=0
            NsumN=0

            do k1=1,5
               do k2=1,5
                  if (gpcc_0p5(i*5-5+k1,j*5-5+k2).GE.0) then
                     sum=sum+gpcc_0p5(i*5-5+k1,j*5-5+k2)
                     Nsum=Nsum+1
                     sumN=sumN+gpccN_0p5(i*5-5+k1,j*5-5+k2)
                     NsumN=NsumN+1
                  end if
               end do
            end do
            
            if (Nsum .GT. 0) then
               gpcc_2p5(i,j)=sum/Nsum
               gpccN_2p5(i,j)=sumN/NsumN
            end if
         end do
      end do
      


C     Translate the data to match the GPCC coordinate system.
c     Also use this step to gor from 3-dimensional inputs to 2d outputs
      DO j = 1, 72
         DO i = 1, 144
            IF  (i .LE. 72) THEN
               gauge(i,j) = gpcc_2p5(i + 72, j)
               gaugen(i,j) = gpccN_2p5(i + 72, j)
            ELSE
               gauge(i,j) = gpcc_2p5(i - 72, j)
               gaugen(i,j) = gpccN_2p5(i - 72, j)
            END IF
            IF (gauge(i,j) .LT. 0) THEN
               gauge(i,j)=-99999.99
               gaugen(i,j)=-99999.99
            END IF
         END DO
      END DO






      end subroutine Read_GPCC_FG_NetCDF



