      subroutine Read_GPCCv7_NetCDF(cinfname, csflag, iyear, imonth,
     +      gauge, gaugen)
c     Code to read GPCC Full V7 and Monitoring V5 data from NetCDF
c
c     Inputs:
c       cinfname   Name of input file
c       gauge      GPCC gauge precip
c       gaugen     GPCC number of gauges

      implicit none
      include 'netcdf.inc'

      CHARACTER*256     cinfname
      CHARACTER*1       csflag
      integer iyear, imonth

      integer i, j
      REAL*4  gauge(144,72), gaugen(144,72), 
     +     tmpgauge(144,72), tmpgaugen(144,72)

      integer status, ncid, ncid_p, ncid_s
      integer start(3), count(3)

      data start /1, 1, 1/    ! start at first value
      data count /144, 72, 1/ ! orientation is lon, lat, time



c     FUll V7 data is in a single file, so adjust start to read desired month
      if (csflag .EQ. 'F') then

c       Figure out field number to read based on month and year
c       First field is Jan 1901, last is Dec 2013 (index=113*12=1356)
        start(3)=(iyear-1901)*12+imonth
        if (start(3) .LT. 1 .OR. start(3) .GT. 1356) then
           WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &                 'Index for Netcf file is out of bounds'
           STOP
        endif
       end if






c     Open file
      status=NF_OPEN(cinfname, 0, ncid)
      if (status .NE. NF_NOERR)  then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &               'Could not open NetCDF file'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif


c       Get handle for precip and read
        status=NF_INQ_VARID (ncid, 'p', ncid_p)
        if (status .NE. NF_NOERR) then
           WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &                 'Could not read NetCDF handle for gauge'
           PRINT *, NF_STRERROR(STATUS)
           STOP
        endif

        status=NF_GET_VARA_REAL (ncid, ncid_p, start, count, tmpgauge)
        if (status .NE. NF_NOERR)  then
           WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &                 'Could not read NetCDF gauge'
           PRINT *, NF_STRERROR(STATUS)
           STOP
        endif

c       Get handle for number of stations and read
        status=NF_INQ_VARID (ncid, 's', ncid_s)
        if (status .NE. NF_NOERR) then
           WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &                 'Could not read NetCDF handle for gaugen'
           PRINT *, NF_STRERROR(STATUS)
           STOP
        endif

        status=NF_GET_VARA_REAL (ncid, ncid_s, start, count, tmpgaugen)
        if (status .NE. NF_NOERR) then
           WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &                 'Could not read NetCDF gaugen'
           PRINT *, NF_STRERROR(STATUS)
           STOP
        endif


c     Close file
      status=NF_CLOSE(ncid)
      if (status .NE. NF_NOERR) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &               'Could not close NetCDF file'
         PRINT *, NF_STRERROR(STATUS)
         STOP
      endif





C     Translate the data to match the GPCC coordinate system.
c     Also use this step to gor from 3-dimensional inputs to 2d outputs
      DO j = 1, 72
         DO i = 1, 144
            IF  (i .LE. 72) THEN
               gauge(i,j) = tmpgauge(i + 72, j)
               gaugen(i,j) = tmpgaugen(i + 72, j)
            ELSE
               gauge(i,j) = tmpgauge(i - 72, j)
               gaugen(i,j) = tmpgaugen(i - 72, j)
            END IF
            IF (gauge(i,j) .LT. 0) THEN
               gauge(i,j)=-99999.99
               gaugen(i,j)=-99999.99
            END IF
         END DO
      END DO






      end subroutine Read_GPCCv7_NetCDF


