	SUBROUTINE ADJNG (imonth, iyear, gauge, cgauge, gaugen)
C************************************************************************
C* ADJNG  Compute the gauge quality					*
C*									*
C* Compute the gauge quality from the precipitation and population, and *
C* replace the population values with the quality estimate.  First,     *
C* compute the gauge sampling error. Second, compute the gauge quality	*
C* based on the sampling error.						*
C*									*
C* Input parameters:							*
C*									*
C*   imonth	INT	month of data to be processed			*
C*   iyear	INT	year of data to be processed			*
C*   gauge	REAL	month of gauge analysis data			*
C*   cgauge	REAL	month of bias-adjusted gauge analysis data	*
C*   gaugen	REAL	month of gauge population data			*
C*									*
C* Output parameters:							*	
C*									*
C*   gaugen	REAL	month of gauge quality data			*
C*									*
C**									*
C* Log:									*
C* D.Bolvin/SSAI	10/98	First version				*
C************************************************************************
C
	IMPLICIT	NONE
C
	INTEGER*4	i, j, imonth, iyear, LN/144/, LT/72/
	INTEGER*4	ihaveg(144,72)
	INTEGER*4	imon2hr(12)/744,672,744,720,744,720,744,744,720,
     +				    744,720,744/
C
	REAL*4		gnorth, gsouth, clos, hr, ebias, H/0.0075/, 
     +			S/8.0/
	REAL*4		gauge(144,72), gaugen(144,72), gaugeerr(144,72),
     +			cgauge(144,72)
C
C***********************************************************************
C	Compute gauge error with one field.  Gaugen is in number of
C	gauges in the box, but due to the analysis technique the "true"
C	number of gauges should reflect the presence of gauges in 
C	surrounding boxes.  This influence is crudely accounted for
C	by using ( gaugen + clos / 4 ) as the number of gauges, where
C	clos is the number of surrounding boxes with at least one
C	gauge.  Bias error (due to using the Legates climatological
C	correction) is *not* handled here.  Start by finding clos; 
C	all boxes are neighbors at the poles.
C
	IF  (MOD ((iyear - 1978), 4) .NE. 0) THEN
	    hr = imon2hr(imonth)
	ELSE
	    IF  (imonth .EQ. 2) THEN
	    	hr = 696
	    ELSE
	    	hr = imon2hr(imonth)
	    END IF
	END IF
C
	DO j = 1, LT
	    DO i = 1, LN
		IF  (gaugen(i,j) .EQ. 0.0)  THEN
		    ihaveg(i,j) = 0
		ELSE
		    ihaveg(i,j) = 1
		END IF
	    END DO
	END DO
C
	gnorth = 0.0
	gsouth = 0.0
C
	DO i = 1, LN
	    gnorth = gnorth + ihaveg (i,1)
	    gsouth = gsouth + ihaveg (i,LT)
	END DO
C
C***********************************************************************
C	Check how many neighboring boxes have at least one gauge.
C	Boxes on the lateral boundaries are neighbors to the 
C	opposite side and the poles use gnorth and gsouth.
C
	DO j = 1, LT
	    DO i = 1, LN
		IF  (j .EQ. 1)  THEN
		    IF  (i .EQ. 1)  THEN
			clos = ihaveg (LN,1) + ihaveg (2,1) +
     +			       ihaveg (LN,2) + ihaveg (2,2) +
     +			       gnorth        + ihaveg (1,2)
		    ELSE IF  (i .EQ. LN)  THEN
			clos = ihaveg (LN-1,1) + ihaveg (1,1) +
     +			       ihaveg (LN-1,2) + ihaveg (1,2) +
     +			       gnorth          + ihaveg (LN,2)
		    ELSE
			clos = ihaveg (i-1,1) + ihaveg (i+1,1) +
     +			       ihaveg (i-1,2) + ihaveg (i+1,2) +
     +			       gnorth         + ihaveg (i,2)
		    END IF
		ELSE IF  (j .EQ. LT)  THEN
		    IF  (i .EQ. 1)  THEN
			clos = ihaveg (LN,LT-1) + ihaveg (2,LT-1) +
     +			       ihaveg (LN,LT)   + ihaveg (2,LT) +
     +			       ihaveg (1,LT-1)  + gsouth
		    ELSE IF  ( i .EQ. LN )  THEN
			clos = ihaveg (LN-1,LT-1) + ihaveg (1,LT-1) +
     +			       ihaveg (LN-1,LT)   + ihaveg (1,LT) +
     +			       ihaveg (LN,LT-1)   + gsouth
		    ELSE
			clos = ihaveg (i-1,LT-1) + ihaveg (i+1,LT-1) +
     +			       ihaveg (i-1,LT)   + ihaveg (i+1,LT) +
     +			       ihaveg (i,LT-1)   + gsouth
		    END IF
		ELSE IF  (i .EQ. 1)  THEN
		    clos = ihaveg (LN,j-1) + ihaveg (2,j-1) +
     +			   ihaveg (LN,j)   + ihaveg (2,j) +
     +			   ihaveg (LN,j+1) + ihaveg (2,j+1) +
     +			   ihaveg (1,j-1)  + ihaveg (1,j+1)
		  ELSE IF  (i .EQ. LN)  THEN
		    clos = ihaveg (LN-1,j-1) + ihaveg (1,j-1) +
     +			   ihaveg (LN-1,j)   + ihaveg (1,j) +
     +			   ihaveg (LN-1,j+1) + ihaveg (1,j+1) +
     +			   ihaveg (LN,j-1)   + ihaveg (LN,j+1)
		  ELSE
		    clos = ihaveg (i-1,j-1) + ihaveg (i+1,j-1) +
     +			   ihaveg (i-1,j)   + ihaveg (i+1,j) +
     +			   ihaveg (i-1,j+1) + ihaveg (i+1,j+1) +
     +			   ihaveg (i,j-1)   + ihaveg (i,j+1)
		END IF
C
C***********************************************************************
C	Set gauge error; missing if no estimate, or no gauges in the box
C	or nearby.
C
		IF  (((gaugen(i,j) + clos * 0.125) .LE. 0.0) .OR.
     +		     (cgauge(i,j) .LT. 0.0))  THEN
		    gaugeerr(i,j) = -9999.9
		  ELSE
		    gaugeerr(i,j) = ((S + cgauge(i,j)) * 
     +				    H * (1.0 * hr + 
     +				    10.0 * (hr * cgauge(i,j) ) ** 0.5 )/
     +				    (gaugen(i,j) + clos * 0.125)) ** 0.5
		END IF
C
C***********************************************************************
C	Adjust the estimated gauge error to 40% of the bias adjustment
C	if warranted.
C
	    	IF  (gaugeerr(i,j) .GT. 0.0) THEN
	    	    IF  (cgauge(i,j) .GT. 0.0) THEN
	    	    	ebias = 0.4 * ABS (cgauge(i,j) - gauge(i,j))
	    	    ELSE
	    	    	ebias = 0.0
	    	    END IF
	    	    gaugeerr(i,j) = AMAX1 (gaugeerr(i,j), ebias)
	    	END IF
C
C***********************************************************************
C	Set gauge quality.
C
		IF  (gaugeerr(i,j) .GT. 0.0)  THEN
		    gaugen(i, j) = 
     +		    (S + cgauge(i,j)) * H * 
     +		    (1.0 * HR + 10.0 * (hr * cgauge(i,j)) ** 0.5) / 
     +		    gaugeerr(i,j) ** 2
	      	ELSE
		    gaugen(i,j) = -9999.9
	    	END IF

	    END DO
	END DO
C
	RETURN
	END
