c     subroutine that aggregates adjusted, gridded AIRS data
c
c     Original header:
C* This program adds up input data files and writes out the average    *
C* precipitation (.rav), number of observations (.num), and percent    *
C* raining (.per) fields.  The standard 'coin' program couldn't be used*
C* because it doesn't save number-of-raining grid boxes, and thus .per,*
C* information.  The GPROF "g_bin6.0" programs couldn't be used        *
C* because they are tailored to receive GPROF orbit file format.       *

      subroutine GPCPcdr_m2_add_month(basedir, Ndays)

      IMPLICIT      NONE
      real*4        rain(360,180), per(360,180), sum(360,180),
     &              avg(360,180),ratio(180)
      integer*4     l_bd, iret, i, j
      integer*4     iday, Ndays, ios
      integer*4     num(360,180), numrain(360,180)
      character*2   cday
      character*256 basedir

c     read in the ratio to modify the AIRS IR only data
      open(unit=13,file='/home/jwang/GPCP_CDR/monthly/GPCPcdr_m2/GPCPcdr
     &_m2_Get_AIRS_TOVS/ratio_MWoverIR',status='unknown')
      do j = 1, 180
         read(13,111) ratio(j)
 111     format(f6.4)
      end do
      close(63)
c     get length of indir and outfile
      do l_bd = 1,len_trim(basedir)
         if (basedir(l_bd:l_bd) .eq. ' ') then
            continue
         end if
      end do
      l_bd=l_bd-1

c     initialize some arays
      do j = 1, 180
      do i = 1, 360
         sum(i,j) = 0.
         num(i,j) = 0
         numrain(i,j) = 0
         avg(i,j) = -0.1
         per(i,j) = -1.
      end do
      end do


c     Loop over days, construct filename and aggregate data
      do iday=1,Ndays
         
         do j = 1, 180
            do i = 1, 360
               rain(i,j)=-999.9
            end do
         end do
            
         write(cday, '(I2.2)' ) iday

         open(23,file=basedir(1:l_bd)//cday//'.avg',iostat=ios,
     &        form='unformatted',access='direct', recl=4*360)

         if ( ios .NE. 0 ) then
            print *, 'Error in GPCPcdr_m2_add_month.f:'//
     &           'Failed to open input file (', ios, ')'
            print *, basedir(1:l_bd)//cday//'.avg'
            continue
         else
            do j = 1, 180
               read(23,rec=j, iostat=ios) (rain(i,j), i=1,360)
               if ( ios .NE. 0 ) then
                  print *, 'Error in GPCPcdr_m2_add_month.f: could'//
     &                 'not read data for j=', j, ' (error=', ios, ')'
                  print *, basedir(1:l_bd)//cday//'.avg'
                  continue
               end if
            end do
c     
            do j = 1, 180
               do i = 1, 360
                  if (rain(i,j) .gt. 0.) then
                     sum(i,j) = sum(i,j) + rain(i,j)
                     numrain(i,j) = numrain(i,j) + 1
                     num(i,j) = num(i,j) + 1
                  else if (rain(i,j) .eq. 0.) then
                     num(i,j) = num(i,j) + 1
                  endif
               end do
            end do

         endif
c
         close(23)

      enddo


      do j = 1, 180
      do i = 1, 360
         if (num(i,j).gt.0) then
            avg(i,j) = sum(i,j)/float(num(i,j))*ratio(j)
            per(i,j) = 100. * float(numrain(i,j))/float(num(i,j))
         endif
      end do
      end do
c


C     Write the Average Precip
      open(24,file=basedir(1:l_bd)//'.rav',form='unformatted',
     &     access='direct',recl=4*64800,status='replace',iostat=ios)
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to open output file for Avg'
         STOP
      endif
      write(24,rec=1,iostat=ios) avg
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to write output file for Avg'
         STOP
      endif
      close(24,iostat=ios)
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to close output file for Avg'
         STOP
      endif


c     Write the %
      open(25,file=basedir(1:l_bd)//'.per',form='unformatted',
     &     access='direct',recl=4*64800,status='replace',iostat=ios)
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to open output file for %'
         STOP
      endif
      write(25,rec=1,iostat=ios) per
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to write output file for %'
         STOP
      endif
      close(25,iostat=ios)
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to close output file for %'
         STOP
      endif


C     Write N
      open(26,file=basedir(1:l_bd)//'.num',form='unformatted',
     &     access='direct',recl=4*64800,status='replace',iostat=ios)
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to open output file for N'
         STOP
      endif
      write(26,rec=1,iostat=ios) num
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to write output file for N'
         STOP
      endif
      close(26,iostat=ios)
      if ( ios .NE. 0 ) then
         write(*,*) 'Error in GPCPcdr_m2_add_month.f:'//
     &        'Failed to close output file for N'
         STOP
      endif



c      stop

      end subroutine GPCPcdr_m2_add_month
