c     PROTTYPE version... correction not applied!
c     subroutine that reads gridded daily AIRS and and calibrates to TOVS
c     temporary output to interim files
c
c     Original header:
C* This program reads in an AIRS daily 1x1-degree file and re-assigns  *
C* the precipitation value at various grid boxes to the corresponding  *
C* TOVS value via histogram matching.  Separate histograms are         *
C* maintained for:                                                     *
C*                  1. L = land, defined as 0-15% ocean grid boxes     *
C*                            (all latitudes)                          *
C*                  2. M = mixed, defined as 15-75% ocean grid boxes   *
C*                            (all latitudes)                          *
C*                  3. O = ocean, defined as 75-100% ocean grid boxes, *
C*                          and separately for:                        *
C*                             a) 60-90N  (O1)                         *
C*                             b) 30-60N  (O2)                         *
C*                             c) 30N-30S (O3)                         *
C*                             d) 30-60S  (O4)                         *
C*                             e) 60-90S  (O5)                         *
C*                                                                     *
C* The histogram files were generated with the program "comphist.f".   *
C*                                                                     *

      subroutine GPCPcdr_m2_airs2tovs (infile,outfile)

      implicit none
      character*256 infile, outfile

      real*4 airs_orig(360,180), airs_new(360,180)
      integer*4 l_if, l_of, iret
      integer*4 i, j, ios
C



c     get length of infile, outfile and histroot
      do l_if = 1,len_trim(infile)
         if (infile(l_if:l_if) .eq. ' ') then
            continue
         end if
      end do
      l_if=l_if-1
      do l_of = 1,len_trim(outfile)
         if (outfile(l_of:l_of) .eq. ' ') then
            continue
         end if
      end do



C Read in the input AIRS file. 
C
      open(21,file=infile(1:l_if),form='unformatted',status='old',
     &     access='direct',recl=4*64800, iostat = ios)

      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m2_airs2tovs:'//
     &        'Missing file (moving to next): ' // infile(1:l_if)
         RETURN
      end if
      
      read(21,rec=1) airs_orig
      close(21)


C Start by copying all values from "airs_orig" to "airs_new".  Then all
C grid boxes that aren't being altered by a histogram file (should be
C only grid boxes with missing values) will remain as is.
C
      do j = 1, 180
      do i = 1, 360
         airs_new(i,j) = airs_orig(i,j)
      end do
      end do



C Write out the resulting file.
C
      open(29,file=outfile(1:l_of),form='unformatted',status='replace',
     &     access='direct',recl=4*360)
      do j = 1, 180
         write(29,rec=j) (airs_new(i,j), i=1,360)
      end do
      close(29)


      return
      end

