C
C     Code to get TOVS/AIRS data
C     TOVS and AIRS must be processed separately, but the ouput 
C     files are the same format - annual binary files containing
C     1-deg monthly fields
C     TOVS data are simply copied into the annual file.
C     AIRS data are gridded in the code GPCPcdr_m2_Grid_AIRS
C     As of V2.3, calibration of AIRS to TOVS is done later 
C     when the data are sub sampled to 2.5-deg resolution
C     in XOVER
C
      program Main_GPCPcdr_m2_TOVS_AIRS



c     ******************************************************
c     Declarations

      IMPLICIT NONE

      integer*4     iyear, imonth, iday, Ndays
      integer*4     Ninputs, strlen, strlen1, ProcFlag
      real*4        tovsdata(144,72)
      character*1   cOverRideFlag
      character*4   cyear
      character*2   cmonth, cday
      CHARACTER*256 InterimPATH, InterimPath_orig, InputPATH
      CHARACTER*256 infile, outfile, histroot
      logical       ex
      integer*4     NdaysInMonth(12)
     &     / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/


c     ******************************************************
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .GT. 4 ) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Get_TOVS_AIRS:'//
     &        ' Incorrect number of input arguments (should be 6)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, cyear)
      call getarg(2, cmonth)
      call getarg(3, InputPath)
      call getarg(4, InterimPath)


c     Convert month and year to numbers
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth


c     get length of InterimPATH and InputPATH
      do strlen = 1,len_trim(InterimPath)
         if (InterimPath(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1

      do strlen1 = 1,len_trim(InputPath)
         if (InputPath(strlen1:strlen1) .eq. ' ') then
            continue
         end if
      end do
      strlen1=strlen1-1



c     ******************************************************
c     Setup parameters
c     Sapiano, Apr 2015: Transition form TOVS to AIRS was
c         changed so that TOVS is used until end 2002 with
c         AIRS V6 used thereafter. AIRS V6 is calibrated to
c         TOVS data using last ~6 months of 2002 (TOVS data 
c         appears unstable).  Note that the new correction
c         is no longer seasonal.


C     Set processing flag
      ProcFlag=0

c     TOVS period: 1987/08 to 2002/12 
      if ((iyear .EQ. 1987 .AND. imonth .GE. 8) .OR.
     &     (iyear .GE. 1988 .AND. iyear .LE. 2002)) then
         ProcFlag=1
      
c     AIRS V6 period: 2003/01 onwards
      elseif (iyear .GE. 2003) then
         ProcFlag=4
      endif



C     establish empty annual file if one does not already exist
C     If already exists, relevant months will be overwritten
      outfile=InterimPATH(1:strlen)//
     &        '/AIRS_TOVS/monthlies.rain.'//cyear//'.am.pm.avg'
      inquire(file=outfile, exist=ex)
      if ( .NOT. ex ) then
         call GPCPcdr_m2_make_empty_TOVS_AIRS(outfile)
      endif 


c     Determine number of days in month
      Ndays=NdaysInMonth(imonth)
      if ( imonth.EQ.2 ) then 
         if ( mod(iyear,400).EQ.0 ) then
            Ndays=29
         elseif ( mod(iyear,100).EQ.0 ) then
            Ndays=28
         elseif ( mod(iyear,4).EQ.0 ) then
            Ndays=29
         endif
      endif





c     ******************************************************
c     TOVS
c     Copy TOVS data from original format file to new file
      if (ProcFlag .EQ. 1) then
         write (*,*) 'Processing TOVS'
         infile=InputPATH(1:strlen1)//
     &        '/TOVS/monthlies.rain.'//cyear//'.am.pm.avg'
         call Read_Binary_File(infile,360,180,imonth,tovsdata)
         outfile=InterimPATH(1:strlen)//
     &        '/AIRS_TOVS/monthlies.rain.'//cyear//'.am.pm.avg'
         call Write_binary_TOVS_AIRS(outfile,tovsdata,360,180,imonth)


c     ******************************************************
c     AIRS V6
c     Calibrate AIRS and copy into annual file
      elseif (ProcFlag .EQ. 4) then
         write (*,*) 'Processing AIRS V6'
         histroot='Ancillary/V6_hists_test/'

C        This version does not do calibration, data are
c        just copied over - final version will need to be different!
c         do iday=1,Ndays
c            write(cday, '(I2.2)' ) iday
c            infile=InterimPATH_orig(1:strlen2)//'/AIRS_V6_1DD/'//cyear
c     &           //'/'//cmonth//'/3A_AIRS.V6L2G_1DD.'
c     &           //cyear//cmonth//cday//'.avg'
c            outfile=InterimPATH(1:strlen)//
c     &           '/AIRS_tmp/AIRS_HIST.'//cyear//cmonth//cday
cc            call GPCPcdr_m2_airs2tovs (infile,outfile,seas,histroot,6)
c            call GPCPcdr_m2_airs2tovs (infile,outfile)
c         enddo

c        Aggregate daily data to monthly file
         infile=InterimPATH(1:strlen)//'/AIRS_V6_1DD/'//cyear
     &        //'/'//cmonth//'/3A_AIRS.V6L2G_1DD.'//cyear//cmonth 
c         infile=InterimPATH(1:strlen)//
c     &     '/AIRS_tmp/AIRS_HIST.'//cyear//cmonth
         call GPCPcdr_m2_add_month(infile, Ndays)


c        Copy data for available months into TOVS/AIRS file
c         infile=InterimPATH(1:strlen)//
c     &        '/AIRS_tmp/AIRS_HIST.'//cyear//cmonth//'.rav'
         infile=InterimPATH(1:strlen)//'/AIRS_V6_1DD/'//cyear
     &        //'/'//cmonth//'/3A_AIRS.V6L2G_1DD.'
     &        //cyear//cmonth//'.rav'
         outfile=InterimPATH(1:strlen)//
     &        '/AIRS_TOVS/monthlies.rain.'//cyear//'.am.pm.avg'
         call GPCPcdr_m2_AIRS_Add(infile,outfile,cmonth)


c     Failure option for the processing flag
      else
         write (*,*) 'Could not interpret ProcFlag:'//
     &        ' No TOVS/AIRS data for this period?'
         return
      endif


      end program Main_GPCPcdr_m2_TOVS_AIRS
