C***********************************************************************
C* program airsV6_2_tovshist                                         *
C* COPY of /snow/nelkin/AIRS/vs_tovshist/airsV6_2_tovshist.f, which*
C* was itself was a COPY of /snow/nelkin/AIRS/vs_airsV5orig/           *
C* airsV6_2_airsV5.f, but here a direct calibration against TOVS is    *
C* done, now that V6 AIRS L2G for 2004 is available.                   *
C* Modified from /rain/nelkin/airs/vs_tovs/HIST4_V5/airs2tovs.f        *
C* NOTE: In this code, "airs" = AIRS V6 and "tovs" = TOVS              *
C*                                                                     *
C* ifort -o airs2tovs airs2tovs.f swapr4.f st_lib.a                    *
C*                                                                     *
C* This program reads an AIRS V6 daily 1x1-degree file and re-assigns  *
C* the precipitation value at various grid boxes to the corresponding  *
C* TOVS value via histogram matching.  Separate histograms are         *
C* maintained for:                                                     *
C*                  1. L = land, defined as 0-15% ocean grid boxes     *
C*                            (all latitudes)                          *
C*                  2. M = mixed, defined as 15-75% ocean grid boxes   *
C*                            (all latitudes)                          *
C*                  3. O = ocean, defined as 75-100% ocean grid boxes, *
C*                          and separately for:                        *
C*                             a) 60-90N  (O1)                         *
C*                             b) 30-60N  (O2)                         *
C*                             c) 30N-30S (O3)                         *
C*                             d) 30-60S  (O4)                         *
C*                             e) 60-90S  (O5)                         *
C*                                                                     *
C* The histogram files were generated with the program "comphist.f".   *
C*                                                                     *
C* Written by:    Eric Nelkin/SSAI                                     *
C* Date Written:  March 3, 2006 (original airs2tovs.f program)         *
C* Modifications: 5/23/2013 EJN  AIRS V6 to AIRS V5 version, in LINUX; *
C*                               byte-swap to big-endian if specified; *
C*                               use 2012 instead of 2004 (may re-do   *
C*                               when AIRS V6 for 2004 is available)   *
C*                5/24/2013 EJN  AIRS V6 to AIRS_HIST V5 version       *
C*		  7/12/2013 EJN  AIRS V6 to TOVS directly, using 2004, *
C*				 with byte-swap option to big-endian;  *
C*				 use l_hfA, l_hfT for differing lengths*
C***********************************************************************
      implicit none
      character*256 infile, outfile, histfileAL, histfileTL,
     &       histfileAM, histfileTM, histfileAO1, histfileTO1,
     &       histfileAO2, histfileTO2, histfileAO3, histfileTO3,
     &       histfileAO4, histfileTO4, histfileAO5, histfileTO5
      character*3   season
      real*4 airs_orig(360,180), airs_new(360,180), pct_ocean(360,180)
      real*8 airsL(0:10000), tovsL(0:10000), airsM(0:10000),
     &       tovsM(0:10000), airsO1(0:10000), tovsO1(0:10000),
     &       airsO2(0:10000), tovsO2(0:10000), airsO3(0:10000),
     &       tovsO3(0:10000), airsO4(0:10000), tovsO4(0:10000),
     &       airsO5(0:10000), tovsO5(0:10000)
      real*4 aval, tovs_interp
      integer*4 l_if, l_of, l_hfA, l_hfT, iret
      integer*4 i, j
      character*1 input_endian, output_endian
C
      call getarg(1, infile)
      call getarg(2, outfile)
      call getarg(3, season)
      call getarg(4, input_endian)
      call getarg(5, output_endian)
C
      CALL ST_LSTR(infile, l_if, iret)
      CALL ST_LSTR(outfile, l_of, iret)
C
      if (season(1:3) .EQ. 'djf' .OR. season(1:3) .EQ. 'DJF') then
         histfileAL  = 'airsV6hist.200401.L'
         histfileTL  = 'tovshist.200401.L'
         histfileAM  = 'airsV6hist.200401.M'
         histfileTM  = 'tovshist.200401.M'
         histfileAO1 = 'airsV6hist.200401.O.60-90N'
         histfileTO1 = 'tovshist.200401.O.60-90N'
         histfileAO2 = 'airsV6hist.200401.O.30-60N'
         histfileTO2 = 'tovshist.200401.O.30-60N'
         histfileAO3 = 'airsV6hist.200401.O.30N-30S'
         histfileTO3 = 'tovshist.200401.O.30N-30S'
         histfileAO4 = 'airsV6hist.200401.O.30-60S'
         histfileTO4 = 'tovshist.200401.O.30-60S'
         histfileAO5 = 'airsV6hist.200401.O.60-90S'
         histfileTO5 = 'tovshist.200401.O.60-90S'
      else if (season(1:3) .EQ. 'mam' .OR. season(1:3) .EQ. 'MAM') then
         histfileAL  = 'airsV6hist.200404.L'
         histfileTL  = 'tovshist.200404.L'
         histfileAM  = 'airsV6hist.200404.M'
         histfileTM  = 'tovshist.200404.M'
         histfileAO1 = 'airsV6hist.200404.O.60-90N'
         histfileTO1 = 'tovshist.200404.O.60-90N'
         histfileAO2 = 'airsV6hist.200404.O.30-60N'
         histfileTO2 = 'tovshist.200404.O.30-60N'
         histfileAO3 = 'airsV6hist.200404.O.30N-30S'
         histfileTO3 = 'tovshist.200404.O.30N-30S'
         histfileAO4 = 'airsV6hist.200404.O.30-60S'
         histfileTO4 = 'tovshist.200404.O.30-60S'
         histfileAO5 = 'airsV6hist.200404.O.60-90S'
         histfileTO5 = 'tovshist.200404.O.60-90S'
      else if (season(1:3) .EQ. 'jja' .OR. season(1:3) .EQ. 'JJA') then
         histfileAL  = 'airsV6hist.200407.L'
         histfileTL  = 'tovshist.200407.L'
         histfileAM  = 'airsV6hist.200407.M'
         histfileTM  = 'tovshist.200407.M'
         histfileAO1 = 'airsV6hist.200407.O.60-90N'
         histfileTO1 = 'tovshist.200407.O.60-90N'
         histfileAO2 = 'airsV6hist.200407.O.30-60N'
         histfileTO2 = 'tovshist.200407.O.30-60N'
         histfileAO3 = 'airsV6hist.200407.O.30N-30S'
         histfileTO3 = 'tovshist.200407.O.30N-30S'
         histfileAO4 = 'airsV6hist.200407.O.30-60S'
         histfileTO4 = 'tovshist.200407.O.30-60S'
         histfileAO5 = 'airsV6hist.200407.O.60-90S'
         histfileTO5 = 'tovshist.200407.O.60-90S'
      else if (season(1:3) .EQ. 'son' .OR. season(1:3) .EQ. 'SON') then
         histfileAL  = 'airsV6hist.200410.L'
         histfileTL  = 'tovshist.200410.L'
         histfileAM  = 'airsV6hist.200410.M'
         histfileTM  = 'tovshist.200410.M'
         histfileAO1 = 'airsV6hist.200410.O.60-90N'
         histfileTO1 = 'tovshist.200410.O.60-90N'
         histfileAO2 = 'airsV6hist.200410.O.30-60N'
         histfileTO2 = 'tovshist.200410.O.30-60N'
         histfileAO3 = 'airsV6hist.200410.O.30N-30S'
         histfileTO3 = 'tovshist.200410.O.30N-30S'
         histfileAO4 = 'airsV6hist.200410.O.30-60S'
         histfileTO4 = 'tovshist.200410.O.30-60S'
         histfileAO5 = 'airsV6hist.200410.O.60-90S'
         histfileTO5 = 'tovshist.200410.O.60-90S'
      else
         write(*,*) 'Invalid entry for season!'
         stop
      endif
C
C The length of the character string is larger by two than before for
C AIRS ("airsV6" instead of "airs"), while still the same for TOVS.
C
      l_hfA = 26
      l_hfT = 24
C
C Read in the input AIRS file. 
C
      open(21,file=infile(1:l_if),form='unformatted',status='old',
     &     access='direct',recl=64800)
      read(21,rec=1) airs_orig
      close(21)
C
C Byte-swap if specified on command line.
C
        if (input_endian .EQ. 'B' .OR. input_endian .EQ. 'b')
     &     CALL SWAPR4( airs_orig, 360, 180 )
C
C Open the ocean mask file.
C
C     open(24,file='/rain/rain1/gprof/code/surfrac/surfrac.1.0.edited',
      open(24,file='/home/nelkin/XOVOL/surfrac.1.0',
     &     form='unformatted',access='direct',recl=360,status='old')
      do j = 1, 180
         read(24,rec=j) (pct_ocean(i,j), i=1,360)
      end do
      close(24)
C
C Open and read the histogram files.
C
C AIRS V6 LAND:
      open(51,file=histfileAL(1:l_hfA-7),form='formatted',status='old')
      do i = 0, 10000
         read(51,200) airsL(i)
      end do
      close(51)
  200 format(10x,f11.0)
C
C TOVS LAND:
      open(52,file=histfileTL(1:l_hfT-7),form='formatted',status='old')
      do i = 0, 10000
         read(52,200) tovsL(i)
      end do
      close(52)
C
C AIRS V6 MIXED:
      open(53,file=histfileAM(1:l_hfA-7),form='formatted',status='old')
      do i = 0, 10000
         read(53,200) airsM(i)
      end do
      close(53)
C
C TOVS MIXED:
      open(54,file=histfileTM(1:l_hfT-7),form='formatted',status='old')
      do i = 0, 10000
         read(54,200) tovsM(i)
      end do
      close(54)
C
C AIRS V6 OCEAN 60-90N:
      open(55,file=histfileAO1(1:l_hfA),form='formatted',status='old')
      do i = 0, 10000
         read(55,200) airsO1(i)
      end do
      close(55)
C
C TOVS OCEAN 60-90N:
      open(56,file=histfileTO1(1:l_hfT),form='formatted',status='old')
      do i = 0, 10000
         read(56,200) tovsO1(i)
      end do
      close(56)
C
C AIRS V6 OCEAN 30-60N:
      open(57,file=histfileAO2(1:l_hfA),form='formatted',status='old')
      do i = 0, 10000
         read(57,200) airsO2(i)
      end do
      close(57)
C
C TOVS OCEAN 30-60N:
      open(58,file=histfileTO2(1:l_hfT),form='formatted',status='old')
      do i = 0, 10000
         read(58,200) tovsO2(i)
      end do
      close(58)
C
C AIRS V6 OCEAN 30N-30S:
      open(59,file=histfileAO3(1:l_hfA+1),form='formatted',status='old')
      do i = 0, 10000
         read(59,200) airsO3(i)
      end do
      close(59)
C
C TOVS OCEAN 30N-30S:
      open(60,file=histfileTO3(1:l_hfT+1),form='formatted',status='old')
      do i = 0, 10000
         read(60,200) tovsO3(i)
      end do
      close(60)
C
C AIRS V6 OCEAN 30-60S:
      open(61,file=histfileAO4(1:l_hfA),form='formatted',status='old')
      do i = 0, 10000
         read(61,200) airsO4(i)
      end do
      close(61)
C
C TOVS OCEAN 30-60S:
      open(62,file=histfileTO4(1:l_hfT),form='formatted',status='old')
      do i = 0, 10000
         read(62,200) tovsO4(i)
      end do
      close(62)
C
C AIRS V6 OCEAN 60-90S:
      open(63,file=histfileAO5(1:l_hfA),form='formatted',status='old')
      do i = 0, 10000
         read(63,200) airsO5(i)
      end do
      close(63)
C
C TOVS OCEAN 60-90S:
      open(64,file=histfileTO5(1:l_hfT),form='formatted',status='old')
      do i = 0, 10000
         read(64,200) tovsO5(i)
      end do
      close(64)
C
C Start by copying all values from "airs_orig" to "airs_new".  Then all
C grid boxes that aren't being altered by a histogram file (should be
C only grid boxes with missing values) will remain as is.
C
      do j = 1, 180
      do i = 1, 360
         airs_new(i,j) = airs_orig(i,j)
      end do
      end do
C
C Loop through the values in the input AIRS V6 file. For all non-missing
C values, look up the corresponding value in the appropriate histogram
C file.  The histogram file is chosen according to the surface type
C definition for land/mixed/ocean (see program header), and for ocean,
C by latitude band.  In the subroutine histo_lookup, use interpolation
C to pick out the correct bin value, even though this really amounts to
C just deciding whether to take the lower or higher of two adjacent
C precipitation rates that are 0.01 mm/day apart.
C
      do j = 1, 180
      do i = 1, 360
         if ( airs_orig(i,j) .GE. 0. ) then
            aval = airs_orig(i,j)
            if ( pct_ocean(i,j) .LT. 15. ) then
               call histo_lookup(aval, airsL, tovsL, tovs_interp, i, j)
            else if (pct_ocean(i,j) .LT. 75. ) then
               call histo_lookup(aval, airsM, tovsM, tovs_interp, i, j)
            else
               if ( j .GE. 1 .AND. j .LE. 30 ) then
                  call histo_lookup(aval, airsO1, tovsO1, tovs_interp,
     &                              i, j)
               else if ( j .GE. 31 .AND. j .LE. 60 ) then
                  call histo_lookup(aval, airsO2, tovsO2, tovs_interp,
     &                              i, j)
               else if ( j .GE. 61 .AND. j .LE. 120 ) then
                  call histo_lookup(aval, airsO3, tovsO3, tovs_interp,
     &                              i, j)
               else if ( j .GE. 121 .AND. j .LE. 150 ) then
                  call histo_lookup(aval, airsO4, tovsO4, tovs_interp,
     &                              i, j)
               else if ( j .GE. 151 .AND. j .LE. 180 ) then
                  call histo_lookup(aval, airsO5, tovsO5, tovs_interp,
     &                              i, j)
               endif
            endif
C
            airs_new(i,j) = tovs_interp
C
         endif
      end do 
      end do
C
C Byte-swap if specified on command line.
C
        if (output_endian .EQ. 'B' .OR. output_endian .EQ. 'b')
     &     CALL SWAPR4( airs_new, 360, 180 )
C
C Write out the resulting file.
C
      open(29,file=outfile(1:l_of),form='unformatted',status='new',
     &     access='direct',recl=360)
      do j = 1, 180
         write(29,rec=j) (airs_new(i,j), i=1,360)
      end do
      close(29)
      write(*,*) 'Done writing file ', outfile(1:l_of)
C
      stop
      end
C
C
      subroutine histo_lookup(aval, airscounts, tovscounts, tovs_interp,
     &                        ii, jj)
      implicit none
      real*8 airscounts(0:10000), tovscounts(0:10000), target, asum,
     &       tsum, tsum_previous
      real*4 aval, tovs_interp, binlow, frac
      real*4 binsize/0.01/
      integer*4 iabin, itbin, i, ii, jj
C
C Determine which AIRS histogram bin (0 to 10000) that aval falls into.
C
      if (aval .LT. 0.005) then
         iabin = 0
      else if (aval .GT. 100.00) then
         write(*,300) aval
  300    format('Found AIRS rain rate of ',f10.3,' > 100.00 mm/day!')
         write(*,301) ii, jj
  301    format('at (i,j) = (',i3,',',i3,').  Program aborted.')
         STOP
      else
         iabin = NINT(aval*100)
      endif
C
C Beginning at the low end, sum the AIRS-histogram occurrences up until
C the bin before the input "aval" rate's bin (iabin).
C
      asum = 0.
      do i = 0, iabin-1
         asum = asum + airscounts(i)
      end do 
C
C Define the lower limit of the bin.  Bin 0 represents rain rates of
C 0 to 0.0049999, bin 1 represents 0.005 to 0.0149999, etc.
C
      if (iabin .EQ. 0) then
         binlow = 0.
      else
         binlow = float(iabin)/100. - binsize/2.
      endif
C
C Linearly interpolate to determine how far into the next bin will match
C the input (aval) rain rate.  Call this index value "target".
C
      target = asum +
     &         ( ( (aval - binlow) / binsize ) * airscounts(iabin) )

C Next, beginning at the low end, sum the TOVS occurrences until the
C target value is met or exceeded.
C
      tsum = 0.
      itbin = -1.
   50 itbin = itbin + 1 
      tsum = tsum + tovscounts(itbin)
      if (tsum .LT. target) go to 50
C
C Define the lower limit of the bin that contained (and ultimately
C exceeded) the target number of occurrences.
C
      if (itbin .EQ. 0) then
         binlow = 0.
      else
         binlow = float(itbin)/100. - binsize/2.
      endif
C
C Compute the sum of the occurrences up through the previous bin, i.e.,
C subtract back out the population of the current bin of interest.
C
      tsum_previous = tsum - tovscounts(itbin)
C
C Finally, linearly interpolate to determine how far into itbin's range
C of rain rates (spanning from binlow to binlow+0.01) will match the
C same number of occurrences that were found in AIRS ("target").
C This "tovs_interp" value is returned to the main program, where it is
C assigned to the "airs_new" array at that (i,j) grid box.
C
C Note that bins 0 (range 0-0.0049999) and 10000 (range 99.9949999 to
C 100.00) are half the width of the other bins and must therefore be
C treated separately.
C
      if (itbin .EQ. 0) then
         tovs_interp = target/tovscounts(0) * 0.005
      else if (itbin .EQ. 10000) then
         tovs_interp = binlow +
     &  ( ( (target - tsum_previous) / tovscounts(itbin) ) * binsize/2.)
      else
         tovs_interp = binlow +
     &  ( ( (target - tsum_previous) / tovscounts(itbin) ) * binsize )
      endif
C
      return
      end
