c     subroutine that reads gridded daily AIRS and and calibrates to TOVS
c     temporary output to interim files
c
c     Original header:
C* This program reads in an AIRS daily 1x1-degree file and re-assigns  *
C* the precipitation value at various grid boxes to the corresponding  *
C* TOVS value via histogram matching.  Separate histograms are         *
C* maintained for:                                                     *
C*                  1. L = land, defined as 0-15% ocean grid boxes     *
C*                            (all latitudes)                          *
C*                  2. M = mixed, defined as 15-75% ocean grid boxes   *
C*                            (all latitudes)                          *
C*                  3. O = ocean, defined as 75-100% ocean grid boxes, *
C*                          and separately for:                        *
C*                             a) 60-90N  (O1)                         *
C*                             b) 30-60N  (O2)                         *
C*                             c) 30N-30S (O3)                         *
C*                             d) 30-60S  (O4)                         *
C*                             e) 60-90S  (O5)                         *
C*                                                                     *
C* The histogram files were generated with the program "comphist.f".   *
C*                                                                     *

      subroutine GPCPcdr_m2_airs2tovs (infile,outfile,season,
     &     histroot,AIRSvers)

      implicit none
      character*256 infile, outfile, histroot, histfileAL, histfileTL,
     &       histfileAM, histfileTM, histfileAO1, histfileTO1,
     &       histfileAO2, histfileTO2, histfileAO3, histfileTO3,
     &       histfileAO4, histfileTO4, histfileAO5, histfileTO5
      character*3   season
      integer*4 AIRSvers
      real*4 airs_orig(360,180), airs_new(360,180), pct_ocean(360,180)
      real*8 airsL(0:10000), tovsL(0:10000), airsM(0:10000),
     &       tovsM(0:10000), airsO1(0:10000), tovsO1(0:10000),
     &       airsO2(0:10000), tovsO2(0:10000), airsO3(0:10000),
     &       tovsO3(0:10000), airsO4(0:10000), tovsO4(0:10000),
     &       airsO5(0:10000), tovsO5(0:10000)
      real*4 aval, tovs_interp
      integer*4 l_if, l_of, l_hf, l_af, l_hr, iret
      integer*4 i, j, ios
C



c     get length of infile, outfile and histroot
      do l_if = 1,len_trim(infile)
         if (infile(l_if:l_if) .eq. ' ') then
            continue
         end if
      end do
      l_if=l_if-1
      do l_of = 1,len_trim(outfile)
         if (outfile(l_of:l_of) .eq. ' ') then
            continue
         end if
      end do
      l_of=l_of-1
      do l_hr = 1,len_trim(histroot)
         if (histroot(l_hr:l_hr) .eq. ' ') then
            continue
         end if
      end do
      l_hr=l_hr-1


C
      if (season(1:3) .EQ. 'djf' .OR. season(1:3) .EQ. 'DJF') then
         histfileTL  = 'tovshist.200401.L'
         histfileTM  = 'tovshist.200401.M'
         histfileTO1 = 'tovshist.200401.O.60-90N'
         histfileTO2 = 'tovshist.200401.O.30-60N'
         histfileTO3 = 'tovshist.200401.O.30N-30S'
         histfileTO4 = 'tovshist.200401.O.30-60S'
         histfileTO5 = 'tovshist.200401.O.60-90S'
         if (AIRSvers .EQ. 6) then
            histfileAL  = 'airsV6hist.200401.L'
            histfileAM  = 'airsV6hist.200401.M'
            histfileAO1 = 'airsV6hist.200401.O.60-90N'
            histfileAO2 = 'airsV6hist.200401.O.30-60N'
            histfileAO3 = 'airsV6hist.200401.O.30N-30S'
            histfileAO4 = 'airsV6hist.200401.O.30-60S'
            histfileAO5 = 'airsV6hist.200401.O.60-90S'
         else
            histfileAL  = 'airshist.200401.L'
            histfileAM  = 'airshist.200401.M'
            histfileAO1 = 'airshist.200401.O.60-90N'
            histfileAO2 = 'airshist.200401.O.30-60N'
            histfileAO3 = 'airshist.200401.O.30N-30S'
            histfileAO4 = 'airshist.200401.O.30-60S'
            histfileAO5 = 'airshist.200401.O.60-90S'
         endif

      else if (season(1:3) .EQ. 'mam' .OR. season(1:3) .EQ. 'MAM') then
         histfileTL  = 'tovshist.200404.L'
         histfileTM  = 'tovshist.200404.M'
         histfileTO1 = 'tovshist.200404.O.60-90N'
         histfileTO2 = 'tovshist.200404.O.30-60N'
         histfileTO3 = 'tovshist.200404.O.30N-30S'
         histfileTO4 = 'tovshist.200404.O.30-60S'
         histfileTO5 = 'tovshist.200404.O.60-90S'
         if (AIRSvers .EQ. 6) then
            histfileAL  = 'airsV6hist.200404.L'
            histfileAM  = 'airsV6hist.200404.M'
            histfileAO1 = 'airsV6hist.200404.O.60-90N'
            histfileAO2 = 'airsV6hist.200404.O.30-60N'
            histfileAO3 = 'airsV6hist.200404.O.30N-30S'
            histfileAO4 = 'airsV6hist.200404.O.30-60S'
            histfileAO5 = 'airsV6hist.200404.O.60-90S'
         else
            histfileAL  = 'airshist.200404.L'
            histfileAM  = 'airshist.200404.M'
            histfileAO1 = 'airshist.200404.O.60-90N'
            histfileAO2 = 'airshist.200404.O.30-60N'
            histfileAO3 = 'airshist.200404.O.30N-30S'
            histfileAO4 = 'airshist.200404.O.30-60S'
            histfileAO5 = 'airshist.200404.O.60-90S'
         endif
         
      else if (season(1:3) .EQ. 'jja' .OR. season(1:3) .EQ. 'JJA') then
         histfileTL  = 'tovshist.200407.L'
         histfileTM  = 'tovshist.200407.M'
         histfileTO1 = 'tovshist.200407.O.60-90N'
         histfileTO2 = 'tovshist.200407.O.30-60N'
         histfileTO3 = 'tovshist.200407.O.30N-30S'
         histfileTO4 = 'tovshist.200407.O.30-60S'
         histfileTO5 = 'tovshist.200407.O.60-90S'
         if (AIRSvers .EQ. 6) then
            histfileAL  = 'airsV6hist.200407.L'
            histfileAM  = 'airsV6hist.200407.M'
            histfileAO1 = 'airsV6hist.200407.O.60-90N'
            histfileAO2 = 'airsV6hist.200407.O.30-60N'
            histfileAO3 = 'airsV6hist.200407.O.30N-30S'
            histfileAO4 = 'airsV6hist.200407.O.30-60S'
            histfileAO5 = 'airsV6hist.200407.O.60-90S'
         else
            histfileAL  = 'airshist.200407.L'
            histfileAM  = 'airshist.200407.M'
            histfileAO1 = 'airshist.200407.O.60-90N'
            histfileAO2 = 'airshist.200407.O.30-60N'
            histfileAO3 = 'airshist.200407.O.30N-30S'
            histfileAO4 = 'airshist.200407.O.30-60S'
            histfileAO5 = 'airshist.200407.O.60-90S'
         end if
         
      else if (season(1:3) .EQ. 'son' .OR. season(1:3) .EQ. 'SON') then
         histfileTL  = 'tovshist.200410.L'
         histfileTM  = 'tovshist.200410.M'
         histfileTO1 = 'tovshist.200410.O.60-90N'
         histfileTO2 = 'tovshist.200410.O.30-60N'
         histfileTO3 = 'tovshist.200410.O.30N-30S'
         histfileTO4 = 'tovshist.200410.O.30-60S'
         histfileTO5 = 'tovshist.200410.O.60-90S'
         if (AIRSvers .EQ. 6) then
            histfileAL  = 'airsV6hist.200410.L'
            histfileAM  = 'airsV6hist.200410.M'
            histfileAO1 = 'airsV6hist.200410.O.60-90N'
            histfileAO2 = 'airsV6hist.200410.O.30-60N'
            histfileAO3 = 'airsV6hist.200410.O.30N-30S'
            histfileAO4 = 'airsV6hist.200410.O.30-60S'
            histfileAO5 = 'airsV6hist.200410.O.60-90S'
         else
            histfileAL  = 'airshist.200410.L'
            histfileAM  = 'airshist.200410.M'
            histfileAO1 = 'airshist.200410.O.60-90N'
            histfileAO2 = 'airshist.200410.O.30-60N'
            histfileAO3 = 'airshist.200410.O.30N-30S'
            histfileAO4 = 'airshist.200410.O.30-60S'
            histfileAO5 = 'airshist.200410.O.60-90S'
         endif
         
      else
         write(*,*) 'Invalid entry for season!'
         stop
      endif



c     Set lengths of histogram file names
      l_hf = 24
      if (AIRSvers .EQ. 6) then
         l_af = 26
      else
         l_af = 26
      end if

C
C Read in the input AIRS file. 
C
      open(21,file=infile(1:l_if),form='unformatted',status='old',
     &     access='direct',recl=4*64800, iostat = ios)

      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m2_airs2tovs:'//
     &        'Missing file (moving to next): ' // infile(1:l_if)
         RETURN
      end if
      
      read(21,rec=1) airs_orig
      close(21)
C
C Open the ocean mask file.
C
      open(24,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     'Ancillary/surfrac.1.0',
     &     form='unformatted',access='direct',recl=4*360,status='old')
      do j = 1, 180
         read(24,rec=j) (pct_ocean(i,j), i=1,360)
      end do
      close(24)
C
C Open and read the histogram files.
C
C AIRS LAND:
      open(51,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileAL(1:l_af-7),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(51,200) airsL(i)
      end do
      close(51)
  200 format(10x,f11.0)
C
C TOVS LAND:
      open(52,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileTL(1:l_hf-7),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(52,200) tovsL(i)
      end do
      close(52)
C
C AIRS MIXED:
      open(53,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileAM(1:l_af-7),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(53,200) airsM(i)
      end do
      close(53)
C
C TOVS MIXED:
      open(54,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileTM(1:l_hf-7),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(54,200) tovsM(i)
      end do
      close(54)
C
C AIRS OCEAN 60-90N:
      open(55,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileAO1(1:l_af),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(55,200) airsO1(i)
      end do
      close(55)
C
C TOVS OCEAN 60-90N:
      open(56,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileTO1(1:l_hf)
     &     ,form='formatted',status='old')
      do i = 0, 10000
         read(56,200) tovsO1(i)
      end do
      close(56)
C
C AIRS OCEAN 30-60N:
      open(57,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileAO2(1:l_af),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(57,200) airsO2(i)
      end do
      close(57)
C
C TOVS OCEAN 30-60N:
      open(58,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileTO2(1:l_hf),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(58,200) tovsO2(i)
      end do
      close(58)
C
C AIRS OCEAN 30N-30S:
      open(59,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileAO3(1:l_af+1),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(59,200) airsO3(i)
      end do
      close(59)
C
C TOVS OCEAN 30N-30S:
      open(60,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileTO3(1:l_hf+1),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(60,200) tovsO3(i)
      end do
      close(60)
C
C AIRS OCEAN 30-60S:
      open(61,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileAO4(1:l_af),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(61,200) airsO4(i)
      end do
      close(61)
C
C TOVS OCEAN 30-60S:
      open(62,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileTO4(1:l_hf),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(62,200) tovsO4(i)
      end do
      close(62)
C
C AIRS OCEAN 60-90S:
      open(63,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileAO5(1:l_af),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(63,200) airsO5(i)
      end do
      close(63)
C
C TOVS OCEAN 60-90S:
      open(64,file='GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/'//
     &     histroot(1:l_hr)//histfileTO5(1:l_hf),
     &     form='formatted',status='old')
      do i = 0, 10000
         read(64,200) tovsO5(i)
      end do
      close(64)
C
C Start by copying all values from "airs_orig" to "airs_new".  Then all
C grid boxes that aren't being altered by a histogram file (should be
C only grid boxes with missing values) will remain as is.
C
      do j = 1, 180
      do i = 1, 360
         airs_new(i,j) = airs_orig(i,j)
      end do
      end do
C
C Loop through the values in the input AIRS file.  For all non-missing
C values, look up the corresponding value in the appropriate histogram
C file.  The histogram file is chosen according to the surface type
C definition for land/mixed/ocean (see program header), and for ocean,
C by latitude band.  In the subroutine histo_lookup, use interpolation
C to pick out the correct bin value, even though this really amounts to
C just deciding whether to take the lower or higher of two adjacent
C precipitation rates that are 0.01 mm/day apart.
C
      do j = 1, 180
      do i = 1, 360
         if ( airs_orig(i,j) .GE. 0. ) then
            aval = airs_orig(i,j)
            if ( pct_ocean(i,j) .LT. 15. ) then
               call histo_lookup(aval, airsL, tovsL, tovs_interp, i, j)
            else if (pct_ocean(i,j) .LT. 75. ) then
               call histo_lookup(aval, airsM, tovsM, tovs_interp, i, j)
            else
               if ( j .GE. 1 .AND. j .LE. 30 ) then
                  call histo_lookup(aval, airsO1, tovsO1, tovs_interp,
     &                              i, j)
               else if ( j .GE. 31 .AND. j .LE. 60 ) then
                  call histo_lookup(aval, airsO2, tovsO2, tovs_interp,
     &                              i, j)
               else if ( j .GE. 61 .AND. j .LE. 120 ) then
                  call histo_lookup(aval, airsO3, tovsO3, tovs_interp,
     &                              i, j)
               else if ( j .GE. 121 .AND. j .LE. 150 ) then
                  call histo_lookup(aval, airsO4, tovsO4, tovs_interp,
     &                              i, j)
               else if ( j .GE. 151 .AND. j .LE. 180 ) then
                  call histo_lookup(aval, airsO5, tovsO5, tovs_interp,
     &                              i, j)
               endif
            endif
C
            airs_new(i,j) = tovs_interp
C
         endif
      end do 
      end do
C
C Write out the resulting file.
C
      open(29,file=outfile(1:l_of),form='unformatted',status='replace',
     &     access='direct',recl=4*360)
      do j = 1, 180
         write(29,rec=j) (airs_new(i,j), i=1,360)
      end do
      close(29)
c      write(*,*) 'Done writing file ', outfile(1:l_of)
C
c      stop
      return
      end
C
C
      subroutine histo_lookup(aval, airscounts, tovscounts, tovs_interp,
     &                        ii, jj)
      implicit none
      real*8 airscounts(0:10000), tovscounts(0:10000), target, asum,
     &       tsum, tsum_previous
      real*4 aval, tovs_interp, binlow, frac
      real*4 binsize/0.01/
      integer*4 iabin, itbin, i, ii, jj
C
C Determine which AIRS histogram bin (0 to 10000) that aval falls into.
C
      if (aval .LT. 0.005) then
         iabin = 0
      else if (aval .GT. 100.00) then
         write(*,300) aval
  300    format('Found AIRS rain rate of ',f10.3,' > 100.00 mm/day!')
         write(*,301) ii, jj
  301    format('at (i,j) = (',i3,',',i3,').  Program aborted.')
         STOP
      else
         iabin = NINT(aval*100)
      endif
C
C Beginning at the low end, sum the AIRS-histogram occurrences up until
C the bin before the input "aval" rate's bin (iabin).
C
      asum = 0.
      do i = 0, iabin-1
         asum = asum + airscounts(i)
      end do 
C
C Define the lower limit of the bin.  Bin 0 represents rain rates of
C 0 to 0.0049999, bin 1 represents 0.005 to 0.0149999, etc.
C
      if (iabin .EQ. 0) then
         binlow = 0.
      else
         binlow = float(iabin)/100. - binsize/2.
      endif
C
C Linearly interpolate to determine how far into the next bin will match
C the input (aval) rain rate.  Call this index value "target".
C
      target = asum +
     &         ( ( (aval - binlow) / binsize ) * airscounts(iabin) )

C Next, beginning at the low end, sum the TOVS occurrences until the
C target value is met or exceeded.
C
      tsum = 0.
      itbin = -1.
   50 itbin = itbin + 1 
      tsum = tsum + tovscounts(itbin)
      if (tsum .LT. target) go to 50
C
C Define the lower limit of the bin that contained (and ultimately
C exceeded) the target number of occurrences.
C
      if (itbin .EQ. 0) then
         binlow = 0.
      else
         binlow = float(itbin)/100. - binsize/2.
      endif
C
C Compute the sum of the occurrences up through the previous bin, i.e.,
C subtract back out the population of the current bin of interest.
C
      tsum_previous = tsum - tovscounts(itbin)
C
C Finally, linearly interpolate to determine how far into itbin's range
C of rain rates (spanning from binlow to binlow+0.01) will match the
C same number of occurrences that were found in AIRS ("target").
C This "tovs_interp" value is returned to the main program, where it is
C assigned to the "airs_new" array at that (i,j) grid box.
C
C Note that bins 0 (range 0-0.0049999) and 10000 (range 99.9949999 to
C 100.00) are half the width of the other bins and must therefore be
C treated separately.
C
      if (itbin .EQ. 0) then
         tovs_interp = target/tovscounts(0) * 0.005
      else if (itbin .EQ. 10000) then
         tovs_interp = binlow +
     &  ( ( (target - tsum_previous) / tovscounts(itbin) ) * binsize/2.)
      else
         tovs_interp = binlow +
     &  ( ( (target - tsum_previous) / tovscounts(itbin) ) * binsize )
      endif
C
      return
      end
