c     Code to grid original AIRS data
c     This code can be slow, so it is a separate module
c     to the other routines that do the intermediate AIRS
c     processing.

      program Main_GPCPcdr_m2_Grid_Airs
      
c     From the original airsg_grid_1dd.f file:
C     *									*
C* This program re-maps AIRS Level 2G data to a 1.0-deg daily global    *
C* grid.  An intermediate fine grid is used to approximately represent  *
C* the areal coverage of each AIRS FOV, then these boxes are averaged to*
C* the 1.0-deg scale.  The averaging is implemented on approximate      *
C* 100x100 km templates for uniform smoothness, so adjacent 1.0-deg grid*
C* boxes at high latitudes are not totally independent.  The implemen-  *
C* tation is intended to account for the nonlinear transformation from  *
C* pixel location to grid location, and for the change in resolution of *
C* the pixels as scan angle increases, while avoiding hideous           *
C* coordinate transformations.  In this spirit, separate rectilinear    *
C* grids are used at the poles for the fine scale to avoid messy        *
C* navigation issues.                                                   *
C*                                                                      *
C* The AIRS L2G file has 24 layers, one for each hour; this program     *
C* grids one hour at a time, accumulating along the way, then outputs   *
C* a daily 1x1-degree 00-24 UTC day file.				*
C*									*
C* AIRSG_GRID_DAY swathfile outpath_g  					*
C*									*
C* Input command-line arguments:					*
C*   swathfile	CHAR	SRT subsetted AIRS precip file name		*
C*   outpath_g  CHAR    Path (directory) for gridded output, including	*
C*                      trailing slash (/)				*

      IMPLICIT NONE
      
      INTEGER*4	m, n, i, j, nscan, iunit, l_sf, l_on, l_outdir
      INTEGER*4       l_fullpath, l_h, Ninputs, iargc
      INTEGER*4       iscan1, ifov1, ifov2, ihour, ii, jj
      INTEGER*4       jloop, lin, dayscan, hrscan, loscan, hiscan,
     +     ioffset
      REAL*4		c, x, y
      LOGICAL		in_use, jftime_set_yet(6), date_set_yet
      DATA            jftime_set_yet/.FALSE.,.FALSE.,.FALSE.,.FALSE.,
     +     .FALSE.,.FALSE./
      DATA            date_set_yet/.FALSE./
      CHARACTER*1     output_endian
C     
C     NFGFR, NCEDFR are the number of FG and CED grid intervals in a 
C     deg. of lat/lon.
C     
      INTEGER*4	LATEDGE, NFGFR, NCEDFR, NCX,  NCY, NNSX, 
     +     NNSY,    MXFOV, MXSCAN, MXLN, MXLT
      INTEGER*2	I2MISS
      REAL*4		RE, SZDEG
      PARAMETER	( LATEDGE = 75,
     +     NFGFR   = 16,
     +     NCEDFR  = 4,
     +     NCX     = 360              * NFGFR,
     +     NCY     = LATEDGE          * NFGFR * 2,
     +     NNSX    = ( 90 - LATEDGE ) * NFGFR * 2,
     +     NNSY    = ( 90 - LATEDGE ) * NFGFR * 2,
     +     RE      = 6371.,
     +     SZDEG   = 3.1415926 * 2. * RE / 360.,
     +     MXFOV   = 30,
     +     MXSCAN  = 500,       ! enough for 10+ granules/day
     +     I2MISS  = -31999,
     +     MXLN    = 360 * NCEDFR,
     +     MXLT    = 180 * NCEDFR )
      REAL*4		flon (MXFOV, MXSCAN), flat (MXFOV, MXSCAN),
     +     w (MXFOV), s (MXFOV),
     +     xorn   (MXFOV),         yorn (MXFOV),
     +     xorc   (MXFOV),         yorc (MXFOV),
     +     xors   (MXFOV),         yors (MXFOV)
      INTEGER*2	ifp    (MXFOV, MXSCAN), ipn    (NNSX, NNSY),
     +     ipc    (NCX,   NCY   ), ips    (NNSX, NNSY),
     +     ip1    (360,  180)
      INTEGER*1	ift    (MXFOV, MXSCAN), itn    (NNSX, NNSY),
     +     itc    (NCX,   NCY   ), its    (NNSX, NNSY),
     +     influn (NNSX, NNSY   ), influc (NCX,  NCY ),
     +     influs (NNSX, NNSY   )
      INTEGER*1	iqual  (MXFOV, MXSCAN), ifov (MXFOV, MXSCAN),
     +     iscn   (MXFOV, MXSCAN), isfc (MXFOV, MXSCAN)
      REAL*4          sum(360, 180), rnum(360, 180), 
     +     airs1dd(360, 180)
C     
C     AIRS L2G input file arrays:
C     
      INTEGER*1       iscind (MXLN, MXLT),    ipxind(MXLN, MXLT),
     +     igran  (MXLN, MXLT)
      INTEGER*2       i2gran (MXLN, MXLT)
      INTEGER*2       iqflag (MXLN, MXLT)
      REAL*4          precip (MXLN, MXLT),    alat  (MXLN, MXLT),
     +     alon   (MXLN, MXLT)
      REAL*8          time   (MXLN, MXLT)
C     
C     Set of variables for "bogus" scan lines iscan1-1 and nscan+1;
C     needed in order to choose only pixels near the elliptical center
C     along the first and last scan lines actually processed.
C     
      INTEGER*1       iftb   (MXFOV, 2)
      INTEGER*2       ifpb   (MXFOV, 2)
      REAL*4          flatb  (MXFOV, 2), flonb  (MXFOV, 2)
C     
      INTEGER*4	iret
      INTEGER*4       iftime (6, MXFOV, MXSCAN), jftime(6)
      INTEGER*2       itime(6, MXLN, MXLT)
      
      CHARACTER*256	swathfile, oname, outpath_g
      CHARACTER*256   out_file
C     
      LOGICAL         use_this_scan(MXSCAN)
      DATA            use_this_scan/MXSCAN*.FALSE./
C     
      iret = 0
C     
C     Initialize daily values.
C     
      DO  j = 1, 180
         DO  i = 1, 360
	    sum(i,j)     =     0.
            rnum(i,j)    =     0.
            airs1dd(i,j) = -9999.
         END DO
      END DO
C     
C     Initialize all character strings to blanks.
C     
      DO  j = 1, 256
         swathfile (j:j) = ' '
      END DO
      oname     = swathfile
      outpath_g = swathfile
      out_file  = swathfile
      
      
c     Added to fix leap second issue:
      DO j = 1, 6
         jftime(j) = -999
      END DO
      
        
c     ******************************************************
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 2 ) then
         WRITE(*,*) 'Error in GPCPcdr_m2_Grid_Airs:'//
     &        ' Incorrect number of input arguments (should be 2)'
         STOP
      endif
      
C     Get arguments and the length of each string
      call getarg(1, swathfile)
      call getarg(2, outpath_g)
      
      do l_sf = 1,len_trim(swathfile)
         if (swathfile(l_sf:l_sf) .eq. ' ') then
            continue
         end if
      end do
      l_sf=l_sf-1
      
      do l_outdir = 1,len_trim(outpath_g)
         if (outpath_g(l_outdir:l_outdir) .eq. ' ') then
            continue
         end if
      end do
      l_outdir=l_outdir-1



C     Compute the FOV size arrays for a scan.
C     
      CALL FOV_DIM ( w, s, iret )
C     
      DO ihour = 1, 24
C     
C     Initialize the fine grid arrays.
C     
         DO  n = 1, NNSY
            DO  m = 1, NNSX
               influn (m, n) = 20
               ipn    (m, n) = I2MISS
               itn    (m, n) = 127
            END DO
         END DO
         DO  n = 1, NCY
            DO  m = 1, NCX
               influc (m, n) = 20
               ipc    (m, n) = I2MISS
               itc    (m, n) = 127
            END DO
         END DO
         DO  n = 1, NNSY
            DO  m = 1, NNSX
               influs (m, n) = 20
               ips    (m, n) = I2MISS
               its    (m, n) = 127
            END DO
         END DO
C     
C     Get the data.
C     
         WRITE (*, *) 'Open/read swathfile: ', swathfile (1:l_sf)
C     
         CALL READ_AIRS( swathfile, ihour, MXLN, MXLT, precip, time,
     +        alat, alon, iscind, ipxind, igran, iqflag )


C     
C-----------------------------------------------------------------------
C     NOTE: FORTRAN does not support unsigned bytes.  Therefore, the
C     fill and missing byte values that Jason Li reports to be 255 are
C     read as -1 on our machines.
C-----------------------------------------------------------------------
C     
C       Convert igran array from signed byte to unsigned integer*2
C       by adding 256 if negative.  This will preserve the 1-240 format,
C       as otherwise FORTRAN would wrap values above 127 to negatives.
C
         DO j = 1, MXLT
            DO i = 1, MXLN
               IF (igran(i,j) .LT. 0) THEN
                  i2gran(i,j) = igran(i,j) + 256
               ELSE
                  i2gran(i,j) = igran(i,j)
               END IF
            END DO
         END DO
C
C       Convert times from real*8 seconds since 00Z 01 January 1993 to
C       integer*4 year/month/day/hour/minute/second.
C
         CALL CONVERT_TIME( time, MXLN, MXLT, itime )
C     C
CC      This is the format that the backward-gridder code expects.
CC      For AIRS L2G, the data are already on a 1440 x 720 grid.
CC      Therefore, the gridding must be "undone" and put back into
CC      scan/pixel format before the backward gridding can proceed.
CC
C       Initialize the working range of scans [iscan1, nscan]; these
C       will be modified in the loop below.  Also initialize loscan and
C       hiscan, which will represent the minimum and maximum scan
C       indices, respectively, regardless of whether there is valid
C       precipitation at that scan/FOV.
C     
         iscan1 = 20000
         loscan = 20000
         nscan  =     0
         hiscan =     0
C     
C       Loop through the entire 2-D array at the specified hour,
C       identifying the minimum and maximum scan values that contain
C       valid navigation data, and those that contain valid
C       precipitation data.
C
         DO j = 1, MXLT
            DO i = 1, MXLN
C     Missing float values are -7777. (outside swath range)
CDEBUG
C          if (itime(1,i,j).GT.0) write(227,*) i, j,
C    +                                        (itime(m,i,j), m = 1,6)
C
C       FOV's are numbered 0-29, and scans are numbered 0-44.  Add one
C       to convert to 1-30 and 1-45, respectively.
C***********************************************************************
C***    NOTE:  Fill and missing values of -1 will be promoted to 0.  ***
C***           This is OK because the code will load the arrays      ***
C***           only if ii > 0 and jj > 0.                            ***
C***********************************************************************
C     
               ii = -100
               jj = -100
               IF ( ipxind(i,j) .GE. 0 ) ii = ipxind(i,j) + 1
               IF ( iscind(i,j) .GE. 0 ) jj = iscind(i,j) + 1
C     
C       At ihour = 1 (00 to 01 UTC), granule #240 can sometimes have data
C       from both the end of the previous day and the beginning of the
C       current day.  Re-assign the granule number to zero.
C       NOTE: This can result in negative values for dayscan/loscan, but
C       this is OK, as ultimately loscan will be re-set to 1 regardless
C       of the sign of ioffset (= loscan-1).
C
               IF ( ihour .EQ. 1 .AND. i2gran(i,j) .EQ. 240) 
     +              i2gran(i,j) = 0
               
               IF ( ii .GT. 0 .AND. jj .GT. 0 ) THEN
C     
                  dayscan = (i2gran(i,j) - 1 ) * 45 + jj
                  IF ( dayscan .LT. loscan ) loscan = dayscan
                  IF ( dayscan .GT. hiscan ) hiscan = dayscan
C     
                  IF ( precip(i,j) .GE. 0. .AND. iqflag(i,j) .LT. 2 ) 
     +                 THEN
C     DEBUG
C                write(*,*) i, j, ii, jj, dayscan, i2gran(i,j)
C
C       Update the values of iscan1 and/or nscan as appropriate.
C       When the double do loop is finished, the range [iscan1, nscan]
C       will represent the range of scan indices at that hour with a
C       valid precipitation amount.  Also set the values of ifov1 and
C       ifov2, to be used in setting the start and end times in the
C       output file name.
C
                     IF ( dayscan .LT. iscan1 ) THEN
                        iscan1 = dayscan
                        ifov1  = ii
                     END IF
                     IF ( dayscan .GT.  nscan ) THEN
                        nscan  = dayscan
                        ifov2  = ii
                     END IF
                  END IF
               END IF
            END DO
         END DO
         
C
C     If no valid navigation and/or precipitation values were found,
C       abort the program.  In such a case, the initial values remain:
C       loscan = 20000, hiscan = 0 and/or iscan1 = 20000, nscan = 0.
C
         IF ( loscan .GT. hiscan .OR. iscan1 .GT. nscan ) THEN
            GO TO 999
         END IF
C
         WRITE (*, 200) iscan1, nscan, loscan, hiscan
 200     FORMAT('iscan1 = ',i6,' nscan = ',i6,' loscan = ',i6,
     +        ' hiscan = ',i6)
C
C       To this point, the scan values have been defined by their
C       position within the entire day, i.e. roughly 1 (at hour 1) to
C       10,800 (= 240 granules/day * 45 scans/granule, at hour 24).
C       Reassign the limiting values based on their position within the
C     hour.
C     
         ioffset = loscan - 1
C     
         loscan = loscan - ioffset
         hiscan = hiscan - ioffset
         iscan1 = iscan1 - ioffset
         nscan  = nscan  - ioffset
C     
C     Loop through the entire 2-D array a second time, using the
C     ioffset value to assign arrays to scan positions within the
C     hour.
C     
         DO j = 1, MXLT
            DO i = 1, MXLN
C     
C       FOV's are numbered 0-29, and scans are numbered 0-44.  Add one
C       to convert to 1-30 and 1-45, respectively.
C***********************************************************************
C***    NOTE:  Fill and missing values of -1 will be promoted to 0.  ***
C***           This is OK because the code will load the arrays      ***
C***           only if ii > 0 and jj > 0.                            ***
C***********************************************************************
C
               ii = -100
               jj = -100
               IF ( ipxind(i,j) .GE. 0 ) ii = ipxind(i,j) + 1
               IF ( iscind(i,j) .GE. 0 ) jj = iscind(i,j) + 1
C
C       At ihour = 1 (00 to 01 UTC), granule #240 can sometimes have data
C       from both the end of the previous day and the beginning of the
C       current day.  Re-assign the granule number to zero.
C     
               IF ( ihour .EQ. 1 .AND. i2gran(i,j) .EQ. 240) 
     +              i2gran(i,j) = 0
c     
               IF ( ii .GT. 0 .AND. jj .GT. 0 ) THEN
C     
                  hrscan = ( i2gran(i,j) - 1 ) * 45 + jj - ioffset
                  use_this_scan(hrscan) = .TRUE.
C     
                  flat (ii,hrscan)  = alat(i,j)
                  flon (ii,hrscan)  = alon(i,j)
                  iqual(ii,hrscan)  = iqflag(i,j)
C 4/16/13 EJN - Split up jftime calculation; year/month/day needs to
C     test on .GT.0, whereas hour/minute/second needs to test on .GE.0.
                  DO  m = 1, 3
                     iftime(m, ii, hrscan) = itime(m, i, j)
                     IF ( iftime(m, ii, hrscan) .GT. 0 .AND.
     +                    .NOT. date_set_yet ) THEN
                        jftime(m) = iftime(m, ii, hrscan)
                        jftime_set_yet(m) = .TRUE.
                     ENDIF
                  END DO
                  DO  m = 4, 6
                     iftime(m, ii, hrscan) = itime(m, i, j)
                     IF ( iftime(m, ii, hrscan) .GE. 0 .AND.
     +                    .NOT. date_set_yet ) THEN
                        jftime(m) = iftime(m, ii, hrscan)
                        jftime_set_yet(m) = .TRUE.
                     ENDIF
                  END DO
C     4/16/13 EJN - Once jftime is defined for all 6 elements, the date to
C     be used in MK_ONAME is set, so no longer define jftime array on
C     subsequent invocations of this loop.
C     
                  IF ( jftime_set_yet(1) .AND. jftime_set_yet(2) .AND.
     +                 jftime_set_yet(3) .AND. jftime_set_yet(4) .AND.
     +                 jftime_set_yet(5) .AND. jftime_set_yet(6) )
     +                 date_set_yet = .TRUE.
C     DEBUGTM
C             write(*,*) (jftime(m), m = 1, 6)
CDEBUGTMP 

c Sapiano May2013 - older code replaced by previous edit:
c             DO  m = 1, 6
c                  iftime(m, ii, hrscan) = itime(m, i, j)
c     END DO
                  
                  IF ( precip(i,j) .GE. 0. .AND. iqflag(i,j) .LT. 2 )
     +                 ifp(ii,hrscan)  = precip(i,j)


                 hrscan = ( i2gran(i,j) - 1 ) * 45 + jj - ioffset
                  use_this_scan(hrscan) = .TRUE.
C                                                                                                                   
               END IF
            END DO
         END DO

C     
C       The time field "ift" represents minutes since the start time,
C	so it couldn't be computed until we knew what the start time
C       would be.  Compute it only for the range of data we will be 
C       using, i.e. scan lines iscan1 to nscan.  The variables 
C	stacked in iftime are year, month, day, hour, minute, second.
C
         DO  n = iscan1, nscan
            DO  m = 1, MXFOV
             IF  ( iftime (1, m, n) .EQ. iftime (1, ifov1, iscan1) .AND. 
     +            iftime (2, m, n) .EQ. iftime (2, ifov1, iscan1) .AND.
     +            iftime (3, m, n) .EQ. iftime (3, ifov1, iscan1) ) THEN
                  ift (m, n) = 
     +             ( iftime (4, m, n) - iftime (4, ifov1, iscan1) ) * 60
     +               + ( iftime (5, m, n) - iftime (5, ifov1, iscan1) )
               ELSE
                  ift (m, n) = 
     +             ( iftime (4, m, n) - iftime (4, ifov1, iscan1) ) * 60
     +               + ( iftime (5, m, n) - iftime (5, ifov1, iscan1) )
     +                 + 1440
               END IF
            END DO
         END DO


C
C       Loop through the scans, first computing the FOV orientation
C       along the whole scan, then processing each FOV.  The zones
C       are separated into successive DO's.
C
C	The pad of 2 deg. on the call to FOV_ORNS ensures orientation 
C	information for the call to FOV2FGNS; that call only has a 
C	0.25-deg pad because it's pixel-by-pixel.  An extra scan-
C	center test is done N and S to catch a possible protrusion in
C	the middle due to curved coordinates.
C
C       Process only those scans for which assignments to flat, flon,
C       etc. have been made, i.e., for which use_this_scan is .TRUE.
C     
         DO  n = iscan1, nscan
            IF (use_this_scan(n)) THEN
               IF  ( flat (1,       n) .GT.  LATEDGE-2. .OR.
     +              flat (MXFOV/2, n) .GT.  LATEDGE-2. .OR.
     +              flat (MXFOV,   n) .GT.  LATEDGE-2. )
     +             CALL FOV_ORNS ( flon (1, n), flat (1, n), xorn, yorn,
     +              iret )
               DO  m = 1, MXFOV
                  IF  ( iqual(m, n) .LT. 2 .AND.
     +                 flat (m, n) .GT.  LATEDGE-0.25 )
     +                 CALL FOV2FGNS ( flon (m, n), flat (m, n),
     +                 ift  (m, n), ifp  (m, n),  xorn (m),
     +                 yorn (m),    w (m), s (m), influn,
     +                 ipn, itn,    iret )
               END DO
            END IF
         END DO
         DO  n = iscan1, nscan
            IF (use_this_scan(n)) THEN
               IF  ( ( flat (1,     n) .LT.  LATEDGE+2. .OR.
     +              flat (MXFOV, n) .LT.  LATEDGE+2. ) .AND.
     +              ( flat (1,     n) .GT. -LATEDGE-2. .OR.
     +              flat (MXFOV, n) .GT. -LATEDGE-2. ) )
     +             CALL FOV_ORC  ( flon (1, n), flat (1, n), xorc, yorc,
     +              iret )
               DO  m = 1, MXFOV
                  IF  ( iqual(m, n) .LT. 2 .AND.
     +                 flat (m, n) .LT.  LATEDGE+0.25 .AND.
     +                 flat (m, n) .GT. -LATEDGE-0.25 )
     +                 CALL FOV2FGC  ( flon (m, n), flat (m, n),
     +                 ift  (m, n), ifp  (m, n),  xorc (m),
     +                 yorc (m),    w (m), s (m), influc,
     +                 ipc, itc,    iret )
               END DO
            END IF
         END DO
         DO  n = iscan1, nscan
            IF (use_this_scan(n)) THEN
               IF  ( flat (1,       n) .LT. -LATEDGE+2. .OR.
     +              flat (MXFOV/2, n) .LT. -LATEDGE+2. .OR.
     +              flat (MXFOV,   n) .LT. -LATEDGE+2. )
     +             CALL FOV_ORNS ( flon (1, n), flat (1, n), xors, yors,
     +              iret )
               DO  m = 1, MXFOV
                  IF  ( iqual(m, n) .LT. 2 .AND.
     +                 flat (m, n) .LT. -LATEDGE+0.25 )
     +                 CALL FOV2FGNS ( flon (m, n), flat (m, n),
     +                 ift  (m, n), ifp  (m, n),  xors (m),
     +                 yors (m),    w (m), s (m), influs,
     +                 ips, its,    iret )
               END DO
            END IF
         END DO
         
C
C       Generate "bogus" (synthetic) scan lines at iscan1-1 and
C       nscan+1.  Use these values to repeat the procedure above.
C       This will result in certain pixels being assigned to the
C       bogus lines, and then not counted in the gridding procedure
C       fg_avg.
C
C       Synthetic lat/lon values may be obtained using the actual
C       values of lines iscan1-1 and nscan+1, as long as they are
C       not all zeroes, which sometimes happens.  If they are,
C       extrapolate.  NOTE: By definition, loscan = 1.
C
         IF  ( iscan1 .EQ. loscan )  GO TO 48
         DO  m = 1, MXFOV
            flatb (m, 1) = flat (m, iscan1-1)
            flonb (m, 1) = flon (m, iscan1-1)
         END DO
C     
C       Bypass the extrapolation as soon as an FOV is encountered
C       that is NOT lat=lon=0.  However, check against small numbers
C       because it is possible to get lat/lon values all set to 1.E-38.
C
         DO  m = 1, MXFOV
            IF  ( ABS ( flatb (m, 1) ) .GT. 0.0001 .OR.
     +           ABS ( flonb (m, 1) ) .GT. 0.0001 )
     +           GO TO 50
         END DO
C     
C     For scan line iscan1, extrapolate lat/lon if everything was
C       set to zeroes.
C
 48      DO  m = 1, MXFOV
            flatb (m, 1) = flat (m, iscan1) +
     +           ( flat (m, iscan1) - flat (m, iscan1+1) )
            flonb (m, 1) = flon (m, iscan1) +
     +           ( flon (m, iscan1) - flon (m, iscan1+1) )
            IF  ( flonb (m, 1) .GT. 180. )  THEN
               flonb (m, 1) = flonb (m, 1) - 360.
            ELSE IF  ( flonb (m, 1) .LT. -180. )  THEN
               flonb (m, 1) = flonb (m, 1) + 360.
            END IF
         END DO
C     
 50      CONTINUE
         IF  ( nscan .EQ. hiscan )  GO TO 52
         DO  m = 1, MXFOV
            flatb (m, 2) = flat (m, nscan+1)
            flonb (m, 2) = flon (m, nscan+1)
         END DO
C     
C     Bypass the extrapolation as soon as an FOV is encountered
C     that is NOT lat=lon=0.  However, check against small numbers
C     because it is possible to get lat/lon values all set to 1.E-38.
C     
         DO  m = 1, MXFOV
            IF  ( ABS ( flatb (m, 2) ) .GT. 0.0001 .OR.
     +          ABS ( flonb (m, 2) ) .GT. 0.0001 )  GO TO 54
         END DO
C     
C     For scan line nscan+1, extrapolate lat/lon if everything was
C     set to zeroes.
C     
 52      DO  m = 1, MXFOV
            flatb (m, 2) = flat (m, nscan) +
     +           ( flat (m, nscan) - flat (m, nscan-1) )
            flonb (m, 2) = flon (m, nscan) +
     +           ( flon (m, nscan) - flon (m, nscan-1) )
            IF  ( flonb (m, 2) .GT. 180. )  THEN
               flonb (m, 2) = flonb (m, 2) - 360.
            ELSE IF  ( flonb (m, 1) .LT. -180. )  THEN
               flonb (m, 2) = flonb (m, 2) + 360.
            END IF
         END DO
C     
 54      CONTINUE
C     
C     Set the time field for the first bogus line to the first value 
C     at line iscan1 and for the second bogus line to the first value
C     at line nscan.  Set ifp to a fake value of -98.
C     
         DO  m = 1, MXFOV
            iftb (m, 1) = ift (m, iscan1)
            iftb (m, 2) = ift (m, nscan )
            ifpb (m, 1) = -98
            ifpb (m, 2) = -98
         END DO
C             
        
         DO  n = 1, 2
            IF  ( flatb(1,       n) .GT.  LATEDGE-2. .OR.
     +           flatb(MXFOV/2, n) .GT.  LATEDGE-2. .OR.
     +           flatb(MXFOV,   n) .GT.  LATEDGE-2. )
     +           CALL FOV_ORNS ( flonb(1, n), flatb(1, n), xorn, yorn,
     +           iret )
            DO  m = 1, MXFOV
               IF  ( iqual(m, n) .LT. 2 .AND.
     +              flatb(m, n) .GT.  LATEDGE-0.25 )
     +              CALL FOV2FGNS ( flonb(m, n), flatb(m, n),
     +              iftb (m, n), ifpb (m, n),  xorn (m),
     +              yorn (m),    w (m), s (m), influn,
     +              ipn, itn,    iret )
            END DO
         END DO
         DO  n = 1, 2
            IF  ( ( flatb(1,     n) .LT.  LATEDGE+2. .OR.
     +           flatb(MXFOV, n) .LT.  LATEDGE+2. ) .AND.
     +           ( flatb(1,     n) .GT. -LATEDGE-2. .OR.
     +           flatb(MXFOV, n) .GT. -LATEDGE-2. ) )
     +           CALL FOV_ORC  ( flonb(1, n), flatb(1, n), xorc, yorc,
     +           iret )
            DO  m = 1, MXFOV
               IF  ( iqual(m, n) .LT. 2 .AND.
     +              flatb(m, n) .LT.  LATEDGE+0.25 .AND.
     +              flatb(m, n) .GT. -LATEDGE-0.25 )
     +              CALL FOV2FGC  ( flonb(m, n), flatb(m, n),
     +              iftb (m, n), ifpb (m, n),  xorc (m),
     +              yorc (m),    w (m), s (m), influc,
     +              ipc, itc,    iret )
            END DO
         END DO
         DO  n = 1, 2
            IF  ( flatb(1,       n) .LT. -LATEDGE+2. .OR.
     +           flatb(MXFOV/2, n) .LT. -LATEDGE+2. .OR.
     +           flatb(MXFOV,   n) .LT. -LATEDGE+2. )
     +           CALL FOV_ORNS ( flonb(1, n), flatb(1, n), xors, yors,
     +           iret )
            DO  m = 1, MXFOV
               IF  ( iqual(m, n) .LT. 2 .AND.
     +              flatb(m, n) .LT. -LATEDGE+0.25 )
     +              CALL FOV2FGNS ( flonb(m, n), flatb(m, n),
     +              iftb (m, n), ifpb (m, n),  xors (m),
     +              yors (m),    w (m), s (m), influs,
     +              ips, its,    iret )
            END DO
         END DO
C     
C     Now average the fine grid to the 1.0-deg grid.
C
 70      CALL FG_AVG_1DEG ( ipn,  ipc,  ips,  itn, itc, its, ip1, iret )
C     
C       Add this hour's totals to the daily accumulation, after first
C       dividing by 100. to get back mm/hr.
C
         DO j = 1, 180
            DO i = 1, 360
               IF ( ip1(i,j) .GE. 0 ) THEN
                  sum(i,j)  = sum(i,j) + FLOAT(ip1(i,j))/100.
                  rnum(i,j) = rnum(i,j) + 1.
               END IF
            END DO
         END DO
C     DEBUG
         write(*,*) 'hour ', ihour, '; (180,90) ip1 = ', ip1(180,90),
     &        ' sum = ', sum(180,90), ' num = ', rnum(180,90)
C     END DEBUG
C     
C     Loop back up to process the next hour of the day.
C     
 999  END DO
C     
C     Compute the daily average.
C
	DO j = 1, 180
	DO i = 1, 360
	   IF ( rnum(i,j) .GT. 0.) airs1dd(i,j) = 24.*sum(i,j)/rnum(i,j)
	END DO
	END DO
C
C       Create the output file name, find a free FORTRAN unit number,
C       and write the file.
C

c     Sapiano, may 2013: changed to still print output for partial day  
c     CALL MK_ONAME_1DD ( iftime (1, ifov1, iscan1), oname, iret )
C
        IF ( jftime(1) .LE. 1990 .OR. jftime(2) .LE. 0 .OR.
     +       jftime(3) .LE. 0 ) STOP
C
        CALL MK_ONAME_1DD ( jftime, oname, iret )                                                         
       
        do l_on = 1,len_trim(oname)
           if (oname(l_on:l_on) .eq. ' ') then
              continue
           end if
        end do


        IF  ( iret .NE. 0 )  THEN
           STOP
        ENDIF
        WRITE (*, *) 'Open/write output file: ', oname (1:l_on)
C
        out_file   = outpath_g (1:l_outdir) // oname (1:l_on)
        l_fullpath = l_outdir + l_on
C
        in_use = .TRUE.
        iunit  = 2
        DO  WHILE ( in_use )
            iunit = iunit + 1
            INQUIRE ( iunit, OPENED=in_use )
        END DO

        print *, out_file(1:l_fullpath)

        OPEN ( iunit, FILE=out_file(1:l_fullpath), STATUS='replace',
     +         FORM='UNFORMATTED', ACCESS='DIRECT',RECL=4*64800 )
C
        write (iunit, rec=1) airs1dd
        CLOSE (iunit)
CDEBUG
        write(*,*) 'sum/num/avg at (180,90) = ', sum(180,90),
     +             rnum(180,90), airs1dd(180,90)
CEND DEBUG
C


      end program Main_GPCPcdr_m2_Grid_Airs

