	SUBROUTINE CONVERT_TIME ( time1993, MXLN, MXLT, itime )
C***********************************************************************
C* SEC2K2T  Convert seconds from 1993010100 to 6-integer date/time     *
C*								       *
C* This subroutine converts seconds since the start of 1 Jan. 1993 to  *
C* 6-integer time (YYYY,MM,DD,HH,MM,SS).			       *
C*								       *
C* CONVERT_TIME ( time1993 itime )			               *
C*								       *
C* Input parameters:						       *
C*   time1993   REAL8  Number of seconds from the start of 1993 (may   *
C*		       be <0)					       *
C*  MXLN        INT     X-dimension of input/output arrays (1440)      *
C*  MXLT        INT     Y-dimension of input/output arrays (720)       *
C*                                                                     *
C* Output parameters:						       *
C*   itime	INT2   Corresponding 6-integer time:YYYY,MM,DD,HH,MM,SS*
C**								       *
C* Log:								       *
C* G.Huffman/SSAI	 2/04	Work backwards from T2SEC2K.F	       *
C* E.Nelkin/SSAI         2/04   Adapted from /usr/user0/huffman/src6/  *
C*                              T2SEC2K.F, but from start of 1993.     *
C*                              Changed data types of input/output.    *
C* E.Nelkin/SSAI         3/04   Add do_debug for write statements      *
C* E.Nelkin/SSAI         2/07   Adapted from /rain/nelkin/amsr/code/   *
C*                              DELIVERY_V6.4/convert_time.F           * 
C* E.Nelkin/SSAI         3/07   Pass in MXLN, MXLT                     *
C***********************************************************************
	IMPLICIT	NONE
C
        INTEGER*4       i, j, k
        INTEGER*4       MXLN, MXLT
C       PARAMETER       ( MXLN = 1440, MXLT = 720 )
        REAL*8          time1993(MXLN, MXLT)
	INTEGER*2	itime (6, MXLN, MXLT)
        INTEGER*4       nsec
	INTEGER*4	iymd,      jyyyy, jday, jsec, leap1993,
     +			leapyyyy,  iday,  isec, nday, leapdex,
     +			MONTH (12, 2)
	DATA		MONTH
     +			/ 0,31,59,90,120,151,181,212,243,273,304,334,
     +			  0,31,60,91,121,152,182,213,244,274,305,335 /
C       INTEGER*4       ierroutn
C
        DO  j = 1, MXLT
        DO  i = 1, MXLN
C
C       If the grid box is outside the swath, and hence time is missing,
C       simply assign the missing value from time1993 to all six
C       elements of itime, then jump to the bottom of the loop.
C
        IF  ( time1993(i,j) .LT. 0. ) THEN
            itime(1, i, j) = time1993(i,j)
            itime(2, i, j) = time1993(i,j)
            itime(3, i, j) = time1993(i,j)
            itime(4, i, j) = time1993(i,j)
            itime(5, i, j) = time1993(i,j)
            itime(6, i, j) = time1993(i,j)
            GO TO 90
        END IF
C
C       First convert the real*8 time1993 array value to integer*4 nsec.
C
        nsec = IDINT(time1993(i,j))
C
C	Pull out number of seconds into the day and the number of whole
C	days.  Then convert day seconds into minutes, hours, seconds.
C	The test on jsec<0 is the only mod needed to handle nsec
C	negative (i.e., before the start of 1993).
C
	jsec      = MOD ( nsec, 86400 )
	IF  ( jsec .LT. 0 )  jsec = jsec + 86400
	nday      = ( nsec - jsec ) / 86400
	itime (4, i, j) = jsec / 3600
	itime (5, i, j) = ( jsec - itime (4, i, j) * 3600 ) / 60
	itime (6, i, j) = ( jsec - itime (4, i, j) * 3600
     +                                           - itime (5, i, j) * 60)
C
C	Take a first cut at the year and jday.  Work backwards from:
C	leapyyyy = (jyyyy - 1) / 4 + 1
C	nday     = (jyyyy - 1993) * 365 + leapyyyy - leap1993 + jday - 1
C
	jyyyy    = nday / 365 + 1993
	leap1993 = (1993  - 1) / 4 + 1
	leapyyyy = (jyyyy - 1) / 4 + 1
	jday     = nday - ( ( jyyyy - 1993 ) * 365 +
     +			    leapyyyy - leap1993 - 1 )
C
C	jday below 1 means we need to back up a year; a value past
C	the end of the year (noting leap year!) adds a year.  In either
C	case, recompute affected variables.  Finally, load the year
C	array value.
C
	IF  ( jday .LE. 0 )  THEN
	    jyyyy    = jyyyy - 1
	    leapyyyy = (jyyyy - 1) / 4 + 1
	    jday     = nday - ( ( jyyyy - 1993 ) * 365 +
     +				leapyyyy - leap1993 - 1 )
	  ELSE IF ( ( jday .GT. 365 .AND. MOD ( jyyyy, 4 ) .NE. 0 ) .OR.
     +		    ( jday .GT. 366 .AND. MOD ( jyyyy, 4 ) .EQ. 0 ) )
     +								    THEN
	    jyyyy    = jyyyy + 1
	    leapyyyy = (jyyyy - 1) / 4 + 1
	    jday     = nday - ( ( jyyyy - 1993 ) * 365 +
     +				leapyyyy - leap1993 - 1 )
	END IF
	itime (1, i, j) = jyyyy
C
C	Noting leap year, work backwards from December to compare jday
C	to the Julian day at the end of the previous month to pull out
C	the month index; the day of the month is the remainder.
C
	IF  ( MOD ( jyyyy, 4 ) .NE. 0 )  THEN
	    leapdex = 1
	  ELSE
	    leapdex = 2
	END IF
	DO  k = 12, 1, -1
	    IF  ( jday .GT. MONTH (k, leapdex) )  GO TO 10
	END DO
C       ierroutn = E_3B42_ERROR_CONVERTING_TIME
C       call reprt_error_amsre(ierroutn)
   10	CONTINUE
	itime (2, i, j) = k
	itime (3, i, j) = jday - MONTH (k, leapdex)
	IF  ( itime (2, i, j) .EQ. 12 .AND. itime (3, i, j) .GT. 31 )
     +                                                              THEN
C           ierroutn = E_3B42_ERROR_CONVERTING_TIME
C           call reprt_error_amsre(ierroutn)
	END IF
C
   90   END DO
        END DO
C
	RETURN
	END
