	SUBROUTINE FG_AVG_1DEG ( ipn,  ipc,  ips,  itn,  itc, its, ip1,
     +	 			 iret)
C************************************************************************
C* FG_AVG  Average the fine-grid AIRS to a uniform CED			*
C*									*
C* This subroutine averages the fine-grid AIRS data to a uniform CED at	*
C* 1.0-deg resolution.	This involves whole-box averaging in all zones:	*
C* 1. In the central zone whole fine-grid boxes are averaged to yield	*
C*    an approximate 1.0x1.0-deg-of-lat resolution.			*
C* 2. In the north and south zones the 16x16 template of fine-grid boxes*
C*    most nearly centered on the 1.0-deg CED box are averaged.	        *
C*									*
C* FG_AVG_1DEG  ( ipn,  ipc, ips, itn, itc, its, ip1, iret )		*
C*									*
C* Input parameters:							*
C*   ipn	INT2*	Precip on the fine north grid			*
C*   ipc	INT2*	Precip on the fine central grid			*
C*   ips	INT2*	Precip on the fine south grid			*
C*   itn	INT1*	Time after start on the fine north grid		*
C*   itc	INT1*	Time after start on the fine central grid	*
C*   its	INT1*	Time after start on the fine south grid		*
C*									*
C* Output parameters:							*
C*   ip1	INT2*	Precip on the 1.0-deg CED grid			*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 5/06	Adapt amsu_grid/fg_avg.f (new special	*
C*				value tests, assoc. documentation);	*
C*				remove scaling precip by 10 - full 100x	*
C*				scaling now in READSWATH		*
C* E.Nelkin/SSAI         3/07   L2G: Full 100x scaling done in READ_AIRS*
C* E.Nelkin/SSAI         9/07   1x1-degree version, to match up vs TOVS,*
C*                              returns precip field only               *
C* G.Huffman/SSAI	10/07	Correct variable grid sizes (in		*
C*				PARAMETERs and code); only do avg if	*
C*				at least half the FG contribute		*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i, j, m, n, mm, ma, mb, na, nb, ipsum, itsum, 
     +			nsum, nambig, nrain, mxtra, nfgx, nfgy
	REAL*4		c, x, y, cedrad, cedthet
C
C	NFGFR, NCEDFR, MCED2FG are the number of FG and CED grid 
C	intervals in a deg. of lat/lon, and the multiplier from CED to
C	FG.
C
	INTEGER*4	LATEDGE, NFGFR, NCEDFR, MCED2FG, 
     +			NCX,     NCY,   NNSX,   NNSY, 
     +			MXLN,    MXLT,  NSSZ 
	INTEGER*2	I2MISS
	REAL*4		CED2FG,  RE,    SZDEG
	PARAMETER	( LATEDGE = 75,
     +			  NFGFR   = 16,
     +			  NCEDFR  = 1,
     +			  NCX     = 360              * NFGFR,
     +			  NCY     = LATEDGE          * NFGFR * 2,
     +			  NNSX    = ( 90 - LATEDGE ) * NFGFR * 2,
     +			  NNSY    = ( 90 - LATEDGE ) * NFGFR * 2,
     +			  RE      = 6371.,
     +			  SZDEG   = 3.1415926 * 2. * RE / 360.,
     +			  I2MISS  = -31999,
     +			  MXLN    = 360 * NCEDFR,
     +			  MXLT    = 180 * NCEDFR,
     +			  NSSZ    = NCEDFR * ( 90-LATEDGE ) )
	INTEGER*2	ipn    (NNSX,  NNSY), 
     +			ipc    (NCX,   NCY ), ips (NNSX, NNSY),  
     +			ip1    (MXLN,  MXLT)
	INTEGER*1	itn    (NNSX,  NNSY), 
     +			itc    (NCX,   NCY ), its  (NNSX, NNSY)
	INTEGER*4	iret
C
c     Need to convert degrees to radians - SAPIANO
        REAL deg2rad
        deg2rad = 4*ATAN(1.0)/180

c	Moved these from parameter declaration to resolve bug 
c       caused by function use in parameter declaration; Sapiano
	CED2FG  = NFGFR / FLOAT ( NCEDFR )
	MCED2FG = NINT ( CED2FG )


	iret = 0
C
C	Average up the boxes on the center zone.  Wrapping across the 
C	lateral boundaries is needed to accomodate the reach in X done
C	to approximately preserve area.  
C	  The outer loops are over the CED boxes covered by the central
C	fine grid.
C
	DO  j = 1, MXLT - NSSZ*2
	    na = ( j - 1 ) * MCED2FG + 1
	    nb = j * MCED2FG
	    mxtra = NINT ( 0.5 * MCED2FG / 
     +			   COS ( ( LATEDGE - (j - 0.5) / NCEDFR ) *deg2rad) )
	    DO  i = 1, MXLN
		ma     = NINT ( ( i - 0.5 ) * MCED2FG - mxtra )
		mb     = NINT ( ( i - 0.5 ) * MCED2FG + mxtra )
		ipsum  = 0
		itsum  = 0
		nsum   = 0
		nambig = 0
		nrain  = 0
Cc		Test for real rain; umambiguous zero (-11 desert); -10 
Cc		undetermined because of i) high-lat. frozen sfc (defined
Cc		here as outside 75N-S, or the north and south zones, so
Cc		not relevant to the center, treated as ambiguous zero), 
Cc		and ii) retrieval non-convergence (defined here as 
Cc		inside 75N-S, or the central zone, so not relevant to 
Cc		the north and south, treated as unambiguous zero); and 
Cc		ambiguous zero (-7 snow, -8 seaice, -9 high-lat. coast, 
Cc		-12 high elevation, -13 high-lat. frozen surface, and  
Cc		all remaining rain flags other than "missing" (I2MISS 
Cc		explicit, -99 algorithm missing, -98 start/stop 
Cc		padding).  
C
		DO  n = na, nb
		DO  m = ma, mb
		    IF  ( m .GT. NCX )  THEN
			mm = m - NCX
		      ELSE IF  ( m .LT. 1 )  THEN
			mm = m + NCX
		      ELSE
			mm = m
		    END IF
		    IF  ( ipc (mm, n) .GE. 0 )  THEN
			ipsum  = ipsum  + ipc (mm, n)
			itsum  = itsum  + itc (mm, n)
			nsum   = nsum   + 1
			IF  ( ipc (mm, n) .GT. 0 )
     +			    nrain = nrain + 1
cc		      ELSE IF  ( ipc (mm, n) .EQ. -11 )  THEN
cc			itsum  = itsum  + itc (mm, n)
cc			nsum   = nsum   + 1
cc		      ELSE IF  ( ipc (mm, n) .EQ. -10 )  THEN
cc			itsum  = itsum  + itc (mm, n)
cc			nsum   = nsum   + 1
cc		      ELSE IF  ( ipc (mm, n) .NE. I2MISS .AND.
cc     +				 ipc (mm, n) .NE. -99    .AND.
cc     +				 ipc (mm, n) .NE. -98 )  THEN
cc			itsum  = itsum  + itc (mm, n)
cc			nsum   = nsum   + 1
cc			nambig = nambig + 1
		    END IF
		END DO
		END DO
C
C		If at least half of the possible boxes are reported,
C		compute average.  Precipitation no longer scaled by 10.
C               to get 100.*mm/hr - done in READSWATH.
C
		IF  ( nsum .GE. (mb-ma+1) * (nb-na+1) / 2 )  THEN
		    ip1 (i, j+NSSZ) = NINT ( ipsum / FLOAT ( nsum ) )
		  ELSE
		    ip1 (i, j+NSSZ) = I2MISS
		END IF
	    END DO
	END DO
C
C	In the north zone find the FG box corner closest to a CED box
C	center, then average over the MCED2FGxMCED2FG template most
C	nearly centered on that corner.
C	  The outer loops are over the CED grid boxes covered by the 
C	north Tangent Polar grid.
C	  Converting lat/lon. to polar coordinates centered on the 
C	North Pole, cedrad is in the FG 1/16-deg-of-lat increments and
C	cedthet is in degrees.
C	  The fine grid is a y-increasing-down left-handed coordinate,
C	so the conversion from polar coordinates enters nfgy with a 
C	minus sign.
C         Wrapping across lateral boundaries is not a concern in the
C       north zone.
C
CC
CC	test of adding a strip in the north polar zone.
CC
CC	DO  j = NNSY / 2 - 15, NNSY / 2 + 16
CC	DO  i = 1, NNSX / 2
CC	    ipn (i, j) = 100
CC	    itn (i, j) = 10
CC	END DO
CC	END DO
CC
	DO  j = 1, NSSZ
	    cedrad  = ( j - 0.5 ) * MCED2FG
	    DO  i = 1, MXLN
		cedthet = ( i - 0.5 ) / NCEDFR
		nfgx    = NINT (  cedrad * COS ( deg2rad*cedthet ) + NNSX*0.5 )
		nfgy    = NINT ( -cedrad * SIN ( deg2rad*cedthet ) + NNSY*0.5 )
		ma      = MAX ( nfgx - MCED2FG / 2 + 1, 1    )
		mb      = MIN ( nfgx + MCED2FG / 2    , NNSX )
		na      = MAX ( nfgy - MCED2FG / 2 + 1, 1    )
		nb      = MIN ( nfgy + MCED2FG / 2    , NNSY )
		ipsum   = 0
		itsum   = 0
		nsum    = 0
		nambig  = 0
		nrain   = 0
CCDEBUG
CC              IF (i.eq.1) write(*,*) MCED2FG, NNSX, NNSY, NCEDFR,
CC   +			j, cedrad, cedthet, nfgx, nfgy, ma, mb,
CC   &                                 na, nb
CCEND DEBUG
C
Cc		Test for real rain; umambiguous zero (-11 desert); -10 
Cc		undetermined because of i) high-lat. frozen sfc (defined
Cc		here as outside 75N-S, or the north and south zones, so
Cc		not relevant to the center, treated as ambiguous zero), 
Cc		and ii) retrieval non-convergence (defined here as 
Cc		inside 75N-S, or the central zone, so not relevant to 
Cc		the north and south, treated as unambiguous zero); and 
Cc		ambiguous zero (-7 snow, -8 seaice, -9 high-lat. coast, 
Cc		-12 high elevation, -13 high-lat. frozen surface, and  
Cc		all remaining rain flags other than "missing" (I2MISS 
Cc		explicit, -99 algorithm missing, -98 start/stop 
Cc		padding).  
C               m is not converted to mm because wrapping across lateral 
C		boundaries is not a concern in the north zone.
C
		DO  n = na, nb
		DO  m = ma, mb
		    IF  ( ipn (m, n) .GE. 0 )  THEN
			ipsum  = ipsum  + ipn (m, n)
			itsum  = itsum  + itn (m, n)
			nsum   = nsum   + 1
			IF  ( ipn (m, n) .GT. 0 )
     +			    nrain = nrain + 1
cc		      ELSE IF  ( ipn (m, n) .EQ. -11 )  THEN
cc			itsum  = itsum  + itn (m, n)
cc			nsum   = nsum   + 1
cc		      ELSE IF  ( ipn (m, n) .EQ. -10 )  THEN
cc			itsum  = itsum  + itn (m, n)
cc			nsum   = nsum   + 1
cc			nambig = nambig + 1
cc		      ELSE IF  ( ipn (m, n) .NE. I2MISS .AND.
cc     +				 ipn (m, n) .NE. -99    .AND.
cc     +				 ipn (m, n) .NE. -98 )  THEN
cc			itsum  = itsum  + itn (m, n)
cc			nsum   = nsum   + 1
cc			nambig = nambig + 1
		    END IF
		END DO
		END DO
C
C		If at least half of the possible boxes are reported,
C		compute average.  Precipitation no longer scaled by 10.
C               to get 100.*mm/hr - done in READSWATH.
C
		IF  ( nsum .GE. (mb-ma+1) * (nb-na+1) / 2 )  THEN
		    ip1 (i, j) = NINT ( ipsum / FLOAT ( nsum ) )
		  ELSE
		    ip1 (i, j) = I2MISS
		END IF
	    END DO
	END DO
C
C	Now repeat for the south zone.  The code is identical to the
C	north zone except for:
C	* j ranging over the southern zone,
C	* j and cedrad incrementing in opposite directions,
C	* i and cedthet incrementing in opposite directions, and
C	* reference to ips and its,
C
	DO  j = MXLT - NSSZ + 1, MXLT
	    cedrad  = ( MXLT - j + 0.5 ) * MCED2FG
	    DO  i = 1, MXLN
		cedthet = ( MXLN - i + 0.5 ) / NCEDFR
		nfgx    = NINT (  cedrad * COS ( deg2rad*cedthet ) + NNSX*0.5 )
		nfgy    = NINT ( -cedrad * SIN ( deg2rad*cedthet ) + NNSY*0.5 )
		ma      = MAX ( nfgx - MCED2FG / 2 + 1, 1    )
		mb      = MIN ( nfgx + MCED2FG / 2    , NNSX )
		na      = MAX ( nfgy - MCED2FG / 2 + 1, 1    )
		nb      = MIN ( nfgy + MCED2FG / 2    , NNSY )
		ipsum   = 0
		itsum   = 0
		nsum    = 0
		nambig  = 0
		nrain   = 0
C
Cc		Test for real rain; umambiguous zero (-11 desert); -10 
Cc		undetermined because of i) high-lat. frozen sfc (defined
Cc		here as outside 75N-S, or the north and south zones, so
Cc		not relevant to the center, treated as ambiguous zero), 
Cc		and ii) retrieval non-convergence (defined here as 
Cc		inside 75N-S, or the central zone, so not relevant to 
Cc		the north and south, treated as unambiguous zero); and 
Cc		ambiguous zero (-7 snow, -8 seaice, -9 high-lat. coast, 
Cc		-12 high elevation, -13 high-lat. frozen surface, and  
Cc		all remaining rain flags other than "missing" (I2MISS 
Cc		explicit, -99 algorithm missing, -98 start/stop 
Cc		padding).  
C               m is not converted to mm because wrapping across lateral 
C		boundaries is not a concern in the  south zone.
C
		DO  n = na, nb
		DO  m = ma, mb
		    IF  ( ips (m, n) .GE. 0 )  THEN
			ipsum  = ipsum  + ips (m, n)
			itsum  = itsum  + its (m, n)
			nsum   = nsum   + 1
			IF  ( ips (m, n) .GT. 0 )
     +			    nrain = nrain + 1
cc		      ELSE IF  ( ips (m, n) .EQ. -11 )  THEN
cc			itsum  = itsum  + its (m, n)
cc			nsum   = nsum   + 1
cc		      ELSE IF  ( ips (m, n) .EQ. -10 )  THEN
cc			itsum  = itsum  + its (m, n)
cc			nsum   = nsum   + 1
cc			nambig = nambig + 1
cc		      ELSE IF  ( ips (m, n) .NE. I2MISS .AND.
cc     +				 ips (m, n) .NE. -99    .AND.
cc     +				 ips (m, n) .NE. -98 )  THEN
cc			itsum  = itsum  + its (m, n)
cc			nsum   = nsum   + 1
cc			nambig = nambig + 1
		    END IF
		END DO
		END DO
C
C		If at least half of the possible boxes are reported,
C		compute average.  Precipitation no longer scaled by 10.
C               to get 100.*mm/hr - done in READSWATH.
C
		IF  ( nsum .GE. (mb-ma+1) * (nb-na+1) / 2 )  THEN
		    ip1 (i, j) = NINT ( ipsum / FLOAT ( nsum ) )
		  ELSE
		    ip1 (i, j) = I2MISS
		END IF
	    END DO
	END DO
C
	RETURN
	END
