	SUBROUTINE FOV_DIM  ( w, s, iret )
C************************************************************************
C* FOV_DIM  Compute the dimensions of AIRS FOVs for a scan		*
C*									*
C* This subroutine approximates the AMSU FOV sizes in a scan.  The	*
C* geometry is based on program fov_B.c, sent by Ralph Ferraro (NESDIS	*
C* ORA) on 9/8/03.							*
C*									*
C* FOV_DIM  ( w, s, iret )						*
C*									*
C* Output parameters:							*
C*   w		REAL	FOV width (cross-scan) in km			*
C*   s		REAL	FOV length (along-scan) in km			*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 5/06	Adapt amsu_grid/fov_dim.f; add FRSTANG;	*
C*				change MXFOV, SCANINC, RSAT, sc_ang	*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i
	REAL*4		sc_ang, s_ec2s, co_ec2s
	REAL*8		xx, xy, yy, xor2, xoryor2, yor2, hor2s2, hor2w2
C
	INTEGER*4	LATEDGE, NFGFRAC, NCX, NCY, MXFOV
	REAL*4		RE, SZDEG, SCANINC, RSAT, FRSTANG
	PARAMETER	( LATEDGE = 75,
     +			  NFGFRAC = 16,
     +			  NCX     = 360     * NFGFRAC,
     +			  NCY     = LATEDGE * NFGFRAC * 2,
     +			  RE      = 6371.,
     +			  SZDEG   = 3.1415926 * 2. * RE / 360.,
     +			  MXFOV   = 30,
     +			  SCANINC = 3.3,
     +			  RSAT    = 705.3,
     +			  FRSTANG = -47.9 )
C
	REAL*4		w (MXFOV), s (MXFOV)
	INTEGER*4	iret
C
	REAL*4		e_a (MXFOV+1), d (MXFOV+1)
C

c     Need to convert degrees to radians - SAPIANO
        REAL deg2rad
        deg2rad = 4*ATAN(1.0)/180


CZZZZZSTRT
c	WRITE (*, *) 'i, sc_ang, s_cozen, ang11, e_a (i), d (i)'
CZZZZZSTOP
	iret = 0
C
C	sc_ang  = scan angle (measured from nadir, in deg)
C	s_ec2s  = sine of Earth-center-to-satellite angle at FOV
C	co_ec2s = 180deg - Earth-center-to-satellite angle at FOV (ATAN
C		  returns in the interval [-90.90], but ec2s is > 90deg)
C	e_a     = Earth angle from satellite to FOV in deg
C	d       = satellite-to-FOV distance in km
C
	DO  i = 1, MXFOV+1


	    sc_ang  = SCANINC * ( i - 1.5 ) + FRSTANG
	    s_ec2s  = ( RE + RSAT ) * SIN ( sc_ang *deg2rad ) / RE
	    co_ec2s = ATAN ( ( s_ec2s / SQRT ( 1. - s_ec2s*s_ec2s ) ) 
     +         )/deg2rad 
	    e_a (i) = co_ec2s - sc_ang

	    IF  ( sc_ang .EQ. 0. )  THEN
		d (i) = RSAT
	      ELSE
		d (i) = RE * SIN ( e_a (i)*deg2rad ) / SIN ( sc_ang*deg2rad )
	    END IF
	    IF  ( i .GT. 1 )  THEN
		s (i-1) = ABS ( e_a (i) - e_a (i-1) ) * RE      * 
     +			  3.1415926 / 180.
		w (i-1) = ( d (i-1) + d (i) ) * 0.5   * SCANINC * 
     +			  3.1415926 / 180.
CZZZZZSTRT
c	    WRITE (*, *) i, sc_ang, e_a (i), d (i)
CZZZZZSTOP
	    END IF
	END DO
C
	RETURN
	END
