	SUBROUTINE FOV_ORC  ( flon, flat, xor, yor, iret )
C************************************************************************
C* FOV_ORC  Find AIRS FOV orientations on the central fine grid		*
C*									*
C* This subroutine approximates the (x,y) orientation of each FOV in a	*
C* scan for the central fine grid.  We just use the location of the	*
C* adjacent FOVs, except for a one-sided computation for the edge FOVs.	*
C* NOTE: The grid is left-handed because y increases down.		*
C*									*
C* FOV_ORC  ( flon, flat, xor, yor, iret )				*
C*									*
C* Input parameters:							*
C*   flon	REAL	FOV central longitude				*
C*   flat	REAL	FOV central latitude				*
C*									*
C* Output parameters:							*
C*   xor	REAL	X component of FOV orientation in FG coord.	*
C*   yor	REAL	Y component of FOV orientation in FG coord.	*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 5/06	Adapt amsu_grid/fov_orc.f; change MXFOV	*
C*				to 30					*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	nx,  ny,  nxa, nxb, nya, nyb, i, j
	INTEGER*1	i_d2
C
	INTEGER*4	LATEDGE, NFGFR, NCX, NCY, MXFOV
	REAL*4		WTEST,   WRAP
	PARAMETER	( LATEDGE = 75,
     +			  NFGFR   = 16,
     +			  NCX     = 360     * NFGFR,
     +			  NCY     = LATEDGE * NFGFR * 2,
     +			  MXFOV   = 30,
     +			  WTEST   = 350. * NFGFR,
     +			  WRAP    = 360. * NFGFR )
	REAL*4		flon (MXFOV), flat (MXFOV), xor (MXFOV), 
     +			yor (MXFOV)
	INTEGER*4	iret
C
	REAL*4		fx0 (MXFOV), fy0 (MXFOV)
C
	iret = 0
C
C	Unlike FOV2FGC, there is no actual gridding, so we don't need
C	to keep longitude in the range 0-360.
C
	DO  i = 1, MXFOV
	    fx0 (i) = flon (i) * NFGFR
	    fy0 (i) = ( LATEDGE - flat (i) ) * NFGFR
	END DO
C
C	Compute the X and Y orientation in fine grid coordinates.
C	Unlike FOV2FGC, it does matter if neighborinng FOVs jump
C	across the lateral grid wrapping boundary (the test on 
C	+/-WTEST).  The test is done here so that we catch both
C	the -180/+180 and 0/360 boundaries.  Do centered differences on 
C	the interior of the scan, then single-sided differences at the 
C	ends, setting to missing if the data aren't available.
C
	DO  i = 2, MXFOV-1
	    xor (i) = fx0 (i+1) - fx0 (i-1)
	    IF  ( xor (i) .LT. -WTEST )  xor (i) = xor (i) + WRAP
	    IF  ( xor (i) .GT.  WTEST )  xor (i) = xor (i) - WRAP
	    yor (i) = fy0 (i+1) - fy0 (i-1)
	END DO
	xor (1) = fx0 (2) - fx0 (1)
	IF  ( xor (1) .LT. -WTEST )  xor (1) = xor (1) + WRAP
	IF  ( xor (1) .GT.  WTEST )  xor (1) = xor (1) - WRAP
	yor (1) = fy0 (2) - fy0 (1)
	xor (MXFOV) = fx0 (MXFOV) - fx0 (MXFOV-1)
	IF  ( xor (MXFOV) .LT. -WTEST )  xor (MXFOV) = xor (MXFOV) + 
     +						       WRAP
	IF  ( xor (MXFOV) .GT.  WTEST )  xor (MXFOV) = xor (MXFOV) - 
     +						       WRAP
	yor (MXFOV) = fy0 (MXFOV) - fy0 (MXFOV-1)
C
	RETURN
	END
