	SUBROUTINE FOV_ORNS  ( flon, flat, xor, yor, iret )
C************************************************************************
C* FOV_ORNS  Find AIRS FOV orientations on the north, south fine grids	*
C*									*
C* This subroutine approximates the (x,y) orientation of each FOV in a	*
C* scan for the Tangent Equidistant north and south fine grids.  We	*
C* just use the location of the adjacent FOVs, except for a one-sided	*
C* computation for the edge FOVs.					*
C* NOTE: The routine makes no checks that the latitudes are appropriate	*
C* to the hemisphere's grids being passed in.				*
C* NOTE: The grid is left-handed because y increases down.		*
C*									*
C* FOV_ORNS  ( flon, flat, xor, yor, iret )				*
C*									*
C* Input parameters:							*
C*   flon	REAL	FOV central longitude				*
C*   flat	REAL	FOV central latitude				*
C*									*
C* Output parameters:							*
C*   xor	REAL	X component of FOV orientation in FG coord.	*
C*   yor	REAL	Y component of FOV orientation in FG coord.	*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 5/06	Adapt amsu_grid/fov_orns.f; change	*
C*				MXFOV to 30				*
C* E.Nelkin/SSAI	 8/12	LINUX Version				*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	nxa, nxb, nya, nyb, i, j
	INTEGER*1	i_d2
	REAL*4		hemi
C
	INTEGER*4	LATEDGE, NFGFR, NX, NY, MXFOV
	PARAMETER	( LATEDGE = 75,
     +			  NFGFR    = 16,
     +			  NX      = ( 90 - LATEDGE ) * NFGFR * 2,
     +			  NY      = ( 90 - LATEDGE ) * NFGFR * 2,
     +			  MXFOV   = 30 )
	CHARACTER	zone
	REAL*4		flon (MXFOV), flat (MXFOV), xor (MXFOV), 
     +			yor (MXFOV)
	INTEGER*4	iret
C
	REAL*4		fx0 (MXFOV), fy0 (MXFOV)
C

c     Need to convert degrees to radians - SAPIANO
        REAL deg2rad
        deg2rad = 4*ATAN(1.0)/180


	iret = 0
C
C	Unlike FOV2FGC, there is no actual gridding, so we don't need
C	to keep longitude in the range 0-360.
C
	DO  i = 1, MXFOV
	    IF  ( flat (i) .GE. 0. )  THEN
		hemi = 1
	      ELSE
		hemi = -1
	    END IF
	    fx0 (i) = NX * 0.5 + ( 90. - hemi * flat (i) ) * 
     +				 COS (deg2rad*hemi * flon (i)) * NFGFR
	    fy0 (i) = NY * 0.5 - ( 90. - hemi * flat (i) ) * 
     +				 SIN (deg2rad*hemi * flon (i)) * NFGFR
	END DO
C
C	Compute the X and Y orientation in fine grid coordinates.
C	Like FOV2FGC, there is no Prime Meridian jump in X.  Do 
C	centered differences on the interior of the scan, then 
C	single-sided differences at the ends.
C
	DO  i = 2, MXFOV-1
	    xor (i) = fx0 (i+1) - fx0 (i-1)
	    yor (i) = fy0 (i+1) - fy0 (i-1)
	END DO
	xor (1) = fx0 (2) - fx0 (1)
	yor (1) = fy0 (2) - fy0 (1)
	xor (MXFOV) = fx0 (MXFOV) - fx0 (MXFOV-1)
	yor (MXFOV) = fy0 (MXFOV) - fy0 (MXFOV-1)
C
	RETURN
	END
