	SUBROUTINE GETL2GFD (isd_id, iindex, ihour, outdata)
C
C***********************************************************************
C* GETL2GFD	Reads a REAL*4 2-dimensional array of AIRS L2G data.   *
C*								       *
C* Called by:	READ_AIRS					       *
C*   								       *
C* Calls:	GSDSINFO					       *
C*  								       *
C* GETL2GFD extracts and returns a REAL*4 2-dimensional data array of  *
C* AIRS L2G data.						       *
C*  								       *
C* Input Parameters:						       *
C*								       *
C*  isd_id 	INT	AIRS HDF File ID			       *
C*  iindex 	INT	Index of SDS to be read			       *
C*  ihour 	INT	Hour of data to be read 		       *
C* 								       *
C* Output Parameters:						       *
C*								       *
C*  outdata 	REAL*4	array of output AIRS data		       *
C* 								       *
C**								       *
C* Log:								       *
C* D.Bolvin/SSAI	02/07	First Version			       *
C* E.Nelkin/SSAI	08/12	LINUX Version			       *
C***********************************************************************
C
	IMPLICIT	NONE
C
C***********************************************************************
C	Include header file for FORTRAN HDF parameter definitions.
C
C	INCLUDE		"/usr/apps/32/hdf4.1r1/include/hdf.inc"
C	INCLUDE		"/usr/local/hdf4/include/hdf.inc"
C       Changed by Sapiano
c	INCLUDE		"/usr/local/hdf-4.2.6/include/hdf.inc"
	INCLUDE		"hdf.inc"
C
C***********************************************************************
C	Declaration of internal subroutine variables.
C
	INTEGER*4	i, j, k, ihour
	INTEGER*4	isd_id, isds_id, istatus
	INTEGER*4	istart(3), istride(3), iedges(3)
        INTEGER*4	iindex
	INTEGER*4	sfrdata, sfendacc, heprnt
C
	REAL*4		outdata(1440,720), temp(24,1440,720)
C
C***********************************************************************
C	Define the start and stride, which is the same for all the 
C	arrays which will be read.
C
	istart(1) = 0
	istart(2) = 0
	istart(3) = 0
	istride(1) = 1
	istride(2) = 1
	istride(3) = 1
C
C***********************************************************************
C	Get the SDS ID for the data of interest.
C
	CALL GSDSINFO (isd_id, iindex, isds_id, iedges)
C
C***********************************************************************
C	Read the SDS into an array.
C
	istatus = sfrdata (isds_id, istart, istride, iedges, temp)
C
	IF  (istatus .EQ. FAIL) THEN
	    WRITE (*,*) 'ERROR: UNABLE TO GET READ SDS DATA; ',
     +			'TERMINATING EXECUTION; SUBROUTINE ',
     +			'GETL2GFD'
	    istatus = heprnt (0)
	    STOP
	END IF
C
C***********************************************************************
C	Reorient the data to standard GPCP format (N-S flip and 
C	180-degree translate in width).
C
	DO i = 1, 1440
	    DO j = 1, 720
		IF  (i .LE. 720) THEN
	    	    outdata(i,j) = temp(ihour,i + 720,721 - j)
	        ELSE
	            outdata(i,j) = temp(ihour,i - 720,721 - j)
	        END IF 
	    END DO
	END DO
C
C***********************************************************************
C	Close the SDS.
C
	istatus = sfendacc (isds_id)
C
	IF  (istatus .EQ. FAIL) THEN
	    WRITE (*,*) 'ERROR: UNABLE TO CLOSE METADATA SDS;',
     +			' TERMINATING EXECUTION; SUBROUTINE ',
     +			'GTBDATA'
	    istatus = heprnt (0)
	    STOP
	END IF
C
	RETURN
	END
