#define RMISS		-999

#define TRUE		1
#define FALSE		0

#define LOW_SAMPS	90
#define HI_SAMPS	180

#define AB_SCAN	        2
#define LOW_CHANS	5
#define HI_CHANS	2

#define HISTSCALE	2.5
#define TOPLAT		65
#define BOTLAT		(-65)

#define NO_OF_BINS	        195
#define NO_OF_RAIN_RATES	64
#define NO_OF_FREEZE_LEVELS	60

#define NO_OF_COLS	((int)(360/HISTSCALE))	           /*  No of longitude cols  */
#define NO_OF_ROWS	((int)((TOPLAT-BOTLAT)/HISTSCALE)) /*  No of latitude  rows  */

#define RETURN_FAIL 1

enum ReadRtn	{ READ_OK, READ_EOF };
enum isOK		{ OK, NOT_OK };



/** ---------------------- V6 SSMIS Variables ---------------------- **/

struct UnpackString_header_v6 {
   long			ksat, numscan;
};

struct UnpackString_v6 {
   int 		 	scanrecord;
   unsigned char	btime[24];   	//time for the begin of the b-scan
   int			ztime;
   int			zorbit;
   int			zsclat, zsclon;   //spacecraft latitude and east longitude (deg)
   int			zscalt;          //spacecraft altitude (km)
   short		zflags[15];
   short		lat[LOW_SAMPS];		//latitude of low-freq footprint (deg; requires scaling)
   short		lon[LOW_SAMPS];		//longitude of low-freq footprint (deg; requires scaling)
   short		ztht[LOW_SAMPS];		//cell incidence angle (deg; requires scaling)
   short		zeaz[LOW_SAMPS];		//earth azimuth angle (deg; requires scaling)
   short		zsun[LOW_SAMPS];		//sun glitter angle (deg; requires scaling)
   short		ztoil[LOW_SAMPS];	//surface flag
   short		ztb[LOW_SAMPS][LOW_CHANS];  // 19v 19h 22v 37v 37h
   short		lat_hi[HI_SAMPS];	//latitude of 92GHz footprint (deg; requires scaling)
   short		lon_hi[HI_SAMPS];	//longitude of 92GHz footprint (deg; requires scaling)
   short		ztht_hi[HI_SAMPS];	//cell incidence angle of 92GHz footprint (deg; requires scaling)
   short		zeaz_hi[HI_SAMPS];	//earth azimuth angle of 92GHz footprint (deg; requires scaling)
   short		zsun_hi[HI_SAMPS];	//sun glitter angle of 92GHz footprint (deg; requires scaling)
   short		ztoil_hi[HI_SAMPS];	//surface flag of 92GHz footprint
   short		ztb_hi[HI_SAMPS][HI_CHANS];	// 92v 92h

};

struct Histogram {
   unsigned short	v19[NO_OF_BINS];
   unsigned short	h19[NO_OF_BINS];
   unsigned short	v22[NO_OF_BINS];
   unsigned short	v37[NO_OF_BINS];
   unsigned short	h37[NO_OF_BINS];
   unsigned short	v85[NO_OF_BINS];
   unsigned short	h85[NO_OF_BINS];
   unsigned short	PC1[NO_OF_BINS];
   unsigned short	RainRate[NO_OF_RAIN_RATES];
   char			unused1[32];
   unsigned short	FreezeLevel[NO_OF_FREEZE_LEVELS];
   char			r_0 [4];	/* Will hold a float value when written out */
   char			r_s [4];	/* Will hold a float value when written out */
   char			r_d [4];	/* Will hold a float value when written out */
   char			s2_0[4];	/* Will hold a float value when written out */
   char			s2_s[4];	/* Will hold a float value when written out */
   char			s2_d[4];	/* Will hold a float value when written out */
   unsigned short	ntot;
   unsigned short	n_s;
   unsigned short	n_d;
   char			unused2[80];
};

