#include <stdio.h>
#include <stdlib.h>
#include <netcdf.h>
#include <string.h>

#include "Define_RSSv6Tb_ssmis_5deg.h"

#define footprint_number_hires  180
#define footprint_number_lores  90
#define max_scan                10000
#define eleven_flags            11
#define four_flags              4 
#define numchar                 20



/* Required for error checking reading NetCDF files */
#define ERR(e) {fprintf(stderr,"Error (METH RSS Tb Read): %s\n", nc_strerror(e)); return 2;;}



int ReadNC_RSS_SSMIS_Tb_v6_nscan(char *fname)
{
  int     ncid;   /* Netcdf file unit */
  int     retval; /* Status flag for netcdf files */
  char    recname[NC_MAX_NAME+1]; /* dummy var used by nc_inq_dim */
  int     nscan_varid;
  size_t  nscan;

  /* Open nc file and get number of scans from header */
  if (retval = nc_open(fname, NC_NOWRITE, &ncid)) {
    fprintf(stderr,"Can't open RSS V6 file: %s\nNC Error: %s\n",
	    fname,nc_strerror(retval));
    return;
  }

  /* get number of scans dimension */
  if (retval = nc_inq_dimid(ncid, "scan_number",&nscan_varid))      ERR(retval);
  if (retval = nc_inq_dim(ncid, nscan_varid, recname, &nscan)) ERR(retval);

  /* Close netcdf file */
  if ((retval = nc_close(ncid))) ERR(retval);

  return((int)nscan);
}




int ReadNC_RSS_SSMIS_Tb_v6(char *fname, struct UnpackString_header_v6 *RSStb_header, struct UnpackString_v6 *RSStb)
{

  int     ncid;   /* Netcdf file unit */
  int     retval; /* Status flag for netcdf files */
  char    recname[NC_MAX_NAME+1]; /* dummy var used by nc_inq_dim */
  char    platform[100]; /* Platform attribute from file */
  int     iscan;
  int     i, j;

  char   *tmptext;
  char    sattext[2];
  char    date[numchar];
  float   offset;

  /* variables used to convert julian day to yyyy, mm */
  int JD, Z, W, X, A, B, C, D, E, mm, yyyy;

  /* Declare variable IDs */
  int nscan_varid;
  int iorbit_varid;
  int scan_time_hires_varid;
  int orbit_position_varid;
  int sc_lat_varid;
  int sc_lon_varid;
  int sc_alt_varid;
  int iscn_flag_varid;
  int ical_flag_lores_varid;
  int latitude_hires_varid;
  int longitude_hires_varid;
  int earth_incidence_angle_hires_varid;
  int earth_azimuth_angle_hires_varid;
  int sun_glitter_angle_hires_varid;
  int land_flag_hires_varid;
  int ice_flag_hires_varid;
  int fcdr_brightness_temperature_92V_varid;
  int fcdr_brightness_temperature_92H_varid;
  int latitude_lores_varid;
  int longitude_lores_varid;
  int earth_incidence_angle_lores_varid;
  int earth_azimuth_angle_lores_varid;
  int sun_glitter_angle_lores_varid;
  int land_flag_lores_varid;
  int ice_flag_lores_varid;
  int fcdr_brightness_temperature_19V_varid;
  int fcdr_brightness_temperature_19H_varid;
  int fcdr_brightness_temperature_22V_varid;
  int fcdr_brightness_temperature_37V_varid;
  int fcdr_brightness_temperature_37H_varid;

  /* Declare variables to be read */
  size_t nscan;
  int iorbit;
  double scan_time_hires[max_scan];
  double orbit_position[max_scan];
  float sc_lat[max_scan];
  float sc_lon[max_scan];
  float sc_alt[max_scan];
  char iscn_flag[eleven_flags][max_scan];
  char ical_flag_lores[four_flags][max_scan];
  short int latitude_hires[max_scan][footprint_number_hires];
  short int longitude_hires[max_scan][footprint_number_hires];
  short int earth_incidence_angle_hires[max_scan][footprint_number_hires];
  short int earth_azimuth_angle_hires[max_scan][footprint_number_hires];
  short int sun_glitter_angle_hires[max_scan][footprint_number_hires];
  signed char land_flag_hires[max_scan][footprint_number_hires];
  signed char ice_flag_hires[max_scan][footprint_number_hires];
  float fcdr_brightness_temperature_92V[max_scan][footprint_number_hires];
  float fcdr_brightness_temperature_92H[max_scan][footprint_number_hires];
  short int latitude_lores[max_scan][footprint_number_lores];
  short int longitude_lores[max_scan][footprint_number_lores];
  short int earth_incidence_angle_lores[max_scan][footprint_number_lores];
  short int earth_azimuth_angle_lores[max_scan][footprint_number_lores];
  short int sun_glitter_angle_lores[max_scan][footprint_number_lores];
  signed char land_flag_lores[max_scan][footprint_number_lores];
  short int ice_flag_lores[max_scan][footprint_number_lores];
  float fcdr_brightness_temperature_19V[max_scan][footprint_number_lores];
  float fcdr_brightness_temperature_19H[max_scan][footprint_number_lores];
  float fcdr_brightness_temperature_22V[max_scan][footprint_number_lores];
  float fcdr_brightness_temperature_37V[max_scan][footprint_number_lores];
  float fcdr_brightness_temperature_37H[max_scan][footprint_number_lores];


  /* Open nc file and get sat number and number of scans from header */
  if (retval = nc_open(fname, NC_NOWRITE, &ncid)) {
    fprintf(stderr,"Can't open RSS V6 file: %s\nNC Error: %s\n",
	    fname,nc_strerror(retval));
    return;
  }

  /* Read some global attributes */
  if (retval = nc_get_att_text(ncid, NC_GLOBAL, 
			       "platform", platform))                ERR(retval);

  /* get number of scans dimension */
  if (retval = nc_inq_dimid(ncid, "scan_number",&nscan_varid))       ERR(retval);
  if (retval = nc_inq_dim(ncid, nscan_varid, recname,&nscan))        ERR(retval);




  /************************************************
 *    * Read variables from file                     */
  if (nscan == 0) { // Do not continue if there is no data to be read
    printf("Zero scans in file: nothing to read;"); return 2;
  }


  /* Get variable ids */
  if (retval = nc_inq_varid(ncid, "iorbit", 
			    &iorbit_varid))                          ERR(retval);
  if (retval = nc_inq_varid(ncid, "scan_time_hires", 
			    &scan_time_hires_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "orbit_position", 
			    &orbit_position_varid))                  ERR(retval);
  if (retval = nc_inq_varid(ncid, "sc_lat", 
			    &sc_lat_varid))                          ERR(retval);
  if (retval = nc_inq_varid(ncid, "sc_lon", 
			    &sc_lon_varid))                          ERR(retval);
  if (retval = nc_inq_varid(ncid, "sc_alt", 
			    &sc_alt_varid))                          ERR(retval);
  if (retval = nc_inq_varid(ncid, "iscn_flag", 
			    &iscn_flag_varid))                       ERR(retval);
  if (retval = nc_inq_varid(ncid, "ical_flag_lores", 
			    &ical_flag_lores_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "latitude_hires", 
			    &latitude_hires_varid))                  ERR(retval);
  if (retval = nc_inq_varid(ncid, "longitude_hires", 
			    &longitude_hires_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "earth_incidence_angle_hires", 
			    &earth_incidence_angle_hires_varid))     ERR(retval);
  if (retval = nc_inq_varid(ncid, "earth_azimuth_angle_hires", 
			    &earth_azimuth_angle_hires_varid))       ERR(retval);
  if (retval = nc_inq_varid(ncid, "sun_glitter_angle_hires", 
			    &sun_glitter_angle_hires_varid))         ERR(retval);
  if (retval = nc_inq_varid(ncid, "land_flag_hires", 
			    &land_flag_hires_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "ice_flag_hires", 
			    &ice_flag_hires_varid))                  ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_92V", 
			    &fcdr_brightness_temperature_92V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_92H", 
			    &fcdr_brightness_temperature_92H_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "latitude_lores", 
			    &latitude_lores_varid))                  ERR(retval);
  if (retval = nc_inq_varid(ncid, "longitude_lores", 
			    &longitude_lores_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "earth_incidence_angle_lores", 
			    &earth_incidence_angle_lores_varid))     ERR(retval);
  if (retval = nc_inq_varid(ncid, "earth_azimuth_angle_lores", 
			    &earth_azimuth_angle_lores_varid))       ERR(retval);
  if (retval = nc_inq_varid(ncid, "sun_glitter_angle_lores", 
			    &sun_glitter_angle_lores_varid))         ERR(retval);
  if (retval = nc_inq_varid(ncid, "land_flag_lores", 
			    &land_flag_lores_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "ice_flag_lores", 
			    &ice_flag_lores_varid))                  ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_19V", 
			    &fcdr_brightness_temperature_19V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_19H", 
			    &fcdr_brightness_temperature_19H_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_22V", 
			    &fcdr_brightness_temperature_22V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_37V", 
			    &fcdr_brightness_temperature_37V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_37H", 
			    &fcdr_brightness_temperature_37H_varid)) ERR(retval);


    
  /* Read variables */
  if (retval = nc_get_var_int(ncid, iorbit_varid, 
			       &iorbit))                                 ERR(retval);
  if (retval = nc_get_var_double(ncid, scan_time_hires_varid, 
			       &scan_time_hires[0]))                     ERR(retval);
  if (retval = nc_get_var_double(ncid, orbit_position_varid, 
			       &orbit_position[0]))                      ERR(retval);
  if (retval = nc_get_var_float(ncid, sc_lat_varid, 
			       &sc_lat[0]))                              ERR(retval);
  if (retval = nc_get_var_float(ncid, sc_lon_varid, 
			       &sc_lon[0]))                              ERR(retval);
  if (retval = nc_get_var_float(ncid, sc_alt_varid, 
			       &sc_alt[0]))                              ERR(retval);
  if (retval = nc_get_var_schar(ncid, iscn_flag_varid, 
			       &iscn_flag[0][0]))                        ERR(retval);
  if (retval = nc_get_var_schar(ncid, ical_flag_lores_varid, 
			       &ical_flag_lores[0][0]))                  ERR(retval);
  if (retval = nc_get_var_short(ncid, latitude_hires_varid, 
			       &latitude_hires[0][0]))                   ERR(retval);
  if (retval = nc_get_var_short(ncid, longitude_hires_varid, 
			       &longitude_hires[0][0]))                  ERR(retval);
  if (retval = nc_get_var_short(ncid, earth_incidence_angle_hires_varid, 
			       &earth_incidence_angle_hires[0][0]))      ERR(retval);
  if (retval = nc_get_var_short(ncid, earth_azimuth_angle_hires_varid, 
			       &earth_azimuth_angle_hires[0][0]))        ERR(retval);
  if (retval = nc_get_var_short(ncid, sun_glitter_angle_hires_varid, 
			       &sun_glitter_angle_hires[0][0]))          ERR(retval);
  if (retval = nc_get_var_schar(ncid, land_flag_hires_varid, 
			       &land_flag_hires[0][0]))                  ERR(retval);
  if (retval = nc_get_var_schar(ncid, ice_flag_hires_varid, 
			       &ice_flag_hires[0][0]))                   ERR(retval);
  if (retval = nc_get_var_float(ncid, fcdr_brightness_temperature_92V_varid, 
			       &fcdr_brightness_temperature_92V[0][0]))  ERR(retval);
  if (retval = nc_get_var_float(ncid, fcdr_brightness_temperature_92H_varid, 
			       &fcdr_brightness_temperature_92H[0][0]))  ERR(retval);
  if (retval = nc_get_var_short(ncid, latitude_lores_varid, 
			       &latitude_lores[0][0]))                   ERR(retval);
  if (retval = nc_get_var_short(ncid, longitude_lores_varid, 
			       &longitude_lores[0][0]))                  ERR(retval);
  if (retval = nc_get_var_short(ncid, earth_incidence_angle_lores_varid, 
			       &earth_incidence_angle_lores[0][0]))      ERR(retval);
  if (retval = nc_get_var_short(ncid, earth_azimuth_angle_lores_varid, 
			       &earth_azimuth_angle_lores[0][0]))        ERR(retval);
  if (retval = nc_get_var_short(ncid, sun_glitter_angle_lores_varid, 
			       &sun_glitter_angle_lores[0][0]))          ERR(retval);
  if (retval = nc_get_var_schar(ncid, land_flag_lores_varid, 
			       &land_flag_lores[0][0]))                  ERR(retval);
  if (retval = nc_get_var_short(ncid, ice_flag_lores_varid, 
			       &ice_flag_lores[0][0]))                   ERR(retval);
  if (retval = nc_get_var_float(ncid, fcdr_brightness_temperature_19V_varid, 
			       &fcdr_brightness_temperature_19V[0][0]))  ERR(retval);
  if (retval = nc_get_var_float(ncid, fcdr_brightness_temperature_19H_varid, 
			       &fcdr_brightness_temperature_19H[0][0]))  ERR(retval);
  if (retval = nc_get_var_float(ncid, fcdr_brightness_temperature_22V_varid, 
			       &fcdr_brightness_temperature_22V[0][0]))  ERR(retval);
  if (retval = nc_get_var_float(ncid, fcdr_brightness_temperature_37V_varid, 
			       &fcdr_brightness_temperature_37V[0][0]))  ERR(retval);
  if (retval = nc_get_var_float(ncid, fcdr_brightness_temperature_37H_varid, 
			       &fcdr_brightness_temperature_37H[0][0]))  ERR(retval);
 
  /* Close netcdf file */
  if ((retval = nc_close(ncid))) ERR(retval);


  /****************************************************
 *    * Copy data into structure for output to main call */

  /* Copy data into SSMI header */
  (*RSStb_header).numscan=nscan;

  /* Get satellite number from global attributes 
 *    * Need to find start of satellite number first  */ 
  tmptext=(char*)memchr(platform,'F',strlen(platform));
  memcpy(sattext,&tmptext[1],2);
  (*RSStb_header).ksat=atoi(sattext);
 
    
  /* copy data into main output array
   *    * nscan is the record length  */
  for (iscan=0; iscan<nscan; iscan++) {

    /* Use algorithm to get days since epoch (2000-01-01 00:00:00) 
       Note: 2000-01-01 00:00:00 = 2451544.5 Julian Days */
    JD=(int)(scan_time_hires[iscan]/86400);
    Z = JD+0.5+2451544.5;
    W = (Z - 1867216.25)/36524.25;
    X = W/4;
    A = Z+1+W-X;
    B = A+1524;
    C = (B-122.1)/365.25;
    D = 365.25*C;
    E = (B-D)/30.6001;
    mm = E-1;
    if (mm>12) {
      mm = E-13;
    }
    if (mm<=2) {
      yyyy = C-4715;
    }
    else {
      yyyy = C-4716;
    }
    /* Get year and month and store in btime YYYY-MM-DD HH-MM-SSZ */
    sprintf(RSStb[iscan].btime,"%04d-%02d-00 00-00-00Z",yyyy,mm);
    
    RSStb[iscan].scanrecord=iscan+1;
    RSStb[iscan].ztime=0;
    RSStb[iscan].zorbit=(int)orbit_position[iscan];

    /* Following are stored as scaled long to be compatible with METH SSMI code */
    RSStb[iscan].zsclat=(long)(sc_lat[iscan]*1.e3);
    RSStb[iscan].zsclon=(long)(sc_lon[iscan]*1.e3);
    RSStb[iscan].zscalt=(long)(sc_alt[iscan]*1.e3);

    for (i=0; i<11; i++) {
      RSStb[iscan].zflags[i]=(int)iscn_flag[i][iscan];
    }
    for (i=11; i<15; i++) {
      RSStb[iscan].zflags[i]=(int)ical_flag_lores[i][iscan];
    }

    for (i=0; i<LOW_SAMPS; i++) {
      RSStb[iscan].lat[i]=latitude_lores[iscan][i];

      /* METH code expects lon to be between 0:360, then scaled and offset 
 *        * by -180. This is a legacy of the code and is very tricky to fix... */
      if ( ((longitude_lores[iscan][i]*0.01)>=-180.) && 
           ((longitude_lores[iscan][i]*0.01)<0.) ) {
        RSStb[iscan].lon[i]=(int)(((float)longitude_lores[iscan][i]*0.01+360-180)/0.01);
      }
      else {
        RSStb[iscan].lon[i]=(int)(((float)longitude_lores[iscan][i]*0.01-180)/0.01);
      }

      /*
      if ( (i==45) & (RSStb[iscan].lat[i]>=500) & (RSStb[iscan].lat[i]<1000) ) {
	printf("%d  ",RSStb[iscan].lat[i]);
	for (j=0; j<11; j++) printf("%d ",RSStb[iscan].zflags[j]);
	printf("  %f %d %d %d\n",orbit_position[iscan],RSStb[iscan].zorbit,yyyy,mm);
	}*/

      RSStb[iscan].ztht[i]=(earth_incidence_angle_lores[iscan][i]*0.01-45)/2.E-3;
      RSStb[iscan].zeaz[i]=earth_azimuth_angle_lores[iscan][i];
      RSStb[iscan].zsun[i]= sun_glitter_angle_lores[iscan][i];

      /* Combine surface type and sea ice: 0=open_ocean, 1=near_coast, 
 *        * 2=land, 4=ocean+ice, 5=coast+ice, 6=land+ice/snow            
 *               * Fill values are 10 for each variable - set to -9 if either occurs */
       RSStb[iscan].ztoil[i]=(int) land_flag_lores[iscan][i]+4*(int) ice_flag_lores[iscan][i];
      if ((land_flag_lores[iscan][i]==10)|(ice_flag_lores[iscan][i]==10)) {
        RSStb[iscan].ztoil[i]=-9;
      }

      RSStb[iscan].ztb[i][0]=(int)(fcdr_brightness_temperature_19V[iscan][i]*100);
      RSStb[iscan].ztb[i][1]=(int)(fcdr_brightness_temperature_19H[iscan][i]*100);
      RSStb[iscan].ztb[i][2]=(int)(fcdr_brightness_temperature_22V[iscan][i]*100);
      RSStb[iscan].ztb[i][3]=(int)(fcdr_brightness_temperature_37V[iscan][i]*100);
      RSStb[iscan].ztb[i][4]=(int)(fcdr_brightness_temperature_37H[iscan][i]*100);
    }


    for (i=0; i<HI_SAMPS; i++) {
      RSStb[iscan].lat_hi[i]= latitude_hires[iscan][i];

      /* METH code expects lon to be between 0:360, then scaled and offset 
 *        * by -180. This is a legacy of the code and is very tricky to fix... */
      if ( ((longitude_hires[iscan][i]*0.01)>=-180) && 
           ((longitude_hires[iscan][i]*0.01)<0) ) {
        RSStb[iscan].lon_hi[i]=(int)(((float)longitude_hires[iscan][i]*0.01+360-180)/0.01);
      }
      else {
        RSStb[iscan].lon_hi[i]=(int)(((float)longitude_hires[iscan][i]*0.01-180)/0.01);
      }

      RSStb[iscan].ztht_hi[i]=(earth_incidence_angle_hires[iscan][i]*0.01-45)/2.E-3;
      RSStb[iscan].zeaz_hi[i]=earth_azimuth_angle_hires[iscan][i];
      RSStb[iscan].zsun_hi[i]= sun_glitter_angle_hires[iscan][i];

      /* Combine surface type and sea ice: 0=open_ocean, 1=near_coast, 
 *        * 2=land, 4=ocean+ice, 5=coast+ice, 6=land+ice/snow            
 *               * Fill values are 10 for each variable - set to -9 if either occurs */
       RSStb[iscan].ztoil_hi[i]=(int) land_flag_hires[iscan][i]+4*(int) ice_flag_hires[iscan][i];
      if ((land_flag_hires[iscan][i]==10)|(ice_flag_hires[iscan][i]==10)) {
        RSStb[iscan].ztoil_hi[i]=-9;
      }

      RSStb[iscan].ztb_hi[i][0]=(int)(fcdr_brightness_temperature_92V[iscan][i]*100);
      RSStb[iscan].ztb_hi[i][1]=(int)(fcdr_brightness_temperature_92H[iscan][i]*100);
    }
  }


  return(0);
}


