#include   <stdio.h>
#include   <string.h>
#include   <math.h>
#include   <inttypes.h>
#include   <unistd.h>
#include   <fcntl.h>
#include   "Define_RSSv6Tb_ssmis_5deg.h"

extern struct UnpackString_v6		t6;
struct Histogram hist[NO_OF_COLS][NO_OF_ROWS], hist2[NO_OF_COLS][NO_OF_ROWS];   
float		RainRateScan[LOW_SAMPS];
float	FreezeLevel[LOW_SAMPS];
static float	RainRateBins[NO_OF_RAIN_RATES];
float	r_0[NO_OF_COLS][NO_OF_ROWS], s2_0[NO_OF_COLS][NO_OF_ROWS];
float	r_s[NO_OF_COLS][NO_OF_ROWS], s2_s[NO_OF_COLS][NO_OF_ROWS];
float	r_d[NO_OF_COLS][NO_OF_ROWS], s2_d[NO_OF_COLS][NO_OF_ROWS];

#define CUT1 0.5
#define CUT2 20.

#define BEG19V_WATER	100
#define BEG19H_WATER	80
#define BEG22V_WATER	100
#define BEG37V_WATER	100
#define BEG37H_WATER	80
#define BEG85V_WATER	100
#define BEG85H_WATER	100
#define BEGPC1_WATER	100		/* Pseudo Channel 1 [2*19V - 22V] */

#define DEBUG_RAINRATEBINS_OFF

enum isOK UpdateBins(int iver) {

   float lat, lon, tb19v, tb19h, tb22v, tb37v, tb37h, tb92v, tb92h;
	short row, col;
	int index, i, u, scan; 
   signed char sflag, landocean, seaice;

   /** call RainRate() to get rainrates and freeze levels for this scan **/
   RainRate();
	
   /**************************** Low Channels ****************************/
   /* Low channels (19v 19h 22v 37v 37h) are taken                       */ 
	/* only during odd (1,3,5,...) scan positions along the A-scan.       */ 
	for (i=0;i<LOW_SAMPS;i++) {
	   //In C, index begins with 0, so the index of odd scan will be 0,2,4,...instead
		index = i*2;
		//lat = t6.lat[0][index]*0.01;
		//lon = t6.lon[0][index]*0.01+180;      
		lat = t6.lat[i]*0.01;
		lon = t6.lon[i]*0.01+180;

      // Update the bins only between TOPLAT and BOTLAT
      // Update the bins only open ocean and no sea ice found
      if (lat<TOPLAT && lat>BOTLAT) {
         //sflag = t6.sflag[0][index];
         sflag = t6.ztoil[i];
         landocean = sflag-4*(sflag/4); //0 = ocean, 1 = coast, 2 = land
         seaice = sflag/4; //0 = no possibility of sea ice, 1 = possible sea ice         
         if (seaice == 0 && landocean == 0) { 
	    tb19v = t6.ztb[i][0]*0.01;
            tb19h = t6.ztb[i][1]*0.01;
	    tb22v = t6.ztb[i][2]*0.01;
            tb37v = t6.ztb[i][3]*0.01;
            tb37h = t6.ztb[i][4]*0.01;

            // If iver = 6, convert the RSS SSMIS V7 brightness temperatures
            // to the V6 SSM/I standard.

            if ( iver == 6 ) {
               tb19v = -0.143481 + 1.003207*tb19v + 0.005177*tb19h;
               tb19h = -0.090130 - 0.001055*tb19v + 1.009745*tb19h;
               tb22v = -0.387787 + 1.005719*tb22v;
               tb37v = -0.091821 + 0.985167*tb37v + 0.011956*tb37h;
               tb37h = -0.344555 + 0.005039*tb37v + 0.992195*tb37h;
            }

            row = (TOPLAT - lat) / HISTSCALE;
	         col = lon / HISTSCALE;
            //fprintf(stderr,"lat=%.3f row=%d lon=%.3f col=%d ",lat,row,lon,col);
            //printf("low channels lat=%.2f lon=%.2f ",lat, lon);
		      //printf("tb19v=%.3f ",tb19v);
		      //printf("tb22v=%.3f ",tb22v);
            u = tb19v - BEG19V_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist[col][row].v19[u]++;
            u = tb19h - BEG19H_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist[col][row].h19[u]++;
            u = tb22v - BEG22V_WATER;
            if (u>=0 && u<NO_OF_BINS) hist[col][row].v22[u]++;
            u = tb37v - BEG37V_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist[col][row].v37[u]++;
            u = tb37h - BEG19H_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist[col][row].h37[u]++;
            u = (tb19v + tb19v - tb22v) - BEGPC1_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist[col][row].PC1[u]++;

            if( RainRateScan[i] != -999 ) {
               unsigned short	bin;
               int ilevel = FreezeLevel[i] * 10;

               if( ilevel >= NO_OF_FREEZE_LEVELS ) ilevel = NO_OF_FREEZE_LEVELS-1;
	            else if ( ilevel < 0 ) ilevel = 0;

               hist[col][row].FreezeLevel[ilevel]++;

               if( ((double)RainRateScan[i]) == 0 ) {
                  hist[col][row].RainRate[0]++;
               } else {
                  for( bin=1; bin<NO_OF_RAIN_RATES; bin++ ) {
                     if( RainRateScan[i] < RainRateBins[bin] ) {
                        hist[col][row].RainRate[bin]++;
                        break;
                     }
                  }
               }

               /*** Keep statistics on the rain rates.  The double values will be
	            converted to floats and included in the histogram when it is written
	            out at the end of the program. ***/
               hist[col][row].ntot++;
               r_0[col][row]  += RainRateScan[i];
               s2_0[col][row] += RainRateScan[i] * RainRateScan[i];
               if( RainRateScan[i] >= CUT1 ) {
                  double	rln = log( (double)RainRateScan[i] );
                  r_s[col][row]  += rln;
                  s2_s[col][row] += rln * rln;
                  hist[col][row].n_s++;
	                   
	               if( RainRateScan[i] <= CUT2 ) {
                     r_d[col][row]  += rln;
                     s2_d[col][row] += rln * rln;
                     hist[col][row].n_d ++;
	               }
               }

            }
         }
      }      
      //printf("\n");	   
   } // End updating bins for Low channels (19v 19h 22v 37v 37h)


   /**************************** High Channels ****************************/
   /* Update the bins for High Channels (92v, 92h) */
   for (i=0;i<HI_SAMPS;i++) {
      //for (scan=0;scan<AB_SCAN;scan++) {
         //lat = t6.lat[scan][i]*0.01;
	 //lon = t6.lon[scan][i]*0.01+180;      
         lat = t6.lat_hi[i]*0.01;
         lon = t6.lon_hi[i]*0.01+180;
         // Update the bins only between TOPLAT and BOTLAT
         // Update the bins only open ocean and no sea ice found
         if (lat<TOPLAT && lat>BOTLAT) {
            sflag = t6.ztoil_hi[i];
            landocean = sflag-4*(sflag/4); //0 = ocean, 1 = coast, 2 = land
            seaice = sflag/4; //0 = no possibility of sea ice, 1 = possible sea ice         
            if (seaice == 0 && landocean == 0) {
               //tb85v = t6.tb85[scan][i][0]*0.01;
               //tb85h = t6.tb85[scan][i][1]*0.01;
               tb92v = t6.ztb_hi[i][0]*0.01;
               tb92h = t6.ztb_hi[i][1]*0.01;

            // If iver = 6, convert the RSS SSMIS V7 brightness temperatures
            // to the V6 SSM/I standard.

            if ( iver == 6 ) {
               tb92v =  -0.067575 + 0.982756*tb92v + 0.015267*tb92h;
               tb92h =  -0.050578 + 0.010066*tb92v + 0.988008*tb92h;
            }

               row = (TOPLAT - lat) / HISTSCALE;
	            col = lon / HISTSCALE;               
               //u = tb85v - BEG85V_WATER;
               u = tb92v - BEG85V_WATER;
	            if (u>=0 && u<NO_OF_BINS) hist[col][row].v85[u]++;
               //u = tb85h - BEG85H_WATER;
               u = tb92h - BEG85H_WATER;
	            if (u>=0 && u<NO_OF_BINS) hist[col][row].h85[u]++;
            }
         }
     // }
   } // End updating bins for High channels (92v, 92h)
   
	//hist[row][col].ntot++;

	//printf("\n");
	return(OK);
}



enum isOK UpdateBins2(int iver) {

   float lat, lon, tb19v, tb19h, tb22v, tb37v, tb37h, tb92v, tb92h;
	short row, col;
	int index, i, u, scan; 
   signed char sflag, landocean, seaice;

   /** call RainRate() to get rainrates and freeze levels for this scan **/
   RainRate();
	
   /**************************** Low Channels ****************************/
   /* Low channels (19v 19h 22v 37v 37h) are taken                       */ 
	/* only during odd (1,3,5,...) scan positions along the A-scan.       */ 
	for (i=0;i<LOW_SAMPS;i++) {
	   //In C, index begins with 0, so the index of odd scan will be 0,2,4,...instead
		index = i*2;
		//lat = t6.lat[0][index]*0.01;
		//lon = t6.lon[0][index]*0.01+180;      
		lat = t6.lat[i]*0.01;
		lon = t6.lon[i]*0.01+180;

      // Update the bins only between TOPLAT and BOTLAT
      // Update the bins only open ocean and no sea ice found
      if (lat<TOPLAT && lat>BOTLAT) {
         //sflag = t6.sflag[0][index];
         sflag = t6.ztoil[i];
         landocean = sflag-4*(sflag/4); //0 = ocean, 1 = coast, 2 = land
         seaice = sflag/4; //0 = no possibility of sea ice, 1 = possible sea ice         
         if (seaice == 0 && landocean == 0) { 
	    tb19v = t6.ztb[i][0]*0.01;
            tb19h = t6.ztb[i][1]*0.01;
	    tb22v = t6.ztb[i][2]*0.01;
            tb37v = t6.ztb[i][3]*0.01;
            tb37h = t6.ztb[i][4]*0.01;

            // If iver = 6, convert the RSS SSMIS V7 brightness temperatures
            // to the V6 SSM/I standard.

            if ( iver == 6 ) {
               tb19v = -0.143481 + 1.003207*tb19v + 0.005177*tb19h;
               tb19h = -0.090130 - 0.001055*tb19v + 1.009745*tb19h;
               tb22v = -0.387787 + 1.005719*tb22v;
               tb37v = -0.091821 + 0.985167*tb37v + 0.011956*tb37h;
               tb37h = -0.344555 + 0.005039*tb37v + 0.992195*tb37h;
            }

            row = (TOPLAT - lat) / HISTSCALE;
	         col = lon / HISTSCALE;
            //fprintf(stderr,"lat=%.3f row=%d lon=%.3f col=%d ",lat,row,lon,col);
            //printf("low channels lat=%.2f lon=%.2f ",lat, lon);
		      //printf("tb19v=%.3f ",tb19v);
		      //printf("tb22v=%.3f ",tb22v);
            u = tb19v - BEG19V_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist2[col][row].v19[u]++;
            u = tb19h - BEG19H_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist2[col][row].h19[u]++;
            u = tb22v - BEG22V_WATER;
            if (u>=0 && u<NO_OF_BINS) hist2[col][row].v22[u]++;
            u = tb37v - BEG37V_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist2[col][row].v37[u]++;
            u = tb37h - BEG19H_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist2[col][row].h37[u]++;
            u = (tb19v + tb19v - tb22v) - BEGPC1_WATER;
	         if (u>=0 && u<NO_OF_BINS) hist2[col][row].PC1[u]++;

            if( RainRateScan[i] != -999 ) {
               unsigned short	bin;
               int ilevel = FreezeLevel[i] * 10;

               if( ilevel >= NO_OF_FREEZE_LEVELS ) ilevel = NO_OF_FREEZE_LEVELS-1;
	            else if ( ilevel < 0 ) ilevel = 0;

               hist2[col][row].FreezeLevel[ilevel]++;

               if( ((double)RainRateScan[i]) == 0 ) {
                  hist2[col][row].RainRate[0]++;
               } else {
                  for( bin=1; bin<NO_OF_RAIN_RATES; bin++ ) {
                     if( RainRateScan[i] < RainRateBins[bin] ) {
                        hist2[col][row].RainRate[bin]++;
                        break;
                     }
                  }
               }

               /*** Keep statistics on the rain rates.  The double values will be
	            converted to floats and included in the histogram when it is written
	            out at the end of the program. ***/
               hist2[col][row].ntot++;
               r_0[col][row]  += RainRateScan[i];
               s2_0[col][row] += RainRateScan[i] * RainRateScan[i];
               if( RainRateScan[i] >= CUT1 ) {
                  double	rln = log( (double)RainRateScan[i] );
                  r_s[col][row]  += rln;
                  s2_s[col][row] += rln * rln;
                  hist2[col][row].n_s++;
	                   
	               if( RainRateScan[i] <= CUT2 ) {
                     r_d[col][row]  += rln;
                     s2_d[col][row] += rln * rln;
                     hist2[col][row].n_d ++;
	               }
               }

            }
         }
      }      
      //printf("\n");	   
   } // End updating bins for Low channels (19v 19h 22v 37v 37h)


   /**************************** High Channels ****************************/
   /* Update the bins for High Channels (92v, 92h) */
   for (i=0;i<HI_SAMPS;i++) {
      //for (scan=0;scan<AB_SCAN;scan++) {
         //lat = t6.lat[scan][i]*0.01;
	 //lon = t6.lon[scan][i]*0.01+180;      
         lat = t6.lat_hi[i]*0.01;
         lon = t6.lon_hi[i]*0.01+180;
         // Update the bins only between TOPLAT and BOTLAT
         // Update the bins only open ocean and no sea ice found
         if (lat<TOPLAT && lat>BOTLAT) {
            sflag = t6.ztoil_hi[i];
            landocean = sflag-4*(sflag/4); //0 = ocean, 1 = coast, 2 = land
            seaice = sflag/4; //0 = no possibility of sea ice, 1 = possible sea ice         
            if (seaice == 0 && landocean == 0) {
               //tb85v = t6.tb85[scan][i][0]*0.01;
               //tb85h = t6.tb85[scan][i][1]*0.01;
               tb92v = t6.ztb_hi[i][0]*0.01;
               tb92h = t6.ztb_hi[i][1]*0.01;

            // If iver = 6, convert the RSS SSMIS V7 brightness temperatures
            // to the V6 SSM/I standard.

            if ( iver == 6 ) {
               tb92v =  -0.067575 + 0.982756*tb92v + 0.015267*tb92h;
               tb92h =  -0.050578 + 0.010066*tb92v + 0.988008*tb92h;
            }

               row = (TOPLAT - lat) / HISTSCALE;
	            col = lon / HISTSCALE;               
               //u = tb85v - BEG85V_WATER;
               u = tb92v - BEG85V_WATER;
	            if (u>=0 && u<NO_OF_BINS) hist2[col][row].v85[u]++;
               //u = tb85h - BEG85H_WATER;
               u = tb92h - BEG85H_WATER;
	            if (u>=0 && u<NO_OF_BINS) hist2[col][row].h85[u]++;
            }
         }
     // }
   } // End updating bins for High channels (92v, 92h)
   
	//hist[row][col].ntot++;

	//printf("\n");
	return(OK);
}



void PutStatsInHist()
{
   struct {
      float	r_0, r_s, r_d, s2_0, s2_s, s2_d;
   } x;

   int  col, row;
   float *tmp;

	//fprintf(stderr,"size of x= %d\n", sizeof(x));
	//fprintf(stderr,"size of x.r_0 = %d\n", sizeof(x.r_0));
	//fprintf(stderr,"size of hist[0][0].r_0 = %d\n", sizeof(hist[0][0].r_0));
	//fprintf(stderr,"\nsize of x = %d\tsize of x.r_0 = %d\n", sizeof(x),sizeof(x.r_0));
   for( col=0; col<NO_OF_COLS; col++ ) {
      for( row=0; row<NO_OF_ROWS; row++ ) {

         if( hist[col][row].ntot > 0 ) {
            x.r_0  = r_0[col][row] / (double)hist[col][row].ntot;
            x.s2_0 = s2_0[col][row]  -  x.r_0 * x.r_0;
	      } else
            x.r_0 = x.s2_0 = RMISS;
	 
         if( hist[col][row].n_s > 0 ) {
            x.r_s  = r_s[col][row] / (double)hist[col][row].n_s;
            x.s2_s = s2_s[col][row]  -  x.r_s * x.r_s;
         } else
            x.r_s = x.s2_s = RMISS;
	 
         if( hist[col][row].n_d > 0 ) {
            x.r_d  = r_d[col][row] / (double)hist[col][row].n_d;
            x.s2_d = s2_d[col][row]  -  x.r_d * x.r_d;
         } else
            x.r_d = x.s2_d = RMISS;

         memcpy( &hist[col][row].r_0, &x.r_0, sizeof(x));
         tmp = (float *) hist[col][row].s2_d;
         //tmp = &hist[col][row].s2_d[0];
         //fprintf(stderr,"row=%d col=%d x.s2_d=%f hist.s2_d=%f\n",row,col,x.s2_d,*tmp);
         //fprintf(stderr,"row=%d col=%d x.r_0=%f hist.r_0=%f\n",row,col,x.r_0,hist[col][row].r_0);
         //fprintf(stderr,"row=%d col=%d x.s2_0=%f hist.s2_0=%f\n",row,col,x.s2_0,hist[col][row].s2_0);
         //fprintf(stderr,"row=%d col=%d x.r_s=%f hist.r_s=%f\n",row,col,x.r_s,hist[col][row].r_s);
         //fprintf(stderr,"row=%d col=%d x.s2_s=%f hist.s2_s=%f\n",row,col,x.s2_s,hist[col][row].s2_s);
         //fprintf(stderr,"row=%d col=%d x.r_d=%f hist.r_d=%f\n",row,col,x.r_d,hist[col][row].r_d);
         //fprintf(stderr,"row=%d col=%d x.s2_d=%f hist.s2_d=%f\n",row,col,x.s2_d,hist[col][row].s2_d);
      }
   }
}



void PutStatsInHist2()
{
   struct {
      float	r_0, r_s, r_d, s2_0, s2_s, s2_d;
   } x2;

   int  col, row;
   float *tmp;

	//fprintf(stderr,"size of x= %d\n", sizeof(x));
	//fprintf(stderr,"size of x.r_0 = %d\n", sizeof(x.r_0));
	//fprintf(stderr,"size of hist[0][0].r_0 = %d\n", sizeof(hist[0][0].r_0));
	//fprintf(stderr,"\nsize of x = %d\tsize of x.r_0 = %d\n", sizeof(x),sizeof(x.r_0));
   for( col=0; col<NO_OF_COLS; col++ ) {
      for( row=0; row<NO_OF_ROWS; row++ ) {

         if( hist2[col][row].ntot > 0 ) {
            x2.r_0  = r_0[col][row] / (double)hist2[col][row].ntot;
            x2.s2_0 = s2_0[col][row]  -  x2.r_0 * x2.r_0;
	      } else
            x2.r_0 = x2.s2_0 = RMISS;
	 
         if( hist2[col][row].n_s > 0 ) {
            x2.r_s  = r_s[col][row] / (double)hist2[col][row].n_s;
            x2.s2_s = s2_s[col][row]  -  x2.r_s * x2.r_s;
         } else
            x2.r_s = x2.s2_s = RMISS;
	 
         if( hist2[col][row].n_d > 0 ) {
            x2.r_d  = r_d[col][row] / (double)hist2[col][row].n_d;
            x2.s2_d = s2_d[col][row]  -  x2.r_d * x2.r_d;
         } else
            x2.r_d = x2.s2_d = RMISS;

         memcpy( &hist2[col][row].r_0, &x2.r_0, sizeof(x2));
         tmp = (float *) hist[col][row].s2_d;
         //tmp = &hist2[col][row].s2_d[0];
         //fprintf(stderr,"row=%d col=%d x.s2_d=%f hist.s2_d=%f\n",row,col,x.s2_d,*tmp);
         //fprintf(stderr,"row=%d col=%d x.r_0=%f hist.r_0=%f\n",row,col,x.r_0,hist[col][row].r_0);
         //fprintf(stderr,"row=%d col=%d x.s2_0=%f hist.s2_0=%f\n",row,col,x.s2_0,hist[col][row].s2_0);
         //fprintf(stderr,"row=%d col=%d x.r_s=%f hist.r_s=%f\n",row,col,x.r_s,hist[col][row].r_s);
         //fprintf(stderr,"row=%d col=%d x.s2_s=%f hist.s2_s=%f\n",row,col,x.s2_s,hist[col][row].s2_s);
         //fprintf(stderr,"row=%d col=%d x.r_d=%f hist.r_d=%f\n",row,col,x.r_d,hist[col][row].r_d);
         //fprintf(stderr,"row=%d col=%d x.s2_d=%f hist.s2_d=%f\n",row,col,x.s2_d,hist[col][row].s2_d);
      }
   }
}


/* Compute the upper rain rate limit for the rain rate bins. */
void ComputeRainRateBins( )
{
   short	j;

   RainRateBins[1] = 0.25;
   for(j=2; j<NO_OF_RAIN_RATES; j++)
      RainRateBins[j] = RainRateBins[j-1] * 1.1;
      
   RainRateBins[NO_OF_RAIN_RATES-1] = 9999;

   #ifdef DEBUG_RAINRATEBINS_ON /* check whether the value get read correctly */
	for(j=0;j<NO_OF_RAIN_RATES;j++) {
		fprintf(stderr,"RainRateBins[%d] = %f\n",j,RainRateBins[j]);
	}
   #endif
}
