#include   <fcntl.h>
#include   <string.h>
#include   <stdio.h>
#include   <math.h>
#include   "Define_RSSv6Tb_ssmi.h"


#define T_ZERO		174.7
#define NTT		2028
#define NFL		301

/*** Prototypes ***/
void exit();
void rain_1922( double *yy, double *fl, double *rate1, int *kflag );

extern struct UnpackString_v6		t6;
float		rt[NFL][NTT], tt[NTT], frl[NFL], tmaxn[NFL][2];
extern double	FreezeLevel[LOW_SAMPS];
extern float	RainRateScan[LOW_SAMPS];

void InitializeRainRate_25deg(int month, char *rtTableDSN,
			                     char *rtMaxMinDSN, char *FreezeTableDSN)
{
   int		j, k, handle;
   /* Had to change rt_temp, tmaxn_temp & h_clim to short to be compatable with rest of data */
   int     rt_temp[NFL][NTT];   		/* temporary variable to store rt data */
   int     tmaxn_temp[NFL][2];   		/* temporary variable to store tmaxn data */
   int     h_clim[12][NO_OF_ROWS];		/* Climate for 12 months */
   float	   temp;

   /******************************************************************************************/
   /****************************************    RT    ****************************************/
   if( (handle=open( rtTableDSN, O_RDONLY ) ) == -1 ) {
      perror("Open error for rtTable");
      exit(1);
   }
   if( read( handle, rt_temp, sizeof(rt_temp) ) != (sizeof(rt_temp)) ) { //read in to rt_temp instead of rt
      perror("Read error for rtTable");
      exit(1);
   }
   close( handle );

  /* convert big-endian data to little-endian data (swapping bytes) */
   for( j=0;j<(NFL);j++ ) {		
     for( k=0;k<(NTT);k++ ) {
       rt_temp[j][k] = ntohl(rt_temp[j][k]);
       memcpy(&rt[j][k], &rt_temp[j][k], sizeof(rt[j][k])); 
     }
   }
   /* end of conversion */



   /******************************************************************************************/
   /***************************************    Tmax    ***************************************/
   if( (handle=open( rtMaxMinDSN, O_RDONLY ) ) == -1 ) {
      perror("Open error for rtMaxMin");
      exit(1);
   }
   if( read( handle, tmaxn_temp, sizeof(tmaxn_temp) ) != sizeof(tmaxn_temp) ) {	// read in to tmaxn_temp instead of tmax
      perror("Read error for rtMaxMin");
      exit(1);
   }
   close( handle );

   /* convert big-endian data to little-endian data (swapping bytes) */
	for( j=0;j<(NFL);j++ ) {
		for( k=0;k<(2);k++ ) {
			tmaxn_temp[j][k] = ntohl(tmaxn_temp[j][k]);
			memcpy(&tmaxn[j][k], &tmaxn_temp[j][k], sizeof(tmaxn[j][k])); 
		}
	}
   /* end of conversion */



   /******************************************************************************************/
   /***********************************    FreezeLevel    ************************************/
   if( (handle=open( FreezeTableDSN, O_RDONLY ) ) == -1 ) {
      perror("Open error for FreezeTable");
      exit(1);
   }
   if( read( handle, h_clim, sizeof(h_clim) ) != sizeof(h_clim) ) {
      perror("Read error for FreezeTable");
      exit(1);
   }
   close( handle );

	/* convert big-endian data to little-endian data (swapping bytes) */
   for(j=0; j<NO_OF_ROWS; j++ ) {
   	/* ntohl - converts big-endian data to little endian */ 
     	h_clim[month-1][j] = ntohl(h_clim[month-1][j]);
   	/* requires a temporary variable because double has different size than float */
     	memcpy(&temp, &h_clim[month-1][j], sizeof(temp)); 
     	FreezeLevel[j] = (double)(temp); 
   }
   /* end of conversion */

   /*
   for(j=0; j<NO_OF_ROWS; j++ ) {
     printf("%i %f \n",j,FreezeLevel[j]);
   }
   */


   /******************************************************************************************/
   /*************************************    TT & FRL    *************************************/
   for(j=0; j<NTT; j++)
     tt[j] = T_ZERO + j * 0.05;

   for(j=0; j<NFL; j++)
     frl[j] = 2.0 + j * 0.01;
}

void RainRate( )

   /***
     Logic:

     if t1922 >= tbmax for the fl
     .	Use t85 and take max of r85 and r1922 at tbmax, where the tbmax is set
     .	at about the saturation point or the max of the rt curve for lower fl.

     else use t1922
     .	if 0 < r1922 < 1 and p85 > 0.46, set rate=0. (low clw)
     .	if t1922 > table max for h, use t85
     .		if r85 < max r1922  for the h, take the max

     r1922 has freezing height dependent beam filling correction.
     .				           Jie Meng 6/24/97
     c
     c     The subroutine rain_1922 is completely replaced again.
     c     The routine uses direct look up table, instead of the inverse table.
     c     call get_rt_table in the main routine.
     c     /data4/meng/onedeg/inst_rain/rt_table.bin and rt_maxmin.bin are used,
     c     instead of model_rt.table.
     c                                 Jie Meng, 6/11/97
     c
     c ***   The program is based on Al's raincal.
     c       The major changes in the algorithm:
     c         1. Use model R_T curves, instead of fitted curves.
     c         2. Use freezing height climatology, instead of calculating it
     c            from water vapor.
     c         3. Compute wv from freezing height, intead of vice verse.
     c         4. The subroutine rain_1922 is comletely new.
     c            Use polynomial interpolation in the inverse estimatation
     c            of R(T,h) from the model R_T curves T(R,h).
     c            In addition, bisection searching of table is used.
     c
     c                                            4/30/97
     c **    Compute rain over ocean water only.
     c       kflag : =1, rain from t1922.
     c               =2, rain from t85, when  tb > t1922_max
     c               =3, rain from t85, when tb > tbmax, wich is at about
     c				saturation point or the max of the rt table.
     c               =4, rate1=0, for low clw (<25mg) and r1922 < 1.
     c               =8, missing, due to t19v > 276, (for the tracking purpose)
     c               =9, missing, due to land or sea ice or bad Tb
     c
     c                                 Jie Meng, 4/21/97		***/



{
   //static char *rcs="$c: RainRate_25deg.c,v 1.1 1997/08/08 15:05:57 powell $";

   double	wv, delt85, yy, p851, p852, p853, p854, tbmax;
   double	yx, r1, r2, r3, r4, r85, rate1, fl;
   int		ii, ix, jx, kflag;
   unsigned short	sampLow, sampHigh;
   float	t19v, t22v, t85v1, t85v2, t85v3, t85v4, t85h1, t85h2, t85h3, t85h4;

   signed char sflag, landocean, seaice; // added by kij
   float lat, lon; // added by kij
   short row, col; // added by kij
   float t19h, t37v, t37h; // added by kij

   
   for( sampLow=0,sampHigh=0; sampLow<LOW_SAMPS; sampLow++,sampHigh+=2 ) {
      rate1 = RMISS; // RMISS is set to -999
      kflag = 989;

         /* old codes
                 if( SkipSample[sampLow]    == TRUE ) {
	                 kflag = 959;
	                 goto Label900;
                 }
                 if( WaterFlagScan[sampLow] == LAND ) {
	                 kflag = 969;
	                 goto Label900;
                 } 
          */


      /************* check if lat and lon is in range *************/
      // lat and lon for A-scan for low channels      
      lat = t6.lat[0][sampHigh]*0.01;
      lon = t6.lon[0][sampHigh]*0.01+180;
      if (!(lat<TOPLAT && lat>BOTLAT)) {
         kflag = 959;
	      goto Label900; //set to RMISS
      }

      /********** check if this observation is over water **********/
      sflag = t6.sflag[0][sampHigh];
      landocean = sflag-4*(sflag/4); //0 = ocean, 1 = coast, 2 = land
      seaice = sflag/4; //0 = no possibility of sea ice, 1 = possible sea ice         
      if (!(seaice == 0 && landocean == 0)) {
	kflag = 959;
	goto Label900; //set to RMISS   
      }
      

      /******** come here if this observation is over water. ********/
      t19v = t6.tb[sampLow][0]*0.01;
      t19h = t6.tb[sampLow][1]*0.01;
      t22v = t6.tb[sampLow][2]*0.01;
      t37v = t6.tb[sampLow][3]*0.01;
      t37h = t6.tb[sampLow][4]*0.01;

           
         /* old codes
            if(t.tb19v[sampLow]>(290*16) || t.tb19h[sampLow]>(290*16) ||
	            t.tb22v[sampLow]>(290*16) || t.tb37v[sampLow]>(290*16) ||
	            t.tb37h[sampLow]>(290*16) ) {
	            kflag = 979;
               goto Label900;
            }
         */


      /*** Range checks ***/ 
      if( t19v>290 || t19h>290 || t22v>290 || t37v>290 || t37h>290 ) {
	kflag = 979;
	goto Label900; //set to RMISS
      }
      


      // 85 vertical
      t85v1 = t6.tb85[0][sampHigh+0][0]*0.01; //A-scan
      t85v2 = t6.tb85[0][sampHigh+1][0]*0.01; //A-scan
      t85v3 = t6.tb85[1][sampHigh+0][0]*0.01; //B-scan
      t85v4 = t6.tb85[1][sampHigh+1][0]*0.01; //B-scan

      // 85 horizontal
      t85h1 = t6.tb85[0][sampHigh+0][1]*0.01; //A-scan
      t85h2 = t6.tb85[0][sampHigh+1][1]*0.01; //A-scan
      t85h3 = t6.tb85[1][sampHigh+0][1]*0.01; //B-scan
      t85h4 = t6.tb85[1][sampHigh+1][1]*0.01; //B-scan

      	/* old codes
      			t85v1 = t.tb85va[sampHigh+0] * (1./16.);
      			t85v2 = t.tb85va[sampHigh+1] * (1./16.);
      			t85v3 = t.tb85vb[sampHigh+0] * (1./16.);
      			t85v4 = t.tb85vb[sampHigh+1] * (1./16.);

      			t85h1 = t.tb85ha[sampHigh+0] * (1./16.);
      			t85h2 = t.tb85ha[sampHigh+1] * (1./16.);
      			t85h3 = t.tb85hb[sampHigh+0] * (1./16.);
      			t85h4 = t.tb85hb[sampHigh+1] * (1./16.);
      	*/


      /********** get freezing level from the table. **********/
      row = (TOPLAT - lat) / HISTSCALE;
      col = lon / HISTSCALE; 
      fl = (float)FreezeLevel[row];

      if( fl > frl[NFL-1]  ||  fl < frl[0] ) {
	      fprintf(stderr,"\nFreezing level out of range: %f\n", fl );
	      exit(1);
      }


      /******** Check for bad data after all filtering ********/
      if( (yy=(2.0 * t19v - t22v)) > 276.0 ) {
	      kflag = 8;
	      goto Label900;
      }

      if( yy <= tt[0] ) {
         rate1 = 0.0;
         kflag = 1;
         goto Label900;
      }

      /*** Heavy rain over water, beyond the saturation or beyond the maximum
	        of the table. ***/
      jx = (fl - 2) * 100;
      tbmax = tmaxn[jx][1];
      if( yy >= tbmax ) {
         kflag = 3;
         ix = (tbmax - T_ZERO) * 20;
         yx = rt[jx][ix];

         if( yx > 0.0 )
	         rate1 = yx * (1.0 + ((0.44/5.0) * fl) );
	      else if( yx <= 0.0 ) {
	         for( ii=ix; ii>=ix-100; ii-- ) {
	            if(ii < 0 ) {
                  fprintf(stderr,"\tii is too small: %i\n", ii);
   	            exit(1);
	            }
	            if( rt[jx][ii] > 0.0 ) {
		            rate1 = rt[jx][ii] * (1.0 + ((0.44/5.0) * fl) );
		            break;
	            }
	         }
	      }
	      goto Label70;
      } 
      else {
         kflag = 1;
      }

      /*********** use 2*19v-22v to calculate rain rate ***********/
      rain_1922(&yy, &fl, &rate1, &kflag);

      /**** use t85 to check CLW & set low rain < 1 to no rain ****/
      if( (rate1>0) && (rate1<1.0) ) {

	      /*** Compute wv from freezing height. ***/
	      wv = 10. * exp( fl * 0.39);

	      delt85=exp(4.27-0.0271*wv);

         if(delt85 != 0.0) {
            p851 = (t85v1-t85h1) * (1/delt85);
            p852 = (t85v2-t85h2) * (1/delt85);
            p853 = (t85v3-t85h3) * (1/delt85);
            p854 = (t85v4-t85h4) * (1/delt85);
	      } else
            p851 = p852 = p853 = p854 = 0.5;


         /***	less than 25 mg clw -- no rain ***/         
	 if(p851>0.46 && p852>0.46 && p853>0.46 && p854>0.46 ) {
	   rate1 = 0.0;
	   kflag = 4;
	   goto Label900;
	 }
      }


      /*** For high rain rate from scattering, use Adler's 85 GHz formula. ***/
      Label70:
      if( (kflag==2) || (kflag==3) ) {
         r1 = (251.0-t85h1) * (1.0/2.09);
         r1 = r1<0.0 ? 0.0 : r1;

         r2 = (251.0-t85h2) * (1.0/2.09);
         r2 = r2<0.0 ? 0.0 : r2;

         r3 = (251.0-t85h3) * (1.0/2.09);
         r3 = r3<0.0 ? 0.0 : r3;

         r4 = (251.0-t85h4) * (1.0/2.09);
         r4 = r4<0.0 ? 0.0 : r4;

         r85 = (r1+r2+r3+r4) * 0.25;

         if(r85 > rate1 && rate1 >= 0) rate1 = r85;	     
      }

      Label900: ;
      RainRateScan[sampLow] = rate1;
   }
   #ifdef DEBUG
      fputs("\nRain Results:\n",stderr);
      for( sampLow=0; sampLow<LOW_SAMPS; sampLow++ ) {
	      fprintf(stderr,"%02d_%.2f\t", sampLow+1, RainRateScan[sampLow] );
	      if( sampLow%10 == 9 ) fputc('\n',stderr);
      }
      fputc( '\n',stderr);
      exit(1);
   #endif
}

void rain_1922( double *tb, double *h, double *rr, int *kflag )
{
   /*** Given tb and h, estimate rr using rt direct lookup table rt[301][2028], via
     one step of direct linear interpolations without table searching.

     The rt table is R(T,h) and generated by many steps of linear interpolations from
     the model inverst rt table T(`R,h).  -1.0 is set to the points off the curves in
     the rt table.  This routine will replace rain1922 in raincal.  Call get_rt_table
     in the main program.

     kflag=1: rr is interpolated from 1922 model R-T curve.
     kflag=2: rr is going to be caluclated using 85 Ghz.

     Beam filling correction is dependent on freezing height h.

     Jie Meng, 6/11/97	*/

   int		i1, i2, j1;
   double	x1, x2, y1, y2, bfc;

   if( *h > frl[NFL-1] || *h < frl[0] ) {
      fprintf(stderr,"Invalid freezing level: %lf\n", *h);
      exit(1);
   }

   if( *tb > 276.3 ) {
      *kflag = 999;
      *rr = RMISS;
      return;
   }

   if( *tb <= tt[0] ) {
      *kflag = 1;
      *rr = 0.0;
      return;
   }

   /*** Locate the tb and h in the table. ***/
   i1 = (*tb - T_ZERO) * 20;
   i2 = i1 + 1;
   j1 = (*h - 2.0) * 100 + 0.5;

   #ifdef DEBUG_TEST
   if( (i1<0) || (i2>=NTT) )
      fprintf(stderr,"\tBad size for i1 & i2, i1: %i\n", i1);
   if( (j1<0) || (j1>=NFL) )
      fprintf(stderr,"\tBad size for j1: %i\n", j1);
   #endif

   /*** tb is outside of the curve for the h. ***/
   if( *tb < tmaxn[j1][0] ) {
      *rr = 0.0;
      *kflag = 1;
      return;
   }
   if( *tb >= tmaxn[j1][1] ) {
      *kflag = 2;
      *rr = rt[j1][i1];
      goto Label400;
   }

   /*** h is close to fj[j1]. ***/
   x1 = tt[i1];
   x2 = tt[i2];
   y1 = rt[j1][i1];
   y2 = rt[j1][i2];

   if( y1 < 0.0 ) {
      if( *tb >= tmaxn[j1][1] ) {
         *kflag = 2;
         *rr = rt[j1+1][i1-1];
         goto Label400;
      } else {
         *rr = RMISS;
         *kflag = 9;
         return;
      }
   }

   if( *tb == x1 ) {
      *rr = y1;
      goto Label400;
   }

   if( y1 > 0.0 && y2 < 0.0 ) {
      *rr = y1;
      *kflag = 2;
      goto Label400;
   }

   *rr = y1 + (*tb-x1) * (y2-y1) / (x2-x1);
   *kflag = 1;


   /*** Beam filling correction, dependent on h. ***/
   Label400:
   bfc = (0.44/5.0) * (*h) + 1.0;
   *rr *= bfc;
}
