#include   <stdio.h>
#include   <math.h>
#include   <string.h>
#include   "Define_RSSv6Tb_ssmi.h"


extern struct UnpackString_v6 t6;
struct Histogram              hist[NO_OF_COLS][NO_OF_ROWS];
float                         RainRateScan[LOW_SAMPS];
double                        FreezeLevel[LOW_SAMPS];
static float                  RainRateBins[NO_OF_RAIN_RATES];
double  r_0[NO_OF_COLS][NO_OF_ROWS], s2_0[NO_OF_COLS][NO_OF_ROWS];
double  r_s[NO_OF_COLS][NO_OF_ROWS], s2_s[NO_OF_COLS][NO_OF_ROWS];
double  r_d[NO_OF_COLS][NO_OF_ROWS], s2_d[NO_OF_COLS][NO_OF_ROWS];


#define CUT1 0.5
#define CUT2 20.

#define BEG19V_WATER	100
#define BEG19H_WATER	80
#define BEG22V_WATER	100
#define BEG37V_WATER	100
#define BEG37H_WATER	80
#define BEG85V_WATER	100
#define BEG85H_WATER	100
#define BEGPC1_WATER	100		/* Pseudo Channel 1 [2*19V - 22V] */


enum isOK UpdateBins() {

   float lat, lon, tb19v, tb19h, tb22v, tb37v, tb37h, tb85v, tb85h;
	short row, col;
	int index, i, u, scan; 
   signed char sflag, landocean, seaice;


   /** call RainRate() to get rainrates and freeze levels for this scan **/
   RainRate();

   /**************************** Low Channels ****************************/
   /* Low channels (19v 19h 22v 37v 37h) are taken                       */ 
	/* only during odd (1,3,5,...) scan positions along the A-scan.       */ 
	for (i=0;i<LOW_SAMPS;i++) {
	   //In C, index begins with 0, so the index of odd scan will be 0,2,4,...instead
		index = i*2;
		lat = t6.lat[0][index]*0.01;
		lon = t6.lon[0][index]*0.01+180;      

      // Update the bins only between TOPLAT and BOTLAT
      // Update the bins only open ocean and no sea ice found
      if (lat<TOPLAT && lat>BOTLAT) {

	

         sflag = t6.sflag[0][index];
         landocean = sflag-4*(sflag/4); //0 = ocean, 1 = coast, 2 = land
         seaice = sflag/4; //0 = no possibility of sea ice, 1 = possible sea ice   

         if (seaice == 0 && landocean == 0) { 
            tb19v = t6.tb[i][0]*0.01;
            tb19h = t6.tb[i][1]*0.01;
            tb22v = t6.tb[i][2]*0.01;
            tb37v = t6.tb[i][3]*0.01;
            tb37h = t6.tb[i][4]*0.01;

            row = (TOPLAT - lat) / HISTSCALE;
	    col = lon / HISTSCALE;

	    u = tb19v - BEG19V_WATER;
	    if (u>=0 && u<NO_OF_BINS) hist[col][row].v19[u]++;
	    u = tb19h - BEG19H_WATER;
	    if (u>=0 && u<NO_OF_BINS) hist[col][row].h19[u]++;
	    u = tb22v - BEG22V_WATER;
	    if (u>=0 && u<NO_OF_BINS) hist[col][row].v22[u]++;
	    u = tb37v - BEG37V_WATER;
	    if (u>=0 && u<NO_OF_BINS) hist[col][row].v37[u]++;
	    u = tb37h - BEG19H_WATER;
	    if (u>=0 && u<NO_OF_BINS) hist[col][row].h37[u]++;
	    u = (tb19v + tb19v - tb22v) - BEGPC1_WATER;
	    if (u>=0 && u<NO_OF_BINS) hist[col][row].PC1[u]++;

            if( RainRateScan[i] != -999 ) {
	       unsigned short	bin;
	       int ilevel = FreezeLevel[i] * 10;
	      
               if( ilevel >= NO_OF_FREEZE_LEVELS ) ilevel = NO_OF_FREEZE_LEVELS-1;
	            else if ( ilevel < 0 ) ilevel = 0;

               hist[col][row].FreezeLevel[ilevel]++;

               if( ((double)RainRateScan[i]) == 0 ) {
                  hist[col][row].RainRate[0]++;
               } else {
                  for( bin=1; bin<NO_OF_RAIN_RATES; bin++ ) {
                     if( RainRateScan[i] < RainRateBins[bin] ) {
                        hist[col][row].RainRate[bin]++;
                        break;
                     }
                  }
               }

               /*** Keep statistics on the rain rates.  The double values will be
	            converted to floats and included in the histogram when it is written
	            out at the end of the program. ***/
               hist[col][row].ntot++;
               r_0[col][row]  += RainRateScan[i];
               s2_0[col][row] += RainRateScan[i] * RainRateScan[i];

               if( RainRateScan[i] >= CUT1 ) {
                  double	rln = log( (double)RainRateScan[i] );
                  r_s[col][row]  += rln;
                  s2_s[col][row] += rln * rln;
                  hist[col][row].n_s++;
	                   
	               if( RainRateScan[i] <= CUT2 ) {
                     r_d[col][row]  += rln;
                     s2_d[col][row] += rln * rln;
                     hist[col][row].n_d ++;

	               }
               }

            }
         }
      }      
   } // End updating bins for Low channels (19v 19h 22v 37v 37h)



   /**************************** High Channels ****************************/
   /* Update the bins for High Channels (85v, 85h) for both A and B scan  */
   for (i=0;i<HI_SAMPS;i++) {
      for (scan=0;scan<AB_SCAN;scan++) {
         lat = t6.lat[scan][i]*0.01;
		   lon = t6.lon[scan][i]*0.01+180;      
         // Update the bins only between TOPLAT and BOTLAT
         // Update the bins only open ocean and no sea ice found
         if (lat<TOPLAT && lat>BOTLAT) {
            sflag = t6.sflag[scan][i];
            landocean = sflag-4*(sflag/4); //0 = ocean, 1 = coast, 2 = land
            seaice = sflag/4; //0 = no possibility of sea ice, 1 = possible sea ice         
            if (seaice == 0 && landocean == 0) {
               tb85v = t6.tb85[scan][i][0]*0.01;
               tb85h = t6.tb85[scan][i][1]*0.01;
               row = (TOPLAT - lat) / HISTSCALE;
	       col = lon / HISTSCALE;    
           
               u = tb85v - BEG85V_WATER;
	            if (u>=0 && u<NO_OF_BINS) hist[col][row].v85[u]++;
               u = tb85h - BEG85H_WATER;
	            if (u>=0 && u<NO_OF_BINS) hist[col][row].h85[u]++;
            }
         }
      }
   } // End updating bins for High channels (85v, 85h)

  
	//hist[row][col].ntot++;

	//printf("\n");
	return(OK);
}


void PutStatsInHist()
{
   struct {
      float	r_0, r_s, r_d, s2_0, s2_s, s2_d;
   } x;

   int  col, row;
   float *tmp;

   for( col=0; col<NO_OF_COLS; col++ ) {
      for( row=0; row<NO_OF_ROWS; row++ ) {

         if( hist[col][row].ntot > 0 ) {
            x.r_0  = r_0[col][row] / (double)hist[col][row].ntot;
            x.s2_0 = s2_0[col][row]  -  x.r_0 * x.r_0;
	      } else
            x.r_0 = x.s2_0 = RMISS;
	 
         if( hist[col][row].n_s > 0 ) {
            x.r_s  = r_s[col][row] / (double)hist[col][row].n_s;
            x.s2_s = s2_s[col][row]  -  x.r_s * x.r_s;
         } else
            x.r_s = x.s2_s = RMISS;
	 
         if( hist[col][row].n_d > 0 ) {
            x.r_d  = r_d[col][row] / (double)hist[col][row].n_d;
            x.s2_d = s2_d[col][row]  -  x.r_d * x.r_d;
         } else
            x.r_d = x.s2_d = RMISS;

         memcpy( &hist[col][row].r_0, &x.r_0, sizeof(x));
         tmp = &hist[col][row].s2_d[0];
      }
   }
}

/* Compute the upper rain rate limit for the rain rate bins. */
void ComputeRainRateBins( )
{
   short	j;

   RainRateBins[1] = 0.25;
   for(j=2; j<NO_OF_RAIN_RATES; j++)
      RainRateBins[j] = RainRateBins[j-1] * 1.1;
      
   RainRateBins[NO_OF_RAIN_RATES-1] = 9999;

   #ifdef DEBUG_RAINRATEBINS_ON /* check whether the value get read correctly */
	for(j=0;j<NO_OF_RAIN_RATES;j++) {
		fprintf(stderr,"RainRateBins[%d] = %f\n",j,RainRateBins[j]);
	}   
   #endif   
}
