#define RMISS		-999

#define TRUE		1
#define FALSE		0

#define LOW_SAMPS	64
#define HI_SAMPS	128

#define AB_SCAN	        2
#define LOW_CHANS	5
#define HI_CHANS	2

#define HISTSCALE	5
#define TOPLAT		50
#define BOTLAT		(-50)

#define NO_OF_BINS	195
#define NO_OF_RAIN_RATES	64
#define NO_OF_FREEZE_LEVELS	60

#define NO_OF_COLS	72  /*  No of longitude cols  */
#define NO_OF_ROWS	20  /*  No of latitude  rows  */

#define RETURN_FAIL 1

enum ReadRtn	{ READ_OK, READ_EOF };
enum isOK		{ OK, NOT_OK };



/** ---------------------- V6 SSMI Variables ---------------------- **/

struct UnpackString_header_v6 {
   long			ksat, iorbit, numscan;
   char			blank[3756];
};

struct UnpackString_v6 {
   long          scanrecord;
   unsigned char btime[24];                          //time for the begin of the b-scan
   signed char   qflag[12];                          //quality flag 
   long          fop;                                //fractional orbit position (-0.02 to 1.02)
   long          sclat, sclon;                       //spacecraft latitude and east longitude (deg)
   long          scalt;                              //spacecraft altitude (km)
   short         lat[AB_SCAN][HI_SAMPS];             //latitude of 85GHz footprint (deg)
   short         lon[AB_SCAN][HI_SAMPS];             //longitude of 85GHz footprint (deg)
   signed char   sflag[AB_SCAN][HI_SAMPS];           //surface flag
   short         eiangle[HI_SAMPS];                  //earth incidence angle for 85GHz footprint (deg)
   short         azimuth[HI_SAMPS];                  //earth azimuth angle for 85GHz footprint (deg)
   short         sunang[HI_SAMPS];                   //sun glitter angle for 85GHz footprint (deg)
   short         tb[LOW_SAMPS][LOW_CHANS];           /* 19v 19h 22v 37v 37h */
   short         tb85[AB_SCAN][HI_SAMPS][HI_CHANS];  /* 85v 85h and a-scan b-scan */
};

struct Histogram {
   unsigned short	v19[NO_OF_BINS];
   unsigned short	h19[NO_OF_BINS];
   unsigned short	v22[NO_OF_BINS];
   unsigned short	v37[NO_OF_BINS];
   unsigned short	h37[NO_OF_BINS];
   unsigned short	v85[NO_OF_BINS];
   unsigned short	h85[NO_OF_BINS];
   unsigned short	PC1[NO_OF_BINS];
   unsigned short	RainRate[NO_OF_RAIN_RATES];
   char			unused1[32];
   unsigned short	FreezeLevel[NO_OF_FREEZE_LEVELS];
   char			r_0 [4];	/* Will hold a float value when written out */
   char			r_s [4];	/* Will hold a float value when written out */
   char			r_d [4];	/* Will hold a float value when written out */
   char			s2_0[4];	/* Will hold a float value when written out */
   char			s2_s[4];	/* Will hold a float value when written out */
   char			s2_d[4];	/* Will hold a float value when written out */
   unsigned short	ntot;
   unsigned short	n_s;
   unsigned short	n_d;
   char			unused2[80];
};
