#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <inttypes.h>
#include "../../../OptionsM.h"
#include "Define_RSSv6Tb_ssmi_5deg.h"

struct UnpackString_header_v6   t6_header;
struct UnpackString_v6          t6;     /* Array of single Tb record */
struct UnpackString_v6          *RSStb; /* Array of all Tb records   */
struct Histogram	hist2[NO_OF_COLS][NO_OF_ROWS], hist[NO_OF_COLS][NO_OF_ROWS];


void exit();
void PutStatsInHist();
void PutStatsInHist2();

/* This code makes 5deg Tb histograms using the METH algorithm */

int main(int argc, char *argv[])
{
  int   yyyy, yy, mm;       //Input year and month
  char  sat[3];         //3-digit satellite code
  char  *rtTableDsn, *rtMaxMinDsn, *FreezeTableDsn;
  char  HistFile[500], tmpname[500]; 
  FILE  *fopen(), *fp_filelist, *fp_RSSv6, *fp_histfile;
  char  RSSv6_filelist[500], fname[500];
  char  HistHeader[390];     // header for output histogram file
  int 	len, i, iscan, ReadFlag, btimeY, btimeM, tmptime[24];
  float lat, lon;
  int   nscan_lores;
  int satlat[3800]; // Used to determine asc/des scans

  char TestFile[500];
  FILE *fp_test;
  int  ii;
  ii=0;

  /* Convert inputs to integer and check that they are valid */
  yyyy = atoi(argv[1]);
  yy=yyyy;
  if (yy<2000) yy=yy-1900;
  if (yy>=2000) yy=yy-2000;
  mm   = atoi(argv[2]);
  sscanf(argv[3],"%3s",&sat);

  if ((yyyy<1979)||(yyyy>2050)) {
    fprintf(stderr,"Year (%d) outside of valid range [1979-2050])\n",yyyy);
  }

  if ((mm<1)||(mm>12)) {
    fprintf(stderr,"month (%d) outside of valid range\n",mm);
  }



  /* construct output file names */
  sprintf(HistFile,"%s/METH_Hist_5deg/%sY%02dM%02d_5.0deg_hist_v6",InterimPATH,sat,yy,mm);

  /* get names of ancillary files required for processing METH */
  rtTableDsn = CODEPATH "GPCPcdr_m2/GPCPcdr_m2_SSMI_Ocean/GPCPcdr_m2_SSMI_Ocean_Hist_5deg/rt_table.bin_97-06-25_fromJie";
  rtMaxMinDsn = CODEPATH "GPCPcdr_m2/GPCPcdr_m2_SSMI_Ocean/GPCPcdr_m2_SSMI_Ocean_Hist_5deg/rt_maxmin.bin_97-06-25_fromJie";
  FreezeTableDsn = CODEPATH "GPCPcdr_m2/GPCPcdr_m2_SSMI_Ocean/GPCPcdr_m2_SSMI_Ocean_Hist_5deg/hclim95_5.0deg_50N.table_Gao";




  /*************************************************
   * Initialize variables and allocate memory      */
  InitializeRainRate_25deg(mm, rtTableDsn, rtMaxMinDsn, FreezeTableDsn);
  
  /* Compute the boundary values for the rain rate bins. */
  /* Compute the upper rain rate limit for the rain rate bins. */
  ComputeRainRateBins();

  /* Zero out histograms */
  memset(hist,0,sizeof(hist));
  memset(hist2,0,sizeof(hist2));

  /* Open filename listing */
  sprintf(RSSv6_filelist,"%s/file_listings/RSS_Tb_%3s_%04i%02i.list",
	  CODEPATH,sat,yyyy,mm);
  fp_filelist = fopen(RSSv6_filelist,"r");
  if (fp_filelist == NULL) {
    fprintf(stderr,"Can't open file: %s\n",RSSv6_filelist);
    exit(EXIT_FAILURE);
  }

  /* scroll though listing of files */
  while (fgets(fname,sizeof(fname),fp_filelist) != NULL ){

    /* remove newline left by fgets */
    len = strlen(fname);
    if (fname[len-1] == '\n') fname[len-1] = 0;	


    /* Read NetCDF header and and get nscan so that the output
     * array of structures can be allocated.  May be OK to do this
     * in ReadNC_RSS_SSMI_Tb_v6 (this avoiding opening the nc file
     * twice), but this seems most memory leakproof                */
    nscan_lores=ReadNC_RSS_SSMI_Tb_v6_nscan(fname);

    /* Allocate space for main RSS array */
    RSStb = calloc(nscan_lores,sizeof(struct UnpackString_v6));
    
    /* Read NetCDF file into temporary structure */
    ReadNC_RSS_SSMI_Tb_v6(fname,&t6_header,RSStb);

    /* store SClats - used for des/asc calculation*/
    for(iscan=0; iscan<t6_header.numscan; iscan++) { 
      satlat[iscan]=RSStb[iscan].sclat;
    }


    /* scroll through number of scans */
    for(iscan=0; iscan<t6_header.numscan; iscan++) { 
      /* Copy record into t6 for use by other functions */
      t6=RSStb[iscan];

      /* Read month and year from btime (YYYY-MM-DD HH-MM-SSZ) */
      for (i=0; i<24; i++) tmptime[i]=t6.btime[i]-'0'; 
      btimeY=1000*tmptime[0]+100*tmptime[1]+10*tmptime[2]+tmptime[3];
      btimeM=10*tmptime[5]+tmptime[6];

      /* Tests for valid data
       * 1. RSS pads ~5% scans to file: Remove scans where 0 >= fop > =1
       * 2. Check Month and Year to remove data from other months/years
       * 3. Exlude scans where first 4 RSS flags are ==1                 */
      if ( ( ( ((float)t6.fop/1.e6 >= 0) && ((float)t6.fop/1.e6 <= 1) ) ) &&
	   ( btimeY == yyyy && btimeM == mm ) &&
	   ( (t6.qflag[0]!= 1) && (t6.qflag[1]!= 1) &&
	     (t6.qflag[2]!= 1) && (t6.qflag[3]!= 1)) ) {
	

	// Update the bins for the channel 19v, 22v and pc1
	// 5deg code only: do separately for asc/des scans
	if (iscan==0){
	  if (satlat[iscan+1]-satlat[iscan]>0) {
	    UpdateBins();
	  }
	  else {
	    UpdateBins2();
	  }
	}
	if (iscan>0) {
	  if (satlat[iscan]-satlat[iscan-1]>0) {
	    UpdateBins();
	  }
	  else {
	    UpdateBins2();
	  }
	}

      }
    } // end loop over scans

  } // end loop over data files
  fclose(fp_filelist); // close file listing file
  




  /*****************************************************
   * Write histogram to file                           */
  // Make header for the histogram.  Be sure its length is always 390 bytes.

  fp_histfile = fopen(HistFile,"w");
  if (fp_histfile == NULL) {
    fprintf(stderr,"Can't open output file: %s\n",HistFile);
    exit(EXIT_FAILURE);
  }

  // Add ascii header to start of output histogram file
  sprintf(HistHeader,"SUN0230  SSM/I %s 5.0 Degree Calendar Monthly Histogram      ",sat);
  sprintf(HistHeader,"%62s\nYear: %04d   Month: %02d",HistHeader,yyyy,mm);
  fprintf(fp_histfile,"%-390s",HistHeader);

  // Copy statistics to histogram
  PutStatsInHist();
  // 5deg code only: additional copy
  PutStatsInHist2();

  // Save to file 
  fwrite(hist,sizeof(hist),1,fp_histfile);
  // 5deg code only: additional write
  fwrite(hist2,sizeof(hist2),1,fp_histfile);
  
  fclose(fp_histfile); // Close file  

  free(RSStb);
  
  return(0);
  
}




