#include <stdio.h>
#include <stdlib.h>
#include <netcdf.h>
#include <string.h>

#include "Define_RSSv6Tb_ssmi_5deg.h"

#define pixel_lores   64
#define pixel_hires   128
#define max_scan      5000
#define max_scan2     10000
#define twelve_flags  12
#define numchar       20


/* Required for error checking reading NetCDF files */
#define ERR(e) {fprintf(stderr,"Error (METH RSS Tb Read): %s\n", nc_strerror(e)); return 2;;}



int ReadNC_RSS_SSMI_Tb_v6_nscan(char *fname)
{
  int     ncid;   /* Netcdf file unit */
  int     retval; /* Status flag for netcdf files */
  char    recname[NC_MAX_NAME+1]; /* dummy var used by nc_inq_dim */
  int     nscan_lores_varid;
  size_t  nscan_lores;

  /* Open nc file and get number of scans from header */
  if (retval = nc_open(fname, NC_NOWRITE, &ncid)) {
    fprintf(stderr,"Can't open RSS V6 file: %s\nNC Error: %s\n",
	    fname,nc_strerror(retval));
    return;
  }

  /* get number of scans dimension */
  if (retval = nc_inq_dimid(ncid, "nscan_lores",&nscan_lores_varid))      ERR(retval);
  if (retval = nc_inq_dim(ncid, nscan_lores_varid, recname,&nscan_lores)) ERR(retval);

  /* Close netcdf file */
  if ((retval = nc_close(ncid))) ERR(retval);

  return((int)nscan_lores);
}




int ReadNC_RSS_SSMI_Tb_v6(char *fname, struct UnpackString_header_v6 *RSStb_header, struct UnpackString_v6 *RSStb)
{

  int     ncid;   /* Netcdf file unit */
  int     retval; /* Status flag for netcdf files */
  char    recname[NC_MAX_NAME+1]; /* dummy var used by nc_inq_dim */
  char    platform[100]; /* Platform attribute from file */
  int     iscan;
  int     i, j;

  char   *tmptext;
  char    sattext[2];
  char    date[numchar];
  float   offset;

  /* Declare variable IDs */
  int nscan_lores_varid;
  int quality_flags_varid;
  int scan_time_lores_varid;
  int scan_time_since87_lores_varid;
  int latitude_lores_varid;
  int longitude_lores_varid;
  int latitude_hires_varid;
  int longitude_hires_varid;
  int fcdr_brightness_temperature_19V_varid;
  int fcdr_brightness_temperature_19H_varid;
  int fcdr_brightness_temperature_22V_varid;
  int fcdr_brightness_temperature_37V_varid;
  int fcdr_brightness_temperature_37H_varid;
  int fcdr_brightness_temperature_85V_varid;
  int fcdr_brightness_temperature_85H_varid;
  int sea_ice_hires_varid;
  int surface_type_hires_varid;
  int fractional_orbit_position_varid;
  int spacecraft_latitude_varid;
  int spacecraft_longitude_varid;
  int spacecraft_altitude_varid;
  int earth_incidence_angle_hires_varid;
  int earth_azimuth_angle_hires_varid;
  int earth_sunglitter_angle_hires_varid;

  /* Declare variables to be read */
  size_t nscan_lores;
  char   scan_time_lores[max_scan][numchar];
  double scan_time_since87_lores[max_scan];
  float fractional_orbit_position[max_scan];
  float spacecraft_latitude[max_scan];
  float spacecraft_longitude[max_scan];
  float spacecraft_altitude[max_scan];
  signed char quality_flags[max_scan][twelve_flags];
  short int latitude_lores[max_scan][pixel_lores];
  short int longitude_lores[max_scan][pixel_lores];
  signed char sea_ice_hires[max_scan2][pixel_hires];
  short int fcdr_brightness_temperature_19V[max_scan][pixel_lores];
  short int fcdr_brightness_temperature_19H[max_scan][pixel_lores];
  short int fcdr_brightness_temperature_22V[max_scan][pixel_lores];
  short int fcdr_brightness_temperature_37V[max_scan][pixel_lores];
  short int fcdr_brightness_temperature_37H[max_scan][pixel_lores];
  short int latitude_hires[max_scan2][pixel_hires];
  short int longitude_hires[max_scan2][pixel_hires];
  signed char surface_type_hires[max_scan2][pixel_hires];
  short int fcdr_brightness_temperature_85V[max_scan2][pixel_hires];
  short int fcdr_brightness_temperature_85H[max_scan2][pixel_hires];
  short int earth_incidence_angle_hires[max_scan][pixel_hires];
  float earth_azimuth_angle_hires[max_scan][pixel_hires];
  short int earth_sunglitter_angle_hires[max_scan][pixel_hires];

  /* Open nc file and get sat number and number of scans from header */
  if (retval = nc_open(fname, NC_NOWRITE, &ncid)) {
    fprintf(stderr,"Can't open RSS V6 file: %s\nNC Error: %s\n",
	    fname,nc_strerror(retval));
    return;
  }

  /* Read some global attributes */
  if (retval = nc_get_att_text(ncid, NC_GLOBAL, 
			       "platform", platform))       ERR(retval);

  /* get number of scans dimension */
  if (retval = nc_inq_dimid(ncid, "nscan_lores", 
			    &nscan_lores_varid))        ERR(retval);
  if (retval = nc_inq_dim(ncid, nscan_lores_varid, recname,  
			  &nscan_lores))            ERR(retval);




  /************************************************
   * Read variables from file                     */
  if (nscan_lores == 0) { // Do not continue if there is no data to be read
    printf("Zero scans in file: nothing to read;"); return 2;
  }


  /* Get variable ids */
  if (retval = nc_inq_varid(ncid, "quality_flags", 
			    &quality_flags_varid))                   ERR(retval);
  if (retval = nc_inq_varid(ncid, "scan_time_lores", 
			    &scan_time_lores_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "scan_time_since87_lores", 
			    &scan_time_since87_lores_varid))         ERR(retval);
  if (retval = nc_inq_varid(ncid, "latitude_lores", 
			    &latitude_lores_varid))                  ERR(retval);
  if (retval = nc_inq_varid(ncid, "longitude_lores", 
			    &longitude_lores_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "latitude_hires",
			    &latitude_hires_varid))                  ERR(retval);
  if (retval = nc_inq_varid(ncid, "longitude_hires", 
			    &longitude_hires_varid))                 ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_19V", 
			    &fcdr_brightness_temperature_19V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_19H", 
			    &fcdr_brightness_temperature_19H_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_22V", 
			    &fcdr_brightness_temperature_22V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_37V", 
			    &fcdr_brightness_temperature_37V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_37H", 
			    &fcdr_brightness_temperature_37H_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_85V", 
			    &fcdr_brightness_temperature_85V_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "fcdr_brightness_temperature_85H", 
			    &fcdr_brightness_temperature_85H_varid)) ERR(retval);
  if (retval = nc_inq_varid(ncid, "surface_type_hires", 
			    &surface_type_hires_varid))              ERR(retval);
  if (retval = nc_inq_varid(ncid, "sea_ice_hires", 
			    &sea_ice_hires_varid))              ERR(retval);
  if (retval = nc_inq_varid(ncid, "fractional_orbit_position", 
			    &fractional_orbit_position_varid))       ERR(retval);
  if (retval = nc_inq_varid(ncid, "spacecraft_latitude", 
			    &spacecraft_latitude_varid))             ERR(retval);
  if (retval = nc_inq_varid(ncid, "spacecraft_longitude", 
			    &spacecraft_longitude_varid))            ERR(retval);
  if (retval = nc_inq_varid(ncid, "spacecraft_altitude", 
			    &spacecraft_altitude_varid))             ERR(retval);
  if (retval = nc_inq_varid(ncid, "earth_incidence_angle_hires", 
			    &earth_incidence_angle_hires_varid))     ERR(retval);
  if (retval = nc_inq_varid(ncid, "earth_azimuth_angle_hires",
			    &earth_azimuth_angle_hires_varid))       ERR(retval);
  if (retval = nc_inq_varid(ncid, "earth_sunglitter_angle_hires",
			    &earth_sunglitter_angle_hires_varid))    ERR(retval);
  
    
  /* Read variables */
  if (retval = nc_get_var_schar(ncid, quality_flags_varid, 
				&quality_flags[0][0]))                      ERR(retval);
  if (retval = nc_get_var_text(ncid, scan_time_lores_varid, 
			       &scan_time_lores[0][0]))                    ERR(retval);
  if (retval = nc_get_var_double(ncid, scan_time_since87_lores_varid,
				 &scan_time_since87_lores[0]))             ERR(retval);
  if (retval = nc_get_var_short(ncid, latitude_lores_varid, 
				&latitude_lores[0][0]))                    ERR(retval);
  if (retval = nc_get_var_short(ncid, longitude_lores_varid, 
				&longitude_lores[0][0]))                   ERR(retval);
  if (retval = nc_get_var_short(ncid, latitude_hires_varid, 
				&latitude_hires[0][0]))                    ERR(retval);
  if (retval = nc_get_var_short(ncid, longitude_hires_varid, 
				&longitude_hires[0][0]))                   ERR(retval);
  if (retval = nc_get_var_short(ncid, fcdr_brightness_temperature_19V_varid,
				&fcdr_brightness_temperature_19V[0][0]))   ERR(retval);
  if (retval = nc_get_var_short(ncid, fcdr_brightness_temperature_19H_varid, 
				&fcdr_brightness_temperature_19H[0][0]))   ERR(retval);
  if (retval = nc_get_var_short(ncid, fcdr_brightness_temperature_22V_varid, 
				&fcdr_brightness_temperature_22V[0][0]))   ERR(retval);
  if (retval = nc_get_var_short(ncid, fcdr_brightness_temperature_37V_varid, 
				&fcdr_brightness_temperature_37V[0][0]))   ERR(retval);
  if (retval = nc_get_var_short(ncid, fcdr_brightness_temperature_37H_varid, 
				&fcdr_brightness_temperature_37H[0][0]))   ERR(retval);
  if (retval = nc_get_var_short(ncid, fcdr_brightness_temperature_85V_varid, 
				&fcdr_brightness_temperature_85V[0][0]))   ERR(retval);
  if (retval = nc_get_var_short(ncid, fcdr_brightness_temperature_85H_varid, 
				&fcdr_brightness_temperature_85H[0][0]))   ERR(retval);
  if (retval = nc_get_var_schar(ncid, surface_type_hires_varid, 
				&surface_type_hires[0][0]))                 ERR(retval);
  if (retval = nc_get_var_schar(ncid, sea_ice_hires_varid, 
				&sea_ice_hires[0][0]))                 ERR(retval);
  if (retval = nc_get_var_float(ncid, fractional_orbit_position_varid, 
				&fractional_orbit_position[0]))            ERR(retval);
  if (retval = nc_get_var_float(ncid, spacecraft_latitude_varid, 
				&spacecraft_latitude[0]))                  ERR(retval);
  if (retval = nc_get_var_float(ncid, spacecraft_longitude_varid, 
				&spacecraft_longitude[0]))                 ERR(retval);
  if (retval = nc_get_var_float(ncid, spacecraft_altitude_varid, 
				&spacecraft_altitude[0]))                  ERR(retval);
  if (retval = nc_get_var_short(ncid, earth_incidence_angle_hires_varid, 
				&earth_incidence_angle_hires[0][0]))       ERR(retval);
  if (retval = nc_get_var_float(ncid, earth_azimuth_angle_hires_varid, 
				&earth_azimuth_angle_hires[0][0]))         ERR(retval);
  if (retval = nc_get_var_short(ncid, earth_sunglitter_angle_hires_varid, 
				&earth_sunglitter_angle_hires[0][0]))      ERR(retval);
 
  /* Close netcdf file */
  if ((retval = nc_close(ncid))) ERR(retval);




  /****************************************************
   * Copy data into structure for output to main call */

  /* Copy data into SSMI header */
  (*RSStb_header).numscan=nscan_lores;

  /* There is no "iorbit" value in these files
   * One option is to estimate it from the original
   * filename contained in the global attribute "id"
   * but we cannot be certain that it would be correct
   * Thus, iorbit is set to zero - frac orbit can be 
   * negative, so this is the most unambiguous option */
  (*RSStb_header).iorbit=0;

  /* Get satellite number from global attributes 
   * Need to find start of satellite number first  */ 
  tmptext=(char*)memchr(platform,'F',strlen(platform));
  memcpy(sattext,&tmptext[1],2);
  (*RSStb_header).ksat=atoi(sattext);

  


  /* copy data into main output array
   * lores scan is the record length  */
  for (iscan=0; iscan<nscan_lores; iscan++) {

    RSStb[iscan].scanrecord=iscan+1;

    /* copy text date and pad last 4 elements with blank */
    for (i=0; i<numchar; i++) {
      RSStb[iscan].btime[i]=scan_time_lores[iscan][i];
    }
    for (i=numchar; i<24; i++) RSStb[iscan].btime[i]=' ';

    for (i=0; i<numchar; i++) {
      RSStb[iscan].qflag[i]=quality_flags[iscan][i];
    }

    /* Following are stored as scaled long to be compatible with METH SSMI code */
    RSStb[iscan].fop=(long)(fractional_orbit_position[iscan]*1.e6);
    RSStb[iscan].sclat=(long)(spacecraft_latitude[iscan]*1.e3);
    RSStb[iscan].sclon=(long)(spacecraft_longitude[iscan]*1.e3);
    RSStb[iscan].scalt=(long)(spacecraft_altitude[iscan]*1.e3);

    for (i=0; i<HI_SAMPS; i++) {
      for (j=0; j<2; j++) {
	RSStb[iscan].lat[j][i]=latitude_hires[iscan*2+j][i];

	/* METH code expects lon to be between 0:360, then scaled and offset 
	 * by -180. This is a legacy of the code and is very tricky to fix... */
	if ( ((longitude_hires[iscan*2+j][i]*0.01)>=-180) && 
	     ((longitude_hires[iscan*2+j][i]*0.01)<0) ) {
	  RSStb[iscan].lon[j][i]=(int)(((float)longitude_hires[iscan*2+j][i]*0.01+360-180)/0.01);
	}
	else {
	  RSStb[iscan].lon[j][i]=(int)(((float)longitude_hires[iscan*2+j][i]*0.01-180)/0.01);
	}

	/* Combine surface type and sea ice: 0=open_ocean, 1=near_coast, 
	 * 2=land, 4=ocean+ice, 5=coast+ice, 6=land+ice/snow            
	 * Fill values are 10 for each variable - set to -9 if either occurs */
	RSStb[iscan].sflag[j][i]=(int)surface_type_hires[iscan*2+j][i]+4*(int)sea_ice_hires[iscan*2+j][i];
	if ((surface_type_hires[iscan*2+j][i]==10)|
	    (sea_ice_hires[iscan*2+j][i]==10)) {
	  RSStb[iscan].sflag[j][i]=-9;
	}

	//	RSStb[iscan].sflag[j][i]=(int)surface_type_hires[iscan*2+j][i]+4*(int)sea_ice_hires[iscan*2+j][i];






      }
      RSStb[iscan].eiangle[i]=(earth_incidence_angle_hires[iscan][i]*0.01-45)/2.E-3;
      RSStb[iscan].azimuth[i]=(int)earth_azimuth_angle_hires[iscan][i];
      RSStb[iscan].sunang[i]=earth_sunglitter_angle_hires[iscan][i];
    }

    for (i=0; i<LOW_SAMPS; i++) {
      RSStb[iscan].tb[i][0]=fcdr_brightness_temperature_19V[iscan][i];
      RSStb[iscan].tb[i][1]=fcdr_brightness_temperature_19H[iscan][i];
      RSStb[iscan].tb[i][2]=fcdr_brightness_temperature_22V[iscan][i];
      RSStb[iscan].tb[i][3]=fcdr_brightness_temperature_37V[iscan][i];
      RSStb[iscan].tb[i][4]=fcdr_brightness_temperature_37H[iscan][i];
    }
    for (i=0; i<HI_SAMPS; i++) {
      for (j=0; j<2; j++) {
	RSStb[iscan].tb85[j][i][0]=fcdr_brightness_temperature_85V[iscan*2+j][i];
	RSStb[iscan].tb85[j][i][1]=fcdr_brightness_temperature_85H[iscan*2+j][i];
      }
    }

  }

  return(0);
}


