      program Main_GPCPcdr_m4_corr2
c     Code to compute AGPI from GPI and SSMI/SSMIS

c     original header:
C* This program computes the additive and ratio adjustments for geo-IR	*
C* and leo-IR from cutouts of geo-IR and total leo-IR, then computes	*
C* the AGPI for geo-IR with leo-IR fill-in.				*
C*									*
C* corr2 ARG <<!							*
C* IR_NAMES								*
C* CUTFILE								*
C* SSMI_NAME								*
C* DATE									*
C* OUTBASE								*
C* SMOOTH								*
C* !									*
C*									*
C* Input command-line arguments:					*
C*   help	CHAR	Request help					*
C*   <null>		No entry flags interactive input		*
C*   X		CHAR	Any other entry flags non-interactive input	*
C*									*
C* Input data (interactive or a script):				* 
C*   IR_NAMES	CHAR    File names of 1x1-deg 3-hrly merged IR data;	*
C*			- files are sorted in ascending date/time,	*
C*			- there can be up to 910 entries,		*
C*			- with the last entry is '999';			*
C*			or the name of a file containing such a list	*
C*			(without the terminating '999') as NAME,file	*
C*   CUTFILE	CHAR	Cut file name; file contains (x,y,node)=	*
C*			(144,32,a|d) entries of time index (1-8) to be	*
C*			cut from 3hr arrays and accumulated in output	*
C*			array; if cnam='none', def = 1|5 for entire a|d	*
C*			arrays; if cnam='all', all data are used (no	*
C*			cutting; option not available in CORR.F);	*
C*			'-' = default ('geo_cuts.f08')			*
C*   SSMI_NAME	CHAR	File name of 2.5x2.5-deg monthly precip field	*
C*			(in mm/hr) for getting rain rate		*
C*   SN_NAME	CHAR	SSMI, TOVS, and/or MSU number-of-samples	*
C*			filename or 'none'				*
C*   SS_NAME	CHAR	SSMI, TOVS, and/or MSU source filename or	*
C*			'none'						*
C*			      | 0-1 SSM/I fields 1 to 2			*
C*			legal | 2-3 SSM/I fields 1 to 2 and TOVS	*
C*			src   | 4   TOVS				*
C*			values| 5   MSU					*
C*			      | 6   MSU and TOVS			*
C*   DATE	CHAR	Date as YYYYMM					*
C*   OUTBASE	CHAR	Base name of output files			*
C*   SMOOTH	CHAR	Size of spatial smoother for IR histograms and	*
C*			SSM/I fractional coverage; 0 = none, 3 = only	*
C*			valid non-zero value				*


      IMPLICIT NONE
      
      
      INTEGER*4	iunit, iostat, i, j, k, m, iss (3), it, jdiff,
     +     npass, l_m, iret1, iret2, iret3, iret4, iret5,
     +     nin,   l_cn, iout, jout,
     +     l_nam, maxg, maxl
      LOGICAL		active (3), more
C     
      INTEGER		LN10,  LT10, LN25, LT25, LBIN, LHR, MAXNAM
      REAL*4		RMISS
      PARAMETER	( LN10   = 360,
     +     LT10   = 80,
     +     LN25   = 144,
     +     LT25   = 40,
     +     LBIN   = 24,
     +     LHR    = 8,
     +     MAXNAM = 31,
     +     RMISS  = -99999. )
C
      CHARACTER*256	iname (MAXNAM), cname, sname, snname,
     +     ssname, bnam, l1, l2, l3, l4, l5, l6, l7, l8,
     +     l9, l10, l11, l12
      CHARACTER*8	cid
      CHARACTER*6	yyyymm
      INTEGER*4	nir, ndays, iret, iir, ihr, jyrir, jdayir,
     +     jminir, jyrss (3), jdayss (3)
      REAL*4		p_mon   (144,72),
     +     cum10   (LN10, LT10), cumg10   (LN10, LT10),    
     +     cuml10  (LN10, LT10), cutg10   (LN10, LT10),
     +     filll10 (LN10, LT10), ratl10   (LN10, LT10),    
     +     cut10   (LN10, LT10, 4), 
     +     cum25   (LN25, LT25), cumg25   (LN25, LT25),    
     +     cuml25  (LN25, LT25), cutg25   (LN25, LT25),
     +     filll25 (LN25, LT25), ratl25   (LN25, LT25),    
     +     adjg    (LN25, LT25), adjl     (LN25, LT25),
     +     adj     (LN25, LT25),
     +     cut25   (144, 32, 2), ss25     (144, 72),
     +     ss25n   (144, 72),    ss25s    (144, 72), 
     +     cumn10  (LN10, LT10), 
     +     cumgn10 (LN10, LT10), cumln10  (LN10, LT10),
     +     cutgn10 (LN10, LT10), fillln10 (LN10, LT10),
     +     cumn25  (LN25, LT25), 
     +     cumgn25 (LN25, LT25), cumln25  (LN25, LT25),
     +     cutgn25 (LN25, LT25), fillln25 (LN25, LT25),
     +     adjn    (LN25, LT25), smg      (LN25, LT25),
     +     sml     (LN25, LT25), sms      (LN25, LT25),
     +     a       (LN25, LT25), b        (LN25, LT25),
     +     c       (LN25, LT25), d        (LN25, LT25),
     +     e       (LN25, LT25), f        (LN25, LT25)
chase
c	REAL*4		tb10 (LN10, LT10, LBIN), tbsum   (LN10, LT10),
c     +			tb25 (LN25, LT25, LBIN), tbsum25 (LN25, LT25)
c	INTEGER*4	isum
c	DATA		tb10, tbsum / 691200*0., 28800*0. /
chase
      INTEGER*4	icut4 (LN10, LT10, 2)
      INTEGER*2	ltb   (LN10, LT10, LBIN),
     +     lza   (LN10, LT10),
     +     leop  (LN10, LT10), 
     +     igeop (LN10, LT10)
      INTEGER*1	icut1 (144,  32,   2)
C     
      INTEGER*4	INIT_DISK, INITW_DISK, READ_DISK, WRITE_DISK,
     +     EXIT_DISK
C     
      REAL*4        smth (11, 11)
      DATA          smth / 121*1 /
      integer*4     nsm/3/
      
c     Added by Sapiano - gfortran has trouble with reals being used
c     as indexes for arays, so use these as dummy variables
      integer*4     iint1, jint1, iint2, jint2
      character*256 fname, InterimPATH, irPATH, irlist
      integer*4     Ninputs, ios, strlen, inunit, l_ip
      logical       opn
      integer*4     iyear, imonth
      integer*4     NdaysInMonth(12)
     &     / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /




c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 4 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_Adjust_Gauge:'//
     &        ' Incorrect number of input arguments (should be 4)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, yyyymm)
      call getarg(2, InterimPATH)
      call getarg(3, irPATH)
      call getarg(4, IRlist)

c     Convert month to a number
      read(yyyymm(1:4), '(i10)' ) iyear
      read(yyyymm(5:6), '(i10)' ) imonth

c     Determine number of days in month
      ndays=NdaysInMonth(imonth)
      if ( imonth.EQ.2 ) then 
         if ( mod(iyear,400).EQ.0 ) then
            ndays=29
         elseif ( mod(iyear,100).EQ.0 ) then
            ndays=28
         elseif ( mod(iyear,4).EQ.0 ) then
            ndays=29
         endif
      endif

c     Establish Ancillary data: cut filename
c     This is hardcoded, because it should not change
      cname='GPCPcdr_m4/GPCPcdr_m4_CORR2/Ancillary/'
     +     //'geo_cuts.f13.1996.le' 


c     Establish root of output filenames
      do strlen = 1,len_trim(InterimPATH)
         if (InterimPATH(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1
      bnam=InterimPATH(1:strlen)//'CORR2/cgt.'//yyyymm

c     Establish names of SSMI/SSMIS monthly fields
c     This code requires precip, N and source
      sname=InterimPATH(1:strlen)//'XOVER/cgt.'//yyyymm//'.rfix'
      snname=InterimPATH(1:strlen)//'XOVER/cgt.'//yyyymm//'.Ifix'
      ssname=InterimPATH(1:strlen)//'XOVER/cgt.'//yyyymm//'.src'

c     Read contents of IRlist
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      open(inunit, file=IRlist, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_corr2:'
     &        //'Failed to open IR file listing file'
         STOP
      end if
      nir=0
      do i=1,MAXNAM
         read(inunit, '(A256)', iostat=ios ) iname(i)
         if ( ios .EQ. 0 ) then
            nir=nir+1
         else
            iname(i) = '999'
         endif
      end do
      if ( nir .EQ. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_corr2:'
     &        //'Failed to read IR file listing file'
         STOP
      end if
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_corr2:'
     &        //'Failed to close IR file listing file'
         STOP
      end if





c************************************************************
C     Initialize fields.  The maximum number of [geo-|leo-]IR images
C     is [8|4] per day.  The IR reader handles file number 
C     internally, because there are 8 fields in each IR file, so 
C     initialize iir,ihr to not valid.  Zero accumulation arrays.
     
      maxg = 8 * ndays
      maxl = 4 * ndays
      iir  = 0
      ihr  = 0

      DO  j = 1, LT10
         DO  i = 1, LN10
            cum10    (i, j) = 0.
	    cumg10   (i, j) = 0.
	    cuml10   (i, j) = 0.
	    cutg10   (i, j) = 0.
	    filll10  (i, j) = 0.
	    cumn10   (i, j) = 0.
	    cumgn10  (i, j) = 0.
	    cumln10  (i, j) = 0.
	    cutgn10  (i, j) = 0.
	    fillln10 (i, j) = 0.
         END DO
      END DO
      DO  j = 1, LT25
         DO  i = 1, LN25
	    cum25    (i, j) = RMISS
	    cumg25   (i, j) = RMISS
	    cuml25   (i, j) = RMISS
	    cutg25   (i, j) = RMISS
	    filll25  (i, j) = RMISS
	    cumn25   (i, j) = 0.
	    cumgn25  (i, j) = 0.
	    cumln25  (i, j) = 0.
	    cutgn25  (i, j) = 0.
	    fillln25 (i, j) = 0.
         END DO
      END DO



c********************************************************************
c     Set up cut arrays. 
c     First, read cut file 

      do l_cn = 1,len_trim(cname)
         if (cname(l_cn:l_cn) .eq. ' ') then
            continue
         end if
      end do
      l_cn=l_cn-1

      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO

      open(inunit,file=cname(1:l_cn), IOSTAT=ios,
     &     form='unformatted',access='direct',recl=4*2304,status='old')
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_corr2: '//
     &        'Failed to open cut file'
         STOP
      end if
      read(inunit,rec=1, IOSTAT=ios) icut1
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_corr2: '//
     &        'Failed to read cut file'
         STOP
      end if
      close(inunit, IOSTAT=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_corr2: '//
     &        'Failed to close cut file'
         STOP
      end if

C     Regrid cut file data from 2.5 to 1 deg.  To handle the day 
C     boundary, cut values 1-4 are shifted up to 9-12 in cut10 
C     fields 3 and 4.  Then in the center of the day the original 
C     values are used, but at the ends of the day the shifted 
C     values are used (corrected to be in the range 1-8).  
C     "Center" is detected with the original array, because the 
C     interpolation of 8 and 1 at the day boundary gives you an 
C     apparent "center" value.

      DO  j = 1, 32
         DO  i = 1, 144
            cut25 (i, j, 1) = icut1 (i, j, 1)
            cut25 (i, j, 2) = icut1 (i, j, 2)
         END DO
      END DO
      CALL REGRID  ( 144, 32, LN10, LT10, cut25 (1, 1, 1), 
     +     cut10 (1, 1, 1),     iret)
      CALL REGRID  ( 144, 32, LN10, LT10, cut25 (1, 1, 2), 
     +     cut10 (1, 1, 2),     iret)
      DO  j = 1, 32
         DO  i = 1, 144
            IF  ( cut25 (i, j, 1) .LT. 5. )
     +           cut25 (i, j, 1) = cut25 (i, j, 1) + 8.
            IF  ( cut25 (i, j, 2) .LT. 5. )
     +           cut25 (i, j, 2) = cut25 (i, j, 2) + 8.
         END DO
      END DO
      CALL REGRID  ( 144, 32, LN10, LT10, cut25 (1, 1, 1), 
     +     cut10 (1, 1, 3),     iret)
      CALL REGRID  ( 144, 32, LN10, LT10, cut25 (1, 1, 2), 
     +     cut10 (1, 1, 4),     iret)
C     
C     cut25 has had the lower 4 positions shifted up by 8, so do 
C     the tests across the day boundary (cut values 8-9).
C     
      DO  j = 1, LT10
         DO  i = 1, LN10
            iint1=1+(i-1)/2.5
            jint1=1+(j-1)/2.5
            iint2=1+(i-1)/2.5
            jint2=1+(j-1)/2.5
            IF  ( cut25 (iint1, jint1, 1) .GT. 7. .AND.
     +           cut25 (iint2, jint2, 1) .LT. 10. )
     +           THEN
               icut4 (i, j, 1) = NINT ( cut10 (i, j, 3) )
               IF  ( icut4 (i, j, 1) .GT. 8 )
     +              icut4 (i, j, 1) = icut4 (i, j, 1) - 8
            ELSE
               icut4 (i, j, 1) = NINT ( cut10 (i, j, 1) )
            END IF
            iint1=1+(i-1)/2.5
            jint1=1+(j-1)/2.5
            iint2=1+(i-1)/2.5
            jint2=1+(j-1)/2.5
            IF  ( cut25 (iint1, jint1, 2) .GT. 7. .AND.
     +           cut25 (iint2, jint2, 2) .LT. 10. )
     +           THEN
               icut4 (i, j, 2) = NINT ( cut10 (i, j, 4) )
               IF  ( icut4 (i, j, 2) .GT. 8 )
     +              icut4 (i, j, 2) = icut4 (i, j, 2) - 8
            ELSE
               icut4 (i, j, 2) = NINT ( cut10 (i, j, 2) )
            END IF
         END DO
      END DO


c*******************************************************************
c     Read monthly SSMI/SSMIS files and remove low-population values


      call Read_Binary_File(sname,ss25,144,72,1)
      call Read_Binary_File(snname,ss25n,144,72,1)
      call Read_Binary_File(ssname,ss25s,144,72,1)

      DO  j = 1, 72
         DO  i = 1, 144
            IF  ( ss25n (i, j) .LE. 10. .AND. ss25s (i, j) .LE. 2. )
     +           ss25 (i, j) = RMISS
         END DO
      END DO




c*******************************************************************
c     Loop over IR dat aand accumulate matchups over time
C     The general concept is 
C     to open each IR time, limit the incoming merged IR to geo by
C     masking with the zenith angle (which is * 10 and integer), and 
C     "cut" the file, then open the next IR and continue; this 
C     proceeds until the IR data are exhausted.  Accumulate total 
C     geo-GPI and leo-GPI, then approx. cutouts of geo-GPI and leo-GPI 
C     that will fill holes in individual geo-GPI images.
C     NOTE: The integer inputs are mm/d scaled by 100, but the sums 
C     are left that way and it's handled when the averages are 
C     computed.
      


C	Open the first IR file.
C
	CALL GET_IRTB ( iname, nir, iir,   ihr,  jyrir, jdayir, jminir, 
     +			ltb,   lza, igeop, leop, iret )

	IF  ( iret .NE. 0 )  STOP
	WRITE (*, *) 'SSM/I and cut file read in, first IR opened; ',
     +		     'ready to loop'
C
C	Loop to accumulate matchups over time.  The general concept is 
C	to open each IR time, limit the incoming merged IR to geo by
C	masking with the zenith angle (which is * 10 and integer), and 
C	"cut" the file, then open the next IR and continue; this 
C	proceeds until the IR data are exhausted.  Accumulate total 
C	geo-GPI and leo-GPI, then approx. cutouts of geo-GPI and leo-GPI 
C	that will fill holes in individual geo-GPI images.
C	NOTE: The integer inputs are mm/d scaled by 100, but the sums 
C	are left that way and it's handled when the averages are 
C	computed.
C
	DO  WHILE ( iir .GT. 0 )

	    DO  j = 1, LT10
	    DO  i = 1, LN10
		IF  ( lza (i, j) .LT. 0 .OR. lza (i, j) .GE. 900 )  
     +		    igeop (i, j) = -1
	    END DO
	    END DO

	    DO  j = 1, LT10
	    DO  i = 1, LN10
		IF  ( igeop (i, j) .GE. 0 )  THEN
		    cum10   (i, j) = cum10   (i, j) + igeop (i, j)
		    cumn10  (i, j) = cumn10  (i, j) + 1
		    cumg10  (i, j) = cumg10  (i, j) + igeop (i, j)
		    cumgn10 (i, j) = cumgn10 (i, j) + 1
		  ELSE IF  ( leop (i, j) .GE. 0 )  THEN
		    cum10   (i, j) = cum10   (i, j) + leop  (i, j)
		    cumn10  (i, j) = cumn10  (i, j) + 1
		END IF
	    END DO
	    END DO
	    DO  j = 1, LT10
	    DO  i = 1, LN10
		IF  ( leop (i, j) .GE. 0 )  THEN
		    cuml10  (i, j) = cuml10  (i, j) + leop  (i, j)
		    cumln10 (i, j) = cumln10 (i, j) + 1
		END IF
	    END DO
	    END DO


c	    IF  ( cname (1:l_cn) .NE. 'all' )  THEN
            DO  j = 1, LT10
               DO  i = 1, LN10
                  IF  ( igeop (i, j) .GE. 0 .AND. 
     +                 ( icut4 (i, j, 1) .EQ. ihr .OR. 
     +                 icut4 (i, j, 2) .EQ. ihr ) )  THEN
                     cutg10  (i, j) = cutg10  (i, j) + igeop (i, j)
                     cutgn10 (i, j) = cutgn10 (i, j) + 1
                  END IF
               END DO
            END DO
c           END IF

	    DO  j = 1, LT10
	    DO  i = 1, LN10
		IF  ( leop (i, j) .GE. 0 .AND. igeop (i, j) .LT. 0 )  
     +								THEN
		    filll10  (i, j) = filll10  (i, j) + leop (i, j)
		    fillln10 (i, j) = fillln10 (i, j) + 1
		END IF
	    END DO
	    END DO

	    CALL GET_IRTB ( iname,  nir,    iir, ihr, jyrir, 
     +			    jdayir, jminir, ltb, lza, igeop,
     +			    leop,   iret )
	    IF  ( iret .NE. 0 )  STOP
	END DO

C	Compute averages in all fields.  At this point we adjust
C	for the 100 multiplier.
	DO  j = 1, LT10
	DO  i = 1, LN10
	    IF  ( cumn10 (i, j) .GT. 0 )  THEN
		cum10  (i, j) = .01 * cum10  (i, j) / cumn10  (i, j)
	      ELSE
		cum10  (i, j) = RMISS
	    END IF
	END DO
	END DO
	DO  j = 1, LT10
	DO  i = 1, LN10
	    IF  ( cumgn10 (i, j) .GT. 0 )  THEN
		cumg10 (i, j) = .01 * cumg10 (i, j) / cumgn10 (i, j)
	      ELSE
		cumg10 (i, j) = RMISS
	    END IF
	END DO
	END DO
	DO  j = 1, LT10
	DO  i = 1, LN10
	    IF  ( cumln10 (i, j) .GT. 0 )  THEN
		cuml10 (i, j) = .01 * cuml10 (i, j) / cumln10 (i, j)
	      ELSE
		cuml10 (i, j) = RMISS
	    END IF
	END DO
	END DO
c	IF  ( cname (1:l_cn) .NE. 'all' )  THEN
        DO  j = 1, LT10
           DO  i = 1, LN10
              IF  ( cutgn10 (i, j) .GT. 0 )  THEN
                 cutg10 (i, j) = .01 * cutg10 (i, j) / cutgn10 (i, j)
              ELSE
                 cutg10 (i, j) = RMISS
              END IF
           END DO
        END DO
c	END IF

	DO  j = 1, LT10
	DO  i = 1, LN10
	    IF  ( fillln10 (i, j) .GT. 0 )  THEN
		filll10 (i, j) = .01 * filll10 (i, j) / fillln10 (i,j)
	      ELSE
		filll10 (i, j) = RMISS
	    END IF
	END DO
	END DO


C	Reduce resolution from 1 to 2.5 deg.  The offset in the 2.5 
C	array is because the 1-deg runs 40N-S, while the 2.5-deg runs
C	50N-S.

	CALL REDUCE ( LN10,   LT10,   0., 0.,  2.5, 2.5, LN10, LT10, 
     +		      LN25,   LT25-8, cum10,   cumn10,   cum25  (1, 5), 
     +		      cumn25  (1, 5), iout,    jout,     iret )
	CALL REDUCE ( LN10,   LT10,   0., 0.,  2.5, 2.5, LN10, LT10, 
     +		      LN25,   LT25-8, cumg10,  cumgn10,  cumg25 (1, 5), 
     +		      cumgn25 (1, 5), iout,    jout,     iret )
	CALL REDUCE ( LN10,   LT10,   0., 0.,  2.5, 2.5, LN10, LT10, 
     +		      LN25,   LT25-8, cuml10,  cumln10,  cuml25 (1, 5), 
     +		      cumln25 (1, 5), iout,    jout,     iret )
c	IF  ( cname (1:l_cn ) .EQ. 'all' )  THEN
c	    DO  j = 1, LT25
c	    DO  i = 1, LN25
c		cutg25  (i, j) = cumg25  (i, j)
c		cutgn25 (i, j) = cumgn25 (i, j)
c	    END DO
c	    END DO
c	  ELSE

        CALL REDUCE 
     +       ( LN10,   LT10,   0., 0.,  2.5, 2.5, LN10, LT10, 
     +       LN25,   LT25-8, cutg10,  cutgn10,  cutg25 (1, 5), 
     +       cutgn25 (1, 5), iout,    jout,     iret )

c	END IF

	CALL REDUCE ( LN10,   LT10,   0., 0.,  2.5, 2.5, LN10, LT10, 
     +		      LN25,   LT25-8, filll10, fillln10, filll25 (1, 5), 
     +		      fillln25(1, 5), iout,    jout,     iret )





c******************************************************************
c     Commented out most of these - not clear which are needed
c     for earlier period (if any at all)

C	Build output file names (for unsmoothed files, even if we
C	smooth later in the code).  Generate a basename from the
C	input basename and current month.  If the input SSM/I file 
C	has 'fix' in the name, put 'fix' in the output name.
C
c	cid = '0       '
c	IF  ( INDEX ( sname, 'fix' ) .EQ. 0 )  THEN
c	    l4  = 'ssful' // cid (1:1)
c	    l5  = 'gir'   // cid (1:1)
c	    l6  = 'lir'   // cid (1:1)
c	    l7  = 'rat'   // cid (1:1)
c	    l8  = 'add'   // cid (1:1)
c	    l9  = 'ratl'  // cid (1:1)
c	    l10 = 'addl'  // cid (1:1)
c	  ELSE
c	    l4  = 'ssfix'   // cid (1:1)
c	    l5  = 'gir'     // cid (1:1)
c	    l6  = 'lir'     // cid (1:1)
c	    l7  = 'ratfix'  // cid (1:1)
c	    l8  = 'addfix'  // cid (1:1)
c	    l9  = 'ratlfix' // cid (1:1)
c	    l10 = 'addlfix' // cid (1:1)
c	END IF
c
c	CALL DMPIMGI  ( cutg25,       bnam, 'r', l5,   iret )
c	CALL DMPIMGI  ( cum25,        bnam, 'r', 'ge', iret )
c	CALL DMPIMGI  ( cuml25,       bnam, 'r', l6,   iret )
c	CALL DMPIMGI  ( ss25 (1, 17), bnam, 'r', l4,   iret )



C	Optionally smooth the geo-IR and SSMI (only in the IR zone
C	40N-S), but not leo-IR.  The boundaries are "missing" north 
C	and south, and periodic east and west.

	npass = -1
	CALL FILLSM  ( ss25, LN25, LT25, LN25, 0, npass, ss25, iret )
	IF  ( nsm .GE. 3 )  THEN

           CALL SMOOTH  ( nsm, smth, 3., cutg25, LN25, LT25, 
     +          smg, iret )

           DO  j = 1, LT25
              DO  i = 1, LN25
                 sml (i, j) = cuml25 (i, j)
                 sms (i, j) = RMISS
              END DO
           END DO
           CALL SMOOTH  ( nsm, smth, 3., ss25 (1, 21), LN25, LT25-8, 
     +          sms (1,5), iret )

C	    When smoothing, output the smoothed fields.  As before,
C	    if the SSM/I has 'fix' in the name, put 'fix' in the 
C	    output name.

c	    WRITE ( cid, 6 ) MOD ( nsm, 10 )
c    6	    FORMAT ( I1, '       ' )
c	    IF  ( INDEX ( sname, 'fix' ) .EQ. 0 )  THEN
c		l4  = 'ssful' // cid (1:1)
c		l5  = 'gir'   // cid (1:1)
c		l6  = 'lir'   // cid (1:1)
c		l7  = 'rat'   // cid (1:1)
c		l8  = 'add'   // cid (1:1)
c		l9  = 'ratl'  // cid (1:1)
c		l10 = 'addl'  // cid (1:1)
c		l11 = 'ratlg' // cid (1:1)
c		l12 = 'addlg' // cid (1:1)
c	      ELSE
c		l4  = 'ssfix'    // cid (1:1)
c		l5  = 'gir'      // cid (1:1)
c		l6  = 'lir'      // cid (1:1)
c		l7  = 'ratfix'   // cid (1:1)
c		l8  = 'addfix'   // cid (1:1)
c		l9  = 'ratlfix'  // cid (1:1)
c		l10 = 'addlfix'  // cid (1:1)
c		l11 = 'ratlgfix' // cid (1:1)
c		l12 = 'addlgfix' // cid (1:1)
c	    END IF
C
c	    CALL DMPIMGI  ( smg, bnam, 'r', l5, iret )
c	    CALL DMPIMGI  ( sml, bnam, 'r', l6, iret )
c	    CALL DMPIMGI  ( sms, bnam, 'r', l4, iret )

C     Otherwise there is no smoothing and the arrays need to be
C     copied to the smoother arrays.
           
        ELSE
           DO  j = 1, LT25
              DO  i = 1, LN25
                 smg (i, j) = cutg25 (i, j)
                 sml (i, j) = cuml25 (i, j)
                 sms (i, j) = ss25   (i, j+16)
              END DO
           END DO
        END IF
      
C     Compute the ratios for the data requested, smooth-fill the 
C     missing areas (missings first have to be set to RMISS), and 
C     dump out the results.

	CALL MONCOR  ( smg, sms, LN25, LT25, a, b, iret )
	CALL MONCOR  ( sml, sms, LN25, LT25, c, d, iret )
	DO  j = 1, LT25
	DO  i = 1, LN25
	    IF  ( a (i, j) .LT. 0. )  THEN
		a (i, j) = RMISS
		b (i, j) = RMISS
	    END IF
	    IF  ( c (i, j) .LT. 0. )  THEN
		c (i, j) = RMISS
		d (i, j) = RMISS
	    END IF
	END DO
	END DO


cc	npass = -1
cc	CALL FILLSM  ( a, LN25, LT25, LN25, 4, npass, a, iret )
cc	CALL FILLSM  ( b, LN25, LT25, LN25, 4, npass, b, iret )
cc	npass = -1
cc	CALL FILLSM  ( c, LN25, LT25, LN25, 4, npass, c, iret )
cc	CALL FILLSM  ( d, LN25, LT25, LN25, 4, npass, d, iret )


c	CALL DMPIMGI ( b, bnam, 'r', l7,  iret )
c	CALL DMPIMGI ( a, bnam, 'r', l8,  iret )
c	CALL DMPIMGI ( d, bnam, 'r', l9,  iret )
c	CALL DMPIMGI ( c, bnam, 'r', l10, iret )

C	Compute the SSM/I-adjusted geo-GPI and leo-GPI separately 
C	(although the leo doesn't actually get used), then
C	calibrate the leo to the geo where it exists and smoothfill the
C	rest (the missings have to be set to RMISS to get smoothfill to
C	work).  Finally, compute a monthly AGPI that's a combination of
C	the geo and leo fill-in (the subset from when there was no
C	geo at a particular time).  At the same time, compute the
C	number of 2.5x2.5 images that the accumulation of geo and leo
C	1x1 images represents (6.25 1x1 for each 2.5x2.5).

	DO  j = 1, LT25
           DO  i = 1, LN25
              IF  ( cumg25 (i, j) .GE. 0. )  THEN
                 adjg (i, j) = a (i, j) + b (i, j) * cumg25 (i, j)
	      ELSE
                 adjg (i, j) = RMISS
              END IF
              IF  ( cuml25 (i, j) .GE. 0. )  THEN
                 adjl (i, j) = c (i, j) + d (i, j) * cuml25 (i, j)
	      ELSE
                 adjl (i, j) = RMISS
              END IF
           END DO
	END DO

c	CALL DMPIMGI  ( adjg, bnam, 'r', 'adjg', iret )
c	CALL DMPIMGI  ( adjl, bnam, 'r', 'adjl', iret )

	CALL SMOOTH  ( nsm, smth, 3., adjl, LN25, LT25, sml, iret )
	CALL SMOOTH  ( nsm, smth, 3., adjg, LN25, LT25, smg, iret )
	CALL MONCOR  ( sml, smg, LN25, LT25, e, f, iret )
	DO  j = 1, LT25
           DO  i = 1, LN25
              IF  ( e (i, j) .LT. 0. )  THEN
                 e (i, j) = RMISS
                 f (i, j) = RMISS
              END IF
           END DO
	END DO
	npass = -1
	CALL FILLSM  ( e, LN25, LT25, LN25, 4, npass, e, iret )
	CALL FILLSM  ( f, LN25, LT25, LN25, 4, npass, f, iret )
c	CALL DMPIMGI  ( f, bnam, 'r', l11, iret )
c	CALL DMPIMGI  ( e, bnam, 'r', l12, iret )

	DO  j = 1, LT25
	DO  i = 1, LN25
	    IF  ( filll25 (i, j) .GE. 0. )  THEN
		adjl (i, j) = e (i, j) + f (i, j) * 
     +			      (c (i, j) + d (i, j) * filll25 (i, j) )
		adj  (i, j) = ( cumgn25  (i, j) / maxg * adjg (i, j) +
     +				fillln25 (i, j) / maxl * adjl (i, j) ) /
     +			      ( cumgn25  (i, j) / maxg +
     +				fillln25 (i, j) / maxl )
	      ELSE
		adjl (i, j) = RMISS
		adj  (i, j) = adjg (i, j)
	    END IF
	    adjn (i, j) = ( cumgn25 (i, j) + fillln25 (i, j) ) / 6.25
	END DO
	END DO

C	Write out adjusted fields, with or without 'fix'.

        do strlen = 1,len_trim(bnam)
           if (bnam(strlen:strlen) .eq. ' ') then
              continue
           end if
        end do
        strlen=strlen-1

        fname=bnam(1:strlen)//'.ragpi.fix'
        call Write_Binary_File(fname, adj, LN25, LT25)

        fname=bnam(1:strlen)//'.Iagpi.fix'
        call Write_Binary_File(fname, adjn, LN25, LT25)



c	IF  ( INDEX ( sname, 'fix' ) .EQ. 0 )  THEN
c	    l1 = 'agpi_g'
c	    l2 = 'agpi_l'
c	    l3 = 'agpi'
c	  ELSE
c	    l1 = 'agpi_g.fix'
c	    l2 = 'agpi_l.fix'
c	    l3 = 'agpi.fix'
c	END IF
c	CALL DMPIMGI ( adjg, bnam, 'r', l1,  iret )
c	CALL DMPIMGI ( adjl, bnam, 'r', l2,  iret )
c	CALL DMPIMGI ( adj,  bnam, 'r', l3,  iret )
c	CALL DMPIMGI ( adjn, bnam, 'I', l3,  iret )


cC
cC	Now scale agpi to mm/hr and dump.
cC
c	DO  j = 1, LT25
c	DO  i = 1, LN25
c	    IF  ( adj (i, j) .GT. 0. )  adj (i, j) = adj (i, j) / 24.
c	END DO
c	END DO
c	IF  ( INDEX ( sname, 'fix' ) .EQ. 0 )  THEN
c	    l3 = 'agpi.per_hr'
c	  ELSE
c	    l3 = 'agpi.fix.per_hr'
c	END IF
c	CALL DMPIMGI ( adj,  bnam, 'r', l3,  iret )
C
C	Reached end of requested data.
C
	WRITE (*, *) 'Reached end of requested data.'
C
  500	CONTINUE
C
	STOP








      end program Main_GPCPcdr_m4_corr2






      subroutine Read_Binary_File (fname, data, ilen, jlen, fieldnum)
c     Simple code to read data from a binary file 

      integer*4     inunit, strlen, ilen, jlen, fieldnum, recnum
      real*4        data(ilen,jlen)
      logical       opn
      character*256 fname


c     get length of string
      do strlen = 1,len_trim(fname)
         if (fname(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1

c     get unused unit number
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO

c     open file
      open(inunit,file=fname, IOSTAT=ios,
     &     form='unformatted',access='direct',recl=4*ilen,status='old')
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to open file'
         STOP
      end if

c     read records from file
      do j = 1, jlen
         recnum=j+(fieldnum-1)*jlen
         read(inunit,rec=recnum, IOSTAT=ios) (data(i,j), i=1,ilen)
         if ( ios .NE. 0 ) then
            WRITE(*,*) 'Error in '//fname//':'//'Failed to read file'
            STOP
         end if
      end do

c     close file
      close(inunit, IOSTAT=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to close file'
         STOP
      end if

      end subroutine Read_Binary_File


      subroutine Write_Binary_File(fname, data, ilen, jlen)
c     Simple subroutine to write dat ato a file


      integer*4     inunit, strlen, ilen, jlen
      real*4        data(ilen,jlen)
      logical       opn
      character*256 fname

c     get length of string
      do strlen = 1,len_trim(fname)
         if (fname(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1

c     get unused unit number
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO

c     open file
      open(inunit,file=fname, IOSTAT=ios, STATUS="replace",
     &     form='unformatted',access='direct',recl=4*ilen*jlen)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to open file'
         STOP
      end if

c     write records tofile
      write(inunit,rec=1, IOSTAT=ios) data
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to write file'
         STOP
      end if

c     close file
      close(inunit, IOSTAT=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to close file'
         STOP
      end if


      end subroutine Write_Binary_File
