      SUBROUTINE GET_IRTB ( iname,  nir, iir, ihr,   jyrir, jdayir,
     +     jminir, ltb, lza, igeop, leop,  iret )
C************************************************************************
C* GET_IRTB  Get the next IR time in a list				*
C*									*
C* This subroutine gets the next 1x1-deg 3-hrly IR data (TB histogram,	*
C* zenith angle, original ZA-corr. GEO-IR GPI, LEO-IR GPI) from		*
C* CPC-format files, either by taking the next time in the current	*
C* file or opening a new file, as needed.  GET_IRTB manages		*
C* incrementing file names and 3-hour index values.  Alternatively,	*
C* [iir=0, ihr one of 1-8] will read that ihr from the first (or only)	*
C* file in iname.  Any subsequent reads would be the consecutively	*
C* following 3-hour fields.		 				*
C*									*
C* GET_IRTB ( iname, nir,  iir, ihr, jyrir, jdayir, jminir, ltb, lza,	*
C*            igeop, leop, iret )					*
C*									*
C* Input parameters:							*
C*   iname	CHAR*	File names of IR (up to 910)			*
C*   nir	INT	Number of IR file names				*
C*									*
C* Input parameters modified on output:					*
C*   iir	INT	File name index (0 = open first file)		*
C*   ihr	INT	Last-read 3-hrly field index (except actual	*
C*			value on opening first file, but 0 is set to 1)	*
C*   jyrir	INT	4-digit year					*
C*   jdayir	INT	Julian day					*
C*   jminir	INT	Minutes into day				*
C*									*
C* Output parameters:							*
C*   ltb	INT*2	24-class IR Tb histogram (fraction * 10000)	*
C*   lza	INT*2	Zenith angle (* 100)				*
C*   igeop	INT*2	Original ZA-corr. GEO-IR GPI (* 100)		*
C*   leop	INT*2	LEO-IR GPI (* 100)				*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = bad file name			*
C*				    -2 = can't find YYMMDD in name	*
C*				    -3 = bad read of data		*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 6/98						*
C* G.Huffman/SSAI	 9/98	Add igeop				*
C* G.Huffman/SSAI	 2/99	Seek .Z or .gz version if file not found*
C* G.Huffman/SSAI	 3/99	Pull out COMPOPTS.F			*
C* G.Huffman/SSAi	 3/01	256-char strings; INQUIRE iunit; move	*
C*				ihr=1, add ihr.EQ.0 test to allow 1-time*
C*				read of any hour index			*
C* D.Bolvin/SSAI	 3/08	Compatibility with latest compopts.f    *
C************************************************************************
      IMPLICIT	NONE
      INTEGER*4	iunit, ipid,  l_n,  l_un, iyy, idate, iostat,
     +     i, j,  k,     irec, l_unname
c      LOGICAL*1	is_c,  is_gz, 
      logical*1 in_use
      CHARACTER*256	unname
C     
      INTEGER*4	LN, LT, LBIN, MAXNAM
      PARAMETER	( LN     = 360,
     +     LT     = 80,
     +     LBIN   = 24,
     +     MAXNAM = 910 )
C     
      INTEGER*4	nir, iir, ihr, jyrir, jdayir, jminir, iret
      INTEGER*2	ltb   (LN, LT, LBIN), lza  (LN, LT),
     +     igeop (LN, LT),       leop (LN, LT)
      CHARACTER*256	iname (MAXNAM)
C     
      CHARACTER*10	digits
      DATA		digits / '0123456789' /
      
      
C     Handle "all done", then update indices and minutes-into-
C     the-day in the usual case, or close an old file (except
C     the first time) set the local variable holding the current
C     file name (so that it can be changed if we have to find the
C     compressed or gzip'ed version), and open a new one.
C     
      iret = 0
      IF  ( iir .EQ. nir .AND. ihr .EQ. 8 )  THEN
         iir = 0
         ihr = 0
c         WRITE (*, *) 'Done with all IR files'
         RETURN
      END IF
      IF  ( iir .NE. 0 .AND. ihr .LT. 8 )  THEN
         ihr    = ihr + 1
         jminir = ( ihr - 1 ) * 180
c        WRITE (*, *) 'Start IR period', ihr
      ELSE
         IF  ( iir .NE. 0 )  THEN
            CLOSE ( iunit )
            
            do l_unname = 1,len_trim(unname)
               if (unname(l_unname:l_unname) .eq. ' ') then
                  continue
               end if
            end do
            l_unname=l_unname-1
            
c            IF  ( ( is_c .OR. is_gz ) .AND.
c     +           ( iname (iir) (1:l_unname) .NE.
c     +           unname      (1:l_unname) ) )
c     +           CALL SYSTEM ('rm ' // unname (1:l_unname) )
            ihr = 1
         ELSE
            IF  ( ihr .EQ. 0 )  ihr = 1
         END IF
C     
C     Find a free FORTRAN unit number and increment the filename
C     index.
C     
         in_use = .TRUE.
         iunit  = 2
         DO  WHILE ( in_use )
            iunit = iunit + 1
            INQUIRE ( iunit, OPENED=in_use )
         END DO
         iir = iir + 1
C     
C     The test between iname and unname above makes very sure
C     the data file is not accidentally deleted.  Now, check
C     on compression status, uncompress (unzip) if needed,
C     and open the data file.
C     
         do l_n = 1,len_trim(iname(iir))
            if (iname(iir)(l_n:l_n) .eq. ' ') then
               continue
            end if
         end do
         l_n=l_n-1

c         CALL COMPOPTS ( iname (iir), l_un, is_c, is_gz, unname,
c     +        iret )
c         IF  ( iret .NE. 0 )  RETURN


c     Assume that files are already uncompressed for now
c     Simply pass current IR filename into unname
         unname=iname(iir)

         do l_unname = 1,len_trim(unname)
            if (unname(l_unname:l_unname) .eq. ' ') then
               continue
            end if
         end do
         l_unname=l_unname-1
         
         WRITE (*, *) 'Open ', unname (1:l_unname)
c         WRITE (*, *) '(orig.file ', iname (iir) (1:l_n), ')'
         OPEN ( iunit,              FILE=unname,
     +        FORM='UNFORMATTED', STATUS='OLD',
     +        ACCESS='DIRECT',    RECL=4*14400,
     +        IOSTAT=iostat,
     +        CONVERT='BIG_ENDIAN')
         if (iostat .NE. 0) then
            WRITE (*, *) 'Error', iostat, ' opening ',
     +           unname (1:l_unname)
            iret = -1
            RETURN
         end if

C     Set date/time information from file name, which is assumed
C     to end with YYMMDD (and maybe .Z or .gz).  Convert each
C     digit, handle year 2000, and get Julian day.  Minutes should
C     always be zero.
C     

         idate = 0
         iyy   = 0
         DO  i = l_n-5, l_n
            j = INDEX ( digits, iname (iir) (i:i) )
            IF  ( j .EQ. 0 )  THEN
               WRITE (*, *) 'Error: Can''t convert ',
     +              iname (iir) (i:i)
               iret = -2
               RETURN
            END IF
            idate = idate * 10 + j - 1
            IF  ( i .LE. l_un-4 )  iyy = iyy * 10 + j - 1
         END DO
C     
         IF  ( iyy .GT. 50 )  THEN
            jyrir = 1900 + iyy
         ELSE
            jyrir = 2000 + iyy
         END IF
         CALL JULDAY ( idate, jdayir )
         jminir = 0
      END IF
C     
C     The actual read of the data.  There are 30 fields total:
C     1-24 are the class fractions warmest-to-coldest (*TENTHOUSAND)
C     25 is mean temp (*TEN)
C     26 is standard deviation (*TEN)
C     27 is polar gpi (*HUNDRED)
C     28 is gpi, zenith angle corrected (*HUNDRED)
C     29 is zenith angle (*TEN)
C     30 is sat-id
C     1 = GMS
C     2 = GOES WEST
C     3 = GOES EAST
C     4 = METEOSAT
C     5 = INSAT
C     6 = NOAA-12 6:40 AM
C     7 = NOAA-12 6:40 PM
C     8 = NOAA-14 2:10 PM
C     9 = NOAA-14 2:10 AM
C     
      DO  k = 1, LBIN
         irec = ( ihr - 1 ) * 30 + k
         READ ( iunit, REC=irec, IOSTAT=iostat )
     +        ( ( ltb (i, j, k), i = 1, LN ), j = 1, LT )
         if (iostat.NE.0) then
            WRITE (*, *) 'Error ', iostat, ' reading record', irec,
     +           ' of IR file ', iname (iir) (1:l_n)
            iret = -3
         endif

      END DO
      irec = ( ihr - 1 ) * 30 + 27
      READ ( iunit, REC=irec,IOSTAT=iostat )
     +     ( ( leop  (i, j), i = 1, LN ), j = 1, LT )
      if (iostat.NE.0) then
         WRITE (*, *) 'Error ', iostat, ' reading record', irec,
     +        ' of IR file ', iname (iir) (1:l_n)
         iret = -3
      endif

      irec = ( ihr - 1 ) * 30 + 28
      READ ( iunit, REC=irec, IOSTAT=iostat )
     +     ( ( igeop (i, j), i = 1, LN ), j = 1, LT )
      if (iostat.NE.0) then
         WRITE (*, *) 'Error ', iostat, ' reading record', irec,
     +           ' of IR file ', iname (iir) (1:l_n)
         iret = -3
      endif

      irec = ( ihr - 1 ) * 30 + 29
      READ ( iunit, REC=irec, IOSTAT=iostat )
     +     ( ( lza   (i, j), i = 1, LN ), j = 1, LT )
      if (iostat.NE.0) then
         WRITE (*, *) 'Error ', iostat, ' reading record', irec,
     +        ' of IR file ', iname (iir) (1:l_n)
         iret = -3
      endif


      RETURN
      END
      
