	SUBROUTINE MONCOR  ( sumic, sumsc, LN, LT, a, b, iret )
C************************************************************************
C* MONCOR  Correct IR to SSM/I						*
C*									*
C* Compute the ratio of the SSM/I to the IR, controlling the value to	*
C* (.02,2), and tighter to (.02,1) in low-precipitation areas.  When the*
C* ratio is adjusted down (i.e., can't fully boost IR to SSM/I) compute	*
C* a positive additive correction to IR.  Don't generate a negative	*
C* additive correction when the ratio is adjusted up (i.e., SSM/I is	*
C* very low compared to IR).  This should only be an issue for low rain	*
C* rates.								*
C*									*
C* MONCOR  ( sumic, sumsc, LN, LT, a, b, iret )				*
C*									*
C* Input parameters:							*
C*   sumic	REAL*	Low-res IR rain in mm/d (size LN,LT)		*
C*   sumsc	REAL*	Low-res SSM/I center in mm/d (size LN,LT)	*
C*									*
C* Output parameters:							*
C*   LN		INT	X dimension					*
C*   LT		INT	Y dimension					*
C*   a		REAL*	Additive correction to IR (size LN,LT)		*
C*   b		REAL*	Ratio of SSM/I to IR (size LN,LT)		*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/USRA	 2/92	Split from program CORR, add avgper,	*
C*				add smoothing				*
C* G.Huffman/USRA	 6/92	Always do rain rate, area		*
C* G.Huffman/USRA	12/92	Add global rain, #i, #?, #miss;		*
C*				use previously accumulated sums		*
C* G.Huffman/USRA	 1/93	Careful on "msg"; dims to SMOOTH	*
C* G.Huffman/USRA	 3/93	Add number of images			*
C* G.Huffman/SSAI	10/94	Split avging to subroutine SSAVG	*
C* G.Huffman/SSAI	 2/95	Introduce a; rename corrl to b		*
C* G.Huffman/SSAI	 4/95	Limit a to 50, taper over (100,150)	*
C* G.Huffman/SSAI	 9/98	Change mm/hr to mm/d; pass in LN,LT	*
C* G.Huffman/SSAI	 2/99	Change lower limit of rat .2->.02	*
C* G.Huffman/SSAI	 9/02	Byte size on declarations		*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i, j,  npass
	REAL*4		clipu, clipl, clipa
C
C	Args are (ln,lt).
C
	REAL*4		sumic (LN, LT), sumsc (LN, LT),
     +			a     (LN, LT), b     (LN, LT)
	INTEGER*4	LN, LT, iret
C
C	Do the ratio.  Start by checking for "no SSM/I data".
C
	DO  j = 1, LT
	DO  i = 1, LN
	    IF  ( sumsc (i, j) .LT. 0. )  THEN
		a (i, j) = -.2
		b (i, j) = -.2
C
C	      Check for "no IR data".
C
	      ELSE
	    	IF  ( sumic (i, j) .LT. 0. )  THEN
		    a (i, j) = -.1
		    b (i, j) = -.1
C
C		  Check for both fields zero, a case not handled below.
C
		  ELSE IF  ( sumsc (i, j) .EQ. 0. .AND.
     +			     sumic (i, j) .EQ. 0. )  THEN
		    a (i, j) = 0.
		    b (i, j) = 1.
C
C		  Set the clipping range.  The limits grow from [.02,1]
C		  at an average rainrate of zero to [.02,2] at an
C		  average rainrate of 1.7 mm/d (50 mm/mo).
C		     clipu = ( sumsc + sumic ) / 2 * ( 1 / 1.7 ) + 1
C
		  ELSE
		    clipu = ( sumsc (i, j) + sumic (i, j) ) / 3.4 + 1.
		    IF  ( clipu .LT. 2. )  THEN
			clipl = .02
		      ELSE
			clipu = 2.
			clipl = .02
		    END IF
C
C		    Clip the ratio to the range [clipl,clipu].  This 
C		    catches all cases of "divide by zero" except both 
C		    fields zero.  When b is clipped down, solve for a 
C		    in the formula
C		        sumsc = a + b * sumic
C		    and clip a to 1.7 for avg R <3.4, tapered to 0 for
C		    avg r >5 (50, 100, 150 mm/mo, respectively).
C
		    IF  ( sumsc (i, j) .LT. 
     +			  sumic (i, j) * clipl )  THEN
			a (i, j) = 0.
			b (i, j) = clipl
		      ELSE IF  ( sumsc (i, j) .GT. 
     +				 sumic (i, j) * clipu )  THEN
			clipa    = 5. - .5 * ( sumsc (i, j) + 
     +					       sumic (i, j) )
			IF  ( clipa .GT. 1.7 )  THEN
			    clipa = 1.7
			  ELSE  IF ( clipa .LT. 0. )  THEN
			    clipa = 0.
			END IF
			a (i, j) = AMIN1 ( clipa, sumsc (i, j) - 
     +					          sumic (i, j) * clipu )
			b (i, j) = clipu
		      ELSE
			a (i, j) = 0.
			b (i, j) = sumsc (i, j) / sumic (i, j)
		    END IF
		END IF
C
	    END IF
	END DO
	END DO
C
	RETURN
	END
