	SUBROUTINE REGRID  ( LNA, LTA, LNB, LTB, a, b, iret)
C************************************************************************
C* REGRID  Regrid a field from 2.5 to 1 deg				*
C*									*
C* This subroutine regrids a real field from 2.5x2.5 deg to 1x1 deg.	*
C* it is assumed that the upper left corners for the first box coincide	*
C* in the two fields.							*
C*									*
C* REGRID  ( LNA, LTA, LNB, LTB, a, b, iret)				*
C*									*
C* Input parameters:							*
C*   LNA	INT	X dimension of a				*
C*   LTA	INT	Y dimension of a				*
C*   LNB	INT	X dimension of b				*
C*   LTB	INT	y dimension of b				*
C*   a		REAL*	2.5x2.5 deg array (size LNA,LTA)		*
C*									*
C* Output parameters:							*
C*   b		REAL*	1x1 deg array (size LNB,LTB)			*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 3/98	Adapted from REGRIDT1 (Bolvin)		*
C* G.Huffman/SSAI	 3/06	Last 2 INTEGER are *4			*
C* Sapiano/UMD           11/12  Removed output commands                 *
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i, j, i1, i2, j1, j2, k, l, icount
     	REAL*4		accum
C
	INTEGER*4	LNA, LTA, LNB,    LTB, iret
	REAL*4		a (LNA, LTA),     b (LNB, LTB)
C
	REAL*4		dx, dy, RMISS
	DATA		dx, dy, RMISS / 2.5, 2.5, -99999. /
C
c	INTEGER*4	INITW_DISK, WRITE_DISK, EXIT_DISK
	INTEGER*4	iunit, nbyte
C
C	Check that we don't try to reach past the edge of the input.
C
	i2 = INT ((LNB * 0.999999) / dx) + 1
	j2 = INT ((LTB * 0.999999) / dy) + 1
	IF  ( i2 .GT. LNA .OR. j2 .GT. LTA )  THEN
	    WRITE (*, *) 'Error: input array too small; size implied ',
     +			 'by output array is', i2, ' by', j2
	    iret = -1
	    RETURN
	END IF
C
C	1-deg boxes that lie totally within a 2.5-deg box are simply
C	assigned the value of that 2.5-deg box.  1-deg boxes that 
C	straddle the boundaries of 2 or 4 2.5-deg boxes get the average 
C	over those 2.5-deg boxes.  Any 2.5-deg box that has no data 
C	(.LT. 0) is given no weight in the averages.  1-deg boxes which 
C	contain no valid data are set to RMISS.
C
	DO  j = 1, LTB
	    j1 = INT (((j - 1.0) * 1.000001) / dy) + 1
	    j2 = INT ((j * 0.999999) / dy) + 1
C
	    DO  i = 1, LNB
	    	i1 = INT (((i - 1.0) * 1.000001) / dx) + 1
	    	i2 = INT ((i * 0.999999) / dx) + 1
C
C		If the 1-deg box lies completely within a 2.5-deg box,
C		assign the 2.5-deg value.  If the 1-deg box straddles
C		the north-south boundaries of 2 2.5-deg boxes, cpmpute
C		the averages.  Exclude any missing 2.5-deg boxes from 
C		the computation. 
C
	    	IF  (i1 .EQ. i2) THEN
	    	    IF  (j1 .EQ. j2) THEN
	    	    	IF  ( a (i1, j1) .GE. 0. )  THEN
	    	    	    b (i, j) = a (i1, j1)
	    	    	  ELSE
	    	    	    b (i, j) = RMISS
	    	    	END IF
	    	      ELSE
	    	    	IF  ( a (i1, j1) .GE. 0. .AND.
     +			      a (i1, j2) .GE. 0. )  THEN
	    	    	    b (i, j) = ( a (i1, j1) + 
     +					 a (i1, j2) ) * 0.5
     			  ELSE IF ( a (i1, j1) .GE. 0. .AND.
     +				    a (i1, j2) .LT. 0. )  THEN 		
     			    b (i, j) = a (i1, j1)
     			  ELSE IF ( a (i1, j1) .LT. 0. .AND.
     +				    a (i1, j2) .GE. 0. )  THEN
     			    b (i, j) = a (i1, j2)
     			  ELSE
     			    b (i, j) = RMISS
     			END IF
     		    END IF
C
C		  The 1-deg box straddles the east-west boundaries of 2 
C		  2.5-deg boxes, so compute the averages.  Exclude any 
C		  missing 2.5-deg boxes from the computation.  If the 
C		  1-deg box also straddles the north-south boundaries of 
C		  2 2.5-deg boxes (meaning the 1-deg box straddles a 
C		  total of 4 2.5-deg boxes), compute the averages.  
C		  Again, exclude any missing 2.5-deg boxes.
C
     		  ELSE
     		    IF  (j1 .EQ. j2) THEN
     		    	IF  ( a (i1, j1) .GE. 0. .AND.
     +    	    	      a (i2, j1) .GE. 0. )  THEN
     		    	    b (i, j) = ( a (i1, j1) + 
     +					 a (i2, j1) ) * 0.5
     			  ELSE IF ( a (i1, j1) .GE. 0. .AND.
     +				    a (i2, j1) .LT. 0. )  THEN
     			    b (i, j) = a (i1, j1)
     			  ELSE IF ( a (i1, j1) .LT. 0. .AND.
     +				    a (i2, j1) .GE. 0. )  THEN
     			    b (i, j) = a (i2, j1)
     			ELSE
     			    b (i, j) = RMISS
     			END IF
     		      ELSE
     		    	accum  = 0.
     		    	icount = 0
     		    	DO  l = j1, j2
     		    	DO  k = i1, i2
     		    	    IF  ( a (k, l) .GE. 0. )  THEN
     		    	        accum  = accum  + a (k, l)
     		    	    	icount = icount + 1
     		    	    END IF
     		    	END DO
     		    	END DO
     		    	IF  ( icount .GT. 0 )  THEN
     		    	    b (i, j) = accum / FLOAT ( icount )
			  ELSE
     		    	    b (i ,j) = RMISS
     		    	END IF
     		    END IF
C
     		END IF
	    END DO
	END DO
C
c	iunit = INITW_DISK ( 'regrid_a' )
c	IF  ( iunit .LE. 0 )  THEN
c	    WRITE (*, *) 'Error', iunit, ' opening regrid_a'
c	    STOP
c	END IF
c	nbyte = WRITE_DISK (iunit, a, LNA * LTA * 4 )
c	IF  ( nbyte .LT. LNA * LTA * 4 )  THEN
c	    WRITE (*, *)  'Error: short file in regrid_a'
c	    STOP
c	END IF
c	iret = EXIT_DISK ( iunit )
c	IF  ( iret .LT. 0 )  THEN
c	    WRITE (*, *)  'Error', iret, ' in closing regrid_a'
c	    STOP
c	END IF
c	iunit = INITW_DISK ( 'regrid_b' )
c	IF  ( iunit .LE. 0 )  THEN
c	    WRITE (*, *) 'Error', iunit, ' opening regrid_b'
c	    STOP
c	END IF
c	nbyte = WRITE_DISK (iunit, b, LNB * LTB * 4 )
c	IF  ( nbyte .LT. LNB * LTB * 4 )  THEN
c	    WRITE (*, *)  'Error: short file in regrid_b'
c	    STOP
c	END IF
c	iret = EXIT_DISK ( iunit )
c	IF  ( iret .LT. 0 )  THEN
c	    WRITE (*, *)  'Error', iret, ' in closing regrid_b'
c	    STOP
c	END IF
C
	iret = 0
	RETURN
	END
