      program Main_GPCPcdr_m4_corr_adj
c     Code to compute AGPI from SSMI and polar orbiter IR
c     This code is a mix of three separate original code elements
c     each of which is now a separate subroutine called by this code:
c     mk_rssfix - simply trims rfix to 50S/N
c     corr - Compute correlation of SSM/I and polar orbiter IR 
c     adj - Compute adjustment of geo-IR by a ratio


      IMPLICIT	NONE
      
      INTEGER*4	    i, j
      REAL*4        opi(144,72), ssmi_rss(144,40)
      character*2   cmonth
      character*4   cyear
      character*256 fname, InputPATH, InterimPATH
      character*256 irPATH, irlist

      CHARACTER*256 snam, inam, anam, bnam, cnam
      INTEGER*4     inst, istrtm, istopm, ismth


      integer*4     ssmi_flag, ir_flag, cut_flag
      integer*4     Ninputs, ios, strlen, inunit, l_ip
      logical       opn
      integer*4     iyear, imonth





c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 4 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_corr_adj:'//
     &        ' Incorrect number of input arguments (should be 4)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, cyear)
      call getarg(2, cmonth)
      call getarg(3, InputPATH)
      call getarg(4, InterimPATH)

c     Convert month to a number
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth





c*******************************************************************
c     Set flags for processing
c     ssmi_flag: 1=pre-ssmi; 2=early ssmi
c     ir_flag:   1=m_geo; 2=m_geo3; 3=m_geo3-da
      ssmi_flag=0
      ir_flag=0

c     pre-ssmi: 1986, 1987 01-07, 12
      if ( (iyear  .EQ. 1986) .OR. 
     &     ( (iyear .EQ. 1987) .AND. (imonth .LE. 7) ) .OR.
     &     ( (iyear .EQ. 1987) .AND. (imonth .EQ. 12) ) ) then
         ssmi_flag=1
         ir_flag=1

c     early ssmi: 1987 08,09,10,11; 1991-1996
      else if ( ( (iyear .EQ. 1987) .AND. 
     &        (imonth .GE. 8) .AND. (imonth .LE. 11) ) .OR.
     &        ( (iyear .GE. 1988) .AND. (iyear .LE. 1996) ) ) then
         ssmi_flag=2
         ir_flag=2

c     Do not process any data outside this range CORR/ADJ is 
c     used for pre SSMI and SSMI before 1996 
      else
         WRITE(*,*) 'Error in GPCPcdr_m4_corr_adj:'//
     &        'This code is for 1986-1996 only'
         STOP
      end if

c     Extra step for ir_flag: use m_geo3 for 1988-1990
      if ( (iyear  .GE. 1988) .AND. (iyear  .LE. 1990) ) then
         ir_flag=3
      end if




c*******************************************************************
c     Input files for CORR and ADJ code

c     If this is pre-SSMI, then the input data is OPI, which
c     must be restricted to 50S/N using code based on mk_rssfix
      if ( ssmi_flag .EQ. 1) then

         call String_length(InputPATH,strlen)
         fname=InputPATH(1:strlen)//'OPI_le/opi.'
     &        //cyear//cmonth//'.rfix'
         call Read_Binary_File(fname,144,72,1,opi)

         do i = 1, 144
	    do j = 1, 40
               ssmi_rss(i,j) = opi(i,j + 16)
	    end do
         end do

         call String_length(InterimPATH,strlen)
         snam=InterimPATH(1:strlen)//'OPI_le_rssfix/opi.'
     &        //cyear//cmonth//'.rssfix'
         call Write_Binary_File(snam, ssmi_rss, 144, 40)

c        Set anam and bnam for ADJ
         anam=InterimPATH(1:strlen)//'CORR_ADJ/opi.'
     &        //cyear//cmonth//'.radfix3.me'
         bnam=InterimPATH(1:strlen)//'CORR_ADJ/opi.'
     &        //cyear//cmonth//'.rcofix3.me'

c        Set ismth to to box3 smoothinng
         ismth=3

c     If this is early-SSM/I, the input is already between 50S/N
c     and can use file produced by XOVER
      else if ( ssmi_flag .EQ. 2) then
         call String_length(InterimPATH,strlen)
         snam=InterimPATH(1:strlen)//'XOVER/cgt.'
     &        //cyear//cmonth//'.rssfix'

c        Set anam and bnam for ADJ
         anam=InterimPATH(1:strlen)//'CORR_ADJ/cgt.'
     &        //cyear//cmonth//'.radfix3.me'
         bnam=InterimPATH(1:strlen)//'CORR_ADJ/cgt.'
     &        //cyear//cmonth//'.rcofix3.me'

c        Set ismth to to box3 smoothinng
         ismth=3

      end if


c     Set inam and inst (CORR only - this is reset for ADJ)
c     inam is the name of the IR data and inst is a type specifier
      call String_length(InputPATH,strlen)
      if ( ir_flag .EQ. 1 ) then
         inam=InputPATH(1:strlen)//'/mergIR_annual_files/m_geo.'
     +        //cyear(3:4)//''
         inst=6
      else if ( ir_flag .EQ. 2 ) then
         inam=InputPATH(1:strlen)//'/mergIR_annual_files/m_geo3-da.'
     +        //cyear(3:4)//''
         inst=14
      else if ( ir_flag .EQ. 3 ) then
         inam=InputPATH(1:strlen)//'/mergIR_annual_files/m_geo3.'
     +        //cyear(3:4)//''
         inst=14
      end if


c     Set cutfile for CORR
c     F08: 1987 08,09,10,11; 1988-1991
      if ( (iyear  .GE. 1987) .AND. (iyear  .LE. 1991) ) then
         cnam='GPCPcdr_m4/GPCPcdr_m4_CORR_ADJ/'
     &        //'Ancillary/geo_cuts.f08'

c     F11: 1992 - 1995/05
      else if ( ( (iyear  .GE. 1992) .AND. (iyear  .LE. 1994) ) .OR.
     &        (iyear  .EQ. 1995) .AND. (imonth  .LE. 5) ) then
         cnam='GPCPcdr_m4/GPCPcdr_m4_CORR_ADJ/'
     &        //'Ancillary/geo_cuts.f11'
         
c     F13: 1995/06-12 1996
      else if ( (iyear  .EQ. 1995) .AND. (imonth  .GE. 6) .OR.
     &        (iyear  .EQ. 1996) ) then
         cnam='GPCPcdr_m4/GPCPcdr_m4_CORR_ADJ/'
     &        //'Ancillary/geo_cuts.f13.1996'
      else
         cnam=''        
      end if








c*******************************************************************
c     CORR
c     snam = name of SSM/I or OPI files
c     inam = IR filename
C     inst = Instrument id; 1 = 'n10',     2 = 'n11',
C                           3 = 'geo',     4 = 'geo-n10-n11',
C                           5 = 'geo-leo', 6 = 'all'
C                           7 = 'n10flux' 14 = '3hr-geo'
C     cnam = Cut file name, only applicable for 3hr-geo;
C            file contains (x,y,node)=(144,32,a|d) entries
C            of time index (1-8) to be cut from 3hr array
C            and accumulated in output array
C     ismth: level of smoothing for boxcar smoother (0 = no smoothing)
      call corr( snam, inam, inst, cnam, ismth,
     &     cyear, cmonth, InterimPATH )





c*******************************************************************
c     Additional settings for ADJ

c     Set inam and inst for ADJ
      inam=InputPATH(1:strlen)//'/mergIR_annual_files/m_geo.'
     +     //cyear(3:4)
      inst=6
c     Turn smoothinng off for ADJ
      ismth=0



c     ADJ
C     anam: A (additive constant) data file name
C     bnam: B (ratio) data file name
C     inam: IR data file name
C     inst: Instrument id; 1 = 'n10',     2 = 'n11',
C                          3 = 'geo',     4 = 'geo-n10-n11',
C                          5 = 'geo-leo', 6 = 'all'
C                         14 = '3hr-geo'
C     ismth: level of smoothing for boxcar smoother (0 = no smoothing)
      call adj( anam, bnam, inam, inst, ismth, 
     &     cyear, cmonth, InterimPATH )







      end program Main_GPCPcdr_m4_corr_adj




