      subroutine adj ( anam, bnam, inam, inst, ismth,
     &     cyear, cmonth, InterimPATH )
c     This subroutine is based on the ADJ program
c
C************************************************************************
C* ADJ  Compute adjustment of geo-IR by a ratio				*
C*									*
C* This program reads geo IR files and computes corrected fields using	*
C* ratios previously computed from SSM/I and matching IR.  The user	*
C* can specify to compute GEO IR averages without doing the adjustment 	*
C* calculation.								*
C*									*
C* adj  RATIO IR PLATFORM ILIMITS START STOP STAT			*
C*									*
C* Input command-line arguments:					*
C*   AFILE	CHAR	A (additive constant) data filename;		*
C*			'-' = default ('add.198708.radfix3.me')		*
C*   BFILE	CHAR	B (ratio) data filename; '-' = default		*
C*			('add.198708.rcofix3.me')			*
C*   IR		CHAR	IR data filename; '-' = default			*
C*			('/usr/rain1/huffman/m_geoza.87')		* 
C*   PLATFORM	CHAR	IR platform as SAT,DO_ZCOR, where SAT is 'n10',	*
C*			'n11', 'geo', 'geo-n10-n11', 'geo-leo', 'all',	*
C*			'n10flux', '3hr-geo', and DO_ZCOR is t|f for	*
C*			do|omit zenith-angle correction on geo;		*
C*			'-' = default ('geo,t')				*
C*   ILIMITG	CHAR	GEO IR bin boundary limits; possible inputs:	*
C*			- a filename holding reals, one to a line	*
C*			- '-' = defaults (1,2,...,MAXBIN-1)		*
C*   ILIMITL	CHAR	LEO IR bin boundary limits; possible inputs:	*
C*			- a filename holding reals, one to a line	*
C*			- '-' = defaults (1,2,...,MAXBIN-1)		*
C*   START	CHAR	Start period; pentad as pYYYYPP, month as	*
C*			mYYYYMM, '-' = default ('m198708')		*
C*   STOP	CHAR	Stop period; pentad as pYYYYPP, month as	*
C*			mYYYYMM, '-' = default ('m198708')		*
C*   AVGTIME	CHAR	Averaging in time; me, ma, p = exact		*
C*			month, approx. month, pentad (no avg'ing),	*
C*			'-' = default ('ma')				*
C*   SMOOTH     CHAR    Smoothing type; possible inputs are:            *
C*                      - SN, where S is one of 'box' boxcar, 'clip'    *
C*                      boxcar with .5 on corners, 'gauss' gaussian	*
C*                      with .5 at diameter N, and N is the smoother    *
C*                      size (NxN) (odd, in the range 3,MXSM)           *
C*                      - a filename holding reals, one to a line       *
C*                      - 'none' for no smoothing                       *
C*			- '-' for the default ('none')			*
C*   STATS	CHAR	Statistic to be computed; r, a are rain rate,	*
C*			rain area, '-' = default ('r')			*
C**									*
C* Log:									*
C* G.Huffman/USRA	12/91	Adapt CORR				*
C* G.Huffman/USRA	 1/92	Split up INCLUDE; allow G & L bins	*
C* G.Huffman/USRA	 1/93	Move (new) SMOOTH, ISTAT outside IPREP	*
C* G.Huffman/USRA	 7/93	Sum number of images (from getic)	*
C* G.Huffman/SSAI	12/94	Add zenith-angle correction		*
C* G.Huffman/SSAI	 2/95	Add additive correction; corrl->b	*
C* G.Huffman/SSAI	 9/95	Update call to SMOOTH			*
C* G.Huffman/SSAI	 5/00	Upgrade to IRIX6, YYYY			*
C* G.Huffman/SSAI	 6/00	Add INSAT to zcor			*
C* G.Huffman/SSAI	 9/02	Dump unsmoothed rain before smoothing;	*
C*				byte size on declarations		*
C************************************************************************
      IMPLICIT	NONE

      INTEGER*4	iret, i, j, k, n, m, ii, jj,
     +     iyyyy,     ipp,  imm,  l_bn,  nstrt, nstop
      CHARACTER	bnamg*256, cid*8
C

c	INCLUDE		'param_2.5.h'
      REAL*4     RMISS,  DLAT,   DLONC,  BNDRY
      INTEGER*4  MXLTC, MXLNC, MAXBIN, MXLTNS
      PARAMETER ( RMISS  = -99999.,
     +     DLAT   = 2.5,
     +     DLONC  = 2.5,
     +     BNDRY  = 50.,
     +     MXLTC  = ( 2 * BNDRY ) / DLAT,
     +     MXLTNS = ( 90. - BNDRY ) / DLAT,
     +     MXLNC  = 360 / DLONC,
     +     MAXBIN = 32)
      
C     Month boundary data.  For each month, the information is:
C     start pentad, days of start in month, stop pentad, days of
C     stop in month, number of days in month.
C     
      INTEGER*4 mbounds (5, 12)
      DATA      mbounds /  1, 5,  7, 1, 31,   7, 4, 12, 4, 28,
     +     12, 1, 18, 5, 31,  19, 5, 24, 5, 30,
     +     25, 5, 31, 1, 31,  31, 4, 37, 1, 30,
     +     37, 4, 43, 2, 31,  43, 3, 49, 3, 31,
     +     49, 2, 55, 3, 30,  55, 2, 61, 4, 31,
     +     61, 1, 67, 4, 30,  67, 1, 73, 5, 31 /
      
      
c     INCLUDE 'param.corr.h'
      INTEGER*4 MXSM
      PARAMETER ( MXSM   = 15 )
      
      
      CHARACTER*256 anam,    bnam,   inam, cnam
      CHARACTER*8   avgper
      REAL*4        binlimg (MAXBIN-1), binliml (MAXBIN-1),
     +     smth (MXSM, MXSM)
	INTEGER*4   n_blg,   n_bll,  inst,   iyyyypp, 
     +     iyyyymm, istat,  ismth,
     +     istrtp,  istrtm, istopp, istopm
	LOGICAL     do_zcor, old
C     
C	kssmi, xssmi args are (ln,lt,type), where type is 1,2,3,4 for
C	raining, "good", not missing, all (this order because xssmi
C	doesn't have types 3,4).  sumk args are (ln,lt,type), where
C	type follows the XNMRATGOCEH input order.
C
	REAL*4	a     (MXLNC, MXLTC),
     +       b     (MXLNC, MXLTC),
     +       sumic (MXLNC, MXLTC),
     +       sumnc (MXLNC, MXLTC),
     +       getic (MXLNC, MXLTC, 3),
     +       crrct (MXLNC, MXLTC),
     +       zcor  (MXLNC, MXLTC, 6), dum
	INTEGER*4  ni    (MXLNC, MXLTC),
     +       ns    (MXLNC, MXLTC)

      integer iyear, imonth, Ninputs, i1, j1
      character*4 cyear
      character*2 cmonth
      character*256 InputPATH, InterimPATH, fname
      integer*4     ios, inunit, strlen
      logical       opn




c     ******************************************************
c     Setup some parameters
      
c     Convert month to a number
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth


      print *, ismth
C     smth: Smoothing array (size MXSM,MXSM)
C     Set to 3 for Pre-SSMI, 0 o/w
      IF (ismth .GT. 0) THEN
         DO  j = 1, ismth
            DO  i = 1, ismth
               smth (i, j) = 1.
            END DO
         END DO
      ELSE
         smth=0
      END IF


c     set output basename
      call String_Length(bnam, strlen)
      i = INDEX ( bnam (1:strlen), '.'//cyear//cmonth )
      call String_Length(InterimPATH, strlen)
      bnamg = InterimPATH(1:strlen)//'CORR_ADJ/'//bnam(i-3:i+6)

c     Set parameters
c     These used to be set in corrarg based on the input file
c     These are the parameters that do not change, so they are hardcoded
c     Elements that get changed for each processing period will 
c     be fed in via file

C     do_zcor: T|F = do|omit zenith-angle correction on geo
      do_zcor=.TRUE.

C     binlimg & n_blg: GEO IR bin limits (size MAXBIN-1) and number of limits
      data binlimg / 269.5, 265.5, 260.5, 255.5, 250.5, 
     +     245.5, 240.5, 235.5, 230.5, 225.5,
     +     220.5, 215.5, 210.5, 200.5, 190.5,
     +     16*RMISS /
      n_blg=15

C     binliml & n_bll: LEO IR bin limits (size MAXBIN-1) and number limits
      data binliml / 235.5, 219.5, 29*RMISS /
      n_bll=2
      
C     avgper: Averaging period
c     'p', 'me', 'ma' = pentad,  month exact, month approximate
      avgper='ma'

C     istat: Statistics to be computed; 1, 2 = rain rate, rain area
      istat=1

c     Set start and stop pentads and months
      istrtp=iyear*100+mbounds(1, imonth)
      istopp=iyear*100+mbounds(3, imonth)
      istrtm=iyear*100+imonth
      istopm=istrtm
  








c*********************************************************C
C	Generate a basename from the ratio name and current pentad
C	or month for the corrected field, and try to shorten the name 
C	for the geo.
C
c	CALL ST_LSTR ( bnam, l_bn, iret )
c	IF  ( INDEX  ( avgper, 'p' ) .NE. 0 )  THEN
c	    WRITE ( cid, 5 ) iyyyypp
c    5	    FORMAT ( '.', I6.6 )
c	  ELSE
c	    WRITE ( cid, 5 ) iyyyymm
c	END IF
c	bnamr = bnam (1:l_bn) // cid (1:7)
c	IF  ( bnam (l_bn-2 :l_bn-2 ) .EQ. '.' .AND. 
c     +	      bnam (l_bn-10:l_bn-10) .EQ. '.' .AND.
c     +	      bnam (l_bn-15:l_bn-15) .EQ. '.' )  THEN
c	    bnamg = bnam (1:l_bn-16) // cid (1:7)
c	  ELSE IF  ( bnam (l_bn-2 :l_bn-2 ) .EQ. '.' .AND. 
c     +		     bnam (l_bn-10:l_bn-10) .EQ. '.' .AND.
c     +		     bnam (l_bn-17:l_bn-17) .EQ. '.' )  THEN
c	    bnamg = bnam (1:l_bn-18) // cid (1:7)
c	  ELSE
c	    bnamg = bnamr
c	END IF









C     Set up some index variables for the loop and loop through the
C     calibration requested (and available).
C     
      old     = .FALSE.
      iyyyy   = istrtp / 100
      ipp     = MOD ( istrtp, 100 )
      imm     = MOD ( istrtm, 100 )
      iyyyypp = istrtp
      iyyyymm = istrtm




C     Get zenith angle correction
      IF  ( do_zcor )  CALL SET_ZC ( iyyyy, ipp, zcor, iret )
C     
      
      
c     Start main while loop (lasts for whole function)
      DO  WHILE ( iyyyypp .LE. istopp )
         
         
         
         
cC     The averaging period is a pentad (no further averaging
cC     needed).
c           IF  ( INDEX  ( avgper, 'p' ) .NE. 0 )  THEN
c              WRITE (*, *) 'Starting pentad', iyyyypp
c              IF  ( anam .NE. 'none' )  THEN
c                 CALL RPREP  ( anam, iyyyypp, 'all', a, iret )
c                 IF  ( iret .LT. 0 )  THEN
c                    WRITE (*, *) 'No more additive correction data ',
c     +                   'in pentad', iyyyypp
c                    GO TO 500
c                 END IF
c              ELSE
c                 DO  j = 1, MXLTC
c                    DO  i = 1, MXLNC
c                       a (i, j) = 0.
c                    END DO
c                 END DO
c              END IF
c              IF  ( bnam .NE. 'none' )  THEN
c                 CALL RPREP  ( bnam, iyyyypp, 'all', b, iret )
c                 IF  ( iret .LT. 0 )  THEN
c                    WRITE (*, *) 'No more ratio data ',
c     +                   'in pentad', iyyyypp
c                    GO TO 500
c                 END IF
c              ELSE
c                 DO  j = 1, MXLTC
c                    DO  i = 1, MXLNC
c                       b (i, j) = 1.
c                    END DO
c                 END DO
c              END IF
c              CALL IPREP  ( inam,    inst,    iyyyypp,  
c     +             binlimg, n_blg,   binliml, n_bll,
c     +             ' ',     do_zcor, zcor,    getic, iret )
c              IF  ( iret .NE. 0 )  THEN
c                 WRITE (*, *) 'No more GEO/merge IR data in pentad',
c     +                iyyyypp
c                 GO TO 500
c              END IF
cC     
cC     Load up sumic and sumnc; IPREP sends rain rate, rain
cC     area, and number of images so sumic and sumnc can't be 
cC     loaded directly.
cC     
c              DO  j = 1, MXLTC
c                 DO  i = 1, MXLNC
c                    sumic (i, j) = getic (i, j, istat)
c                    sumnc (i, j) = getic (i, j, 3    )
c                 END DO
c              END DO
cC     
cC     Dump out the original rain (or coverage) field.
cC     
c              IF  ( istat .EQ. 1 )  THEN
c                 CALL DMPIMGI  ( sumic, bnamg, 'r', 'ge', iret )
c              ELSE
c                 CALL DMPIMGI  ( sumic, bnamg, 'a', 'ge', iret )
c              END IF
cC     
cC     Smooth, if requested.
cC     
c              IF  ( ismth .NE. 0 )  THEN
c                 DO  j = 1, MXLTC
c                    DO  i = 1, MXLNC
c                       IF  ( sumic (i, j) .LT. 0. )
c     +                      sumic (i, j) = RMISS
c                    END DO
c                 END DO
c                 CALL SMOOTH ( ismth, smth,  3.,  sumic, MXLNC,
c     +                MXLTC, sumic, iret )
cC     
cC     Dump out the smoothed rain (or coverage) field.
cC     
c                 IF  ( istat .EQ. 1 )  THEN
c                    CALL DMPIMGI  ( sumic, bnamg, 'r', 'gesm', iret)
c                 ELSE
c                    CALL DMPIMGI  ( sumic, bnamg, 'a', 'gesm', iret)
c                 END IF
c              END IF
C     
c           ELSE

C     The averaging period is a month.  Set up the start and
C     stop fractions of pentads.
     
         IF  ( MOD ( iyyyy, 4 ) .EQ. 0 .AND. ipp .EQ. 12 )  THEN
            nstrt = mbounds (2, imm)
            nstop = mbounds (4, imm) + 1
         ELSE
            nstrt = mbounds (2, imm)
            nstop = mbounds (4, imm)
         END IF
C     
C     The GEO/merge IR is only available in pentads, so we
C     automatically start the average as "approximate".
C     
         WRITE (*, *) 'Starting pentad', iyyyypp
         IF  ( .NOT. old )  THEN
            cnam=' '
            CALL IPREP  ( inam,    inst,    iyyyypp,
     +           binlimg, n_blg,   binliml, n_bll,
     +           cnam,     do_zcor, zcor,    getic, 
     +           iret )
            IF  ( iret .NE. 0 )  THEN
               WRITE (*, *) 'No more GEO/merge IR data in', 
     +              ' pentad', iyyyypp
               GO TO 500
            END IF
         END IF
         DO  j = 1, MXLTC
            DO  i = 1, MXLNC
               sumnc (i, j) = getic (i, j, 3) * nstrt / 5.
               IF  ( getic (i, j, istat) .GE. 0. )  THEN
                  sumic (i, j) = getic (i, j, istat) * nstrt
                  ni    (i, j) = nstrt
               ELSE
                  sumic (i, j) = 0.
                  ni    (i, j) = 0
               END IF
            END DO
         END DO
c     debug
c	j = 24
c	do i = 74, 79
c	write (*, *) 'j=20, i, sumic, getic, sumnc, getic3, nstrt, ni', 
c     +		     i, sumic (i, j), getic (i, j, istat), sumnc (i, j),
c     +		     getic (i, j, 3), nstrt, ni (i, j)
c	end do
cdebug
C
C		The additive and ratio corrections are already averaged 
C		up for the month.  Only a single retrieval is needed here.
C

         call Read_Binary_File(anam,144,40,1,a)
         call Read_Binary_File(bnam,144,40,1,b)

c         IF  ( anam .NE. 'none' )  THEN
c            CALL RPREP  ( anam, iyyyymm, 'all', a, iret )
c            IF  ( iret .LT. 0 )  THEN
c               WRITE (*, *) 'No more additive correction data ',
c     +              'in month', iyyyymm
c               GO TO 500
c            END IF
c         ELSE
c            DO  j = 1, MXLTC
c               DO  i = 1, MXLNC
c                  a (i, j) = 0.
c               END DO
c            END DO
c         END IF
c         IF  ( bnam .NE. 'none' )  THEN
c            CALL RPREP  ( bnam, iyyyymm, 'all', b, iret )
c            IF  ( iret .LT. 0 )  THEN
c               WRITE (*, *) 'No more ratio data ',
c     +              'in month', iyyyymm
c               GO TO 500
c            END IF
c         ELSE
c            DO  j = 1, MXLTC
c               DO  i = 1, MXLNC
c                  b (i, j) = 1.
c               END DO
c            END DO
c         END IF
C     
C     Add in the middle pentads of GEO/merge IR data.
C     
         DO  ipp = mbounds (1, imm) + 1, mbounds (3, imm) - 1
            iyyyypp = iyyyy * 100 + ipp
            WRITE (*, *) 'Starting pentad', iyyyypp
            cnam=' '
            CALL IPREP  ( inam,    inst,    iyyyypp,
     +           binlimg, n_blg,   binliml, n_bll,
     +           cnam,     do_zcor, zcor,    getic, 
     +           iret )
            IF  ( iret .NE. 0 )  THEN
               WRITE (*, *) 'No more GEO/merge IR data in',
     +              ' pentad', iyyyypp
               GO TO 500
            END IF
            DO  j = 1, MXLTC
               DO  i = 1, MXLNC
                  sumnc (i, j) = sumnc (i, j) + getic (i, j, 3)
                  IF  ( getic (i, j, istat) .GE. 0. )  THEN
                     sumic (i, j) = sumic (i, j) +
     +                    getic (i, j, istat) * 5.
                     ni    (i, j) = ni    (i, j) + 5
                  END IF
               END DO
            END DO
c     debug
c	j = 24
c	do i = 74, 79
c	write (*, *) 'j=20, i, sumic, getic, sumnc, getic3, nstrt, ni', 
c     +		     i, sumic (i, j), getic (i, j, istat), sumnc (i, j),
c     +		     getic (i, j, 3), ' 5', ni (i, j)
c	end do
cdebug
         END DO

  
C     Finish the month average:  The GEO/merge IR is only
C     available in pentads, so we automatically end the
C     average as "approximate".
C     
         ipp     = mbounds (3, imm)
         iyyyypp = iyyyy * 100 + ipp
         WRITE (*, *) 'Starting pentad', iyyyypp
         cnam=' '
         CALL IPREP  ( inam,    inst,    iyyyypp,
     +        binlimg, n_blg,   binliml, n_bll,
     +        cnam,     do_zcor, zcor,    getic, iret )
         IF  ( iret .NE. 0 )  THEN
            WRITE (*, *) 'No more GEO/merge IR data in pentad',
     +           iyyyypp
            GO TO 500
         END IF
         DO  j = 1, MXLTC
            DO  i = 1, MXLNC
               sumnc (i, j) = sumnc (i, j) + 
     +              ( getic (i, j, 3) * nstop / 5. )
               IF  ( getic (i, j, istat) .GE. 0. )  THEN
                  sumic (i, j) = ( sumic (i, j) + 
     +                 getic (i, j, istat) * nstop ) /
     +                 ( ni (i, j) + nstop )
               ELSE IF  ( ni (i, j) .GT. 0 )  THEN
                  sumic (i, j) = sumic (i, j) / ni (i, j)
               ELSE
                  sumic (i, j) = -.1
               END IF
            END DO
         END DO
ccdebug
c         j = 24
c         do i = 74, 79
c            write (*, *) 'j=20,i, sumic, getic, sumnc, getic3,nstop,ni', 
c     +           i, sumic (i, j), getic (i, j, istat), sumnc (i, j),
c     +           getic (i, j, 3), nstop, ni (i, j)
c         end do
ccdebug
C
C     Dump out the original rain (or coverage) field.
C     


         call String_Length(bnamg, strlen)
         fname=bnamg(1:strlen)//'.rge'
         call Write_Binary_File(fname, sumic, 144, 40)
         write (*,*) 'Image file ', fname(1:strlen+4), ' written.'

c         IF  ( istat .EQ. 1 )  THEN
c            CALL DMPIMGI  ( sumic, bnamg, 'r', 'ge', iret )
c         ELSE
c            CALL DMPIMGI  ( sumic, bnamg, 'a', 'ge', iret )
c         END IF
C     
C     Smooth, if requested.
C     
         IF  ( ismth .NE. 0 )  THEN
            DO  j = 1, MXLTC
               DO  i = 1, MXLNC
                  IF  ( sumic (i, j) .LT. 0. )
     +                 sumic (i, j) = RMISS
               END DO
            END DO
            CALL SMOOTH ( ismth, smth,  3., sumic, MXLNC,
     +           MXLTC, sumic, iret )
C     
C     Dump out the smoothed rain (or coverage) field.

            call String_Length(bnamg, strlen)
            fname=bnamg(1:strlen)//'.rgesm'
            call Write_Binary_File(fname, sumic, 144, 40)
            write (*,*) 'Image file ', fname(1:strlen+6), ' written.'

c            IF  ( istat .EQ. 1 )  THEN
c               CALL DMPIMGI  ( sumic, bnamg, 'r', 'gesm', iret)
c            ELSE
c               CALL DMPIMGI  ( sumic, bnamg, 'a', 'gesm', iret)
c            END IF
         END IF
         
c     End of if loop for pentad data
c     END IF
C     
C     For all averaging periods, if the data are o.k.,
C     do the correction.
C     
         IF  ( anam .NE. 'none' .OR. bnam .NE. 'none' )  THEN
            DO  j = 1, MXLTC
               DO  i = 1, MXLNC
                  IF  ( a (i, j) .LT. 0. .OR. b (i, j) .LT. 0. .OR.
     +                 sumic (i, j) .LT. 0. )  THEN
                     crrct (i, j) = -.1
                  ELSE
                     crrct (i, j) = a (i, j) + 
     +                    b (i, j) * sumic (i, j)
                  END IF
               END DO
            END DO
         END IF
C     
C     Dump out the corrected rain (or coverage) and number of 
C     samples fields.
C     
         IF  ( anam .NE. 'none' .OR. bnam .NE. 'none' )  THEN
c            IF  ( istat .EQ. 1 )  THEN
c               CALL DMPIMGI  ( crrct, bnamr, 'r', 'cr', iret )
c            ELSE
c               CALL DMPIMGI  ( crrct, bnamr, 'a', 'cr', iret )
c            END IF

            call String_Length(bnamg, strlen)
            fname=bnamg(1:strlen)//'.rcr'
            call Write_Binary_File(fname, crrct, 144, 40)
            write (*,*) 'Image file ', fname(1:strlen+4), ' written.'
        END IF

c        CALL DMPIMGI  ( sumnc, bnamg, 'I', 'ge', iret )
        call String_Length(bnamg, strlen)
        fname=bnamg(1:strlen)//'.Ige'
        call Write_Binary_File(fname, sumnc, 144, 40)
        write (*,*) 'Image file ', fname(1:strlen+4), ' written.'

C     
C     Increment counters for the next interval.
C     
         IF  ( ipp .EQ. 73 )  THEN
            iyyyy = iyyyy + 1
            imm   = 1
            ipp   = 1
         ELSE
            IF  ( ipp .GT. mbounds (3, imm) )  imm = imm + 1
            ipp = ipp + 1
         END IF
         iyyyymm = iyyyy * 100 + imm
         iyyyypp = iyyyy * 100 + ipp
         
         
         
c     End of main while loop
      END DO
C     
C     Reached end of requested data.
C     
      WRITE (*, *) 'Reached end of requested data.'
C     
 500  CONTINUE
C     
      
c     debug
c     j = 24
c     do i = 74, 79
c     write (*, *) 'j=20, i, sumic, sumnc, istat', i, sumic (i, j), 
c     +		     sumnc (i, j), istat
c     end do
cdebug


      END
      
