      subroutine corr( snam, inam, inst, cnam, ismth,
     &     cyear, cmonth, InterimPATH)
c     This subroutine is based on the original corr.f program.
c
C************************************************************************
C* CORR  Compute correlation of SSM/I and polar orbiter IR		*
C*									*
C* This program reads SSMI and matching IR files and computes the	*
C* ratio field.  If the data are in pentad histogram files, they are	*
C* accumulated to the specified period. It is not possible to check	*
C* that pre-processed rain images have compatible creation parameters:	*
C* ==> It is the user's responsibility to guarantee consistency <==	*
C*									*
C* corr  SSMI DIRECTION SLIMITS IR PLATFORM ILIMITS CUTFILE START STOP	*
C*       AVGTIME SMOOTH RATE STATS NOTMATCH				*
C*									*
C* Input command-line arguments:					*
C*   SSMI	CHAR	SSMI data source as SNAM,STYP, where SNAM is	*
C*			either a datafile basename (.yyyypp will be	*
C*			appended) or the name of a low-res SSM/I image	*
C*			file, and STYP is 'd', 'r', or 'a', for data	*
C*			file, rain rate image, or areal coverage image,	*
C*			respectively; '-' = default ('wed,d')		* 
C*   DIRECTION	CHAR	Orbit direction; where a, d, m are ascending,	*
C*			descending, merged a & d; '-' = default ('m')	*
C*   SLIMITS	CHAR	SSMI bin boundary limits; possible inputs are:	*
C*			- a filename holding reals, one to a line	*
C*			- '-' = defaults (1,2,...,MAXBIN-1)		*
C*   IR		CHAR	IR data source as INAM,ITYP, where INAM is	*
C*			either a data file name or the name of a	*
C*			low-res IR image file, and ITYP is 'd', 'r', or	*
C*			'a', for data file, rain rate image, or areal	*
C*			coverage image, respectively; '-' = default	*
C*			('/usr/data3/huffman/olr_histogram_noaa10,d')	* 
C*   PLATFORM	CHAR	IR platform as SAT,DO_ZCOR, where SAT is 'n10',	*
C*			'n11', 'geo', 'geo-n10-n11', 'geo-leo', 'all',	*
C*			'n10flux', '3hr-geo', and DO_ZCOR is t|f for	*
C*			do|omit zenith-angle correction on geo;		*
C*			'-' = default ('3hr-geo,t')			*
C*   ILIMITS	CHAR	IR bin boundary limits; possible inputs are:	*
C*			- a filename holding reals, one to a line	*
C*			- '-' = defaults (1,2,...,MAXBIN-1)		*
C*   CUTFILE	CHAR	Cut file name, only applicable for 3hr-geo;	*
C*			file contains (x,y,node)=(144,32,a|d) entries	*
C*			of time index (1-8) to be cut from 3hr arrays	*
C*			and accumulated in output array;		*
C*			if cnam=' ', def = 1|5 for entire a|d arrays;	*
C*			'-' = default ('geo_cuts.f08')			*
C*   START	CHAR	Start period; pentad as pYYYYPP, month as	*
C*			mYYYYMM, '-' = default ('m198906')		*
C*   STOP	CHAR	Stop period; pentad as pYYYYPP, month as	*
C*			mYYYYMM, '-' = default ('m198906')		*
C*   AVGTIME	CHAR	Averaging in time; ME, MA, MM, P = exact	*
C*			month, approx. month, matched-pentad month,	*
C*			pentad (no avg'ing), '-' = default ('MA')	*
C*   SMOOTH	CHAR	Smoothing type; possible inputs are:		*
C*			- SN, where S is one of 'box' boxcar, 'clip'	*
C*			boxcar with .5 on corners, 'gauss' gaussian	*
C*			with .5 at diameter N, and N is the smoother	*
C*			size (NxN) (odd, in the range 3,MXSM)		*
C*			- a filename holding reals, one to a line	*
C*			- 'none' for no smoothing			*
C*			- '-' for the default ('box3')			*
C*   RATE	CHAR	Multipliers of "basic" rainrate for ocean,	*
C*			land,coast:					*
C*			- reals separated by ',' (rro,rrl,rrc);		*
C*			  rrl, rrc default to 1, (rro+rrl)/2		*
C*			- '-' = default ('2,1,1.5')			*
C*   EXTRAMATCH CHAR	Number of extra pentads before and after the	*
C*			to be included in 'mm' (N/A otherwise):		*
C*			- integers separated by ',' (ibefore,iafter);	*
C*			  iafter defaults to ibefore			*
C*			- '-' = default ('1,1')				*
C*   NOTMATCH	CHAR	Pentads to be skipped in 'mm'; possible inputs:	*
C*			- a filename holding integers, one to a line	*
C*			- 'none' for no not-matching			*
C*			- '-' = default ('notmatch')			*
C**									*
C* Log:									*
C* G.Huffman/USRA	12/91	Adapt SHOW, SHOWI			*
C* G.Huffman/USRA	 1/92	Split up INCLUDE; accomodate  fix	*
C* G.Huffman/USRA	 2/92	Make SSM/I high-res; do var. check;	*
C*				change exact to even avg.; smoothing	*
C* G.Huffman/USRA	 3/92	Change variance editing limit, 		*
C*				add MM to AVGTIME			*
C* G.Huffman/USRA	 5/92	Vary rainrate coeff. depending on sfc	*
C* G.Huffman/USRA	 6/92	Do mismatch skipping correctly;		*
C*				Do N10 flux; always do rate,area	*
C* G.Huffman/USRA	11/92	Add #ice,#?, #miss			*
C* G.Huffman/USRA	12/92	Comment out variance editing; accum.	*
C*				for MONCOR in ADDIN			*
C* G.Huffman/USRA	 1/93	Be more careful about handling missing,	*
C*				use ',' in RATE,EXTRAMATCH		*
C* G.Huffman/USRA	 3/93	New variance				*
C* G.Huffman/SSAI	 8/94	Add 3hr-geo (and so cut files); pass in	*
C*				binlimi and n_bli for geo in IPREP	*
C* G.Huffman/SSAI	10/94	Allow pre-processed rain images		*
C* G.Huffman/SSAI	12/94	Add zenith-angle correction		*
C* G.Huffman/SSAI	 1/95	Add additive IR correction		*
C* G.Huffman/SSAI	 9/95	Update call to SMOOTH; fix equiv.images	*
C* D.Bolvin/SSAI	10/97	Screen and count low number of geo-IR   *
C*				images for each pentad			*
C* D.Bolvin/SSAI	12/97	Add separate obs count screen for "all"	*
C*				option					*
C* G.Huffman/SSAI	 5/00	Upgrade to IRIX6, YYYYY; dim. on SMTH,	*
C*				pass LN,LT to MONCOR			*
C* G.Huffman/SSAI	 6/00	Add INSAT to zcor			*
C* G.Huffman/SSAI	 7/00	Smoothfill SSM/I before MONCOR		*
C* G.Huffman/SSAI	 2/11	Explicit type sizes; 256-byte names	*
C************************************************************************

      IMPLICIT	NONE

      INTEGER*4	iret,  i, j,  k,    n, m, ii,    jj,
     +     iasc,  iyyyy, ipp,  imm,  l_nam, idum,
     +     nstrt, nstop, nday, ippl, iunit, nin,  npass
      REAL*4		days,  ed
      CHARACTER*256	bnam,  cid,   l1,   l2,   l3,   l4,  l5, l6, l7,
     +     l8
C     
c     INCLUDE		'param_.5.h'
c     INCLUDE		'param.corr.h'
        
c     some varibales from param_.5.h and param.corr.h
        
      REAL*4     RMISS
      real*4   DLAT,   DLONC,  DLONNS, BNDRY
c     INTEGER*4	IVERSB, IVERSH, IVERSD, IVERSR, MAXDEX, MXSTAT
      integer*4  MXLTC,  MXLTNS, MXLNC,  MXLNNS, MAXBIN

      PARAMETER	( RMISS  = -99999.,
c     +			  IVERSB = 9311, 
c     +			  IVERSH = 5, 
c     +			  IVERSD = 9311, 
c     +			  IVERSR = 9311, 
     +     DLAT   = 0.5,
     +     DLONC  = 0.5,
     +     DLONNS = 1.,
     +     BNDRY  = 50.,
     +     MXLTC  = ( 2 * BNDRY ) / DLAT,
     +     MXLTNS = ( 90. - BNDRY ) / DLAT,
     +     MXLNC  = 360 / DLONC,
     +     MXLNNS = 360 / DLONNS,
     +     MAXBIN = 32 )
c     +			  MAXDEX = 20,
c     +			  MXSTAT = 20 )
C
C     Month boundary data.  For each month, the information is:
C     start pentad, days of start in month, stop pentad, days of
C     stop in month, number of days in month.
C
      INTEGER*4	mbounds (5, 12)
      DATA      mbounds /  1, 5,  7, 1, 31,   7, 4, 12, 4, 28,
     +     12, 1, 18, 5, 31,  19, 5, 24, 5, 30,
     +     25, 5, 31, 1, 31,  31, 4, 37, 1, 30,
     +     37, 4, 43, 2, 31,  43, 3, 49, 3, 31,
     +     49, 2, 55, 3, 30,  55, 2, 61, 4, 31,
     +     61, 1, 67, 4, 30,  67, 1, 73, 5, 31 /

      INTEGER*4	MXSM,  MXLNCL, MXLTCL, MXLNL, MXLTL, MAXNOT, 
     +     MINIMG, MINIMA
c	REAL*4		DLONL, DLATL
      PARAMETER	( MXSM   = 15,
     +     MXLNCL = MXLNC / 5,
     +     MXLTCL = MXLTC / 5,
     +     MXLNL  = MXLNC / 5,
     +     MXLTL  = ( 2*MXLTNS + MXLTC ) / 5,
     +     MAXNOT = 100,
     +     MINIMG = 15,
     +     MINIMA = 1 )
c     +			  DLONL  = DLONC * 5.,
c     +			  DLATL  = DLAT  * 5. )





C
      CHARACTER*256 snam,   inam,       cnam
      CHARACTER*8   avgper
      CHARACTER*1   stype,  itype
      REAL*4        binlims (MAXBIN-1), binlimi (MAXBIN-1), 
     +     smth (MXSM, MXSM),  rro,    rrl,     rrc
      INTEGER*4	n_bls,  n_bli,      inst,   iyyyypp, iyyyymm,
     +     ismth,  istrtp,     istrtm, istopp,  istopm,
     +     notmat (MAXNOT),    n_nm,   npitch,  MINIM
      LOGICAL*1	skpend, do_zcor
C     
c      INTEGER*4	INIT_DISK, READ_DISK, EXIT_DISK
c      INTEGER*4	INITW_DISK, WRITE_DISK, iiret, inin, iiunit
C     
C	Args are (ln,lt).
C
      REAL*4 xs    (MXLNNS, MXLTNS, 2), 
     +     xxs   (MXLNNS, MXLTNS),
     +     gets  (MXLNNS, MXLTNS, 3),
     +     avgs  (MXLNNS, MXLTNS, 5),
     +     eqvs  (MXLNNS, MXLTNS),
     +     sigs  (MXLNNS, MXLTNS), s_as  (MXLNNS, MXLTNS),
     +     xn    (MXLNNS, MXLTNS, 2), 
     +     xxn   (MXLNNS, MXLTNS),
cc     +	xxn, xxs, xxc,
cc
     +     getn  (MXLNNS, MXLTNS, 3),
     +     avgn  (MXLNNS, MXLTNS, 5),
     +     eqvn  (MXLNNS, MXLTNS),
     +     sign  (MXLNNS, MXLTNS), s_an  (MXLNNS, MXLTNS),
     +     xc    (MXLNC,  MXLTC,  2),
     +     xxc   (MXLNC,  MXLTC),
     +     getc  (MXLNC,  MXLTC,  3),
     +     avgc  (MXLNC,  MXLTC,  5),
     +     eqvc  (MXLNC,  MXLTC),
     +     sigc  (MXLNC,  MXLTC),  s_ac  (MXLNC,  MXLTC),
     +     zcor  (MXLNC,  MXLTC,  6), dum
      REAL*4  sumsc (MXLNCL, MXLTCL, 2),
     +     sums  (MXLNL,  MXLTL,  2),
     +     sumi  (MXLNL,  MXLTL),
     +     sumq  (MXLNL,  MXLTL),
     +     summ  (MXLNL,  MXLTL),
     +     sumim (MXLNL,  MXLTL),
     +     sumic (MXLNCL, MXLTCL, 3),
     +     getic (MXLNCL, MXLTCL, 3),
     +     a     (MXLNCL, MXLTCL, 2),
     +     b     (MXLNCL, MXLTCL, 2),
     +     tmpmat(MXLNCL, MXLTCL)
      INTEGER ns    (MXLNNS, MXLTNS, 4),
     +     nn    (MXLNNS, MXLTNS, 4),
     +     nc    (MXLNC,  MXLTC,  4),
     +     ni    (MXLNCL, MXLTCL)
      INTEGER*2	ngets (MXLNNS, MXLTNS, 4), 
     +     ngetn (MXLNNS, MXLTNS, 4),
     +     ngetc (MXLNC,  MXLTC,  4)
      INTEGER*4	  INSIZ
      PARAMETER	( INSIZ = 144 * 40 * 4 )
C
      REAL*4      RRB
      PARAMETER	( RRB = 4.188 )
C
      integer iyear, imonth, Ninputs, i1, j1
      character*4 cyear
      character*2 cmonth
      character*256 InterimPATH, fname
      integer*4     ios, inunit, strlen
      logical       opn
      integer*4 notmat_tmp


c     ******************************************************
c     Setup some parameters

      
c     Convert month to a number
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth

c     Set parameters
c     These used to be set in corrarg based on the input file
c     These are the parameters that do not change, so they are hardcoded
c     Elements that get changed for each processing period will 
c     be fed in via file


c     stype: SSMI data type, as 'd', 'r', 'a' for data file
c     rain rate image, or areal coverage image
      stype='r'
C     iasc: Orbit direction; 1-3 stand for asc, desc, merged A & D
C     'm'=3
      iasc=3


C     binlims & n_bls: SSMI bin limits (size MAXBIN-1) and number of limits
	data binlims /327.5, 326.5, 325.5, 324.5, 323.5, 322.5,
     +                289.5, 279.5, 269.5, 259.5, 254.5, 252.5,
     +                250.5, 248.5, 247.5, 246.5, 245.5, 243.5,
     +                241.5, 239.5, 234.5, 229.5, 219.5, 209.5,
     +                199.5, 179.5, 159.5, 139.5, 119.5,  99.5,
     +                49.5 /
      n_bls=MAXBIN-1

C     itype: IR data type, as 'd', 'r', 'a' for data file,
C               rain rate image, or areal coverage image
      itype='d'

C     do_zcor: T|F = do|omit zenith-angle correction on geo
      do_zcor=.TRUE.

C     binlimi & n_bli: IR bin limits (size MAXBIN-1) and number of limits
c     If Instrument id (variable inst) is n10, n11, n10flux these should be 
c     set to the LEO version with 3 categories, otherwise, it should be set
c     to the GEO version with 16 categories
c     GEO version:
      data binlimi / 269.5, 265.5, 260.5, 255.5, 250.5, 
     +        245.5, 240.5, 235.5, 230.5, 225.5,
     +        220.5, 215.5, 210.5, 200.5, 190.5,
     +                 16*RMISS /
      n_bli=15

c     LEO Version (inst = n10, n11 or n10flux): 
c      data binlimi /235.5, 219.5, 29*RMISS/
c      n_bli=2

C     avgper: Averaging period; 'p', 'me', 'ma', 'mm' =
C     pentad, month exact, month approximate, month
C     month matched-pentad
      avgper='me'
      
C     smth: Smoothing array (size MXSM,MXSM)
      IF (ismth .GT. 0) THEN
         DO  j = 1, ismth
            DO  i = 1, ismth
               smth (i, j) = 1.
            END DO
         END DO
      ELSE
         smth=0
      END IF

C     rro: Multiplier / rainrate-divisor for ocean
      rro = 2 / RRB

C     rrl: Multiplier / rainrate-divisor for land
      rrl = 1 / RRB

C     rrc: Multiplier / rainrate-divisor for coast
      rrc = 1.5 / RRB



c     Set start and stop pentads and months
      istrtp=iyear*100+mbounds(1, imonth)
      istopp=iyear*100+mbounds(3, imonth)
      istrtm=iyear*100+imonth
      istopm=istrtm
      



c*********************************************************
c     Read text file notmatch and get pentads that are not matched
C     notmat: Pentads to be skipped for 'mm'
C     n_nm: Number of pentads to be skipped

      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      
      fname='GPCPcdr_m4/GPCPcdr_m4_CORR_ADJ/notmatch'
      open(inunit, file=fname, iostat=ios) 
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in corr.f: Failed to open notmatch file'
         STOP
      end if

      n_nm=0
      do i=1, MAXNOT
         read(inunit, '(i4)', iostat=ios ) notmat_tmp
         if ( ( ios .NE. 0 ) .OR. ( i .EQ. MAXNOT ) ) then
            continue
         end if
         notmat(i)= notmat_tmp
         n_nm=n_nm+1
      end do
 
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in corr.f: Failed to close notmatch file'
         STOP
      end if


c*********************************************************

	



c	IF  ( iret .EQ. -2 )  STOP
c	WRITE (*, *) 'snam, stype, iasc, n_bls =', snam, stype, iasc, 
c     +		     n_bls
c	WRITE (*, *) 'binlims =', binlims
c	WRITE (*, *) 'inam, itype, inst, do_zcor, n_bli =', inam, 
c     +		     itype, inst, do_zcor, n_bli
c	WRITE (*, *) 'binlimi =', binlimi
c	WRITE (*, *) 'cnam,istrtp,istopp,istrtm,istopm,avgper,',
c     +		     'ismth,smth,rro,rrl,rrc =',
c     +		     cnam, istrtp, istopp, istrtm, istopm, avgper, 
c     +		     ismth, smth, rro, rrl, rrc
c	WRITE (*, *) 'notmat =', notmat
c	WRITE (*, *) 'n_nm,iret =', n_nm, iret
C	IF  ( iret .eq. -1 )  STOP


C	Set the minimum number of IR observations needed to perform the
C	matchups.  If using "all" data, set it to MINIMA.  If using 
C	"geo" or "3hr-geo", use MINIMG.  MINIMA was added to allow
C	matchups with the flux data during the non-SSM/I period.
C
      IF  (inst .NE. 6) THEN
         MINIM = MINIMG
      ELSE
         MINIM = MINIMA
      END IF
      
C	Set up some index variables for the loop, set up zenith-angle
C	corrections (if requested), and loop through the
C	calibration requested (and available).
C
      iyyyy   = istrtp / 100
      ipp     = MOD ( istrtp, 100 )
      imm     = MOD ( istrtm, 100 )
      iyyyypp = istrtp
      iyyyymm = istrtm


C     Get zenith angle correction
      IF  ( do_zcor )  CALL SET_ZC ( iyyyy, ipp, zcor, iret )

c     Start main while loop (lasts for whole function)
      DO  WHILE ( iyyyypp .LE. istopp )
C
         npitch = 0
C     
C     For matched months, skip first pentads until a match
C     occurs.
C
         IF  ( INDEX ( avgper, 'mm' ) .NE. 0 )  THEN
            DO  i = 1, MAXNOT
               IF  ( iyyyypp .EQ. notmat (i) )  THEN
                  WRITE (*, *) 'Skipping pentad', iyyyypp
                  GO TO 400
               END IF
            END DO
         END IF



C     The averaging period is a pentad (no further averaging
C     needed).
C
c         IF  ( INDEX  ( avgper, 'p' ) .NE. 0 )  THEN
c            WRITE (*, *) 'Starting pentad', iyyyypp
c            IF  ( itype .EQ. 'd' )  THEN
c               CALL IPREP  ( inam,    inst,    iyyyypp, binlimi, 
c     +              n_bli,   binlimi, n_bli,   cnam,  
c     +              do_zcor, zcor,    sumic,   iret )
c               IF  ( iret .NE. 0 )  THEN
c                  WRITE (*, *) 'No more LEO IR data in pentad',
c     +                 iyyyypp
c                  GO TO 500
c               END IF
c            END IF
C
c            IF  ( stype .EQ. 'd' )  THEN
c               CALL SPREPH  ( snam,  iasc,  iyyyypp, binlims, 
c     +              n_bls, rro,   rrl,     rrc,
c     +              'all', gets,  getc,    getn,
c     +              ngets, ngetc, ngetn,   iret )
c               IF  ( iret .LT. -1 )  THEN
c                  WRITE (*, *) 'No more SSMI data in pentad',
c     +                 iyyyypp
c                  GO TO 500
c               END IF
cC     
c               CALL ADDIN  ( .TRUE.,    avgper, gets,  getc,  getn,  
c     +              ngets,     ngetc,  ngetn,
c     +              nday, xs,  xc, xn, xxs,
c     +              xxc,  xxn, ns, nc, nn,    iret )
ccdebug	i = 533
cc	j = 76
cc	WRITE (*, *) '(533,76)getc1-3,ngetc1,xc1-2,xxc,nc1',
cc     +	(getc (i, j, k), k=1, 3), ngetc (i, j, 1), (xc (i, j, k),
cc     +	k=1, 2), xxc (i, j), nc (i, j, 1)
cC
c               DO  j = 1, MXLTNS
c                  DO  i = 1, MXLNNS
c                     IF  ( ns (i, j, 1) .LT. 2 )  THEN
c                        avgs (i, j, 1) = -.2
c                        avgs (i, j, 2) = -.2
c                        avgs (i, j, 3) = -.2
c                        avgs (i, j, 4) = -.2
c                        IF  ( ns (i, j, 4) .GT. 0 )  THEN
c                           avgs (i, j, 5) = 1.
c                        ELSE
c                           avgs (i, j, 5) = -.2
c                        END IF
c                        sigs (i, j) = -.2
c                     ELSE
c                        avgs (i, j, 1) = gets (i, j, 1)
c                        avgs (i, j, 2) = gets (i, j, 2)
c                        avgs (i, j, 3) = FLOAT ( ns (i, j, 2) ) /
c     +                       FLOAT ( ns (i, j, 1) )
c			    avgs (i, j, 4) = FLOAT ( ns (i, j, 3) ) /
c     +                       FLOAT ( ns (i, j, 1) )
c			    avgs (i, j, 5) = FLOAT ( ns (i, j, 4) ) /
c     +					     FLOAT ( ns (i, j, 1) +
c     +                           ns (i, j, 4) )
c			    sigs (i, j) = (
c     +                           ( DBLE ( ns (i, j, 1) ) * xxs (i, j) - 
c     +                           xs (i, j, 1) * xs (i, j, 1) )     /
c     +                      DBLE ( ns (i, j, 1) * ( ns (i, j, 1) - 1 )))
c                         END IF
C     
c                         IF  ( nn (i, j, 1) .LT. 2 )  THEN
c			    avgn (i, j, 1) = -.2
c			    avgn (i, j, 2) = -.2
c			    avgn (i, j, 3) = -.2
c			    avgn (i, j, 4) = -.2
c			    IF  ( nn (i, j, 4) .GT. 0 )  THEN
c                               avgn (i, j, 5) = 1.
c                            ELSE
c                               avgn (i, j, 5) = -.2
c			    END IF
c			    sign (i, j) = -.2
c                         ELSE
c			    avgn (i, j, 1) = getn (i, j, 1)
c			    avgn (i, j, 2) = getn (i, j, 2)
c			    avgn (i, j, 3) = FLOAT ( nn (i, j, 2) ) /
c     +                           FLOAT ( nn (i, j, 1) )
c			    avgn (i, j, 4) = FLOAT ( nn (i, j, 3) ) /
c     +                           FLOAT ( nn (i, j, 1) )
c			    avgn (i, j, 5) = FLOAT ( nn (i, j, 4) ) /
c     +                           FLOAT ( nn (i, j, 1) +
c     +                           nn (i, j, 4) )
c			    sign (i, j) = (
c     +                           ( DBLE ( nn (i, j, 1) ) * xxn (i, j) -
c     +                           xn (i, j, 1) * xn (i, j, 1) )     /
c     +                      DBLE ( nn (i, j, 1) * ( nn (i, j, 1) - 1 )))
c                         END IF
c                      END DO
c                   END DO
cC
c                   DO  j = 1, MXLTC
c                      DO  i = 1, MXLNC
c                         IF  ( nc (i, j, 1) .LT. 2 )  THEN
c			    avgc (i, j, 1) = -.2
c			    avgc (i, j, 2) = -.2
c			    avgc (i, j, 3) = -.2
c			    avgc (i, j, 4) = -.2
c			    IF  ( nc (i, j, 4) .GT. 0 )  THEN
c                               avgc (i, j, 5) = 1.
c                            ELSE
c                               avgc (i, j, 5) = -.2
c			    END IF
c			    sigc (i, j) = -.2
c			  ELSE
c                             avgc (i, j, 1) = getc (i, j, 1)
c			    avgc (i, j, 2) = getc (i, j, 2)
c			    avgc (i, j, 3) = FLOAT ( nc (i, j, 2) ) /
c     +                           FLOAT ( nc (i, j, 1) )
c			    avgc (i, j, 4) = FLOAT ( nc (i, j, 3) ) /
c     +                           FLOAT ( nc (i, j, 1) )
c			    avgc (i, j, 5) = FLOAT ( nc (i, j, 4) ) /
c     +                           FLOAT ( nc (i, j, 1) +
c     +                           nc (i, j, 4) )
c			    sigc (i, j) = (
c     +                           ( DBLE ( nc (i, j, 1) ) * xxc (i, j) -
c     +                           xc (i, j, 1) * xc (i, j, 1) )     /
c     +                      DBLE ( nc (i, j, 1) * ( nc (i, j, 1) - 1 )))
c			END IF
c                     END DO
c                  END DO
c               END IF
c               WRITE (*, *) 'Done averaging for a pentad.'







C     The averaging period is a month.  Set up the start and
C     stop fractions of pentads.  Pentad 12 spans the Feb-March
C     boundary, and contains Feb 29 when it occurs, so the last
C     pentad in Feb gets an extra day.  Matched pentads get
C     the full boundary pentads.
C     
c            ELSE
         IF  ( MOD ( iyyyy, 4 ) .EQ. 0 .AND. ipp .EQ. 12 )  THEN
            IF  ( INDEX ( avgper, 'mm' ) .EQ. 0 )  THEN
               nstrt = mbounds (2, imm)
               nstop = mbounds (4, imm) + 1
            ELSE
               nstrt = 5
               nstop = 5
            END IF
         ELSE
            IF  ( INDEX ( avgper, 'mm' ) .EQ. 0 )  THEN
               nstrt = mbounds (2, imm)
               nstop = mbounds (4, imm)
            ELSE
               nstrt = 5
               nstop = 5
            END IF
         END IF
C     
C     The IR is only available in pentads, so we
C     automatically start the average as "approximate".
C     
         WRITE (*, *) 'Starting pentad', iyyyypp
         IF  ( itype .EQ. 'd' )  THEN
            CALL IPREP  ( inam,    inst,    iyyyypp, binlimi, 
     +           n_bli,   binlimi, n_bli,   cnam,  
     +           do_zcor, zcor,    getic,   iret )
            IF  ( iret .NE. 0 )  THEN
               WRITE (*, *) 'No more LEO IR data in pentad',
     +              iyyyypp
               GO TO 500
            END IF
            DO  j = 1, MXLTCL
               DO  i = 1, MXLNCL
                  IF  ( getic (i, j, 1) .GE. 0. )  THEN
                     IF  ( getic (i, j, 3) .GE. MINIM )  THEN
                        sumic (i, j, 1) = getic (i, j, 1) * nstrt
                        sumic (i, j, 2) = getic (i, j, 2) * nstrt
                        ni    (i, j)    = nstrt
                     ELSE
                        sumic (i, j, 1) = 0.
                        sumic (i, j, 2) = 0.
                        ni    (i, j)    = 0
                        npitch          = npitch + 1
                     END IF
                  ELSE
                     sumic (i, j, 1) = 0.
                     sumic (i, j, 2) = 0.
                     ni    (i, j) = 0
                  END IF
               END DO
            END DO
         END IF
C     
C     Print the number of pentad data points tossed due to the
C     number of images .LT. MINIM and reset the counter for 
C     the next pentad.
C     
         IF  (npitch .NE. 0)  THEN
            WRITE (*,*) 'WARNING: npitch = ', npitch, ' for > ',
     +           MINIM, ' images in pentad ', iyyyypp
         END IF
C     
         npitch = 0
C     
C     Start the SSM/I "approximate" month or "matched-pentad"
C     month, or start the SSM/I "exact" month (flagged as 
C     'strte').  NOTE:  No check is currently performed to 
C     verify that a sufficient number of SSM/I observations
C     have been accumulated into the pentad to perform the
C     matchups with the 3-hourly geo-IR data.  This *should*
C     be added later.
C     
c         IF  ( stype .EQ. 'd' )  THEN
c            IF  ( INDEX  ( avgper, 'e' ) .EQ. 0 .OR.
c     +           imm .EQ. 1 .OR. imm .EQ. 4 .OR. imm .EQ. 5 )  
c     +           THEN
c               CALL SPREPH  ( snam,    iasc,  iyyyypp,
c     +              binlims, n_bls, rro,    rrl, 
c     +              rrc,     'all',
c     +              gets,    getc,  getn,   ngets,
c     +              ngetc,   ngetn, iret )
c            ELSE
c               CALL SPREPH  ( snam,    iasc,  iyyyypp, 
c     +              binlims, n_bls, rro,    rrl, 
c     +              rrc,     'strte',
c     +              gets,    getc,  getn,   ngets,
c     +              ngetc,   ngetn, iret )
c            END IF 
c            IF  ( iret .LT. -1 )  THEN
c               WRITE (*, *) 'No more SSMI data in pentad', 
c     +              iyyyypp
c               GO TO 500
c            END IF
cC     
c            CALL ADDIN  ( .TRUE., avgper,  gets,    getc,
c     +           getn,   ngets,   ngetc,   ngetn,
c     +           nstrt,  xs, xc,  xn, xxs, xxc, xxn,
c     +           ns, nc, nn, iret )
ccdebug	i = 533
cc	j = 76
cc	WRITE (*, *) '(533,76)getc1-3,ngetc1,xc1-2,xxc,nc1',
cc     +	(getc (i, j, k), k=1, 3), ngetc (i, j, 1), (xc (i, j, k),
cc     +	k=1, 2), xxc (i, j), nc (i, j, 1)
c         END IF
     
C     Increment pentad by one (accounting for possible
C     year boundary).
C     
         IF  ( ipp .EQ. 73 )  THEN
            iyyyy = iyyyy + 1
            ipp   = 1
         ELSE
            ipp = ipp + 1
         END IF
         iyyyypp = iyyyy * 100 + ipp
C     
C     Add in the middle pentads.  The .LT. test on istopp
C     leaves the last pentad for separate code (below).
C     Begin by skipping unmatched pentads for the "matched 
C     month" scheme.
C     
         DO  WHILE  ( iyyyypp .LT. istopp )
            IF  ( INDEX ( avgper, 'mm' ) .NE. 0 )  THEN
               DO  i = 1, MAXNOT
                  IF  ( iyyyypp .EQ. notmat (i) )  THEN
                     WRITE (*, *) 'Skipping pentad', iyyyypp
                     GO TO 300
                  END IF
               END DO
            END IF
C     
            IF  ( MOD ( iyyyy, 4 ) .EQ. 0 .AND. ipp .EQ. 12 ) 
     +           THEN
               nday = 6
            ELSE
               nday = 5
            END IF
C     
            WRITE (*, *) 'Starting pentad', iyyyypp
            IF  ( itype .EQ. 'd' )  THEN
               CALL IPREP  ( inam,    inst,    iyyyypp, 
     +              binlimi, n_bli,   binlimi, n_bli, 
     +              cnam,    do_zcor, zcor,    getic, 
     +              iret )
               IF  ( iret .NE. 0 )  THEN
                  WRITE (*, *) 'No more IR data in pentad',
     +                 iyyyypp
                  GO TO 500
               END IF
C     
               DO  j = 1, MXLTCL
                  DO  i = 1, MXLNCL
                     IF  ( getic (i, j, 1) .GE. 0. )  THEN
                        IF  ( getic (i, j, 3) .GE. MINIM )  THEN
                           sumic (i, j, 1) = sumic (i, j, 1) +
     +                          getic (i, j, 1) * nday
                           sumic (i, j, 2) = sumic (i, j, 2) +
     +                          getic (i, j, 2) * nday
                           ni    (i, j)    = ni (i, j) + nday
                        ELSE
                           npitch = npitch + 1
                        END IF
                     END IF
                  END DO
               END DO
            END IF
C     
C     Print the number of pentad data points tossed due to the
C     number of images .LT. MINIM and reset the counter for 
C     the next pentad.
C     
            IF  (npitch .NE. 0)  THEN
               WRITE (*,*) 'WARNING: npitch = ', npitch, 
     +              ' for > ', MINIM, ' images in ',
     +              'pentad ', iyyyypp
            END IF
C     
            npitch = 0
C     
c            IF  ( stype .EQ. 'd' )  THEN
c               CALL SPREPH  ( snam,    iasc,  iyyyypp,    
c     +              binlims, n_bls, rro,    rrl,
c     +              rrc,     'all',
c     +              gets,    getc,  getn,   ngets,
c     +              ngetc,   ngetn, iret )
c               IF  ( iret .LT. -1 )  THEN
c                  WRITE (*, *) 'No more SSMI data in pentad',
c     +                 iyyyypp
c                  GO TO 500
c               END IF
C     
c               CALL ADDIN  ( .FALSE., avgper,  gets,   getc,
c     +              getn,    ngets,   ngetc,  ngetn,
c     +              nday,    xs, xc,  xn,     xxs,
c     +              xxc,     xxn, ns, nc, nn, iret )
cc     debug	i = 533
cc	j = 76
cc	WRITE (*, *) '(533,76)getc1-3,ngetc1,xc1-2,xxc,nc1',
cc     +	(getc (i, j, k), k=1, 3), ngetc (i, j, 1), (xc (i, j, k),
cc     +	k=1, 2), xxc (i, j), nc (i, j, 1)
c            END IF


C     Increment pentad for next pass of middle pentad loop,
C     or increment around an unmatched pentad.
C     
 300        IF  ( ipp .EQ. 73 )  THEN
               iyyyy = iyyyy + 1
               ipp   = 1
            ELSE
               ipp = ipp + 1
            END IF
            iyyyypp = iyyyy * 100 + ipp
         END DO
C
C     Finish the month average:  iyyyy, ipp, iyyyypp are 
C     already set above.  The logical skpend is .TRUE. if the 
C     last pentad is unmatched, and we are doing "matched 
C     month".  The AVHRR IR is only available in pentads, so 
C     we automatically end the average as "approximate".
C     
         IF  ( INDEX ( avgper, 'mm' ) .NE. 0 )  THEN
            DO  i = 1, MAXNOT
               IF  ( iyyyypp .EQ. notmat (i) )  THEN
                  WRITE (*, *) 'Skipping pentad', iyyyypp
                  skpend = .TRUE.
                  GO TO 350
               END IF
            END DO
         END IF
C     
         skpend   = .FALSE.
         WRITE (*, *) 'Starting pentad', iyyyypp
         IF  ( itype .EQ. 'd' )  THEN
            CALL IPREP  ( inam,    inst,    iyyyypp, binlimi, 
     +           n_bli,   binlimi, n_bli,   cnam,  
     +           do_zcor, zcor,    getic,   iret )
            IF  ( iret .NE. 0 )  THEN
               WRITE (*, *) 'No more LEO IR data in pentad',
     +              iyyyypp
               GO TO 500
            END IF
C     
            DO  j = 1, MXLTCL
               DO  i = 1, MXLNCL
                  IF  ( getic (i, j, 1) .GE. 0. )  THEN
                     IF  ( getic (i, j, 3) .GE. MINIM )  THEN
                        sumic (i, j, 1) = sumic (i, j, 1) +
     +                       getic (i, j, 1) *nstop
                        sumic (i, j, 2) = sumic (i, j, 2) +
     +                       getic (i, j, 2) *nstop
                        ni    (i, j)    = ni    (i, j)    +nstop
                     ELSE
                        npitch = npitch + 1
                     END IF
                  END IF
               END DO
            END DO
         END IF
C     
C     This jump to 350 was reworked to compile on IRIX6 and 
C     it's not clear why it ever compiled.  Also, the accum.
C     above was pulled out, since it appears that it would 
C     always run, even if the jump to 350 had occurred.
C     
 350     CONTINUE
         IF  ( itype .EQ. 'd' )  THEN
            DO  j = 1, MXLTCL
               DO  i = 1, MXLNCL
                  IF  ( sumic (i, j, 1) .GE. 0. )  THEN
                     IF  ( ni (i, j) .GT. 0 )  THEN
                        sumic (i, j, 1) = sumic (i, j, 1) / 
     +                       ni (i,j)
                        sumic (i, j, 2) = sumic (i, j, 2) / 
     +                       ni (i,j)
                     ELSE
                        sumic (i, j, 1) = RMISS
                        sumic (i, j, 2) = RMISS
                     END IF
                  ELSE
                     sumic (i, j, 1) = RMISS
                     sumic (i, j, 2) = RMISS
                  END IF
               END DO
            END DO
         END IF
C     
C     Print the number of pentad data points tossed due to the
C     number of images .LT. MINIM and reset the counter for 
C     the next pentad.
C     
         IF  (npitch .NE. 0)  THEN
            WRITE (*,*) 'WARNING: npitch = ', npitch, ' for > ',
     +           MINIM, ' images in pentad ', iyyyypp
         END IF
C     
         npitch = 0
C     
C     Finish the SSM/I "approximate" month, or end the SSM/I
C     "exact" month (flagged as 'stope').
C     
c         IF  ( stype .EQ. 'd' )  THEN
c            IF  ( skpend )  GO TO 360
cC     
c            IF  ( INDEX  ( avgper, 'e' ) .EQ. 0 .OR.
c     +           imm .EQ. 3 .OR. imm .EQ. 4 .OR. imm .EQ. 12 ) 
c     +           THEN
c               CALL SPREPH  ( snam,    iasc,  iyyyypp,    
c     +              binlims, n_bls, rro,    rrl,
c     +              rrc,     'all',
c     +              gets,    getc,  getn,   ngets,
c     +              ngetc,   ngetn, iret )
c            ELSE
c               CALL SPREPH  ( snam,    iasc,  iyyyypp,    
c     +              binlims, n_bls, rro,    rrl,
c     +              rrc,     'stope',
c     +              gets,    getc,  getn,   ngets,
c     +              ngetc,   ngetn, iret )
c            END IF
c            IF  ( iret .LT. -1 )  THEN
c               WRITE (*, *) 'No more SSMI data in pentad',
c     +              iyyyypp
c               GO TO 500
c            END IF
cC     
c            CALL ADDIN  ( .FALSE., avgper, gets,    getc,
c     +           getn,    ngets,  ngetc,   ngetn,
c     +           nstop,   xs, xc, xn, xxs, xxc, xxn,
c     +           ns, nc,  nn,     iret )
ccdebug	i = 533
cc	j = 76
cc	WRITE (*, *) '(533,76)getc1-3,ngetc1,xc1-2,xxc,nc1',
cc     +	(getc (i, j, k), k=1, 3), ngetc (i, j, 1), (xc (i, j, k),
cc     +	k=1, 2), xxc (i, j), nc (i, j, 1)
cC
cC		    If the data are o.k., do the statistics.  Following
cC		    IMSL (but written out for speed),
cC		    xbar     = SUM [ x ] / npts
cC		    sig ** 2 = SUM [ (x-xbar)**2 ] / npts-1
cC			     = [ N*sumxx - sumx**2 ] / [ N * N-1 ]
cC
c 360        CONTINUE
c            DO  j = 1, MXLTNS
c               DO  i = 1, MXLNNS
c                  IF  ( ns (i, j, 1) .LT. 2 )  THEN
c                     avgs (i, j, 1) = -.2
c                     avgs (i, j, 2) = -.2
c                     avgs (i, j, 3) = -.2
c                     avgs (i, j, 4) = -.2
c                     IF  (ns (i, j, 4) .GT. 0 )  THEN
c                        avgs (i, j, 5) = 1.
c                     ELSE
c                        avgs (i, j, 5) = -.2
c                     END IF
c                     sigs (i, j) = -.2
c                  ELSE
c                     avgs (i, j, 1) = xs (i, j, 1) / ns (i, j, 1)
c                     avgs (i, j, 2) = xs (i, j, 2) / ns (i, j, 1)
c                     avgs (i, j, 3) = FLOAT ( ns (i, j, 2) ) /
c     +                    FLOAT ( ns (i, j, 1) )
c                     avgs (i, j, 4) = FLOAT ( ns (i, j, 3) ) /
c     +                    FLOAT ( ns (i, j, 1) )
c                     avgs (i, j, 5) = FLOAT ( ns (i, j, 4) ) /
c     +                    FLOAT ( ns (i, j, 1) + 
c     +                    ns (i, j, 4) )
c                     sigs (i, j) = (
c     +                    ( DBLE ( ns (i, j, 1) ) * xxs (i, j) -
c     +                    xs (i, j, 1) * xs (i, j, 1) )     /
c     +                    DBLE ( ns (i, j, 1) * ( ns (i, j, 1) - 1 )))
c                  END IF
cC     
c                  IF  ( nn (i, j, 1) .LT. 2 )  THEN
c                     avgn (i, j, 1) = -.2
c                     avgn (i, j, 2) = -.2
c                     avgn (i, j, 3) = -.2
c                     avgn (i, j, 4) = -.2
c                     IF  (nn (i, j, 4) .GT. 0 )  THEN
c                        avgn (i, j, 5) = 1.
c                     ELSE
c                        avgn (i, j, 5) = -.2
c                     END IF
c                     sign (i, j) = -.2
c                  ELSE
c                     avgn (i, j, 1) = xn (i, j, 1) / nn (i, j, 1)
c                     avgn (i, j, 2) = xn (i, j, 2) / nn (i, j, 1)
c                     avgn (i, j, 3) = FLOAT ( nn (i, j, 2) ) /
c     +                    FLOAT ( nn (i, j, 1) )
c                     avgn (i, j, 4) = FLOAT ( nn (i, j, 3) ) /
c     +                    FLOAT ( nn (i, j, 1) )
c                     avgn (i, j, 5) = FLOAT ( nn (i, j, 4) ) /
c     +                    FLOAT ( nn (i, j, 1) + 
c     +                    nn (i, j, 4) )
c                     sign (i, j) = (
c     +                    ( DBLE ( nn (i, j, 1) ) * xxn (i, j) -
c     +                    xn (i, j, 1) * xn (i, j, 1) )     /
c     +                    DBLE ( nn (i, j, 1) * ( nn (i, j, 1) - 1 )))
c                  END IF
c               END DO
c            END DO
cC     
c            DO  j = 1, MXLTC
c               DO  i = 1, MXLNC
c                  IF  ( nc (i, j, 1) .LT. 2 )  THEN
c                     avgc (i, j, 1) = -.2
c                     avgc (i, j, 2) = -.2
c                     avgc (i, j, 3) = -.2
c                     avgc (i, j, 4) = -.2
c                     IF  (nc (i, j, 4) .GT. 0 )  THEN
c                        avgc (i, j, 5) = 1.
c                     ELSE
c                        avgc (i, j, 5) = -.2
c                     END IF
c                     sigc (i, j) = -.2
c                  ELSE
c                     avgc (i, j, 1) = xc (i, j, 1) / nc (i, j, 1)
c                     avgc (i, j, 2) = xc (i, j, 2) / nc (i, j, 1)
c                     avgc (i, j, 3) = FLOAT ( nc (i, j, 2) ) /
c     +                    FLOAT ( nc (i, j, 1) )
c                     avgc (i, j, 4) = FLOAT ( nc (i, j, 3) ) /
c     +                    FLOAT ( nc (i, j, 1) )
c                     avgc (i, j, 5) = FLOAT ( nc (i, j, 4) ) /
c     +                    FLOAT ( nc (i, j, 1) + 
c     +                    nc (i, j, 4) )
c                     sigc (i, j) = (
c     +                    ( DBLE ( nc (i, j, 1) ) * xxc (i, j) -
c     +                    xc (i, j, 1) * xc (i, j, 1) )     /
c     +                    DBLE ( nc (i, j, 1) * ( nc (i, j, 1) - 1 )))
c                  END IF
c               END DO
c            END DO
c         END IF
cC     

c     This is the end for the test for avgper='p' from line ~508
c      END IF


ccc
ccc	i = 533
ccc	j = 76
ccc	WRITE (*, *) '(533,76)avgc1-2,xc1-2,nc1-2=', 
ccc     +		     (avgc (i, j, k), k=1, 2),
ccc     +		     (xc (i, j, k), k=1, 2), (nc (i, j, k), k=1, 2)
ccc
C


cC	    Done accumulating pentad or monthly sums/avgs.  Now compute
cC	    the "equivalent number of images" for the SSM/I, which is
cC	    the number of images completely covering the box at the 
cC	    equator with scan-center pixels which are required to produce 
cC	    the observed number of pixel counts:
cC		# center-scan pixels = P   = n / pcc
cC		# full images        = eqv = P * psiz**2 /
cC					     (bsiz**2 * f)
cC	    where pcc  = 20 pixel-counts per center pixel
cC		  psiz = 12.5 km pixel size
cC		  bsiz = 55.5 km box size
cC		  f    = 1|2 for the C|N,S grids (different boxes)
cC		  COS (lat) accounts for changes in a deg of lon
cC	    so
cC		eqv = n / 20 * 12.5**2 / (55.5**2 * f)
cCccc
c	    IF  ( stype .EQ. 'd' )  THEN
c               DO  j = 1, MXLTNS
c                  DO  i = 1, MXLNNS
c                     IF  ( ns (i, j, 1) .LT. 0 )  THEN
c			eqvs (i, j) = -1.
c                     ELSE
c			eqvs (i, j) = ns (i, j, 1) * 156.25 / 123210.
c                     END IF
cC     
c                     IF  ( nn (i, j, 1) .LT. 0 )  THEN
c			eqvn (i, j) = -1.
c                     ELSE
c			eqvn (i, j) = nn (i, j, 1) * 156.25 / 123210.
c                     END IF
c                  END DO
c               END DO
C     
c               DO  j = 1, MXLTC
c                  DO  i = 1, MXLNC
c                     IF  ( nc (i, j, 1) .LT. 0 )  THEN
c			eqvc (i, j) = -1.
c                     ELSE
c			eqvc (i, j) = nc (i, j, 1) * 156.25 / 61605.
c                     END IF
c                  END DO
c               END DO
cC     
cC     Compute scaled variance.
cC     
c               DO  j = 1, MXLTNS
c                  DO  i = 1, MXLNNS
c                     IF  ( ns (i, j, 1) .LT. 2 )  THEN
c			s_as (i, j) = -.2
c                     ELSE
c			IF  ( avgs (i, j, 1) .LT. .0001 )  THEN
c                           s_as (i, j) = RMISS
c                        ELSE
c                           s_as (i, j) = sigs (i, j) /
c     +				( avgs (i, j, 1) * avgs (i, j, 1) )
c			END IF
c                     END IF
cC     
c                     IF  ( nn (i, j, 1) .LT. 2 )  THEN
c			s_an (i, j) = -.2
c                     ELSE
c			IF  ( avgn (i, j, 1) .LT. .0001 )  THEN
c                           s_an (i, j) = RMISS
c                        ELSE
c                           s_an (i, j) = sign (i, j) /
c     +				( avgn (i, j, 1) * avgn (i, j, 1) )
c			END IF
c                     END IF
c                  END DO
c               END DO
cC     
c               DO  j = 1, MXLTC
c                  DO  i = 1, MXLNC
c                     IF  ( nc (i, j, 1) .LT. 2 )  THEN
c			s_ac (i, j) = -.2
c                     ELSE
c			IF  ( avgc (i, j, 1) .LT. .0001 )  THEN
c                           s_ac (i, j) = RMISS
c                        ELSE
c                           s_ac (i, j) = sigc (i, j) /
c     +				( avgc (i, j, 1) * avgc (i, j, 1) )
c			END IF
c                     END IF
c                  END DO
c               END DO
c	    END IF


cC     If working with SSM/I data files, then for any of the 
cC     averaging periods, sum o.k. SSM/I data 
cC     to 2.5 resolution pentad-by-pentad, sum the pentads (at 
cC     least 1!), and compute the averages.
cC     
c	    IF  ( stype .EQ. 'd' )
c     +           CALL  SSAVG  ( avgper, xs, xc, xn, ns,
c     +           nc, nn, sums,   sumsc,  sumi,
c     +           sumq,   summ,   sumim,  iret )


C     Done with data file accumulation code.  Depending on user
C     specification, either output the results, or pull in the IR 
C     and/or SSM/I low-res rain rate images.  They are 
C     assumed to be on a 2.5x2.5 deg lat/lon grid spanning 50N-50S 
C     globally, starting at 50N,0E and incrementing in lon first, 
C     with whole degrees on grid edges, stored in binary SGI 
C     floats.  Rain rate goes into the first array; areal coverage 
C     in the second.
C     
C     Start by building output file names (for unsmoothed files,
C     even if we smooth later in the code).  Generate a basename 
C     from the ssmi basename and current pentad or month.  If the 
C     input SSM/I is an image and has 'fix' in the name, put 'fix' 
C     in the output name.
C     
c
         call String_Length(snam, l_nam)
         call String_Length(InterimPATH, strlen)

 5       FORMAT ( '.', I6.6 )
         WRITE ( cid, 5 ) iyyyymm
         i = INDEX ( snam (1:l_nam), cid (1:7) )
         bnam = InterimPATH(1:strlen)//'CORR_ADJ/'//snam(i-3:i+6)


C
         cid = '0      '
c	    IF  ( stype .EQ. 'd' .OR. 
c     +           INDEX ( snam, 'fix' ) .EQ. 0 )  THEN
c               l1 = 'avfull.'                   // avgper (1:2)
c               l2 = 'var.'                      // avgper (1:2)
c               l3 = 'varsc.'                    // avgper (1:2)
c               l4 = 'ssful' // cid (1:1) // '.' // avgper (1:2)
c               l5 = 'ir'    // cid (1:1) // '.' // avgper (1:2)
c               l6 = 'lofull.'                   // avgper (1:2)
c               l7 = 'co'    // cid (1:1) // '.' // avgper (1:2)
c               l8 = 'ad'    // cid (1:1) // '.' // avgper (1:2)
c            ELSE
c               l1 = 'avfix.'                    // avgper (1:2)
c               l2 = 'varfix.'                   // avgper (1:2)
c               l3 = 'varscfix.'                 // avgper (1:2)
c               l4 = 'ssfix' // cid (1:1) // '.' // avgper (1:2)
c               l5 = 'ir'    // cid (1:1) // '.' // avgper (1:2)
c               l6 = 'lofix.'                    // avgper (1:2)
c               l7 = 'cofix' // cid (1:1) // '.' // avgper (1:2)
c               l8 = 'adfix' // cid (1:1) // '.' // avgper (1:2)
c	    END IF
C     
c	    IF  ( itype .EQ. 'd' )  THEN

         call String_Length(bnam, strlen)

c     Extract part of data to write to file
         do i1=1,144
            do j1=1,40
               tmpmat(i1,j1)=sumic(i1,j1,1)
            end do
         end do
         
         fname=bnam(1:strlen)//'.rir0.'//avgper (1:2)
         call Write_Binary_File(fname, tmpmat, 144, 40)
c               CALL DMPIMGI  ( sumic,           bnam, 'r', l5, iret )

c              Extract part of data to write to file
         do i1=1,144
            do j1=1,40
               tmpmat(i1,j1)=sumic(i1,j1,2)
            end do
         end do

         fname=bnam(1:strlen)//'.air0.'//avgper (1:2)
         call Write_Binary_File(fname, tmpmat, 144, 40)
c               CALL DMPIMGI  ( sumic (1, 1, 2), bnam, 'a', l5, iret )




c            ELSE
c               CALL ST_LSTR  ( inam, l_nam, iret )
c               iunit = INIT_DISK ( inam )
c               IF  ( iunit .LT. 0 )  THEN
c                  WRITE (*, *) 'Open error ', iunit, ' on IR file ',
c     +                 inam (1:l_nam)
c                  iret = EXIT_DISK ( iunit )
c                  STOP
c               END IF
c               IF  ( itype .EQ. 'r' )  THEN
c                  nin = READ_DISK ( iunit, sumic,           INSIZ )
c               ELSE
c                  nin = READ_DISK ( iunit, sumic (1, 1, 2), INSIZ )
c               END IF
c               IF  ( nin .LT. INSIZ )  THEN
c                  WRITE ( *, * ) 'Read error: ', nin, ' < ', INSIZ,
cc     +                 ' on IR file ', inam (1:l_nam)
c                  iret = EXIT_DISK ( iunit )
c                  STOP
c               END IF
c               iret = EXIT_DISK ( iunit )
c	    END IF
C     
c	    IF  ( stype .EQ. 'd' )  THEN
cC     
c               CALL DMPIMGH  ( eqvs,  eqvc, eqvn, bnam, 'I', l1, iret )
c               CALL DMPIMGH  ( avgs,  avgc, avgn, bnam, 'r', l1, iret )
c               CALL DMPIMGH  ( sigs,  sigc, sign, bnam, 'r', l2, iret )
c               CALL DMPIMGH  ( s_as,  s_ac, s_an, bnam, 'r', l3, iret )
c               CALL DMPIMGI  ( sumsc, bnam, 'r',  l4,   iret )
c               CALL DMPIMGH  ( avgs  (1, 1, 2),   avgc  (1, 1, 2),
c     +              avgn  (1, 1, 2),   bnam, 'a', l1, iret )
c               CALL DMPIMGI  ( sumsc (1, 1, 2),   bnam, 'a', l4, iret )
c               CALL DMPIMGH  ( avgs  (1, 1, 3),   avgc  (1, 1, 3),
c     +              avgn  (1, 1, 3),   bnam, 'i', l1, iret )
c               CALL DMPIMGH  ( avgs  (1, 1, 4),   avgc  (1, 1, 4),
c     +              avgn  (1, 1, 4),   bnam, 'q', l1, iret )
c               CALL DMPIMGH  ( avgs  (1, 1, 5),   avgc  (1, 1, 5),
c     +              avgn  (1, 1, 5),   bnam, 'm', l1, iret )
c               CALL DMPIMGL  ( sums,  bnam, 'r',  l6,   iret )
c               CALL DMPIMGL  ( sumi,  bnam, 'i',  l6,   iret )
c               CALL DMPIMGL  ( sumq,  bnam, 'q',  l6,   iret )
c               CALL DMPIMGL  ( summ,  bnam, 'm',  l6,   iret )
c               CALL DMPIMGL  ( sumim, bnam, 'I',  l6,   iret )
c            ELSE

         call String_Length(snam, l_nam)
         call Read_Binary_File(snam,144,40,1,sumsc)
         
         

c               iunit = INIT_DISK ( snam )
c               IF  ( iunit .LT. 0 )  THEN
c                  WRITE (*, *) 'Open error ', iunit, ' on SSM/I file ',
c     +                 snam (1:l_nam)
c                  iret = EXIT_DISK ( iunit )
c                  STOP
c               END IF
c               IF  ( stype .EQ. 'r' )  THEN
c               nin = READ_DISK ( iunit, sumsc,           INSIZ )
ctemp
c		    WRITE (*, *) 'Warning: temp. mm/d fix for SSM/I'
c		    DO  j = 1, MXLTCL
c		    DO  i = 1, MXLNCL
c			IF  ( sumsc (i, j, 1) .GE. 0. )  
c     +			    sumsc (i, j, 1) = sumsc (i, j, 1) * 24.
c		    END DO
c		    END DO
ctemp
c               ELSE
c                  nin = READ_DISK ( iunit, sumsc (1, 1, 2), INSIZ )
c               END IF
c               IF  ( nin .LT. INSIZ )  THEN
c                  WRITE ( *, * ) 'Read error: ', nin, ' < ', INSIZ,
c     +                 ' on IR file ', snam (1:l_nam)
c                  iret = EXIT_DISK ( iunit )
c                  STOP
c               END IF
c               iret = EXIT_DISK ( iunit )
c	    END IF
C
C     mooth-fill SSM/I (in parallel with how we treat 1-deg.
C     in CORR2.F).
C     [Philosophy: Smooth-filled SSM/I and adjustment ratios 
C     should be flat across the Himalayas; re-computing ratios 
C     with the smooth-filled SSM/I will squash the cold sfc 
C     artifact in the IR, while using the smooth-filled ratios 
C     will continue to show it.]
C     
C     Optionally, smooth the IR and SSMI.  The boundaries are
C     "missing" north and south, and periodic east and west.
C     To match SMOOTH, set values <0 to RMISS.
C     
         npass = -1
         IF  ( itype .NE. 'a' .AND. stype .NE. 'a' )
     +        CALL FILLSM  ( sumsc (1, 5, 1), MXLNCL, MXLTCL - 8, 
     +        MXLNCL, 0, npass, sumsc (1, 5, 1),  
     +        iret )
         IF  ( itype .NE. 'r' .AND. stype .NE. 'r' )
     +        CALL FILLSM  ( sumsc (1, 5, 2), MXLNCL, MXLTCL - 8, 
     +        MXLNCL, 0, npass, sumsc (1, 5, 2),  
     +        iret )
         IF  ( ismth .GE. 3 )  THEN
            IF  ( itype .NE. 'a' .AND. stype .NE. 'a' )  THEN
               DO  j = 5, MXLTCL - 4
                  DO  i = 1, MXLNCL
                     IF  ( sumic (i, j, 1) .LT. 0. )  
     +                    sumic (i, j, 1) = RMISS
                     IF  ( sumsc (i, j, 1) .LT. 0. )  
     +                    sumsc (i, j, 1) = RMISS
                  END DO
               END DO
               DO  j = 1, 4
                  DO  i = 1, MXLNCL
                     sumic (i, j,              1) = RMISS
                     sumsc (i, j,              1) = RMISS
                     sumic (i, MXLTCL + 1 - j, 1) = RMISS
                     sumsc (i, MXLTCL + 1 - j, 1) = RMISS
                  END DO
               END DO
               CALL SMOOTH  ( ismth,  smth,    3., sumic, 
     +              MXLNCL, MXLTCL,  sumic,       iret )
               CALL SMOOTH  ( ismth,  smth,    3., sumsc, 
     +              MXLNCL, MXLTCL,  sumsc,       iret )
            END IF
            IF  ( itype .NE. 'r' .AND. stype .NE. 'r' )  THEN
               DO  j = 5, MXLTCL - 4
                  DO  i = 1, MXLNCL
                     IF  ( sumic (i, j, 2) .LT. 0. )  
     +                    sumic (i, j, 2) = RMISS
                     IF  ( sumsc (i, j, 2) .LT. 0. )  
     +                    sumsc (i, j, 2) = RMISS
                  END DO
               END DO
               DO  j = 1, 5
                  DO  i = 1, MXLNCL
                     sumic (i, j,              2) = RMISS
                     sumsc (i, j,              2) = RMISS
                     sumic (i, MXLTCL + 1 - j, 2) = RMISS
                     sumsc (i, MXLTCL + 1 - j, 2) = RMISS
                  END DO
               END DO
               CALL SMOOTH  ( ismth,  smth,    3., sumic (1, 1, 2), 
     +              MXLNCL, MXLTCL, 
     +              sumic (1, 1, 2), iret )
               CALL SMOOTH  ( ismth,  smth,    3., sumsc (1, 1, 2), 
     +              MXLNCL, MXLTCL, 
     +              sumsc (1, 1, 2), iret )
            END IF
C     
C     When smoothing, output the smoothed fields.  As before,
C     if the SSM/I is an image and has 'fix' in the name, put
C     'fix' in the output name.
C
            IF  ( ismth .LT. 10 )  THEN
               WRITE ( cid, 6 ) ismth
 6             FORMAT ( I1, '     ' )
            ELSE
               WRITE ( cid, 7 ) ismth
 7             FORMAT ( I2, '    ' )
            END IF
c               IF  ( stype .EQ. 'd' .OR. 
c     +              INDEX ( snam, 'fix' ) .EQ. 0 )  THEN
c                  l4 = 'ssful' // cid (1:1) // '.' // avgper (1:2)
c                  l5 = 'ir'    // cid (1:1) // '.' // avgper (1:2)
c                  l7 = 'co'    // cid (1:1) // '.' // avgper (1:2)
c                  l8 = 'ad'    // cid (1:1) // '.' // avgper (1:2)
c               ELSE
c               l4 = 'ssfix' // cid (1:1) // '.' // avgper (1:2)
c               l5 = 'ir'    // cid (1:1) // '.' // avgper (1:2)
c               l7 = 'cofix' // cid (1:1) // '.' // avgper (1:2)
c               l8 = 'adfix' // cid (1:1) // '.' // avgper (1:2)
c               END IF
C     
c               IF  ( itype .NE. 'a' )
c     +		    CALL DMPIMGI  ( sumic,           bnam, 'r', l5, 
c     +              iret )
c               IF  ( itype .NE. 'r' )
c     +		    CALL DMPIMGI  ( sumic (1, 1, 2), bnam, 'a', l5, 
c     +              iret )
c               IF  ( stype .NE. 'a' )
c     +		    CALL DMPIMGI  ( sumsc,           bnam, 'r', l4,
c     +              iret )
c               IF  ( stype .NE. 'r' )
c     +		    CALL DMPIMGI  ( sumsc (1, 1, 2),  bnam, 'a',l4,
c     +              iret )


c               itype='d', so always produce  rir3 and air3

            call String_Length(bnam, strlen)
               
c     Extract part of data to write to file for rir3
            do i1=1,144
               do j1=1,40
                  tmpmat(i1,j1)=sumic(i1,j1,1)
               end do
            end do
            fname=bnam(1:strlen)//'.rir'//cid(1:1)//'.'//avgper(1:2)
            call Write_Binary_File(fname, tmpmat, 144, 40)
c              CALL DMPIMGI  ( sumic,     bnam, 'r', l5, iret )

c     Extract part of data to write to file air3
            do i1=1,144
               do j1=1,40
                  tmpmat(i1,j1)=sumic(i1,j1,2)
               end do
            end do
            fname=bnam(1:strlen)//'.air'//cid(1:1)//'.'//avgper(1:2)
            call Write_Binary_File(fname, tmpmat, 144, 40)
c              CALL DMPIMGI  ( sumic (1, 1, 2), bnam, 'a', l5,  iret )

c              stype='r', so always produce rssfix
c              Extract part of data to write to file air3
            do i1=1,144
               do j1=1,40
                  tmpmat(i1,j1)=sumsc(i1,j1,1)
               end do
            end do
            fname=bnam(1:strlen)//'.rssfix'//cid(1:1)//'.'
     +              //avgper(1:2)
            call Write_Binary_File(fname, tmpmat, 144, 40)
c               CALL DMPIMGI  ( sumsc,   bnam, 'r', l4,iret )




         END IF
C     
C     Compute the ratios for the data requested and dump out the
C     results.
C     
c     IF  ( itype .NE. 'a' .AND. stype .NE. 'a' )  THEN
         CALL MONCOR  ( sumic, sumsc, MXLNCL, MXLTCL, a, b, iret)
         
c     Extract part of data to write to file rcofix3
         do i1=1,144
            do j1=1,40
               tmpmat(i1,j1)=b(i1,j1,1)
            end do
         end do
         fname=bnam(1:strlen)//'.rcofix'//cid(1:1)//'.'//avgper(1:2)
         call Write_Binary_File(fname, tmpmat, 144, 40)
c            CALL DMPIMGI ( b, bnam, 'r', l7, iret )

c     Extract part of data to write to file radfix3
         do i1=1,144
            do j1=1,40
               tmpmat(i1,j1)=a(i1,j1,1)
            end do
         end do
         fname=bnam(1:strlen)//'.radfix'//cid(1:1)//'.'//avgper(1:2)
         call Write_Binary_File(fname, tmpmat, 144, 40)
c            CALL DMPIMGI ( a, bnam, 'r', l8, iret )
 





c	    END IF
c	    IF  ( itype .NE. 'r' .AND. stype .NE. 'r' )  THEN
c               CALL MONCOR  ( sumic (1, 1, 2), sumsc (1, 1, 2), 
c     +              MXLNCL, MXLTCL,
c     +              a     (1, 1, 2), b     (1, 1, 2), iret )
c               CALL DMPIMGI ( b (1, 1, 2), bnam, 'a', l7, iret )
c               CALL DMPIMGI ( a (1, 1, 2), bnam, 'a', l8, iret )
c	    END IF
     
C     When working with the SSM/I data, do quality-based control 
C     on the .5-deg SSM/I.  If the fractional coverage is more 
C     than .2 for months or .4 for pentads, set the high-res value 
C     to RMISS and add all of the "good" and "ice" samples to 
C     "missing".  ["ambig" are counted in "good", so they don't 
C     get added to "missing".]
C     
c	    IF  ( stype .EQ. 'd' )  THEN
c               IF  ( INDEX  ( avgper, 'm' ) .NE. 0 )  THEN
c                  ed = .2
c               ELSE
c                  ed = .4
c               END IF
c               DO  j = 1, MXLTNS
cc                  DO  i = 1, MXLNNS
c                    IF  ( avgs (i, j, 3) + avgs (i, j, 4) .GT. ed ) THEN
c			avgs (i, j, 1) = RMISS
cc			avgs (i, j, 2) = RMISS
c			eqvs (i, j)    = RMISS
c			sigs (i, j)    = RMISS
c			s_as (i, j)    = RMISS
c			ns   (i, j, 4) = ns (i, j, 4) + ns (i, j, 1) +
c     +                       ns (i, j, 2)
c			ns   (i, j, 1) = 0
c			ns   (i, j, 2) = 0
c			ns   (i, j, 3) = 0
c                     END IF
c                    IF  ( avgn (i, j, 3) + avgn (i, j, 4) .GT. ed ) THEN
c			avgn (i, j, 1) = RMISS
c			avgn (i, j, 2) = RMISS
c			eqvn (i, j)    = RMISS
c			sign (i, j)    = RMISS
c			s_an (i, j)    = RMISS
c			nn   (i, j, 4) = nn (i, j, 4) + nn (i, j, 1) +
c     +                       nn (i, j, 2)
c			nn   (i, j, 1) = 0
c			nn   (i, j, 2) = 0
c			nn   (i, j, 3) = 0
c                     END IF
c                  END DO
c               END DO
c               DO  j = 1, MXLTC
c                  DO  i = 1, MXLNC
c                    IF  ( avgc (i, j, 3) + avgc (i, j, 4) .GT. ed ) THEN
c			avgc (i, j, 1) = RMISS
c			avgc (i, j, 2) = RMISS
c			eqvc (i, j)    = RMISS
c			sigc (i, j)    = RMISS
c			s_ac (i, j)    = RMISS
c			nc   (i, j, 4) = nc (i, j, 4) + nc (i, j, 1) +
c     +                       nc (i, j, 2)
c			nc   (i, j, 1) = 0
c			nc   (i, j, 2) = 0
c			nc   (i, j, 3) = 0
c                     END IF
c                  END DO
c               END DO
cC     
cC     For quality-edited SSM/I data, any of the averaging 
cC     periods sum o.k. SSM/I data to 2.5 resolution pentad-by-
cC     pentad, sum the pentads (at least 1!), and compute the 
cC     averages.
cC     
c               CALL  SSAVG  ( avgper, xs, xc, xn, ns,
c     +              nc, nn, sums,   sumsc,  sumi,
c     +              sumq,   summ,   sumim,  iret )
cC     
cC     Done with data file code.  Output the results starting 
cC     by building output file names.  Use the same basename 
cC     (SSM/I basename and current pentad or month).
cC     
c               cid = '0     '
c               l1 = 'avfix.'                    // avgper (1:2)
c               l2 = 'varfix.'                   // avgper (1:2)
c               l3 = 'varscfix.'                 // avgper (1:2)
c               l4 = 'ssfix' // cid (1:1) // '.' // avgper (1:2)
c               l6 = 'lofix.'                    // avgper (1:2)
c               l7 = 'cofix' // cid (1:1) // '.' // avgper (1:2)
c               l8 = 'adfix' // cid (1:1) // '.' // avgper (1:2)
cC     
c               CALL DMPIMGH  ( eqvs,  eqvc, eqvn, bnam, 'I', l1, iret )
c               CALL DMPIMGH  ( avgs,  avgc, avgn, bnam, 'r', l1, iret )
cc               CALL DMPIMGH  ( sigs,  sigc, sign, bnam, 'r', l2, iret )
c               CALL DMPIMGH  ( s_as,  s_ac, s_an, bnam, 'r', l3, iret )
c               CALL DMPIMGI  ( sumsc, bnam, 'r',  l4,   iret )
c               CALL DMPIMGH  ( avgs  (1, 1, 2),   avgc  (1, 1, 2),
c     +              avgn  (1, 1, 2),   bnam, 'a', l1, iret )
c               CALL DMPIMGI  ( sumsc (1, 1, 2),   bnam, 'a', l4, iret )
c               CALL DMPIMGL  ( sums,  bnam, 'r',  l6,   iret )
c               CALL DMPIMGL  ( sumi,  bnam, 'i',  l6,   iret )
c               CALL DMPIMGL  ( sumq,  bnam, 'q',  l6,   iret )
c               CALL DMPIMGL  ( summ,  bnam, 'm',  l6,   iret )
c               CALL DMPIMGL  ( sumim, bnam, 'I',  l6,   iret )
cC     
cC     mooth-fill SSM/I (in parallel with how we treat 1-deg.
cC     in CORR2.F) as was done above.
cC     
cC     Optionally smooth the SSMI (IR was done above, if  
cC     needed).  The boundaries are "missing" north and south, 
cC     and periodic east and west.  To match SMOOTH, set 
cC     values <0 to RMISS.
cC     
c               npass = -1
c               IF  ( itype .NE. 'a' .AND. stype .NE. 'a' )
c     +		    CALL FILLSM  ( sumsc (1, 5, 1), MXLNCL, MXLTCL - 8, 
c     +              MXLNCL, 0, npass, sumsc (1, 5, 1),  
c     +              iret )
c               IF  ( itype .NE. 'r' .AND. stype .NE. 'r' )
c     +		    CALL FILLSM  ( sumsc (1, 5, 2), MXLNCL, MXLTCL - 8, 
c     +              MXLNCL, 0, npass, sumsc (1, 5, 2),  
c     +              iret )
cC     
cC     Optionally smooth the SSMI (IR was done above, if needed).  
cC     The boundaries are "missing" north and south, and periodic 
cC     east and west.  To match SMOOTH, set values <0 to RMISS.
cC     
c               IF  ( ismth .GE. 3 )  THEN
c                  IF  ( itype .NE. 'a' .AND. stype .NE. 'a' )  THEN
c                     DO  j = 5, MXLTCL - 4
c			DO  i = 1, MXLNCL
c                           IF  ( sumsc (i, j, 1) .LT. 0. )  
c     +				sumsc (i, j, 1) = RMISS
c			END DO
c                     END DO
c                     DO  j = 1, 4
c			DO  i = 1, MXLNCL
c                           sumsc (i, j,              1) = RMISS
c                           sumsc (i, MXLTCL + 1 - j, 1) = RMISS
c			END DO
c                     END DO
c                     CALL SMOOTH  ( ismth,  smth,    3., sumsc, 
cc     +                    MXLNCL, MXLTCL,  sumsc,   iret )
c                  END IF
c                  IF  ( itype .NE. 'r' .AND. stype .NE. 'r' )  THEN
c                     DO  j = 5, MXLTCL - 4
c			DO  i = 1, MXLNCL
c                           IF  ( sumsc (i, j, 2) .LT. 0. )  
c     +				sumsc (i, j, 2) = RMISS
c			END DO
c                     END DO
cc                     DO  j = 1, 4
c			DO  i = 1, MXLNCL
c                           sumsc (i, j,              2) = RMISS
c                           sumsc (i, MXLTCL + 1 - j, 2) = RMISS
c			END DO
c                     END DO
c                     CALL SMOOTH  ( ismth,  smth,    3., 
c     +                    sumic (1, 1, 2), MXLNCL, MXLTCL, 
c     +                    sumic (1, 1, 2), iret )
c                  END IF
cC     
cC     When smoothing, output the smoothed fields.
cC     
c                  IF  ( ismth .LT. 10 )  THEN
c                     WRITE ( cid, 6 ) ismth
c                  ELSE
c                     WRITE ( cid, 7 ) ismth
c                  END IF
c                  l4 = 'ssfix' // cid (1:1) // '.' // avgper (1:2)
c                  l7 = 'cofix' // cid (1:1) // '.' // avgper (1:2)
c                  l8 = 'adfix' // cid (1:1) // '.' // avgper (1:2)
cC     
c                  IF  ( stype .NE. 'a' )
c     +                 CALL DMPIMGI  ( sumsc,           bnam, 'r', l4,
c     +                 iret )
c                  IF  ( stype .NE. 'r' )
c     +                 CALL DMPIMGI  ( sumsc (1, 1, 2),  bnam, 'a',l4,
c     +                 iret )
c               END IF
cC     
cC     Compute the ratios for the data requested and dump out
cC     the results.
cC     
c               IF  ( itype .NE. 'a' .AND. stype .NE. 'a' )  THEN
c                  CALL MONCOR  ( sumic, sumsc, MXLNCL, MXLTCL, a, b, 
c     +                 iret )
c                  CALL DMPIMGI ( b, bnam, 'r', l7, iret )
c                  CALL DMPIMGI ( a, bnam, 'r', l8, iret )
c               END IF
c               IF  ( itype .NE. 'r' .AND. stype .NE. 'r' )  THEN
c                  CALL MONCOR  ( sumic (1, 1, 2), sumsc (1, 1, 2), 
c     +                 MXLNCL, MXLTCL,  
c     +                 a     (1, 1, 2), b     (1, 1, 2), 
c     +                 iret )
c                  CALL DMPIMGI ( b (1, 1, 2), bnam, 'a', l7, iret)
c                  CALL DMPIMGI ( a (1, 1, 2), bnam, 'a', l8, iret)
c               END IF
cC     
c	    END IF

     
C     End of quality-edited code (only run for stype = 'd').
C     Increment counters for beginning the next interval.  This
C     is also used to skip unmatched beginning pentads for 
C     'mm' calculations.
C     
 400     IF  ( ipp .EQ. 73 )  THEN
            iyyyy = iyyyy + 1
            imm   = 1
            ipp   = 1
         ELSE
            IF  ( ipp .GT. mbounds (3, imm) )  imm = imm + 1
            ipp = ipp + 1
         END IF
         iyyyymm = iyyyy * 100 + imm
         iyyyypp = iyyyy * 100 + ipp
C     
      END DO
C     
C     Reached end of requested data.
C     
      WRITE (*, *) 'Reached end of requested data.'
C     
 500  CONTINUE
C     

      end subroutine corr
      
