	SUBROUTINE INDATI  ( datnam, inst, iyyyypp, cnam, histc, iret )
C************************************************************************
C* INDATI  read in data values						*
C*									*
C* This subroutine reads in a data file and checks the retrieved pentad	*
C* against the requested pentad number for consistency.  In the case of	*
C* merged data, zeros are returned for all fields of each pixel not	*
C* having data from the specified instrument(s).			*
C*									*
C* For 3hr-geo data only, an optional "cut file" specifies box-by-box	*
C* which image time indices (if any) are closest to "ascending" and	*
C* "descending" times and contribute to the return array.  If the cut	*
C* file name is missing, the 1st and 5th times are used.  		*
C*									*
C* INDATI  ( datnam, inst, iyyyypp, cnam, histc, iret )			*
C*									*
C* Input parameters:							*
C*   datnam	CHAR	Data file name					*
C*   inst	INT	Instrument id; 1 = 'n10',     2 = 'n11',	*
C*				       3 = 'geo',     4 = 'geo-n10-n11',*
C*				       5 = 'geo-leo', 6 = 'all'		*
C*				       7 = 'n10flux' 14 = '3-hr geo'	*
C*   iyyyypp	INT	User-specified pentad number, as YYYYPP;	*
C*			0 = record 1					*
C*   cnam	CHAR	Cut file name, only applicable for inst=14;	*
C*			file contains (x,y,node)=(144,32,a|d) entries	*
C*			of time index (1-8) to be cut from 3hr arrays	*
C*			and accumulated in output array;		*
C*			if cnam=' ', def = 1|5 for entire a|d arrays	*
C*									*
C* Output parameters:							*
C*   histc	REAL	Histogram array (size MXLNC,MXLTC,MAXBIN+3)	*
C*			Extra fields are asc frac, platform, # images	*
C*			Platform is coded as:				*
C*			 0 = no data (-9999)            		*
C*			 1 = GMS (1)                    		*
C*			 2 = GOES-W (2)                 		*
C*			 3 = GOES-E (3)                 		*
C*			 4 = GOES-E&-W (23)             		*
C*			 5 = METEOSAT (4)               		*
C*			 6 = NOAA-10 (10)               		*
C*			 7 = NOAA-11 (11)               		*
C*			 8 = NOAA-10&-11 (1011)         		*
C*			 9 = NOAA-9 flux (9000)         		*
C*			10 = NOAA-10 flux (10000)       		*
C*			11 = NOAA-11 flux (11000)       		*
C*			12 = NOAA-9&-10 flux (19000)    		*
C*			13 = NOAA-10&-11 flux (21000)   		*
C*			14 = 3-hr Geo (actual sat id is returned)	*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C*				    -2 = out-of-bounds date; 0'ed data	*
C*				    -3 = cut file not accessible	*
C**									*
C* Log:									*
C* G.Huffman/USRA	11/91	Adapt INRES				*
C* G.Huffman/USRA	 1/92	Split up INCLUDE; enable using merged	*
C* G.Huffman/USRA	 6/92	Include N10 flux			*
C* G.Huffman/USRA	 2/93	Initialize if outside date range	*
C* G.Huffman/SSAI	 8/94	Add 3hr-geo				*
C* G.Huffman/SSAI	 4/97	Only 8801-8973 sequential access, rest	*
C*				direct for 3hr-geo			*
C* G.Huffman/SSAI	 5/97	Account for GMS only pentad-avg (in	*
C*				00Z) before 8713			*
C* D.Bolvin/SSAI	10/97	Don't increment histc counter when ASC	*
C*				or DSC matchup data is missing		*
C* G.Huffman/SSAI	11/97	Pitch 3-hr if coldest class is 10000	*
C* G.Huffman/SSAI	 5/00	Renovate to IRIX6, YYYY, mm/d		*
C* G.Huffman/SSAI	 9/02	Byte size on declarations		*
C* G.Huffman/SSAI	 2/11	Shift cut file based on satellite time	*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	l_dn,   l_cn,   iyyyy, ipp,    irec,  iyyyypp4,
     +			lrecl,  i, j,   k, m,  iostat, idum1, idum2, 
     +			icutij, jsatij, icount, ishift
C
c	INCLUDE		'param_2.5.h'
	REAL*4	        DLAT,   DLONC,  BNDRY
	INTEGER*4	MXLTC, MXLNC, MAXBIN

	PARAMETER	( DLAT   = 2.5,
     +			  DLONC  = 2.5,
     +			  BNDRY  = 50.,
     +			  MXLTC  = ( 2 * BNDRY ) / DLAT,
     +			  MXLNC  = 360 / DLONC,
     +			  MAXBIN = 32)

C
	CHARACTER*256	datnam, cnam
c	CHARACTER*(*)	datnam, cnam
	INTEGER*4	inst, iyyyypp, iret
	CHARACTER*256	opnnam
	REAL*4		histc (MXLNC, MXLTC, MAXBIN + 3)
	REAL*4		datin (144,   40,    5)
	INTEGER*2	jhist (144, 32, 8, 16), ihist (144, 32, 16),
     +			jsat  (144, 32),        iprcp (144, 32), 
     +			iyypp2,                 nimg  (5)
	INTEGER*1	icut  (144, 32, 2),     isat  (144, 32),
     +			nobs  (144, 32),       itmp  (144)
c	STATIC		icut
	EQUIVALENCE	( jhist (1, 1, 1, 1), ihist (1, 1, 1) ),
     +			( jsat  (1, 1),       iprcp (1, 1)    ),
     +			( icut  (1, 1, 1),    isat  (1, 1)    ),
     +			( icut  (1, 1, 2),    nobs  (1, 1)    )

c	DATA		opnnam / ' ' /
cdebug
	INTEGER*4	iunit, nin, INITA_DISK, WRITE_DISK, EXIT_DISK
cdebug
C



	IF  ( datnam .EQ. ' ' )  THEN
	    WRITE ( *, * ) 'Data file name is blank'
	    iret = -1
	    RETURN
	END IF
c	CALL ST_LSTR ( datnam, l_dn, iret )
c	CALL ST_LSTR ( cnam,   l_cn, iret )

	call String_Length(datnam, l_dn)
	call String_Length(cnam, l_cn)

c       Blank out opnnam - was done using DATA statement, but did not seem to work....
	opnnam=' '

C	First type of data: NOAA-10 or NOAA-11.  Their archives begin
C	on pentads 8858 and 8907, respectively, so the direct-access
C	record for any particular YYYYPP is
C
C	  irec = ( YYYY - 1988 ) * 73 + ( PP - 57 ), or
C	  irec = ( YYYY - 1989 ) * 73 * ( PP -  6 )
C
	IF  ( inst .EQ. 1 .OR. inst .EQ. 2 )  THEN
	    IF  ( iyyyypp .EQ. 0 )  THEN
		irec = 1
	      ELSE
		iyyyy = iyyyypp / 100
		ipp   = MOD ( iyyyypp, 100 )
		IF  ( inst .EQ. 1 )  THEN
		    irec = ( iyyyy - 1988 ) * 73 + ( ipp - 57 )
		  ELSE
		    irec = ( iyyyy - 1989 ) * 73 + ( ipp -  6 )
		END IF
	    END IF
C
C	    If the date are out-of-bounds, set missing values for 
C	    histogram, fraction of ascending, platform, and number of 
C	    sampled images.
C
	    IF  ( irec .LT. 1 .OR. irec .GT. 250 )  THEN
		WRITE (*, *) 'Record number =', irec, ' implied by date',
     +			     iyyyypp, ' out of bounds;'
		WRITE (*, *) 'providing zeroed data'
		iret = -2
		iyyyypp4 = -1
		DO  j = 1, 40
		DO  i = 1, 144
		    histc (i, j, 1) = 0.
		    histc (i, j, 2) = 0.
		    histc (i, j, 3) = 0.
		    histc (i, j, MAXBIN + 1) = -1.
		    histc (i, j, MAXBIN + 2) = inst + 4.
		    histc (i, j, MAXBIN + 3) = 0.
		END DO
		END DO
		RETURN
	    END IF
C
C	    Open the data file (error 126 is "unit already in use"; we
C	    assume that that's o.k., in the case where the file is used
C	    multiple times, and proceed).
C
	    OPEN  ( UNIT   = 10,              ACCESS = 'DIRECT', 
     +		    FILE   = datnam (1:l_dn), FORM   = 'UNFORMATTED',
     +		    RECL   = 4*28801,           STATUS = 'OLD',
     +		    ERR    = 1,               IOSTAT = iostat ,
     +              CONVERT='BIG_ENDIAN')
	    GO TO 2
C
    1	        IF  ( iostat .EQ. 126 )  THEN
		    WRITE (*, *) 'Note: tried to open', datnam (1:l_dn),
     +				 ' again; assumed this is o.k.'
		    GO TO 2
		END IF
		WRITE ( *, * ) 'Open error ', iostat, ' on ',
     +			       datnam (1:l_dn)
	        iret = -1
	        RETURN
   2	    CONTINUE
C
C	    Get the data and check the pentad number.  The date in the
C	    data file has 2-digit year, but it's always 19YY.
C
c	    READ  ( 10, REC = irec, END = 5 )  iyyyypp4, datin
	    READ  ( 10, REC = irec)  iyyyypp4, datin
c	    GO TO 6
C
c    5	    WRITE ( *, * ) 'Short data file: ', datnam (1:l_dn)
c	        iret = -1
c	        RETURN
    6	    CONTINUE
	    CLOSE ( 10 )
	    iyyyypp4 = 190000 + iyyyypp4
	    IF  ( iyyyypp4 .NE. iyyyypp .AND. iyyyypp4 .GT. 0 .AND.
     +		  iyyyypp  .NE. 0 )  THEN
	        WRITE (*, *)  'IR pentad found = ', iyyyypp4,
     +			      ' .NE. pentad requested = ', iyyyypp
	        iret = -1
C
C	      If good, load the 3-bin histograms into the internal 
C	      array, and then the fraction of ascending.
C
	      ELSE
		IF  ( iyyyypp4 .LE. 0 )
     +		    WRITE (*, *) 'Warning: pentad', iyyyypp, 
     +				 ' apparently has zeroed IR data'
		DO  k = 1, 3
		DO  j = 1, 40
		DO  i = 1, 144
		    histc (i, j, k) = datin (i, j, k)
		END DO
		END DO
		END DO
C
C	      Load the fraction of ascending images.
C
		DO  j = 1, 40
		DO  i = 1, 144
		    IF  ( datin (i, j, 4) + datin (i, j, 5) .EQ.
     +			  0 )  THEN
			histc (i, j, MAXBIN + 1) = -.1
		      ELSE
			histc (i, j, MAXBIN + 1) = datin (i, j, 4) /
     +						   ( datin (i, j, 4) +
     +						     datin (i, j, 5) )
		    END IF
		END DO
		END DO
C
C		The platform is pretty boring, but here it is!
C
		DO  j = 1, 40
		DO  i = 1, 144
		    IF  ( datin (i, j, 4) + datin (i, j, 5) .EQ.
     +			  0 )  THEN
			histc (i, j, MAXBIN + 2) = 0.
		      ELSE
			histc (i, j, MAXBIN + 2) = inst + 5.
		    END IF
		END DO
		END DO
C
C		Load the sampled image number.
C
		DO  j = 1, 40
		DO  i = 1, 144
		    histc (i, j, MAXBIN + 3) = datin (i, j, 4) +
     +					       datin (i, j, 5)
		END DO
		END DO
	    END IF
C
C	  Second data type:  NOAA-10 flux.  The archive begins
C	  on pentad 8743, so the sequential-access record for 
C	  any particular YYYYPP is
C
C	    irec = ( YYYY - 1987 ) * 73 + ( PP - 42 )
C
	  ELSE IF  ( inst .EQ. 7 )  THEN
	    IF  ( iyyyypp .EQ. 0 )  THEN
		irec = 1
	      ELSE
		iyyyy = iyyyypp / 100
		ipp   = MOD ( iyyyypp, 100 )
		irec = ( iyyyy - 1987 ) * 73 + ( ipp - 42 )
	    END IF
	    IF  ( irec .LT. 1 .OR. irec .GT. 285 )  THEN
		WRITE (*, *) 'Record number =', irec, ' implied by date',
     +			     iyyyypp, ' out of bounds;'
		WRITE (*, *) 'providing zeroed IR data'
		iret = -2
		GO TO 55
	    END IF
C
C	    Open the data file (error 126 is "unit already in use"; we
C	    assume that that's o.k., in the case where the file is used
C	    multiple times, and proceed).
C
	    OPEN  ( UNIT   = 10,              ACCESS = 'SEQUENTIAL', 
     +		    FILE   = datnam (1:l_dn), FORM   = 'FORMATTED',
     +		    STATUS = 'OLD',
     +		    ERR    = 51,              IOSTAT = iostat ,
     +              CONVERT='BIG_ENDIAN')
	    GO TO 52
C
   51	        IF  ( iostat .EQ. 126 )  THEN
		    WRITE (*, *) 'Note: tried to open', datnam (1:l_dn),
     +				 ' again; assumed this is o.k.'
		    GO TO 52
		END IF
		WRITE ( *, * ) 'Open error ', iostat, ' on ',
     +			       datnam (1:l_dn)
	        iret = -1
	        RETURN
   52	    CONTINUE
C
C	    Get the data and check the pentad number.  Internal data
C	    set dates are YY, but are always 19YY.
C
	    READ  ( 10, 54, END = 55 )  iyyyypp4, 
     +		(((datin (i, j, k), i = 1, 144), j = 1, 32), k = 1, 2)
   54	    FORMAT ( I4, (144F8.2) )
	    iyyyypp4 = 190000 + iyyyypp4
	    IF  ( iyyyypp4 .EQ. iyyyypp )  THEN
		GO TO 56
	      ELSE
		GO TO 52
	    END IF
C
C	    Can't find the pentad; set missing values for flux and
C	    number of sampled images.
C
   55		WRITE ( *, * ) 'Warning: pentad', iyyyypp,
     +			       ' apparently has zeroed AVHRR IR data'
		iyyyypp4 = -1
		DO  j = 1, 40
		DO  i = 1, 144
		    histc (i, j, 1         ) = 0.
		    histc (i, j, MAXBIN + 3) = 0.
		END DO
		END DO
C
		GO TO 57
C
C	      Otherwise, found the pentad; load the flux into the 
C	      first position in the internal array, and then the 
C	      sampled image number. 
C
   56		CONTINUE
		CLOSE ( 10 )
C
		DO  k = 1, MAXBIN + 3, MAXBIN + 2
		DO  j = 1, 4
		DO  i = 1, 144
		    histc (i, j,    k) = 0.
		    histc (i, 41-j, k) = 0.
		END DO
		END DO
		END DO
C
		DO  j = 1, 32
		DO  i = 1, 144
		    histc (i, j+4, 1) = datin (i, j, 1)
		END DO
		END DO
C
		DO  j = 1, 32
		DO  i = 1, 144
		    histc (i, j+4, MAXBIN + 3) = datin (i, j, 2)
		END DO
		END DO
C
C	    Load the fraction of ascending images (always N/A).
C
   57	    CONTINUE
	    DO  j = 1, 40
	    DO  i = 1, 144
		histc (i, j, MAXBIN + 1) = -.1
	    END DO
	    END DO
C
C	    The platform is pretty boring, but here it is!
C
	    DO  j = 1, 40
	    DO  i = 1, 144
		histc (i, j, MAXBIN + 2) = inst + 4
	    END DO
	    END DO
C
C	  Third data type:  3hr-geo (merged geo, with no leo, separated 
C	  into 3-hr fields).  The archive is by year, so the direct-
C	  access record for any particular YYYYPP is just PP.  Formerly 
C	  we started 1987 with pentad 37.
C
	 ELSE IF  ( inst .EQ. 14 )  THEN
	    
	    IF  ( iyyyypp .EQ. 0 )  THEN
	       irec = 1
cc	      ELSE IF  ( INT ( iyyyypp * .01 ) .EQ. 1987 )  THEN
cc		irec = MOD ( iyyyypp, 100 ) - 36
	    ELSE
	       irec = MOD ( iyyyypp, 100 )
	    END IF

	    IF  ( irec .LT. 1 .OR. irec .GT. 73 )  THEN
	       WRITE (*, *) 'Record number =', irec, ' implied by date',
     +			     iyyyypp, ' out of bounds'
	       STOP
	    END IF
C
C	    Every time there is a new data file name (usually just the
C	    first time) pull in the cut file (or fill in default) and open 
C	    the new data file.
C

	    IF  ( opnnam .NE. datnam )  THEN
	       opnnam = datnam
C
C		cnam is not blank; pull in cfil.
C
	       IF  ( l_cn .GE. 1 )  THEN
		  OPEN  ( UNIT   = 10,   ACCESS = 'DIRECT', 
     +			    FILE   = cnam, FORM   = 'UNFORMATTED',
     +			    RECL   = 4*2304, STATUS = 'OLD',
     +			    ERR    = 61,   IOSTAT = iostat )
		  GO TO 62
C
 61		  WRITE ( *, * ) 'Open error ', iostat, ' on ',
     +			       cnam (1:l_cn)
		  iret = -3
		  RETURN
 62		  CONTINUE
C
c		    READ  ( 10, REC = 1, END = 65 )  icut
		  READ  ( 10, REC = 1)  icut
cdebug
c	iunit = INITA_DISK ( 'merge.precip' )
c	IF  ( iunit .LE. 0 )  THEN
c	    WRITE (*, *) 'Stopping for error', -iunit
c	    STOP
c	END IF
c	nin   = WRITE_DISK ( iunit, iprcp, 9216 )
c	nin   = EXIT_DISK  ( iunit )
cdebug
		  GO TO 66

 65		  CLOSE ( 10 )
		  WRITE ( *, * ) 'Short data file: ', 
     +				       cnam (1:l_cn)
		  iret = -3
		  RETURN
 66		  CONTINUE
		  CLOSE ( 10 )
C
C		    Shift icut (with wrapping) to accommodate satellite 
C		    overpass times different than for the original cut 
C		    file.
C		    Example: Later   (F08)   6 \ 5 \ 4
C		             Earlier (F13)  6 \ 5 \ 4 \
C		    Increment the X index by 1 for each 10 min later:
C		    min/box = (24hr/day * 60 min/hr) / 144 boxes/day
C		    Treat one line at a time; consider positive and 
C		    negative shfits separately so blocks of the line 
C		    can be copied, first just shifted and then the 
C		    wrapped part.  The temporary line is copied back 
C		    into icut after it's built.
C
C temporary hack to solicit user input for the icut shift value.
C
cc	WRITE (*, *  ) 'Please input the offset for icut:'
c	READ  (*, 909) ishift
c  909	FORMAT ( I2 )
c	WRITE (*, *  ) 'ishift =', ishift, ' orig icut ='
c	WRITE (*, 910) ( ( (icut (i, j, k), i=1,144 ), j=1,32 ), k=1,2 )
c  910	FORMAT ( 72I1 )
C
		  IF  ( ishift .GT. 0 )  THEN
		     DO  k = 1, 2
			DO  j = 1, 32
			   DO  i = 1, 144-ishift
			      itmp (i+ishift)     = icut (i, j, k)
			   END DO
			   DO  i = 145-ishift, 144
			      itmp (i+ishift-144) = icut (i, j, k)
			   END DO
			   DO  i = 1, 144
			      icut (i, j, k) = itmp (i)
			   END DO
			END DO
		     END DO
		  ELSE IF  ( ishift .LT. 0 )  THEN
		     DO  k = 1, 2
			DO  j = 1, 32
			   DO  i = 1-ishift, 144
			      itmp (i+ishift)     = icut (i, j, k)
			   END DO
			   DO  i = 1, 0-ishift
			      itmp (i+ishift+144) = icut (i, j, k)
			   END DO
			   DO  i = 1, 144
			      icut (i, j, k) = itmp (i)
			   END DO
			END DO
		     END DO
C
C			And if ishift is zero, nothing needs to be done.
C
		  END IF
C temporary debug write
		  WRITE (*, *) 'ishift =', ishift, ' shifted icut ='
c	WRITE (*, 910) ( ( (icut (i, j, k), i=1,144 ), j=1,32 ), k=1,2 )
C
C		  cfil is blank; fill the cut file with the default.
C
	       ELSE
		  DO  j = 1, 32
		     DO  i = 1, 144
			icut (i, j, 1) = 1
			icut (i, j, 2) = 5
		     END DO
		  END DO
	       END IF
C
C		Open the data file.  Data files written before 1991 
C		(currently 1988-1990) are written unformatted 
C		sequential, but we are reading unformatted direct, so 
C		there are 4 bytes to dummy at the beginning and end.  
C		1986-1987 and 1991-present are written unformatted 
C		direct.
C

	       IF  ( iyyyypp .LT. 199101 .AND. iyyyypp .GT. 198773 )  
     +								 THEN
		  lrecl = 297221
	       ELSE
		  lrecl = 297219
	       END IF

	       OPEN  ( UNIT   = 10,              ACCESS = 'DIRECT', 
     +			FILE   = datnam (1:l_dn), 
     +			FORM   = 'UNFORMATTED',
     +			RECL   = 4*lrecl,           STATUS = 'OLD',
     +			ERR    = 71,              IOSTAT = iostat  ,
     +                  CONVERT='BIG_ENDIAN')
	       GO TO 72
C
 71	       WRITE ( *, * ) 'Open error ', iostat, ' on ',
     +				   datnam (1:l_dn)
	       iret = -1
	       RETURN
 72	       CONTINUE
	    END IF


C
C	    Get the data record (depending on before/after 199101) and 
C	    check the pentad number.  The dates inside the data set have
C	    YY, so we have to account for 1900 vs. 2000 ( as well as
C	    missing).
C
	    IF  ( iyyyypp .LT. 199101 .AND. iyyyypp .GT. 198773 )  THEN
c		READ  ( 10, REC = irec, END = 75 ) 
c     +		    idum1, iyypp2, nimg, jsat, jhist, idum2
	       READ  ( 10, REC = irec) 
     +		    idum1, iyypp2, nimg, jsat, jhist, idum2
	    ELSE
c		READ  ( 10, REC = irec, END = 75 )
c     +		    iyypp2, nimg, jsat,  jhist
	       READ  ( 10, REC = irec )
     +		    iyypp2, nimg, jsat,  jhist
	    END IF
	    IF  ( iyypp2 .GT. 5000 )  THEN
	       iyyyypp4 = 190000 + iyypp2
	    ELSE IF  ( iyypp2 .GT. 0 )  THEN
	       iyyyypp4 = 200000 + iyypp2
	    ELSE
	       iyyyypp4 = iyypp2
	    END IF
C
C	    Fix to handle GMS with only data in coldest class.
C
	    DO  j = 1, 32
	       DO  i = 1, 144
		  DO  k = 1, 8
		     IF  ( jhist (i, j, k, 16) .EQ. 10000 )
     +		    jhist (i, j, k, 16) = 0
		  END DO
	       END DO
	    END DO

cdebug
c	iunit = INITA_DISK ( 'merge.precip' )
c	IF  ( iunit .LE. 0 )  THEN
c	    WRITE (*, *) 'Stopping for error', -iunit
c	    STOP
c	END IF
c	nin   = WRITE_DISK ( iunit, iprcp, 9216 )
c	nin   = EXIT_DISK  ( iunit )
C	IF  (iyyyypp .EQ. 198701) THEN
C	    iunit = INITA_DISK ('jsat.out')
C	    nin = WRITE_DISK (iunit, jsat, 9216)
C	    nin = EXIT_DISK (iunit)
C	END IF
cdebug
c	    GO TO 76
C
c   75		WRITE ( *, * ) 'Short data file: ', datnam (1:l_dn)
c		iret = -1
c		RETURN
 76	    CONTINUE
C
C	    For the pentads 198601-198712 GMS is only available for 
C	    pentad averages, which are loaded into the 00Z bins.  So, 
C	    in that time period check for GMS and the cut file value 
C	    not zero (indicates no contribution desired at that point/
C	    time) and set it to 1 (INTEGER*1, so have to be careful 
C	    with math).
C
	    IF  ( iyyyypp .LT. 198713 )  THEN
	       DO  j = 1, 32
		  DO  i = 1, 144
		     IF  ( jsat (i, j) .EQ. 1 )  THEN
			IF  ( icut (i, j, 1) .NE. 0 )
     +			    icut (i, j, 1) = 1
			IF  ( icut (i, j, 2) .NE. 0 )
     +			    icut (i, j, 2) = 1
		     END IF
		  END DO
	       END DO
	    END IF
C
C	    Check that the pentad number requested was the one read.
C
	    IF  ( iyyyypp4 .NE. iyyyypp .AND. iyyyypp4 .GT. 0 .AND.
     +		  iyyyypp  .NE. 0 )  THEN
	       WRITE (*, *)  'Pentad found = ', iyypp2,
     +			      ' .NE. pentad requested = ', iyyyypp
	       iret = -1
C
C	      If good, compute 16-bin histograms from cutouts of the
C	      3hr-geo specified by icut and divid by the number of 
C	      samples (usually 2) to get the pentad average.  Icut has 
C	      ascending and descending arrays; if only one is needed, 
C	      the other should be passed in as zeros.  This scheme 
C	      automatically pads the 40-50 N and S lat bands with 0.  
C	      First, zero accumulation arrays
C
	    ELSE
cdebug
c	write (*, *) 'jsat:'
c	WRITE (*, 992) ( i, ( jsat (i, 33-j), j=1,32 ), i=1,144 )
c992	FORMAT ( I4, 32I2 )
c	       i = 1
c	       j = 22
c	write (*, *) 'METEOSAT i,j,jhist(hr,bin) =', i, j, 
c     +	    (( jhist (i, j, k, m), k = 1, 8 ), m = 1, 16)
c	       i = 70
c	       j = 22
c	write (*, *) 'GMS i,j,jhist(hr,bin) =', i, j, 
c     +	    (( jhist (i, j, k, m), k = 1, 8 ), m = 1, 16)
c	       i = 100
c	       j = 22
c	write (*, *) 'GOES-W i,j,jhist(hr,bin) =', i, j, 
c     +	    (( jhist (i, j, k, m), k = 1, 8 ), m = 1, 16)
cdebug
	       IF  ( iyyyypp4 .LE. 0 )
     +		    WRITE (*, *) 'Warning: pentad', iyyyypp, 
     +				 ' apparently has zeroed IR data'
C
	       DO  k = 1, 17
		  DO  j = 1, 40
		     DO  i = 1, 144
			histc (i, j, k) = 0.
		     END DO
		  END DO
	       END DO
	       
c	       j=18
c	       do i=105,115
c		  print *, "0",i,j,histc(i,j,1),histc(i,j,2),histc(i,j,3)
c	       end do
c	       print *, "-"
	       
C       
C		Do ascending, then descending.  icut is INT*1, so it is
C		transferred to icutij (INT*4) for the indexing.
C
	       DO  j = 1, 32
		  DO  i = 1, 144
		     icutij = icut (i, j, 1)
		     IF  ( icutij      .GT. 0 .AND.
     +			  jsat (i, j) .GT. 0 )  THEN
			icount = 0
			DO  m = 1, 16
			   histc (i, j+4, m) = histc (i, j+4, m) +
     +			    jhist (i, j, icutij, m)
     			   icount = icount + jhist (i, j, icutij, m)
			END DO
			IF  (icount .NE. 0) THEN
			   histc (i, j+4, 17) = histc (i, j+4, 17) + 1
			END IF
		     END IF
		  END DO
	       END DO
C


c	       j=18
c	       do i=105,115
c		  print *, "1",i,j,histc(i,j,1),histc(i,j,2),histc(i,j,3)
c	       end do
c	       print *, "-"
	       

	       DO  j = 1, 32
		  DO  i = 1, 144
		     icutij = icut (i, j, 2)
		     IF  ( icutij      .GT. 0 .AND.
     +			  jsat (i, j) .GT. 0 )  THEN
     			icount = 0
			DO  m = 1, 16
			    histc (i, j+4, m) = histc (i, j+4, m) +
     +			    jhist (i, j, icutij, m)
     			    icount = icount + jhist (i, j, icutij, m)
			END DO
			IF  (icount .NE. 0) THEN
			    histc (i, j+4, 17) = histc (i, j+4, 17) + 1
			END IF
		    END IF
		END DO
		END DO
C
C		Compute the average.  Missings already are zeroed.
C		The factor .0001 converts the integer format above.
C
		DO  m = 1, 16
		DO  j = 5, 36
		DO  i = 1, 144
		    IF  ( histc (i, j, 17) .GT. 0. )  THEN
			histc (i, j, m) = histc (i, j, m) * .0001 /
     +					  histc (i, j, 17)
		    END IF
		END DO
		END DO
		END DO
C
C		Still good; load the fraction of ascending (N/A from
C		3hr-geo data), then load the number of images and 
C		platform id into the internal arrays (padding the 40-50 N 
C		and S lat bands with 0).  These are done here because
C		the cutout routines above use MAXBIN+1 to store the number 
C		of cutouts contributing.
C
		DO  j = 1, 40
		DO  i = 1, 144
		    histc (i, j, MAXBIN + 1) = -.1
		END DO
		END DO
C
		DO  j = 1, 4
		DO  i = 1, 144
		    histc (i, j,    MAXBIN + 2) = 0.
		    histc (i, 41-j, MAXBIN + 2) = 0.
		    histc (i, j,    MAXBIN + 3) = 0.
		    histc (i, 41-j, MAXBIN + 3) = 0.
		END DO
		END DO
C
C		jsatij converts the INT*2 jsat to INT*4.
C
		DO  j = 1, 32
		DO  i = 1, 144
		    jsatij = jsat (i, j)

		    IF  ( jsatij .GE. 1 .AND.
     +			  jsatij .LE. 3 )  THEN
			histc (i, j + 4, MAXBIN + 2) = jsatij
			histc (i, j + 4, MAXBIN + 3) = nimg (jsatij)
		      ELSE IF  ( jsatij .EQ. 4 )  THEN
		        histc (i, j + 4, MAXBIN + 2) = 5.
		        histc (i, j + 4, MAXBIN + 3) = nimg (4) 
		      ELSE IF  ( jsatij .EQ. 23 )  THEN
		        histc (i, j + 4, MAXBIN + 2) = 4.
		        histc (i, j + 4, MAXBIN + 3) = nimg (2) + 
     +						       nimg (3)
		      ELSE IF  ( jsatij .EQ. -9999 )  THEN
		        histc (i, j + 4, MAXBIN + 2) = 0.
		        histc (i, j + 4, MAXBIN + 3) = 0.
		      ELSE
		        histc (i, j + 4, MAXBIN + 2) = 0.
		        histc (i, j + 4, MAXBIN + 3) = 0.
		        WRITE (*, *) 'ERROR: At i,j =', i, j,
     +				     ' unknown satellite id =', 
     +				     jsat (i, j)
		    END IF
		END DO
		END DO
	    END IF
C
C	  Fourth data type: Merged geo data.  The archive is by year,
C	  so the direct-access record for any particular YYYYPP is just 
C	  PP.
C
	  ELSE
	    IF  ( iyyyypp .EQ. 0 )  THEN
		irec = 1
	      ELSE
		irec = MOD ( iyyyypp, 100 )
	    END IF
	    IF  ( irec .LT. 1 .OR. irec .GT. 73 )  THEN
		WRITE (*, *) 'Record number =',irec, ' implied by date',
     +			     iyyyypp, ' out of bounds'
		STOP
	    END IF
C
C	    Open the data file.
C
	    OPEN  ( UNIT   = 10,              ACCESS = 'DIRECT', 
     +		    FILE   = datnam (1:l_dn), FORM   = 'UNFORMATTED',
     +		    RECL   = 4*41473,           STATUS = 'OLD',
     +		    ERR    = 111,             IOSTAT = iostat,
     +              CONVERT='BIG_ENDIAN')
	    GO TO 112
C
  111	        WRITE ( *, * ) 'Open error ', iostat, ' on ',
     +			       datnam (1:l_dn)
	        iret = -1
	        RETURN
  112	    CONTINUE
C
C	    Get the data and check the pentad number.  Internal data set
C	    dates are YY, but they are always 19YY.
C
c	    READ  ( 10, REC = irec, END = 115 )  iyypp2, iprcp,  
c     +					  isat, nobs, ihist
	    READ  ( 10, REC = irec)  iyypp2, iprcp,  
     +					  isat, nobs, ihist
	    IF  ( iyypp2 .GT. 5000 )  THEN
		iyyyypp4 = 190000 + iyypp2
	      ELSE IF  ( iyypp2 .GT. 0 )  THEN
		iyyyypp4 = 200000 + iyypp2
	      ELSE
		iyyyypp4 = iyypp2
	    END IF
cdebug
c	iunit = INITA_DISK ( 'merge.precip' )
c	IF  ( iunit .LE. 0 )  THEN
c	    WRITE (*, *) 'Stopping for error', -iunit
c	    STOP
c	END IF
c	nin   = WRITE_DISK ( iunit, iprcp, 9216 )
c	nin   = EXIT_DISK  ( iunit )
cdebug
c	    GO TO 116
C
c  115	    WRITE ( *, * ) 'Short data file: ', datnam (1:l_dn)
c	        iret = -1
c	        RETURN
c  116	    CONTINUE
	    CLOSE ( 10 )
	    IF  ( iyyyypp4 .NE. iyyyypp .AND. iyyyypp4 .GT. 0 .AND.
     +		  iyyyypp  .NE. 0 )  THEN
	        WRITE (*, *)  'Pentad found = ', iyypp2,
     +			      ' .NE. pentad requested = ', iyyyypp
	        iret = -1
C
C	      If good, load the fraction of ascending (N/A from
C	      merged data).
C
	      ELSE
cdebug
c	WRITE (*, 998) ( i, ( isat (i, 33-j), j=1,32 ), i=1,144 )
c998	FORMAT ( I4, 32I2 )
	j = 20
cc	WRITE (*, *) 'isat column', i,' =', ( isat (i, j), j=1,32)
c	do i = 74, 79
c	write (*, *) 'j=20, i, ihist, iprcp', i, 
c     +	    ( ihist (i, j, k), k = 1, 16 ), iprcp (i, j)
c	end do
cdebug
		IF  ( iyyyypp4 .LE. 0 )
     +		    WRITE (*, *) 'Warning: pentad', iyyyypp, 
     +				 ' apparently has zeroed IR data'
C
		DO  j = 1, 40
		DO  i = 1, 144
		    histc (i, j, MAXBIN + 1) = -.1
		END DO
		END DO
C
C		Still good; load the platform value into the internal 
C		array (padding the 40-50 N and S lat bands with 0)
C		for the selected instrument(s).
C
		DO  j = 1, 4
		DO  i = 1, 144
		    histc (i, j,    MAXBIN + 2) = 0.
		    histc (i, 41-j, MAXBIN + 2) = 0.
		END DO
		END DO
C
C		Platform; pull out GEO data only. 
C
		IF  ( inst .EQ. 3 )  THEN
		    DO  j = 1,32
		    DO  i = 1, 144
			IF  ( isat (i, j) .GE. 1 .AND. 
     +			      isat (i, j) .LE. 5 )  THEN
			    histc (i, j + 4, MAXBIN + 2) = isat (i, j)
			  ELSE
			    histc (i, j + 4, MAXBIN + 2) = 0.
			END IF
		    END DO
		    END DO
C
C		  Platform; pull out GEO or N10 *and* N11.
C
		  ELSE IF  ( inst .EQ. 4 )  THEN
		    DO  j = 1,32
		    DO  i = 1, 144
			IF  ( ( isat (i, j) .GE. 1 .AND. 
     +				isat (i, j) .LE. 5 ) .OR.
     +			      isat (i, j) .EQ. 8 )  THEN
			    histc (i, j + 4, MAXBIN + 2) = isat (i, j)
			  ELSE
			    histc (i, j + 4, MAXBIN + 2) = 0.
			END IF
		    END DO
		    END DO
C
C		  Platform; pull out GEO or N10 *and/or* N11.
C
		  ELSE IF  ( inst .EQ. 5 )  THEN
		    DO  j = 1,32
		    DO  i = 1, 144
			IF  ( isat (i, j) .GE. 1 .AND. 
     +			      isat (i, j) .LE. 8 )  THEN
			    histc (i, j + 4, MAXBIN + 2) = isat (i, j)
			  ELSE
			    histc (i, j + 4, MAXBIN + 2) = 0.
			END IF
		    END DO
		    END DO
C
C		  Platform; take all merged data.
C
		  ELSE
		    DO  j = 1,32
		    DO  i = 1, 144
			histc (i, j + 4, MAXBIN + 2) = isat (i, j)
		    END DO
		    END DO
		END IF
C
C		Still good; load the sampled image number into the 
C		internal array (padding the 40-50 N and S lat bands 
C		with 0).  histc (i, j + 4, MAXBIN + 2) already reflects 
C		the instrument choice.
C
		DO  j = 1, 4
		DO  i = 1, 144
		    histc (i, j,    MAXBIN + 3) = 0.
		    histc (i, 41-j, MAXBIN + 3) = 0.
		END DO
		END DO
C
		DO  j = 1, 32
		DO  i = 1, 144
		    IF  ( histc (i, j + 4, MAXBIN + 2) .GT. 0 )  THEN
			histc (i, j + 4, MAXBIN + 3) = nobs (i, j)
		      ELSE
			histc (i, j + 4, MAXBIN + 3) = 0.
		    END IF
		END DO
		END DO
C
C		Still good; load the 16-bin histograms into the internal 
C		array (padding the 40-50 N and S lat bands with 0).  If
C		only fluxes are available load the precip est. into
C		bin 1.  The conversion of 500 is because iprcp is in
C		.01 mm/5 days.  histc (i, j + 4, MAXBIN + 2) already
C		reflects the instrument choice.
C
		DO  k = 1, 16
		DO  j = 1, 4
		DO  i = 1, 144
		    histc (i, j,      k) = 0.
		    histc (i, 41 - j, k) = 0.
		END DO
		END DO
		END DO
C
		DO  j = 1, 32
		DO  i = 1, 144
		    IF  ( histc (i, j + 4, MAXBIN + 2) .LE. 0 )  THEN
			DO  k = 1, 16
			    histc (i, j + 4, k) = 0.
			END DO
		      ELSE IF  ( histc (i, j + 4, MAXBIN + 2) .LE. 8 )
     +								  THEN
			DO  k = 1, 16
			    histc (i, j + 4, k) = ihist (i, j, k)
			END DO
		      ELSE
			histc (i, j + 4, 1) = iprcp (i, j) / 500.
			DO  k = 2, 16
			    histc (i, j + 4, k) = 0.
			END DO
		    END IF
		END DO
		END DO
	    END IF
	END IF
C
c
c	j=18
c	do i=105,115
c	   print *, "2",i,j,histc(i,j,1),histc(i,j,2),histc(i,j,3)
c	end do
	
	RETURN
	END
