	SUBROUTINE IPREP  ( inam,    inst,  iyyyypp, binlimg, n_blg, 
     +			    binliml, n_bll, cnam,    do_zcor, zcor,
     +			    sumic,   iret )
C************************************************************************
C* IPREP  Prepare a pentad of IR data					*
C*									*
C* This subroutine reads in the specified pentad(s) and computes the	*
C* rain rate and area for IR data with zenith-angle correction on 	*
C* geo, if requested.							*
C*									*
C* IPREP  ( inam, inst,    iyyyypp, binlimg, n_blg, binliml, n_bll,	*
C*          cnam, do_zcor, zcor,    sumic,   iret )			*
C*									*
C* Input parameters:							*
C*   inam	CHAR*	IR data file name				*
C*   inst	INT	Instrument id; 1 = 'n10',     2 = 'n11',	*
C*				       3 = 'geo',     4 = 'geo-n10-n11',*
C*				       5 = 'geo-leo', 6 = 'all'		*
C*				       7 = 'n10flux' 14 = '3hr-geo'	*
C*   iyyyypp	INT	User-specified pentad number, as YYYYPP		*
C*   binlimg	REAL	GEO IR bin limits (size MAXBIN-1)		*
C*   n_blg	INT	Number of GEO IR bin limits 			*
C*   binliml	REAL	LEO IR bin limits (size MAXBIN-1)		*
C*			(N/A if no GEO)					*
C*   n_bll	INT	Number of LEO IR bin limits (N/A if no LEO)	*
C*   cnam	CHAR	Cut file name, only applicable for inst=14;	*
C*			file contains (x,y,node)=(144,32,a|d) entries	*
C*			of time index (1-8) to be cut from 3hr arrays	*
C*			and accumulated in output array;		*
C*			if cnam=' ', def = 1|5 for entire a|d arrays	*
C*   do_zcor	LOG	T|F = do|omit zenith-angle correction on geo	*
C*   zcor	REAL	Zenith-angle correction for the various geo	*
C*			satellites (GMS, GOES-W, GOES-E, GOES-W & -E,	*
C*			METEOSAT) (size MXLNC, MXLTC, 6)		*
C*									*
C* Output parameters:							*
C*   sumic	REAL	Rain statistic (size MXLNC,MXLTC,3)		*
C*			NOTE: MXLNC, XMLTC are currently 144,40, not	*
C*			720,200 as in CORR				*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/USRA	12/91	Adapt from SHOWIR			*
C* G.Huffman/USRA	 1/92	Update header; split up INCLUDE		*
C* G.Huffman/USRA	 6/92	Do N10 flux; drop ISMTH,ISTAT & always	*
C*				do rate and area			*
C* G.Huffman/USRA	 6/93	Document FCONST				*
C* G.Huffman/USRA	 7/93	Output number of images in sumic	*
C* G.Huffman/SSAI	 8/94	Add 3hr-geo				*
C* G.Huffman/SSAI	12/94	Add zcor				*
C* G.Huffman/SSAI	10/97	Add trap for missing partial pentad	*
C* G.Huffman/SSAI	 5/00	Update to IRIX6, YYYY, mm/d		*
C* G.Huffman/SSAI	 6/00	Add INSAT to zcor			*
C* G.Huffman/SSAI	 9/02	Byte size on declarations		*
C************************************************************************

	IMPLICIT	NONE
	INTEGER*4	ipp, ndx, i, j, k, kraing, krainl
	CHARACTER*4	cid
C
c	INCLUDE		'param_2.5.h'

	REAL*4	        DLAT,   DLONC,  BNDRY
c	REAL*4		RMISS, DLONNS
	INTEGER*4	MXLTC, MXLNC, MAXBIN
c	INTEGER*4	IVERSN,  MXLTNS, MXLNNS, MAXDEX, MXSTAT
C
c	PARAMETER	( RMISS  = -99999.,
c     +			  IVERSN = 0, 
	PARAMETER	( DLAT   = 2.5,
     +			  DLONC  = 2.5,
c     +			  DLONNS = 5.,
     +			  BNDRY  = 50.,
     +			  MXLTC  = ( 2 * BNDRY ) / DLAT,
c     +			  MXLTNS = ( 90. - BNDRY ) / DLAT,
     +			  MXLNC  = 360 / DLONC,
c     +			  MXLNNS = 360 / DLONNS,
c     +			  MAXDEX = 20,
     +			  MAXBIN = 32)
c     +			  MXSTAT = 20 )

C
	CHARACTER*256	inam, cnam
c	CHARACTER*(*)	inam, cnam
	REAL*4		binlimg (MAXBIN-1),      binliml (MAXBIN-1),
     +			sumic (MXLNC, MXLTC, 3), zcor (MXLNC, MXLTC, 6)
	INTEGER*4	inst, iyyyypp, n_blg,    n_bll, iret
	LOGICAL*1	do_zcor
C
	REAL*4		histc (MXLNC, MXLTC, MAXBIN+3), zcorijk
C
C	airc, xirc args are (ln,lt,type), where type is 1,2 for
C	raining, all.  sumic args are (ln,lt,type), where
C	type is 1,2,3 for rainrate, rain area, number of images.
C
	REAL*4		airc (MXLNC, MXLTC, 2), dum
C
	REAL*4		RTHRES, RCONST, FTHRES, FCONST
	PARAMETER	( RTHRES = 235.,
     +			  RCONST = 72.,
     +			  FTHRES = 255.15,
     +			  FCONST = 3.42 )


c	DO  j = 1, MXLTC
c	   DO  i = 1, MXLNC
c	      print *, i, j, zcor(i,j,1), zcor(i,j,2), zcor(i,j,3),
c    +                       zcor(i,j,4), zcor(i,j,5), zcor(i,j,6)
c	   end do
c	   end do

C
C	Trap the case of the data file missing the last (partial) 
C	pentad because it spans the end of the second or third quarter.
C	This problem doesn't exist at the end of the first and fourth
C	quarters.  If the data file name has the previous pentad named, 
C	assume that the file ends on that pentad, return missings and 
C	just print a warning.  The input data file likely has date as 
C	YYPP.
C
	ipp = MOD ( iyyyypp, 100 )
	IF  ( ipp .EQ. 37 .OR. ipp .EQ. 55 )  THEN
	    WRITE ( cid, 5 ) MOD ( iyyyypp, 10000 ) - 1
    5	    FORMAT ( I4.4 )
	    ndx = INDEX ( inam, cid )
	    IF  ( ndx .NE. 0 )  THEN
		DO  j = 1, MXLTC
		DO  i = 1, MXLNC
		    sumic (i, j, 1) = -1.
		    sumic (i, j, 2) = -.1
		    sumic (i, j, 3) = 0.
		END DO
		END DO
		WRITE (*, *) 'Warning: couldn''t find pentad ', cid, 
     +			     ' in ', inam
		WRITE (*, *) 'continuing with missings returned ',
     +			     'because ', cid
		WRITE (*, *) 'is the end of the second/third quarter'
		RETURN
	    END IF
	END IF


C
C	Read in data file.
C

	CALL INDATI  ( inam, inst, iyyyypp, cnam, histc, iret )
	IF  ( iret .EQ. -1 )  THEN
	    WRITE (*, *) 'Stopping for errors in INDATI'
	    STOP
	END IF
	iret = 0


c	j=18
c	do i=105,115
c	   print *, i,j,histc(i,j,1),histc(i,j,2),histc(i,j,3)
c	end do
	
c	j=18
c	do i=105,115
c	   print *, i,j,sumic(i,j,1),sumic(i,j,2),sumic(i,j,3)
c	end do
	




C
C	Load number of images.
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    sumic (i, j, 3) = histc (i, j, MAXBIN+3)
	END DO
	END DO
C
C	Handle N10 flux case: Following Janowiak & Arkin, 1991: JGR, 96
C	Suppl., pp 3359-3373, and multiplying their FCONST (in mm/5 days)
C	by 5 to get mm/d, the rain rate is 
C	  RR = ( FTHRES - OLR_flux ) / FCONST, except
C	  RR = 0                               OLR_flux > FTHRES.
C	Given this simple form for RR, one can infer rain area from 
C	flux-only data by dividing the flux rainrate by RCONST.
C
	IF  ( inst .EQ. 7 )  THEN
	    DO  j = 1, MXLTC
	    DO  i = 1, MXLNC
		IF  ( histc (i, j, MAXBIN+3) .LT. 1. )  THEN
		    sumic (i, j, 1) = -1.
		    sumic (i, j, 2) = -.1
		  ELSE IF  ( histc (i, j, 1) .GT. FTHRES )  THEN
		    sumic (i, j, 1) = 0.
		    sumic (i, j, 2) = 0.
		  ELSE
		    sumic (i, j, 1) = ( FTHRES - histc (i, j, 1) ) /
     +				      FCONST
		    sumic (i, j, 2) = sumic (i, j, 1) / RCONST
		END IF
	    END DO
	    END DO
ccc	    WRITE (*, *) 'RCONST, airc1, airc2, histc, sumic',
ccc     +	RCONST, airc(98,1,1), airc(98,1,2),
ccc     +	(histc (98,1,i), i=1,MAXBIN+3), sumic(98,1,1)
C
	    RETURN
	END IF

C
C	Done with N10 flux-only.  For all others, begin by zeroing the
C	accumulation array.
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    airc (i, j, 1) = 0.
	END DO
	END DO
C
C	Find the bin containing the rain threshold for GEO and LEO.
C
	kraing = 1
	DO  WHILE ( kraing .LE. n_blg .AND.
     +		    RTHRES .LE. binlimg (kraing) )
	    kraing = kraing + 1
	END DO
C
	krainl = 1
	DO  WHILE ( krainl .LE. n_bll .AND.
     +		    RTHRES .LE. binliml (krainl) )
	    krainl = krainl + 1
	END DO
C
C	Accumulate the "raining" pixels.
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    IF  ( histc (i, j, MAXBIN + 2) .LE. 0. )  THEN
	      ELSE IF ( histc (i, j, MAXBIN + 2) .LE. 5. )  THEN
		DO  k = kraing, n_blg + 1
		    airc (i, j, 1) = airc  (i, j, 1) + histc (i, j, k)
		END DO
	      ELSE IF ( histc (i, j, MAXBIN + 2) .LE. 8. )  THEN
		DO  k = krainl, n_bll + 1
		    airc (i, j, 1) = airc  (i, j, 1) + histc (i, j, k)
		END DO
	      ELSE
		airc (i, j, 1) = histc (i, j, 1)
	    END IF
	END DO
	END DO

C
C	Accumulate totals of values, initializing arrays
C	from raining accumulations.
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    airc (i, j, 2) = airc (i, j, 1)
	END DO
	END DO
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    IF  ( histc (i, j, MAXBIN + 2) .LE. 0. )  THEN
	      ELSE IF ( histc (i, j, MAXBIN + 2) .LE. 5. )  THEN
		DO  k = 1, kraing - 1
		    airc (i, j, 2) = airc  (i, j, 2) + histc (i, j, k)
		END DO
	      ELSE IF ( histc (i, j, MAXBIN + 2) .LE. 8. )  THEN
		DO  k = 1, krainl - 1
		    airc (i, j, 2) = airc  (i, j, 2) + histc (i, j, k)
		END DO
	    END IF
	END DO
	END DO

C
C	Find rain rate and fractional rain area.  For flux-only data in 
C	the merged files, rainrate is simply copied.  Otherwise, 
C	RR = RCONST * rain fraction.  Given the simple form of RR, one
C	can infer rain area from flux-only data by dividing by RCONST.
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    IF  ( histc (i, j, MAXBIN + 2) .GE. 9. )  THEN
		sumic (i, j, 1) = airc (i, j, 1)
		sumic (i, j, 2) = airc (i, j, 1) / RCONST
	      ELSE IF  ( airc (i, j, 2) .LT. 0.99 )  THEN
		sumic (i, j, 1) = -1.
		sumic (i, j, 2) = -.1
	      ELSE IF  ( do_zcor )  THEN
		k = NINT ( histc (i, j, maxbin+2) )
		IF  ( k .GT. 0 .AND. k .LT. 7 )  THEN
		    zcorijk = zcor (i, j, k)
		  ELSE
		    zcorijk = 1.
		END IF
		sumic (i, j, 1) = RCONST * zcorijk *
     +				  airc (i, j, 1) / airc (i, j, 2)
		sumic (i, j, 2) = zcorijk *
     +				  airc (i, j, 1) / airc (i, j, 2)
	      ELSE
		sumic (i, j, 1) = RCONST * 
     +				  airc (i, j, 1) / airc (i, j, 2)
		sumic (i, j, 2) = airc (i, j, 1) / airc (i, j, 2)
	    END IF
	END DO
	END DO


ccc	WRITE (*, *) 'RCONST, airc1, airc2, histc, sumic',
ccc     +	RCONST, airc(98,1,1), airc(98,1,2),
ccc     +	(histc (98,1,i), i=1,MAXBIN+3), sumic(98,1,1)
C
	RETURN
	END
