	SUBROUTINE SET_ZC  ( iyyyy, ipp, zcor, iret )
C************************************************************************
C* SET_ZC  Set zenith correction arrays					*
C*									*
C* This subroutine sets the zenith correction arrays for the various	*
C* geo satellites for a pentad in 1986 through 1994.			*
C*									*
C* SET_ZC  ( iyyyy, ipp, zcor, iret )					*
C*									*
C* Input parameters:							*
C*   iyyyy	INT	Year number as YYYY				*
C*   ipp	INT	Pentad number as PP				*
C*									*
C* Output parameters:							*
C*   zcor	REAL	Zenith-angle correction (size MXLNC,MXLTC,6)	*
C*			The last index is: GMS, GOES-W, GOES-E, both	*
C*			GOES-W and -E, METEOSAT, INSAT (M-5)		*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = stop				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	12/94						*
C* G.Huffman/SSAI	 2/96	Relocate goes_table; extend through 1995*
C* G.Huffman/SSAI	 5/00	Renovate to IRIX6, YYYY; relocate	*
C*				goes_table				*
C* G.Huffman/SSAI	 6/00	Add INSAT				*
C* G.Huffman/SSAI	 9/02	Byte size on declarations		*
C************************************************************************

	IMPLICIT	NONE
	INTEGER*4	i, j, k, iunit, inyyyy, inddd
C
c	INCLUDE		'param_2.5.h'

c       parameters from original include file: 'param_2.5.h'
	REAL*4		RMISS,  DLAT,  DLONC, BNDRY
	INTEGER*4 MXLTC, MXLNC	
	PARAMETER	( RMISS  = -99999.,
     +			  DLAT   = 2.5,
     +			  DLONC  = 2.5,
     +			  BNDRY  = 50.,
     +			  MXLTC  = ( 2 * BNDRY ) / DLAT,
     +			  MXLNC  = 360 / DLONC )

C
	REAL*4		zcor (MXLNC, MXLTC, 6)
	INTEGER		iyyyy, ipp, iret
C
	REAL*4		pixlon (MXLNC, MXLTC),   pixlat (MXLNC, MXLTC),
     +			clongms, clongw, clonge, clonmet, cloni, frac_y, 
     +			frac_in, satlat, sathgt
	CHARACTER*256	table
	DATA		table / 'GPCPcdr_m4/GPCPcdr_m4_CORR_ADJ/goes_table' /

c	INTEGER*4	INIT_DISK, EXIT_DISK

C	Set satellite location.  The GMS, METEOSAT, and INSAT locations 
C	are fixed; GOES-W and -E move, so a file of locations is used to 
C	get those values.  Any time before 1986 or after 1995 is given 
C	fixed values.  Otherwise, read through the file to find the 
C	first entry past the desired time.  [Program dates are in 
C	pentads, while the table is in julian days, so each is converted 
C	to fraction of the year.]  This is not precise, but is probably 
C	good enough.
C
	clongms = 140.
	clonmet = 0.
	cloni   = 63.
	IF  ( iyyyy .LT. 1986 )  THEN
	    clongw = -108.
	    clonge = RMISS
	  ELSE IF  ( iyyyy .GT. 1995 ) THEN
	    clongw = -135.3
	    clonge = -74.3
	  ELSE
	    frac_y  = iyyyy + ( ipp - .5 ) * 5. /365.
	    frac_in = 0.
c	    iunit   = INIT_DISK ( table )
c	    iret    = EXIT_DISK ( iunit )



	    OPEN  ( iunit, FILE=table, ACCESS='SEQUENTIAL',
     +		    FORM='FORMATTED', IOSTAT=iret )
	    if ( iret .NE. 0 ) then
	       WRITE(*,*) 'Error in corr:set_zc.f:'//
     &             'Failed to open goes table file '//table
	      iret = -1
	      RETURN
	    end if

	    DO  WHILE ( frac_y .GT. frac_in )
		READ (iunit, 30, IOSTAT=iret) inyyyy, inddd, clongw, clonge
   30		FORMAT ( I4, I4, 2F6.1 )
		frac_in = inyyyy + ( inddd - .5 ) / 365.

		if ( iret .NE. 0 ) then
		   WRITE(*,*) 'Error in corr:set_zc.f:'//
     &                 'Failed to read goes table file '//table
		   RETURN
		end if
	    END DO

	    CLOSE (iunit, IOSTAT=iret)
	    if ( iret .NE. 0 ) then
	       WRITE(*,*) 'Error in corr:set_zc.f:'//
     &             'Failed to close goes table file '//table
	      RETURN
	    end if





C
C	    The table has missing as -99.9 and the good longitudes in
C	    deg. west, so they need adjusting.
C
	    IF  ( clongw .LT. 0. )  THEN
		clongw = RMISS
	      ELSE
		clongw = -clongw
	    END IF
	    IF  ( clonge .LT. 0. )  THEN
		clonge = RMISS
	      ELSE
		clonge = -clonge
	    END IF
	END IF
C
C	Generate the lat/lon arrays, get the zenith angles for the
C	individual satellites, and compute the correction according
C	to Bob Joyce's (CAC) recommendation (when the satellite is 
C	visible).
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    pixlon (i, j) =       DLONC * ( i - .5 )
	    pixlat (i, j) = 50. - DLAT  * ( j - .5 )
	END DO
	END DO
	satlat = 0.
	sathgt = 0.
	CALL SZENANG  ( MXLNC,  MXLTC,  pixlon, pixlat, clongms, 
     +		        satlat, sathgt, zcor (1, 1, 1), iret )
	CALL SZENANG  ( MXLNC,  MXLTC,  pixlon, pixlat, clongw, 
     +		        satlat, sathgt, zcor (1, 1, 2), iret )
	CALL SZENANG  ( MXLNC,  MXLTC,  pixlon, pixlat, clonge, 
     +		        satlat, sathgt, zcor (1, 1, 3), iret )
	CALL SZENANG  ( MXLNC,  MXLTC,  pixlon, pixlat, clonmet, 
     +		        satlat, sathgt, zcor (1, 1, 5), iret )
	CALL SZENANG  ( MXLNC,  MXLTC,  pixlon, pixlat, cloni, 
     +		        satlat, sathgt, zcor (1, 1, 6), iret )
	DO  k = 1, 6
	    IF  ( k .NE. 4 )  THEN
		DO  j = 1, MXLTC
		DO  i = 1, MXLNC
		    IF  ( zcor (i, j, k) .LE. 90. )  THEN
			zcor (i, j, k) = AMIN1 ( 1., 
     +     zcor (i, j, k) *  (-0.009028) + 1.22891 )
		      ELSE
			zcor (i, j, k) = RMISS
		    END IF
		END DO
		END DO
	    END IF
	END DO
C
C	For locations where both GOES are visible, compute the joint
C	correction as the simple average.  Otherwise, it is missing.
C
	DO  j = 1, MXLTC
	DO  i = 1, MXLNC
	    IF  ( zcor (i, j, 2) .EQ. RMISS .OR.
     +		  zcor (i, j, 3) .EQ. RMISS )  THEN
		zcor (i, j, 4) = RMISS
	      ELSE
	  	zcor (i, j, 4) = .5 * ( zcor (i, j, 2) + 
     +					zcor (i, j, 3) )
	    END IF
	END DO
	END DO
ccc	WRITE (*, *) 'iyyyy,ipp,clongms,clongw,clonge,clonmet,cloni',
ccc     +		     iyyyy,ipp,clongms,clongw,clonge,clonmet,cloni
C
	RETURN
	END
