	SUBROUTINE SZENANG  ( NX, NY, pixlon, pixlat, satlon, satlat,
     +			      sathgt, satza,  iret )
C************************************************************************
C* SZENANG  Compute the zenith angle to a satellite			*
C*									*
C* Given arrays of pixel location and the subsatellite location,	*
C* compute the corresponding array of local zenith angles of the	*
C* satellite from each pixel.						*
C*									*
C* SZENANG  ( NX, NY, pixlon, pixlat, satlon, satlat, sathgt, satza,	*
C*            iret )							*
C*									*
C* Input parameters:							*
C*   NX		INT	X size						*
C*   NY		INT	X size						*
C*   pixlon	REAL*	Pixel lon's (size NX,NY)			*
C*   pixlat	REAL*	Pixel lat's (size NX,NY)			*
C*   satlon	REAL	Satellite subpoint lon.				*
C*   satlat	REAL	Staellite subpoint lat.				*
C*   sathgt	REAL	Satellite height (km); 0 implies the geo value	*
C*									*
C* Output parameters:							*
C*   satza	REAL*	Local zenith angles (size NX,NY)		*
C*   iret	INT	Return code: 0 = normal				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	11/94						*
C* G.Huffman/SSAI	 9/02	Byte size on declarations		*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i,  j, k
	REAL*4		RE, GEOHGT, hgt, satna, arg, c
	DATA		RE, GEOHGT / 6371.2, 35800. /
C
	INTEGER*4	NX, NY, iret
	REAL*4		pixlon (NX, NY), pixlat (NX, NY), satlon,
     +			satlat, sathgt,  satza  (NX, NY)
C

c     Need to convert degrees to radians - SAPIANO
        REAL deg2rad
        deg2rad = 4*ATAN(1.0)/180

	  
	iret = 0
	IF  ( sathgt .EQ. 0. )  THEN
	    hgt = GEOHGT
	  ELSE
	    hgt = sathgt
	END IF
C
C	STEP 1:  Compute earth-center great circle angle between pixels 
C	and subsatellite point (c) from the Law of Cosines for sides of
C	spherical triangles.
C
C	STEP 2:  Doing the plane geometry in the plane defined by the 
C	earth's center, satellite, and pixel, use two right triangles
C	defined by those points and the perpendicular from the center-
C	satellite line to the pixel.  The local zenith angle to the 
C	satellite (satza), the angle of the upper triangle (90-satna),
C	and the angle of the lower triangle (90-c) sum to 180, or
C	  satza = satna + c .
C	Writing out sin (c), cos (c), tan (satna) and substituting, 
C	a form for satna can be derived (the ATAND term below).
C
C	Code for general case first.
C
	IF  ( satlat .NE. 0. )  THEN
	    DO  j = 1, NY
	    DO  i = 1, NX
		arg = SIN ( satlat * deg2rad ) *
     +                SIN ( pixlat (i, j) * deg2rad ) +
     +		      COS ( satlat * deg2rad ) *
     +                COS ( pixlat (i, j) * deg2rad ) * 
     +		      COS ( ( satlon - pixlon (i, j) ) * deg2rad )
		IF  ( arg .GE. -1. .AND. arg .LE. 1. )  THEN
		    c = ACOS ( arg ) / deg2rad 
		  ELSE IF  ( arg .LT. -1. )  THEN
		    c = 180.
		  ELSE
		    c = 0.
		END IF
	        satza (i, j) = c + ATAN ( RE * SIN ( c * deg2rad ) /
     +             ( hgt + RE - RE * COS ( c * deg2rad ) ) ) / deg2rad 
c	WRITE (*, *) 'i,j,plon,arg,c,za', i,j,pixlon(i,j),arg,c,satza(i,j)
	    END DO
	    END DO
C
C	  Simplified code for satlat = 0.
C
	  ELSE
	    DO  j = 1, NY
	    DO  i = 1, NX
		arg = COS ( pixlat (i, j) * deg2rad ) * 
     +		      COS ( ( satlon - pixlon (i, j) ) * deg2rad )
		IF  ( arg .GE. -1. .AND. arg .LE. 1. )  THEN
		    c = ACOS ( arg ) / deg2rad 
		  ELSE IF  ( arg .LT. -1. )  THEN
		    c = 180.
		  ELSE
		    c = 0.
		END IF
	        satza (i, j) = c + ATAN ( RE * SIN ( c * deg2rad ) /
     +              ( hgt + RE - RE * COS ( c * deg2rad ) ) ) / deg2rad 
c	WRITE (*, *) 'i,j,plon,arg,c,za', i,j,pixlon(i,j),arg,c,satza(i,j)
	    END DO
	    END DO
	END IF
C
	RETURN
	END
