      program Main_GPCPcdr_m4_fuse2
c     This program combines MW, IR and gauge data to 
c     create the final GPCP output files

c     Original header info:
C* This program "fuses" the precip estimates from SSMI, IR, gauges, and	*
C* model.  The user can specify smoothing on the bias-adjustment and	*
C* how the SSMI/adjusted GEO offset is to be extended outside 40 deg N	*
C* and S.  Depending on the compilation, work files are on a 2.5x2.5 or	*
C* 1x1-deg lat/lon grid.  All internal computations and output are done	*
C* in mm/d.								*
C*									*
C* AGPI error is not boosted for loss of SSM/I calibration data.	*
C*									*
C* fuse  GNAME      NGNAME   SNAME    SI_NAME SQ_NAME SN_NAME SF_NAME	*
C*       SS_NAME    GEONAME  NGEONAME ADJADDNAME			*
C*       ADJRATNAME CNAME    ECONST   SMOOTH  EXTEND  FUSEBASE		*
C*									*
C* Input command-line arguments:					*
C*   GNAME	CHAR	Gauge analysis filename as gnam,gfmt where	*
C*			gfmt = camsyr|camsmo|gpccpr|gpccv1|gpccyr|gpccfl*
C*			for 12 mo binary on GPCC grid, 1 mo binary on   *
C*			GPCC grid, GPCC preliminary, GPCC version 1,    *
C*			GPCC year (or year-to-date), GPCC full; 'none'; * 
C*			'-' = default					*
C*			('/usr/sgm/data/gpcc/l01-081987.asc,gpccv1')	*
C*   GNNAME	CHAR	Gauge population filename as gnnam,gnfmt where	*
C*			gnfmt=camsyr|camsmo|gpccpr|gpccv1|gpccyr|gpccfl *
C*			as above; 'same' = population included in GNAME	*
C*			(only gnfmt=gpccv1,gpccfl); 'none' (only if     *
C*			GNAME =	'none'; '-' = default ('same')		*
C*   GBNAME	CHAR	Gauge bias error filename or 'none'; '-' =	*
C*			default ('/usr/sgm/data/gpcc/error02.a08')	*
C*   SNAME	CHAR	SSMI, TOVS, and/or MSU precip filename; the	*
C*			units may be embedded in the name as '.mo.',	*
C*			'.day.', '.hr.' with a default of 'hr'; '-' =	*
C*			default ('/usr/sgm/huffman/cg/cg.8708.rfix')	* 
C*   SI_NAME	CHAR	SSMI cold sfc fraction filename or 'none';	*
C*			'-' = default ('none')				* 
C*   SQ_NAME	CHAR	SSMI ambiguous fraction filename or 'none';	*
C*			'-' = default ('none')				* 
C*   SN_NAME	CHAR	SSMI, TOVS, and/or MSU number-of-samples	*
C*			filename and format as snnam,fmt where 		*
C*			fmt = CTT, where C=e|h|l for equivalent images|	*
C*			high-res pixels|low-res pixels and TT=r4|i2 for	*
C*			REAL*4|INTEGER*2 (defaults to 'er4' if missing);*
C*			'-' = default					*
C*			('/usr/sgm/huffman/cg/cg.8708.Ifix')		*
C*   SF_NAME	CHAR	SSMI fractional coverage filename or 'none';	*
C*			'-' = default					*
C*			('/usr/sgm/huffman/cg/cg.8708.afix')		*
C*   SS_NAME	CHAR	SSMI, TOVS, and/or MSU source filename or	*
C*			'none'; '-' = default				*
C*			('/usr/sgm/huffman/cg/cg.8708.info')		*
C*			      | 0-1 SSM/I fields 1 to 2			*
C*			legal | 2-3 SSM/I fields 1 to 2 and TOVS	*
C*			src   | 4   TOVS				*
C*			values| 5   MSU					*
C*			      | 6   MSU and TOVS			*
C*   GEONAME	CHAR	GEO filename and format as genam,fmt; fmt may	*
C*			have the values '2550','2590','1040','1040i',	*
C*			'1090', '1090NNf' where 25|10 indicate		*
C*			2.5|1.0-deg res., 40|50|90 indicate the data	*
C*			set's latitudinal boundary, 'i' indicates	*
C*			integer data, NN is the 2-digit record number of*
C*			the field, and 'f' flags a field flipped top-to-*
C*			bottom; '-' = default ('cg.8708.rge')		* 
C*   NGEONAME	CHAR	GEO image number filename and format as		*
C*			gennam,fmt; fmt is as for GEONAME; '-' =	*
C*			default ('cg.8708.Ige')				* 
C*   ADJADDNAME	CHAR	Additive adj. filename and format as aanam,fmt;	*
C*			fmt is as for GEONAME; 'none' = no add. adj.;	*
C*			'-' = default ('cg.8708.radfix3.me')		* 
C*   ADJRATNAME	CHAR	Adjusting ratio filename and format as		*
C*			arnam,fmt; fmt is as for GEONAME; 'none' = no	*
C*			adj. rat.; '-' = default ('cg.8708.rcofix3.me')	* 
C*   CNAME	CHAR	Computer model filename and file type (b|a|7 =	*
C*			binary, ascii, binary for all months starting	*
C*			8701 in binary) as CNAM,CTYPE; 'none'; '-' =	*
C*			default ('/usr/iss4/ssmi/DAOassim/		*
C*			DAO_monthly.8708.pa.lo,b')			*
C*   ECONST	CHAR	H and S (multiplicative and offset) constants	*
C*			for computing error for SSMI water, SSMI land,	*
C*			MSU, TOVS, IR, gauge, computer; '-' = default	*
C*			('3,3.2,4,.0025,.45,.0075,10000,1,1,1.3,.5,.5,	*
C*			.27,3.3')					* 
C*   SMOOTH     CHAR    Smoothing type; possible inputs are:            *
C*                      - SN, where S is one of 'box' boxcar, 'clip'    *
C*                      boxcar with .5 on corners, 'gauss' gaussian	*
C*                      with .5 at diameter N, and N is the smoother    *
C*                      size (NxN) (odd, in the range 3,MXSM)           *
C*                      - a filename holding reals, one to a line       *
C*                      - 'none' for no smoothing                       *
C*			- '-' for the default ('box5')			*
C*   EXTEND     CHAR    Style of extending SSMI/Adjusted GEO offsets	*
C*			outside 40 N and S; possible inputs are:	*
C*                      - SN, where S is one of 'diff' smoothfill the	*
C*			difference, 'ratio' smoothfill the ratio,	*
C*			'taper' taper the difference with a constant	*
C*			decrease toward zero, and N is the 2-digit num.	*
C*			of deg. lat. to extend past 40 for 'diff' and	*
C*			'ratio', or the 2-digit minimum num. of units	*
C*			to decrease and the 2-digit num. of deg. lat.	*
C*			to extend past 40 for 'taper' (as UUDD)		*
C*                      - 'none' for no extension			*
C*			- '-' for the default ('ratio10')		*
C*   FUSEBASE	CHAR	Basename for fused gauge-satellite output;	*
C*			'-' = default ('cg.8708')			* 

      IMPLICIT	NONE
      INTEGER*4	i, j,   ii,   lno,   lto,  next,  ltskp, npass,
     +     l_gn,   l_cn, l_ss
      REAL*4		ext,    fac,  coeff, qtot, ebias, dum,
     +     ratmax, rat
      LOGICAL*1	dofill
C     
      INTEGER*4	MXSM,  LN,    LT
      REAL*4		SSLIM, QILIM, GENLIM, RMISS
      PARAMETER	( MXSM   = 15,
     +     LN     = 144,
     +     LT     = 72,
     +     SSLIM  = 10.,
     +     QILIM  = .2,
     +     GENLIM = 130.,
     +     RMISS  = -99999. )
C     
C     geosm gets reused for nadjsm, ngeosm, nratsm, nsfsm.
C     
      REAL*4		geosm (MXSM, MXSM), sfbrk, wtlim
      INTEGER*4	nadjsm, ngeosm, nratsm, nsfsm
      LOGICAL*1	mrg40ns
      DATA		nadjsm, ngeosm, nratsm, nsfsm, sfbrk, wtlim, 
     +     mrg40ns / 3,5,5,3,65.,3., .FALSE. /
C     
C     2.5-deg grid values:
C     +			  LN     = 144,
C     +			  LT     = 72,
C     +			/ 3,5,5,3,65.,3. /
C     1-deg grid values:
C     +			  LN     = 360,
C     +			  LT     = 180,
C     +			/ 7,5,5,3,65.,14. /
C     monthly limits:
C     +			  SSLIM  = 10.,
C     +			  QILIM  = .2,
C     +			  GENLIM = 130.,
C     daily limits:
C     +			  SSLIM  = 0.,
C     +			  QILIM  = .6,
C     +			  GENLIM = 4.,
C     


c     No longer used: - may need to keep these and set 
c     them all to RMISS or zero
c       snfmt, gefmt
c       sinam, si        SSMI cold surface fraction filename
c       sqnam, sq        SSMI ambiguous fraction filename
c       sfnam, sf        SSMI fractional coverage filename
c       aanam, add       Additive adjustment (SSMI / matched IR
c       arnam, ratio     Adjusting ratio (SSMI / matched IR)
c       cnam, comp       Computer estimate
c
c      CHARACTER*256 sinam, sqnam, sfnam, aanam, arnam, cnam
c      CHARACTER*8   aafmt, arfmt, gnfmt, snfmt, gefmt,  genfmt
c      CHARACTER*1   ctype
c      real*4         sf(LN, LT), ratio(LN, LT), comp(LN, LT)

      CHARACTER*256 aanam, arnam

      CHARACTER*256 gnam, gbnam, snam, snnam, ssnam, 
     &     genam, gennam, gnnam, fbas
      CHARACTER*6   yyyymm
      REAL*4        aconst (7), econst (7), smth (MXSM, MXSM), 
     +     difext, ratext,  tapext, taper,  days,  smlim
      INTEGER*4	    ismth, iret

      REAL*4 surf (LN, LT), ratio  (LN, LT),
     +     gauge  (LN, LT), gaugen (LN, LT), 
     +     gaugeb (LN, LT), ssmi   (LN, LT), 
     +     sn     (LN, LT), ss     (LN, LT),
     +     geo    (LN, LT), geon   (LN, LT),
     +     tmpgeo (LN, 40), add    (LN, LT),
     +     tmpadd (LN, 40), tmpratio(LN, LT),
     +     sq     (LN, LT), si     (LN, LT),
     +     MSadj_ratio(LN, LT)
     
      REAL*4		s0     (LT),
     +     srat   (LN, LT), sadd   (LN, LT),
     +     diff   (LN, LT),
     +     adgeo  (LN, LT), adssmi (LN, LT),
     +     corga  (LN, LT), smadd  (LN, LT),
     +     smrat  (LN, LT), smss   (LN, LT),
     +     smadge (LN, LT), smge   (LN, LT),
     +     sms    (LN, LT), smg    (LN, LT),
     +     smsf   (LN, LT),
     +     egeo   (LN, LT), ess    (LN, LT), 
     +     egauge (LN, LT), esat   (LN, LT),
     +     ecomp  (LN, LT), 
     +     esg    (LN, LT), esgc   (LN, LT),
     +     qgeo   (LN, LT), qss    (LN, LT), 
     +     qgauge (LN, LT), qsat   (LN, LT),
     +     qcomp  (LN, LT), 
     +     qsg    (LN, LT), qsgc   (LN, LT),
     +     wtge   (LN, LT), wtss   (LN, LT), 
     +     wtg    (LN, LT), wts    (LN, LT),
     +     wtsg   (LN, LT), wtc    (LN, LT),
     +     sat    (LN, LT), tmpsat (LN, LT),
     +     sg     (LN, LT), sgc    (LN, LT),
     +     snadj  (LN, LT), one    (LN, LT),
     +     avaddl (LT),     avaddw (LT),
     +     avratl (LT),     avratw (LT)
      REAL*4		geol   (LN, LT), ssmil  (LN, LT),
     +     ratiol (LN, LT), addl   (LN, LT),
     +     ratlg  (LN, LT), addlg  (LN, LT),
     +     adgeol (LN, LT), avg    (LN, LT)
      INTEGER*4	ihaveg (LN, LT), n_l    (LT),
     +     n_w    (LT)

      real*4        gmask(LN,LT)
      integer*4     strlen, Ninputs, strlen1, strlen2
      character*1   gfmt
      character*2   cmonth
      character*4   cyear
      character*256 InputPATH, InterimPATH, OutputPATH, fname
      integer*4     iyear, imonth
      integer*4     NdaysInMonth(12)
     &     / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30 , 31 /



c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 6 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_Adjust_Gauge:'//
     &        ' Incorrect number of input arguments (should be 6)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, cyear)
      call getarg(2, cmonth)
      call getarg(3, InputPATH)
      call getarg(4, InterimPATH)
      call getarg(5, gnam)
      call getarg(6, gfmt)

c     Convert month to a number
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth

c     Set multiplicative and additive error values
      data econst / 3, 3.2, 0.6,  .0045, .45,  .0075, 10000 /
      data aconst / 1, 1,   1,   1,     0.5,  0.267,      3.33 /

c     Smoother is box5: set the following for this:
      ismth=5
      DO  j = 1, ismth
         DO  i = 1, ismth
            smth (i, j) = 1.
         END DO
      END DO

c     Style of extending SSMI/Adjusted GEO offset is ratio10, so:
      difext = 0.
      ratext = 10.
      tapext = 0.
      taper  = 0.
      
c     Set the basename for the output
      call String_Length(InterimPATH, strlen2)
      fbas=InterimPATH(1:strlen2)//'FUSE2/'

c     Determine number of days in month
      days=NdaysInMonth(imonth)
      if ( imonth.EQ.2 ) then 
         if ( mod(iyear,400).EQ.0 ) then
            days=29
         elseif ( mod(iyear,100).EQ.0 ) then
            days=28
         elseif ( mod(iyear,4).EQ.0 ) then
            days=29
         endif
      endif
      
c     year and month variable
      yyyymm=cyear//cmonth
      
c     Establish matrix of ones
      DO  j = 1, LT
         DO  i = 1, LN
	    one (i, j) = 1.
         END DO
      END DO

c     Set smlim and  ltskp
      smlim = ismth * ismth * .5
      IF  ( LT .EQ. 72 )  THEN
         ltskp = 20
      ELSE
         ltskp = 50
      END IF

c     Read 2.5 degree surface fraction file
      fname='GPCPcdr_m4/GPCPcdr_m4_FUSE2/Ancillary/surfrac.2.5.le'
      call Read_Binary_File(fname,144,72,1,surf)

c     Read the file to correct teh Multi-Satellite at the very end
      fname='GPCPcdr_m4/GPCPcdr_m4_FUSE2/Ancillary/'//
     &     'ratiop_'//cmonth//'.bin.le'
      call Read_Binary_File(fname,144,72,1,MSadj_ratio)



c************************************************
c     Get Gauge data
c     This is done separately since updates to gauge data will 
c     change the format - the filename and format are thus given
c     by the calling script

c     Gauge bias error filename
      gbnam='GPCPcdr_m4/GPCPcdr_m4_FUSE2/Ancillary/error02.a'//cmonth

c     Read_GPCC reads gauge data from Monitoring or Full product
c     and also gets the gauge bias
      call Read_GPCC(gnam, gfmt, gbnam, days, iyear, imonth, 
     &     gauge, gaugen, gaugeb)

c     Read mask for Gauge data
c     GPCC Full V7 and Monitoring V5 include many island/atoll gauges
c     that are not representative of the 2.5 deg grid box they are in.
c     These are removed using a mask based on the gridboxes used in
c     GPCC Full V6
      fname='GPCPcdr_m4/GPCPcdr_m4_FUSE2/Ancillary/GaugeMask_le.bin'
      call Read_Binary_File(fname,144,72,1,gmask)

c     Apply the gauge mask
      DO  j = 1, LT
         DO  i = 1, LN
            if (gmask(i,j) .NE. 1) then
               gauge(i,j)=-99999.99
            end if
         END DO
      END DO




c**************************************************************************
c     Establish names of datasets for each period
      call String_Length(InputPATH,strlen1)

c     snam: SSMI/SSMIS precip filename from XOVER
c     snnam: SSMI equivalent image filename and format (equivalent images, real*4)
c     ssnam: SSMI source data filename
c     genam: GEO filename and format (2.5 deg,50N/S)
c     gennam: GEO image number filename and format (2.5 deg,50N/S)
c     aanam: Additive adjustment filename
c     arnam: Adjusting ratio filename

c     OPI Period: 1979-1985
c     GPCC is basically just gauge corrected OPI 
      if ( (iyear .GE. 1979) .AND. (iyear .LE. 1985) ) then
         snam=InputPATH(1:strlen1)//
     &        'OPI_le/opi.'//cyear//cmonth//'.rfix'
         snnam=InputPATH(1:strlen1)//
     &        'OPI_le/opi.fixed.Ifix'
         ssnam=InputPATH(1:strlen1)//
     &        'OPI_le/opi.fixed.src'
         genam='none'
         gennam='none'
         aanam='none'
         arnam='none'

c     Pre-SSMI period: 1986, 1987 01-07, 12
c     Mix of OPI and old style Geo IR (needs correction)
      else if ( (iyear .EQ. 1986) .OR. 
     &          ( (iyear .EQ. 1987) .AND. 
     &            (imonth .GE. 1) .AND. (imonth .LE. 7) ) .OR.
     &          ( (iyear .EQ. 1987) .AND. (imonth .EQ. 12) ) ) then
         snam=InputPATH(1:strlen1)//
     &        'OPI_le/opi.'//cyear//cmonth//'.rfix'
         snnam=InputPATH(1:strlen1)//
     &        'OPI_le/opi.fixed.Ifix'
         ssnam=InputPATH(1:strlen1)//
     &        'OPI_le/opi.fixed.src'
         genam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/opi.'//cyear//cmonth//'.rge'
         gennam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/opi.'//cyear//cmonth//'.Ige'
         aanam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/opi.'//cyear//cmonth//'.radfix3.me'
         arnam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/opi.'//cyear//cmonth//'.rcofix3.me'

c     Early SSMI period: 1987 08-11, 1988-1996
c     SSMI and old style Geo IR (needs correction)
      else if ( ( (iyear .EQ. 1987) .AND. (imonth .LE. 11) ) .OR.
     &        ( (iyear .GE. 1988) .AND. (iyear .LE. 1996) ) ) then

         snam=InterimPATH(1:strlen2)//
     &        'XOVER/cgt.'//cyear//cmonth//'.rfix'
         snnam=InterimPATH(1:strlen2)//
     &        'XOVER/cgt.'//cyear//cmonth//'.Ifix'
         ssnam=InterimPATH(1:strlen2)//
     &        'XOVER/cgt.'//cyear//cmonth//'.src'
         genam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/cgt.'//cyear//cmonth//'.rge'
         gennam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/cgt.'//cyear//cmonth//'.Ige'
         aanam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/cgt.'//cyear//cmonth//'.radfix3.me'
         arnam=InterimPATH(1:strlen2)//
     &        'CORR_ADJ/cgt.'//cyear//cmonth//'.rcofix3.me'

c     Late SSMI period: 1997 - present
c     SSMI and higher resolution Geo-IR
      else if (iyear .GE. 1997) then
         snam=InterimPATH(1:strlen2)//
     &        'XOVER/cgt.'//cyear//cmonth//'.rfix'
         snnam=InterimPATH(1:strlen2)//
     &        'XOVER/cgt.'//cyear//cmonth//'.Ifix'
         ssnam=InterimPATH(1:strlen2)//
     &        'XOVER/cgt.'//cyear//cmonth//'.src'
         genam=InterimPATH(1:strlen2)//'CORR2/cgt.'
     &        //cyear//cmonth//'.ragpi.fix'
         gennam=InterimPATH(1:strlen2)//'CORR2/cgt.'
     &        //cyear//cmonth//'.Iagpi.fix'
         aanam='none'
         arnam='none'

c     catch any errors
      else
         WRITE(*,*) 'Error in GPCPcdr_m4_FUSE2:'//
     &        ' Invalid date?'
         STOP
      end if







c**************************************************************************
c     Get Microwave data and AGPI data


c     initialize datasets
      do i=1,LN
         do j=1,LT
            geo(i,j)=RMISS
            geon(i,j)=RMISS
            add(i,j)=0
            ratio(i,j)=1
         end do
         do j=1,40
            tmpgeo(i,j)=RMISS
            tmpadd(i,j)=0
            tmpratio(i,j)=1
         end do
      end do


c     SSMI/SSMIS precip filename from XOVER
      call Read_Binary_File(snam,144,72,1,ssmi)

c     SSMI equivalent image filename and format (equivalent images, real*4)	
      call Read_Binary_File(snnam,144,72,1,sn)

c     SSMI source data
      call Read_Binary_File(ssnam,144,72,1,ss)

c     GEO filename and format (2.5 deg,50N/S)
      if ( genam (1:4) .NE. 'none' ) then
         call Read_Binary_File(genam,144,40,1,tmpgeo)
      end if

c     Additive adj
      if ( aanam (1:4) .NE. 'none' ) then
         call Read_Binary_File(aanam,144,40,1,tmpadd)
      end if

c     Adjusting ratio
      if ( arnam (1:4) .NE. 'none' ) then
         call Read_Binary_File(arnam,144,40,1,tmpratio)
      end if

      do i=1,LN
         do j=17,56
            geo(i,j)=tmpgeo(i,j-16)
            add(i,j)=tmpadd(i,j-16)
            ratio(i,j)=tmpratio(i,j-16)
         end do
      end do

c     GEO image number filename and format (2.5 deg,50N/S)
      if ( gennam (1:4) .NE. 'none' ) then
         call Read_Binary_File(gennam,144,40,1,tmpgeo)
      end if
      do i=1,LN
         do j=17,56
            geon(i,j)=tmpgeo(i,j-16)
         end do
      end do


c     Set missing values for precip datasets
      DO  j = 1, LT
         DO  i = 1, LN
	    IF  (ssmi(i,j).LT.0.) ssmi(i,j)=RMISS
	    IF  (sn(i,j).LT.0.)   sn(i,j)=RMISS
	    IF  (ss(i,j).LT.0.)   ss(i,j)=RMISS
          END DO
      END DO




c**********************************************************************
C     Find the j index of the last non-zero-adjustment lat., using
C     N-S grid size = 180 deg. / number of N-S grid boxes.  Also,
C     convert taper to change per grid box.
C     
      IF  ( difext .GT. 0. )  THEN
         ext = difext
      ELSE IF  ( ratext .GT. 0. )  THEN
         ext = ratext
      ELSE IF  ( tapext .GT. 0. )  THEN
         ext = tapext
      ELSE
         ext = 0.
      END IF
     
      IF  ( AMOD ( ext * LT / 180., 1. ) .EQ. 0. )  THEN
         next = INT ( ext * LT / 180. )
      ELSE
         next = INT ( ext * LT / 180. ) + 1
      END IF
      taper = taper / next
      

C     Compute number of SSM/I samples adjusted by snow/ice coverage,
C     and handle cases of marginal or missing SSMI and IR.  In the
C     current scheme snow/ice are handled in building the Chang/Grody
C     SSM/I combination, so they are passed in here as 'none.'
C     Set the SSM/I and adjustment coefficients to missing when SSM/I 
C     is marginally sampled or any negative value; note the simple 
C     number-of-samples-based flagging.  Set geo and coefficients to
C     missing when there is no geo data.  [Resetting ratio and add is 
C     immaterial when not recomputing AGPI.]
      DO  j = 1, LT
         DO  i = 1, LN
            sq(i,j)=0.
            si(i,j)=0.
	    snadj (i, j) =
     +           ( 1. - ( sq (i, j) + si (i, j) ) / QILIM ) * sn (i, j)
	    IF  ( snadj (i, j) .LE. 0. .OR. ssmi (i, j) .LT. 0. .OR.
     +           ( snadj (i, j) .LE. SSLIM .AND. ss (i, j) .LE. 2. ) )  
     +           THEN
               ssmi  (i, j) = RMISS
               snadj (i, j) = RMISS
               ratio (i, j) = RMISS
               add   (i, j) = RMISS
	    END IF
	    IF  ( geon (i, j) .LE. 0. )  THEN
               geo   (i, j) = RMISS
               ratio (i, j) = RMISS
               add   (i, j) = RMISS
	    END IF
         END DO
      END DO


C     When geo-IR exists, smooth-fill SSM/I (in parallel with 1-deg.) 
C     into array smss and smooth it, isolate and smooth the geo-only-
C     IR, and compute the adjustment coefficients.  Done here to avoid 
C     changing CORR.F.
C     [Philosophy: Smooth-filled SSM/I and adjustment ratios should 
C     be flat across the Himalayas; re-computing ratios with the
C     smooth-filled SSM/I will squash the cold sfc artifact in the 
C     IR, while using the smooth-filled ratios will continue to show 
C     it.]
        
c     IF  ( genam (1:4) .NE. 'none' )  THEN
      DO  j = 1, ltskp
         DO  i = 1, LN
            smss (i, j     ) = RMISS
            smss (i, LT+1-j) = RMISS
         END DO
      END DO
      npass = -1
      CALL FILLSM  ( ssmi (1, 1+ltskp), LN, LT-2*ltskp, LN, 0, 
     +     npass, smss (1, 1+ltskp), iret )
c	    DO  j = 1, LT
c	    DO  i = 1, LN
c		IF  ( geon (i, j) .LT. GENLIM )  THEN
c		    smge (i, j) = RMISS
c		  ELSE
c		    smge (i, j) = geo (i, j)
c		END IF
c	    END DO
c	    END DO
      IF  ( nadjsm .NE. 0 )  THEN
         DO  j = 1, nadjsm
            DO  i = 1, nadjsm
               geosm (i, j) = 1.
            END DO
         END DO
         CALL SMOOTH  ( nadjsm, geosm, wtlim, smss (1, 1+ltskp), 
     +        LN,     LT-2*ltskp,   smss (1, 1+ltskp), 
     +        iret )
         IF  ( iret .NE. 0 )  THEN
            WRITE (*, *) 'Error in ''SSM/I'' data'
            STOP
         END IF
c		CALL SMOOTH  ( nadjsm, geosm, wtlim, smge (1, 1+ltskp), 
c     +			       LN,     LT-2*ltskp,   smge (1, 1+ltskp), 
c     +			       iret )
c		IF  ( iret .NE. 0 )  THEN
c		    WRITE (*, *) 'Error in geo-IR data'
c		    STOP
c		END IF
c		CALL DMPIMGF  ( smss, fbas, 'ss3', .FALSE., LN, LT,
c     +				iret )
c		CALL DMPIMGF  ( smge, fbas, 'ir3', .FALSE., LN, LT,
c     +				iret )
      END IF
c	    CALL MONCOR  ( smge (1, 1+ltskp), smss (1, 1+ltskp), 
c     +		           LN, LT-2*ltskp,
c     +		           add (1, 1+ltskp), ratio (1, 1+ltskp), iret )
c	    DO  j = 1+ltskp, LT-ltskp
c	    DO  i = 1, LN
c		IF  ( ratio (i, j) .LT. 0. .OR. 
c     +		      geon  (i, j) .LE. 0. )  THEN
c		    ratio (i, j) = RMISS
c		    add   (i, j) = RMISS
c		END IF
c	    END DO
ctemp	    END DO
c     END IF


C	Set the adjusted geo to missing outside the data area.
C
      DO  j = 1, ltskp
         DO  i = 1, LN
	    adgeo (i, j     ) = RMISS
	    adgeo (i, LT+1-j) = RMISS
         END DO
      END DO



c********************************************************************
C     If there is no adjusting ratio filename assume that the input 
C     GPI (if any!) is intended to serve as the AGPI, so transfer it.

      IF  ( arnam (1:4) .EQ. 'none' )  THEN
         DO  j = 1+ltskp, LT-ltskp
            DO  i = 1,  LN
               adgeo (i, j) = geo (i, j)
            END DO
         END DO


C	  Otherwise, the coefficients are passed in and the AGPI is
C	  recomputed here.  When geo is missing, ratio has been set
C	  to missing.  In a previous version, the adjustments 
C	  were set to RMISS whenever quality was too low.  Set the 
C	  adjusted geo to missing when there is no IR at all; do the 
C	  adjustment when there is geo-IR.
C
      ELSE
         DO  j = 1+ltskp, LT-ltskp
	    DO  i = 1,  LN
               IF  ( geo (i, j) .LT. 0. )  THEN
                  adgeo (i, j) = RMISS
               ELSE IF  ( ratio (i, j) .GE. 0. )  THEN
                  adgeo (i, j) = add   (i, j) + 
     +                 ratio (i, j) * geo (i, j)
               ELSE
                  adgeo (i, j) = RMISS
               END IF
	    END DO
         END DO
         
C     Adjust the leo-IR, if available.  As in the prior version 
C     (and the 1-deg version), use an SSM/I-adjusted leo-IR.  
C     However, don't smooth the leo-IR before computing the
C     adjustment.  This makes it immaterial whether leo-IR
C     exists in the geo-IR region (1-deg data) or not (2.5-deg
C     data).  Also, we now approximate the 1-deg version by 
C     embedding the SSM/I-calibrated leo-IR in smoothed SSM/I, 
C     calibrating that field to the AGPI, and smooth-filling 
C     coefficients across gaps.  [Note that the SSM/I used was
C     smoothed above.]
C     Use missings in ratio (but non-zereo snadj) to mask off 
C     geo-IR in geo and the corresponding ssmi for computing 
C     leo-IR adjustment to SSM/I using *only* data in the leo-IR 
C     region.  The standard procedure is to do a 3x3 smoother 
C     before MONCOR.

         DO  j = 1, LT
            DO  i = 1, LN
               geol  (i, j) = RMISS
               ssmil (i, j) = RMISS
            END DO
         END DO
         DO  j = 1+ltskp, LT-ltskp
            DO  i = 1, LN
               IF  ( ratio (i, j) .LT. 0. )  THEN
                  geol  (i, j) = geo  (i, j)
                  ssmil (i, j) = smss (i, j)
               END IF
            END DO
         END DO
         
         IF  ( nadjsm .NE. 0 )  THEN
            DO  j = 1, nadjsm
               DO  i = 1, nadjsm
                  geosm (i, j) = 1.
               END DO
            END DO
c     CALL SMOOTH  ( nadjsm, geosm, wtlim, ssmil (1, 1+ltskp), 
c		CALL SMOOTH  ( nadjsm, geosm, wtlim, ssmi (1, 1+ltskp), 
c     +			       LN,     LT-2*ltskp,   ssmil (1, 1+ltskp), 
c     +			       iret )
c		CALL SMOOTH  ( nadjsm, geosm, wtlim, geol  (1, 1+ltskp), 
c     +			       LN,     LT-2*ltskp,   geol  (1, 1+ltskp), 
c     +			       iret )
c		IF  ( iret .NE. 0 )  THEN
c		    WRITE (*, *)  'Error: adj smoother size out of',
c     +				  ' bounds or even'
c		    STOP
c		END IF


c         CALL DMPIMGF  ( ssmil, fbas, 'lss3', .FALSE., LN, LT,
c     +        iret )
c         CALL DMPIMGF  ( geol,  fbas, 'lir3', .FALSE., LN, LT,
c     +        iret )
         END IF
         CALL MONCOR  ( geol (1, 1+ltskp), ssmil  (1, 1+ltskp), 
     +        LN, LT-2*ltskp,
     +        addl (1, 1+ltskp), ratiol (1, 1+ltskp), iret)
         
         DO  j = 1+ltskp, LT-ltskp
            DO  i = 1, LN
               IF  ( ratiol (i, j) .LT. 0. )  THEN
                  ratiol (i, j) = RMISS
                  addl   (i, j) = RMISS
               END IF
            END DO
         END DO

c	    npass = -1
c	    CALL FILLSM  ( addl   (1, 1+ltskp), LN, LT-2*ltskp, LN, 0, 
c     +		           npass, addl   (1, 1+ltskp), iret )
c	    CALL FILLSM  ( ratiol (1, 1+ltskp), LN, LT-2*ltskp, LN, 0, 
c     +		           npass, ratiol (1, 1+ltskp), iret )
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( geon (i, j) .LE. 0. .OR.
     +              geon (i, j) .GE. GENLIM )  THEN
                  addl   (i, j) = RMISS
                  ratiol (i, j) = RMISS
               END IF
            END DO
         END DO
c         CALL DMPIMGF  ( addl,   fbas, 'addl', .FALSE., LN, LT,
c     +        iret )
c         CALL DMPIMGF  ( ratiol, fbas, 'ratl', .FALSE., LN, LT,
c     +        iret )
C
C	    Initialize the SSM/I-adjusted leo-IR array with smoothed 
C	    SSM/I and then insert the adjusted IR where leo-IR exists.
C
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( smss (i, j) .GE. 0. )  THEN
                  adgeol (i, j) = smss (i, j)
               ELSE
                  adgeol (i, j) = RMISS
               END IF
            END DO
         END DO
c	    IF  ( nadjsm .NE. 0 )  THEN
c		CALL SMOOTH  ( nadjsm, geosm, wtlim, ssmi   (1,1+ltskp), 
c     +			       LN,     LT-2*ltskp,   adgeol (1,1+ltskp), 
c     +			       iret )
c		IF  ( iret .NE. 0 )  THEN
c		    WRITE (*, *)  'Error: adj smoother size out of',
c     +				  ' bounds or even'
c		    STOP
c		END IF
c	    END IF
         DO  j = 1+ltskp, LT-ltskp
            DO  i = 1, LN
               IF  ( ratiol (i, j) .GT. 0. )  THEN
                  adgeol (i, j) = addl   (i, j) + 
     +                 ratiol (i, j) * geo (i, j)
               END IF
            END DO
         END DO
c         CALL DMPIMGF  ( adgeol, fbas, 'adgeol', .FALSE., LN, LT, 
c     +        iret )

C	    Calibrate the SSM/I-adjusted-leo to the geo AGPI where both 
C	    exist and smoothfill the rest (the missings have to be set 
C	    to RMISS to get smoothfill to work).  Finally, compute a 
C	    monthly AGPI that's a combination of the geo and leo fill-
C	    in (the subset from when there was no geo at a particular 
C	    time).  
C
         
         IF  ( nadjsm .NE. 0 )  THEN
            CALL SMOOTH ( nadjsm, geosm, wtlim, adgeol (1, 1+ltskp), 
     +           LN,     LT-2*ltskp,   adgeol (1, 1+ltskp), 
     +           iret )
            IF  ( iret .NE. 0 )  THEN
               WRITE (*, *)  'Error in leo-IR data'
               STOP
            END IF
            CALL SMOOTH ( nadjsm, geosm, wtlim, adgeo  (1, 1+ltskp), 
     +           LN,     LT-2*ltskp,   smadge (1, 1+ltskp), 
     +           iret )
            IF  ( iret .NE. 0 )  THEN
               WRITE (*, *)  'Error in geo-IR data'
               STOP
            END IF
         END IF
         CALL MONCOR  ( adgeol (1, 1+ltskp), smadge (1, 1+ltskp), 
     +        LN, LT-2*ltskp, 
     +        addlg  (1, 1+ltskp), ratlg  (1, 1+ltskp), 
     +        iret )
         DO  j = 1+ltskp, LT-ltskp
            DO  i = 1, LN
               IF  ( addlg (i, j) .LT. 0. )  THEN
                  addlg (i, j) = RMISS
                  ratlg (i, j) = RMISS
               END IF
            END DO
         END DO
         npass = -1
         
         CALL FILLSM  ( addlg (1, 1+ltskp), LN, LT-2*ltskp, LN, 0, 
     +        npass, addlg (1, 1+ltskp), iret )
         CALL FILLSM  ( ratlg (1, 1+ltskp), LN, LT-2*ltskp, LN, 0, 
     +        npass, ratlg (1, 1+ltskp), iret )
c         CALL DMPIMGF  ( addlg,  fbas, 'addlg', .FALSE., LN, LT, 
c     +        iret )
c         CALL DMPIMGF  ( ratlg,  fbas, 'ratlg', .FALSE., LN, LT, 
c     +        iret )
         DO  j = 1+ltskp, LT-ltskp
            DO  i = 1, LN
               IF  ( adgeo (i, j) .LT. 0. )  THEN
                  adgeo (i, j) = addlg (i, j) + 
     +                 ratlg (i, j) * adgeol (i, j)
               END IF
            END DO
         END DO
      END IF
      

C     Done with IF for internal AGPI re-computation.
C     Smooth SSM/I and IR fields in the region of geo data preparatory 
C     to computing adjustments.
C
      IF  ( ngeosm .NE. 0 .AND. genam (1:4) .NE. 'none' )  THEN
         DO  j = 1, ngeosm
	    DO  i = 1, ngeosm
               geosm (i, j) = 1.
	    END DO
         END DO
         CALL SMOOTH  ( ngeosm, geosm, wtlim, ssmi   (1, 1+ltskp), 
     +        LN,     LT-2*ltskp,   smss   (1, 1+ltskp), 
     +        iret )
         IF  ( iret .NE. 0 )  THEN
            WRITE (*, *)  'Error in ''ssmi'' data'
            STOP
         END IF
         CALL SMOOTH  ( ngeosm, geosm, wtlim, adgeo  (1, 1+ltskp), 
     +        LN,     LT-2*ltskp,   smadge (1, 1+ltskp), 
     +        iret )
         IF  ( iret .NE. 0 )  THEN
            WRITE (*, *)  'Error in geo data'
            STOP
         END IF
         DO  j = 1, ltskp
	    DO  i = 1, LN
               smss   (i, j     ) = RMISS
               smss   (i, LT+1-j) = RMISS
               smadge (i, j     ) = RMISS
               smadge (i, LT+1-j) = RMISS
	    END DO
         END DO
      ELSE
         DO  j = 1, LT
	    DO  i = 1, LN
               smss   (i, j) = ssmi  (i, j)
               smadge (i, j) = adgeo (i, j)
	    END DO
         END DO
      END IF


c********************************************************************
c     Apply adjustmet to SSMI

c     Sapiano: difext is hard-coded to 0, so this not done
c
C     Optionally adjust SSMI to (potentially) fill in missing geo
C     and extend the geo influence into the SSMI region.  Nothing is 
C     done here if difext, ratext, and tapext are zero.  Optionally
C     adjust the SSMI outside the 40N to 40S strip according either 
C     to the smoothfilled difference or ratio between SSMI and 
C     adjusted GEO, or the tapered difference.
C     
C     Smoothfilled difference adjustment.  next is the j index of the
C     last non-zero-difference lat.; zero-fill the rest of diff.
C     
c      IF  ( difext .GT. 0. )  THEN
c         DO  j = next+1, ltskp
c	    DO  i = 1, LN
c               diff   (i, 1 +ltskp-j) = 0.
c               diff   (i, LT-ltskp+j) = 0.
c	    END DO
c         END DO
C
C     Compute the difference (sets RMISS outside the region of 
C     good data, 40N-40S), and do a smooth-fill to get diff in
C     all holes.  Drop extremely small positive values of diff, 
C     since the smooth-fill smears differences a long way across
C     zero regions, and tiny positive values provoke an 
C     unrealistically high occurrence of rain in formerly zero
C     areas.  Slight negatives are not touched because they
C     are clipped in zero areas and unimportant elsewhere.
C     "tiny positive" was gauged by inspecting daily maps (the
C     large areas are handled by .05) and considering that 
C     values <.05 round to 0 to the nearest .1 .
c     
c     Chances are that using smoothed values as the base of the
c     diff extension gives an answer that's too smooth.
c         
c         DO  j = 1+ltskp-next, LT-ltskp+next
c	    DO  i = 1,       LN
c               IF  ( ratio (i, j) .GE. 0. )  THEN
c                  IF  ( smss (i, j) .GE. 0. )  THEN
c                     diff   (i, j) = smadge (i, j) - smss (i, j)
c                  ELSE
c                     diff   (i, j) = RMISS
c                  END IF
c               ELSE
c                  diff   (i, j) = RMISS
c               END IF
c	    END DO
c         END DO
c         npass = -1
c         CALL FILLSM  ( diff, LN, LT, LN, ltskp-next, npass, diff, 
c     +        iret )
c         DO  j = 1, LT
c	    DO  i = 1, LN
c               IF  ( diff (i, j) .GT. 0. .AND. diff (i, j) .LT. 0.05 )  
c     +		    diff (i, j) = 0.
c	    END DO
c         END DO
C     
C     Adjust the SSMI by adding diff (but the sum must be .GE. 0).
C
c         DO  j = 1, LT
c	    DO  i = 1, LN
c               IF  ( ssmi (i, j) .GE. 0. )  THEN
c                  IF  ( ssmi (i, j) + diff (i, j) .GE. 0. )  THEN
c                     adssmi (i, j) = ssmi (i, j) + diff (i, j)
c                  ELSE
c                     adssmi (i, j) = 0.
c                  END IF
c               ELSE
c                  adssmi (i, j) = RMISS
c               END IF
c	    END DO
c         END DO
C



c     Sapiano: ratext is hard-coded to 10. by ratio10 option
c     so the following is done:
c
C     Smoothfilled ratio adjustment.  next is the j index of the 
C     last non-unit-ratio lat.; fill with 1's outside.
C     * NOTE: we are now working with the SSMI/GEO ratio, srat! *
C     
c      ELSE IF  ( ratext .GT. 0. )  THEN
      DO  j = next+1, ltskp
         DO  i = 1, LN
            srat (i, 1 +ltskp-j) = 1.
            srat (i, LT-ltskp+j) = 1.
         END DO
      END DO

C     Compute the ratio (sets RMISS outside the region of good
C     data, 40N-40S), keeping the ratio reasonable (.LE. 2),
C     then do a smooth-fill to get ratio for all holes.
c     Note: following comments for the diff extension, this ratio
c     extend is probably too smooth as well.
         
      DO  j = 1+ltskp-next, LT-ltskp+next
         DO  i = 1,       LN
            IF  ( ratio (i, j) .GE. 0. )  THEN
               IF  ( smss (i, j) .GT. 0. )  THEN
                  srat (i, j) = AMAX1 ( .2,
     +                 AMIN1 ( 2.,
     +                 smadge (i, j) / smss (i, j) ) )
               ELSE IF  ( ssmi (i, j) .EQ. 0. )  THEN
                  srat (i, j) = 1.
               ELSE
                  srat (i, j) = RMISS
               END IF
            ELSE
               srat (i, j) = RMISS
            END IF
         END DO
      END DO

      npass = -1

      CALL FILLSM (srat, LN, LT, LN, ltskp-next, npass, srat, iret)


c         CALL DMPIMGF  ( srat, fbas, 'ssr', .FALSE., LN, LT, iret )
     
C     Adjust the SSMI by multiplying ratio.

      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( ssmi (i, j) .GE. 0. )  THEN
               adssmi (i, j) = ssmi (i, j) * srat (i, j)
            ELSE
               adssmi (i, j) = RMISS
            END IF
         END DO
      END DO


c     Sapiano: this is not done
C	  Tapered difference adjustment.  next is the j index of the 
C	  last non-zero difference lat.; fill with zeros outside.
C	  * NOTE: we are now working with the SSMI/GEO ratio, srat! *
C
c      ELSE IF  ( tapext .GT. 0. )  THEN
C
C	    Compute the difference (sets RMISS outside the region of 
C	    good data, 40N-40S), and do a constant-rate taper to get 
C	    diff in all holes.  As well, after the initial smoothfill
C	    pass the taper is increased along columns to force the diff
C	    to zero at the outer boundary.  Forcing differences toward 
C	    zero as you move away from the TMPI non-missing values solves 
C	    the smoosh that smooth-filling the difference tends to cause.
cc	    Note: following comments for the smoothfill extensions, this 
cc	    taper extend is computed on unsmoothed values, since there's 
cc	    a fair amount of smoothing in the taper scheme, and the 
cc	    initial smoothed difference tended to introduce light precip 
cc	    abutting zero TMPI precip at the boundaries.
C	    All areas outside the TMPI are first set to "missing", the
C	    taper fill is done out to the ext limit, then remaining
C	    "missing"s are set to 0.  This ensures that the outer B.C.
C	    is "missing" during the fill.
C
c	    DO  j = 1, LT
c	    DO  i = 1, LN
c		IF  ( ratio (i, j) .GE. 0. )  THEN
c		    IF  ( ssmi (i, j) .GE. 0. )  THEN
c			diff   (i, j) = adgeo (i, j) - ssmi (i, j)
c		      ELSE
c			diff   (i, j) = RMISS
c		    END IF
c		  ELSE
c		    diff   (i, j) = RMISS
c		END IF
c	    END DO
c	    END DO
c	    CALL FILLTA  ( diff, LN, LT, LN, ltskp-next, taper, next, 
c     +			   diff, iret )
c	    DO  j = 1,LT
c	    DO  i = 1, LN
c		IF  ( diff (i, j) .EQ. RMISS )  diff (i, j) = 0.
c	    END DO
c	    END DO
C
C	    Adjust the SSMI by adding diff (but the sum must be .GE. 0).
C
c	    DO  j = 1, LT
c	    DO  i = 1, LN
c		IF  ( ssmi (i, j) .GE. 0. )  THEN
c		    IF  ( ssmi (i, j) + diff (i, j) .GE. 0. )  THEN
c			adssmi (i, j) = ssmi (i, j) + diff (i, j)
c		      ELSE
c			adssmi (i, j) = 0.
c		    END IF
c		  ELSE
c		    adssmi (i, j) = RMISS
c		END IF
c	    END DO
c	    END DO
C
C	  No adjustment; load adssmi directly.
C
c	  ELSE
c	    DO  j = 1, LT
c	    DO  i = 1, LN
c	        IF  ( ssmi (i, j) .GE. 0. )  THEN
c		    adssmi (i, j) = ssmi (i, j)
c		  ELSE
c		    adssmi (i, j) = RMISS
c		END IF
c	    END DO
c	    END DO
c	END IF



c********************************************************************
C	Compute SSM/I sampling error using its own estimated rain just
C	for output. When there are different SSMI sources (and 
C	consequently errors) for land and water plus TOVS (at least 
C	potentially), three sets of constants are provided.
C
c	IF  ( ssnam (1:l_ss) .EQ. 'none' )  THEN
c	    CALL SAMERR ( 24.,    LN,  LT,     econst (1), aconst (1), 
c     +			  'ssmi', 1,   adssmi, snadj, dum, ess, iret )
c	    IF  ( iret .NE. 0 )  STOP
c	  ELSE
      CALL SAMERR ( 24.,    LN,  LT,     econst,     aconst, 
     +     'ssto', 3,   adssmi, snadj,  ss, ess, iret )

c	    IF  ( iret .NE. 0 )  STOP
c	END IF
C
C	Compute AGPI sampling error using its own estimated rain just
C	for output.  Here and throughout, AGPI error is arbitrarily 
C	boosted by 41% (so variance is approx. 2x higher) for loss of 
C	SSM/I calibration data.
C
      CALL SAMERR ( 24.,    LN, LT,    econst (5), aconst (5), 
     +     'agpi', 1,  adgeo, geon, dum,  egeo, iret )
      IF  ( iret .NE. 0 )  STOP
      DO  j = 1+ltskp, LT-ltskp
         DO  i = 1, LN
	    IF  ( smss (i, j) .LT. 0. )  THEN
               IF  ( egeo (i, j) .GT. 0. )  egeo (i, j) = 1.41 *
     +              egeo (i, j)
	    END IF
         END DO
      END DO

C
C	Compute SSM/I and AGPI quality index, and dump out fields.
C
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, adgeo,
     +     egeo,       qgeo,       iret )
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, adssmi,
     +     ess,        qss,        iret )
c	CALL DMPIMGF  ( egeo,   fbas, 'egeo', .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( ess,    fbas, 'ess',  .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( qss,    fbas, 'qss',  .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( qgeo,   fbas, 'qgeo', .FALSE., LN, LT, iret )


c     Output errors for AGPI and SSMI/TOVS composite
      call String_Length(fbas,strlen)

c     SSMI/TOVS composite error (ess -> ess)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.ess'
      call Write_Binary_File(fname, ess, 144, 72)

c     AGPI error (egeo -> egeo)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.egeo'
      call Write_Binary_File(fname, egeo, 144, 72)

c     GEO precip (geo -> pgp)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.pgp'
      call Write_Binary_File(fname, geo, 144, 72)

c     GEO image number (geon -> ngp)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.ngp'
      call Write_Binary_File(fname, geon, 144, 72)







C	Compute the straight average of the available rain values 
C	that go into the multi-satellite combination; this is only 
C	used for the rain value in calculating input errors for 
C	the multi-satellite combination.  The order of tests is
C	intended to minimize computation.  A false mrg40ns specifies
C	that geo-AGPI is not combined.
C
      DO  j = 1, LT
         DO  i = 1, LN
	    IF  ( adssmi (i, j) .GE. 0. )  THEN
               IF  ( adgeo (i, j) .LT. 0. )  THEN
                  avg (i, j) = adssmi (i, j)
               ELSE IF  ( geon (i, j) .GE. GENLIM .AND.
     +                 .NOT. mrg40ns )  THEN
                  avg (i, j) = adgeo (i, j)
               ELSE
                  avg (i, j) = ( adssmi (i, j) + adgeo (i, j) ) * .5
               END IF
            ELSE IF  ( adgeo (i, j) .LT. 0. )  THEN
               avg (i, j) = RMISS
            ELSE
               avg (i, j) = adgeo (i, j)
	    END IF
         END DO
      END DO

C     Compute SSM/I sampling error using avg rain for multi-satellite
C     combination.  When there are different SSMI sources (and
C     consequently errors) for land and water plus TOVS (at least
C     potentially), three sets of constants are provided.


      IF  ( ssnam (1:l_ss) .EQ. 'none' )  THEN
         CALL SAMERR ( 24.,    LN,  LT,  econst (1), aconst (1), 
     +        'ssmi', 1,   avg, snadj, dum, ess, iret )
         IF  ( iret .NE. 0 )  STOP
      ELSE
         CALL SAMERR ( 24.,    LN,  LT,  econst,     aconst, 
     +        'ssto', 3,   avg, snadj,  ss, ess, iret )
         IF  ( iret .NE. 0 )  STOP
      END IF
C     
C     Compute AGPI sampling error using avg rain for multi-satellite
C     combination.
C
      CALL SAMERR ( 24.,    LN, LT,  econst (5), aconst (5), 
     +     'agpi', 1,  avg, geon, dum,  egeo, iret )
      IF  ( iret .NE. 0 )  STOP
      DO  j = 1+ltskp, LT-ltskp
         DO  i = 1, LN
	    IF  ( smss (i, j) .LT. 0. )  THEN
               IF  ( egeo (i, j) .GT. 0. )  egeo (i, j) = 1.41 *
     +              egeo (i, j)
	    END IF
         END DO
      END DO

C     Make sure the error is only available where the estimate is
C     available.

      DO  j = 1, LT
         DO  i = 1, LN
	    IF  ( adssmi (i, j) .LT. 0. )  ess  (i, j) = RMISS
	    IF  ( adgeo  (i, j) .LT. 0. )  egeo (i, j) = RMISS
         END DO
      END DO
      

        


C	Form the multi-satellite precip estimate.  In the no-merge case
C	the geo-AGPI is used where available and the combination scheme 
C	is used where the geo-AGPI is not available.  Geo-AGPI 
C	availability is flagged by th number of IR images, but could be 
C	fooled by a day or month with few geo images.  However, in that 
C	case, you might well want to do a combination anyway.  [The code 
C	is implemented in the opposite order.]
C
C	In the merge case, use a weighted average of AGPI and SSM/I; 
C	this gets important when there are two SSM/I platforms, and is 
C	the scheme for Pathfinder.
C
      CALL WTCOMB  ( LN, LT, adssmi, ess, adgeo, egeo, sat, esat,
     +     wtss,   wtge,   iret )
C     
      IF  ( .NOT. mrg40ns )  THEN
         DO  j = 1, LT
	    DO  i = 1, LN
               IF  ( geon (i, j) .GE. GENLIM .AND.
     +              egeo (i, j) .GT. 0. )  THEN
                  sat  (i, j) = adgeo (i, j)
                  esat (i, j) = egeo  (i, j)
                  wtss (i, j) = 0.
                  wtge (i, j) = 1.
               END IF
	    END DO
         END DO
      END IF
      

C	Compute S quality index and dump out fields.

      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, sat,
     +     esat,       qsat,       iret )

c      CALL DMPIMGF  ( add,    fbas, 'add',  .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( ratio,  fbas, 'rat',  .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( smss,   fbas, 'smss', .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( smadge, fbas, 'sadge',.FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( adgeo,  fbas, 'adge', .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( adssmi, fbas, 'adss', .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( wtge,   fbas, 'wtge', .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( wtss,   fbas, 'wtss', .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( sat,    fbas, 's',    .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( esat,   fbas, 'es',   .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( qsat,   fbas, 'qs',   .FALSE., LN, LT, iret )
c     AGPI precip (adgeo -> adge)





c******************************************************************
c     Apply adjustment to Multisatellite and output
c     This corrects the polar regions to a climatological estimate
c     and ensures that the values are not way off in poorly covered
c     areas
c     The uncorrected MS is used later, so simply correct a temp version
c     Only do this during the SSMI period
      

      IF ((iyear .GE. 1988) .OR. (iyear .EQ. 1987 
     +     .AND. imonth .GE. 8 .AND. imonth .NE. 12)) THEN
         DO i = 1, LN
            DO j = 1, LT
               IF  (MSadj_ratio(i,j) .GE. 0.0) THEN
                  tmpsat(i,j) = sat(i,j) * MSadj_ratio(i,j)
               END IF
            END DO
         END DO
         write (*,*) "Completed clim adjustment of MS output."
         
      ELSE
         DO i = 1, LN
            DO j = 1, LT
               tmpsat(i,j) = sat(i,j)
            END DO
         END DO
         write (*,*) "No clim adjustment of MS output."
      END IF
      
      
      




c     Output AGPI and Multi-Sat
      call String_Length(fbas,strlen)

c     AGPI precip (adgeo -> adge)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.adge'
      call Write_Binary_File(fname, adgeo, 144, 72)

c     Multi-satellite precip (sat -> s)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.s'
      call Write_Binary_File(fname, tmpsat, 144, 72)

c     Multi-satellite error (esat -> es)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.es'
      call Write_Binary_File(fname, esat, 144, 72)





C	If there is no gauge, just load the SG arrays with multi-
C	satellite and jump around code involving gauge.
c
c      CALL String_Length ( gnam, l_gn )
c      IF  ( gnam (1:l_gn) .EQ. 'none' )  THEN
c         DO  j = 1, LT
c	    DO  i = 1, LN
c               sg  (i, j) = sat  (i, j)
c               esg (i, j) = esat (i, j)
c	    END DO
c         END DO
c         GO TO 900
c      END IF
      


C     Compute the bias-adjusted gauge (bias is already converted 
C     to fraction).  Fix limiting bias correction to 3 instituted
C     to emulate a GPCC fix that apparently missed our correction 
C     files.
C
      DO  j = 1, LT
         DO  i = 1, LN
	    IF  ( gaugeb (i, j) .LE. 3. )  THEN
               corga (i, j) = gauge (i, j) * gaugeb (i, j)
            ELSE
               corga (i, j) = gauge (i, j) * 3.
	    END IF
         END DO
      END DO




C     Compute gauge error using its own rain estimate for output
C     and for use in the weighted smoothing below.  The error of
C     bias correction is currently approximated as:
C     e = 0.4 * ABS ( corga - gauge )
C     The actual value used is the larger of the sampling and bias.
C     egauge positive is taken to mean that the box has data.
C     
      CALL SAMERR ( 24.,    LN, LT,    econst (6),  aconst (6), 
     +     'gaug', 1,  corga, gaugen, dum, egauge, iret )
      IF  ( iret .NE. 0 )  STOP
      DO  j = 1, LT
         DO  i = 1, LN
	    IF  ( egauge (i, j) .GT. 0. )  THEN
               IF  ( corga (i, j) .GT. 0. )  THEN
                  ebias = .4 * ABS ( corga (i, j) - gauge (i, j) )
               ELSE
                  ebias = 0.
               END IF
               egauge (i, j) = AMAX1 ( egauge (i, j), ebias )
	    END IF
         END DO
      END DO

C     Compute gauge quality index and dump gauge fields.
      
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, corga,
     +     egauge,     qgauge,     iret )
c      CALL DMPIMGF  ( gauge,  fbas, 'g',   .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( corga,  fbas, 'corg',.FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( egauge, fbas, 'ega', .FALSE., LN, LT, iret )
c      CALL DMPIMGF  ( qgauge, fbas, 'qga', .FALSE., LN, LT, iret )



c     Output Gauge precip, error and number
      call String_Length(fbas,strlen)

c     Gauge precip (corga -> corg)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.corg'
      call Write_Binary_File(fname,corga , 144, 72)

c     Gauge error (egauge -> ega)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.ega'
      call Write_Binary_File(fname, egauge, 144, 72)

c     Gauge number (gaugen -> nga)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.nga'
      call Write_Binary_File(fname, gaugen , 144, 72)







C     Adjust the multi-satellite estimate to the bias of the gauges.
C     Do this by smoothing each field on a ISMTH x ISMTH boxcar
C     (ISMTH+1 x ISMTH+1 if the gauges are too sparse), computing the
C     ratio, and multiplying the ratio by the multi-satellite field.
C     The sat field is weighted by the gaugen field so that the
C     smoothing is done the same as for the gauges.
C     
C     The ratio is tapered toward 1 linearly for percent water coverage
C     between 65 and 100.
C     
C     Start by smoothing the percent water field to bridge across small
C     features or gaps.
C     

      IF  ( nsfsm .NE. 0 )  THEN
         DO  j = 1, nsfsm
	    DO  i = 1, nsfsm
               geosm (i, j) = 1.
	    END DO
         END DO
         CALL SMOOTH  ( nsfsm, geosm, 3., surf, 
     +        LN,    LT,        smsf, iret )
         IF  ( iret .NE. 0 )  THEN
            WRITE (*, *)  'Error in surface type data'
            STOP
         END IF
      ELSE
         DO  j = 1, LT
	    DO  i = 1, LN
               smsf (i, j) = surf (i, j)
	    END DO
         END DO
      END IF
      

      IF  ( ismth .NE. 0 )  THEN
         CALL SMWT  ( ismth,  smlim,   adssmi, ess, egauge, surf, 
     +        LN, LT, .FALSE., sms,    iret )
         IF  ( iret .NE. 0 )  THEN
            WRITE (*, *)  'Error: smoother size', ismth,
     +           ' out of bounds or even in SMWT'
            STOP
         END IF
         CALL SMWT  ( ismth,  smlim,   adgeo, egeo, egauge, surf, 
     +        LN, LT, .FALSE., smge,  iret )
         CALL SMWT  ( ismth,  smlim,   corga, egauge, esat, surf, 
     +        LN, LT, .FALSE., smg,   iret )

C	    Gauge-adjust SSM/I.  The maximum allowed ratio as a function
C	    of *adjusted* SSM/I (which is coded in *unadjusted* SSM/I): 
C	      ratmax = 2    in [0,7] (0-200 mm/mo)
C	             = 1.25 above 17 (500 mm/mo)
C		     = linearly tapered in [7,17] (200-500 mm/mo)
C	    The maximum allowed additive constant is linearly tapered 
C	    from 1.7 (50mm/mo) for zero smoothed SSM/I to 0 at 7 
C	    (200 mm/mo).
C
         DO  j = 1, LT
	    DO  i = 1, LN
               IF  ( smg    (i, j) .GE. 0. .AND. 
     +              adssmi (i, j) .GE. 0. )  THEN
                  IF  ( sms (i, j) .GT. 0. )  THEN
C     
C     Coastal area; don't change sat, else land area
C     and do change sat with controls.
C     
                     IF  ( smsf (i, j) .GT. sfbrk )  THEN
                        srat (i, j) = 1.
                        sadd (i, j) = 0.
                     ELSE
                        rat    = smg (i, j) / sms (i, j)
                        IF  ( adssmi (i, j) .LE. 7. )  THEN
                           ratmax = 2.
                        ELSE
                           ratmax = AMAX1 ( 1.25,
     +                          2.525 - .075 * adssmi (i, j) )
                        END IF
                        IF  ( rat .LE. ratmax )  THEN
                           srat (i, j) = AMAX1 ( rat, .2 )
                           sadd (i, j) = 0.
                        ELSE
                           srat (i, j) = ratmax
                           sadd (i, j) = AMAX1 ( 0.,
     +                          AMIN1 ( 1.7 * ( 1. - sms (i, j) / 7. ),
     +                          smg (i, j) - ratmax * sms (i, j) ) )
                        END IF
                     END IF
                     adssmi (i, j) = srat (i, j) * adssmi (i, j) +
     +                    sadd (i, j)
C     
C		      Zero smoothed SSMI; don't change in coastal area, 
C		      else substitute smoothed gauge value in land area.
C
                  ELSE  IF  ( surf (i, j) .GT. sfbrk )  THEN
c     c			srat   (i, j) = -( 100. - smsf (i, j) ) /
cc     +				         ( 100. - sfbrk ) + 1.
cc			adssmi (i, j) = ( 1. - srat (i, j) ) * smg (i, j)
                     srat   (i, j) = 1.
                     sadd   (i, j) = 0.
                  ELSE
                     srat   (i, j) = 0.
                     sadd   (i, j) = 0.
                     adssmi (i, j) = smg (i, j)
                  END IF
C
C		  Smoothed gauge or SSMI is missing; don't do adjustment.
C
               ELSE
                  srat (i, j) = RMISS
                  sadd (i, j) = RMISS
               END IF
	    END DO
         END DO
c         CALL DMPIMGF  ( srat, fbas, 'ssrat', .FALSE., LN, LT, iret )
c         CALL DMPIMGF  ( sadd, fbas, 'ssadd', .FALSE., LN, LT, iret )

C	    Gauge-adjust AGPI.

         DO  j = 1, LT
	    DO  i = 1, LN
               IF  ( smg   (i, j) .GE. 0. .AND. 
     +              adgeo (i, j) .GE. 0. )  THEN
                  IF  ( smge (i, j) .GT. 0. )  THEN

C			Coastal area; don't change sat, else land area
C			and do change sat with controls.
                     IF  ( smsf (i, j) .GT. sfbrk )  THEN
                        srat (i, j) = 1.
                        sadd (i, j) = 0.
                     ELSE
                        rat    = smg (i, j) / smge (i, j)
                        IF  ( adgeo (i, j) .LE. 7. )  THEN
                           ratmax = 2.
                        ELSE
                           ratmax = AMAX1 ( 1.25,
     +                          2.525 - .075 * adgeo (i, j) )
                        END IF
                        IF  ( rat .LE. ratmax )  THEN
                           srat (i, j) = AMAX1 ( rat, .2 )
                           sadd (i, j) = 0.
                        ELSE
                           srat (i, j) = ratmax
                           sadd (i, j) = AMAX1 ( 0.,
     +                          AMIN1 ( 1.7 * ( 1. - smge (i, j) / 7. ),
     +                          smg (i, j) - ratmax * smge (i, j) ) )
                        END IF
                     END IF
                     adgeo (i, j) = srat (i, j) * adgeo (i, j) +
     +                    sadd (i, j)

C		      Smoothed AGPI = 0; don't change in coastal area, 
C		      else substitute smoothed gauge value in land area.
                  ELSE  IF  ( surf (i, j) .GT. sfbrk )  THEN
                     srat  (i, j) = 1.
                     sadd  (i, j) = 0.
                  ELSE
                     srat  (i, j) = 0.
                     sadd  (i, j) = 0.
                     adgeo (i, j) = smg (i, j)
                  END IF

C		  Smoothed gauge or SSMI is missing; don't do adjustment.
               ELSE
                  srat (i, j) = RMISS
                  sadd (i, j) = RMISS
               END IF
	    END DO
         END DO
c         CALL DMPIMGF  ( srat, fbas, 'gerat', .FALSE., LN, LT, iret )
c         CALL DMPIMGF  ( srat, fbas, 'geadd', .FALSE., LN, LT, iret )
      END IF
 
C	Compute the straight average of the available rainrates that
C	go into the satellite-gauge combination; this is only used for
C	the rainrate in the error calculation.
C
      DO  j = 1, LT
         DO  i = 1, LN
	    IF  ( adssmi (i, j) .GE. 0. )  THEN
               IF  ( adgeo (i, j) .LT. 0. )  THEN
C		    IF  ( corga (i, j) .LT. 0. )  THEN
                  IF  ( qgauge (i, j) .LT. 0. )  THEN
                     avg (i, j) = adssmi (i, j)
                  ELSE
                     avg (i, j) = ( adssmi (i, j) + 
     +                    corga  (i, j) ) * .5
                  END IF
               ELSE IF  ( geon (i, j) .GE. GENLIM .AND.
     +                 .NOT. mrg40ns )  THEN
C		    IF  ( corga (i, j) .LT. 0. )  THEN
                  IF  ( qgauge (i, j) .LT. 0. )  THEN
                     avg (i, j) = adgeo (i, j)
                  ELSE
                     avg (i, j) = ( adgeo  (i, j) + 
     +                    corga  (i, j) ) * .5
                  END IF
C		  ELSE IF  ( corga (i, j) .LT. 0. )  THEN
               ELSE IF  ( qgauge (i, j) .LT. 0. )  THEN
                  avg (i, j) = ( adssmi (i, j) + adgeo (i, j) ) * .5
               ELSE
                  avg (i, j) = ( adssmi (i, j) + adgeo (i, j) +
     +                 corga  (i, j) ) * .33333333333
               END IF

C	      No-SSM/I cases; don't need to test for mrg40ns.
            ELSE IF  ( adgeo (i, j) .LT. 0. )  THEN
C		IF  ( corga (i, j) .LT. 0. )  THEN
               IF  ( qgauge (i, j) .LT. 0. )  THEN
                  avg (i, j) = RMISS
               ELSE
                  avg (i, j) = corga  (i, j)
               END IF
C	      ELSE IF  ( corga (i, j) .LT. 0. )  THEN
            ELSE IF  ( qgauge (i, j) .LT. 0. )  THEN
               avg (i, j) = adgeo (i, j)
            ELSE
               avg (i, j) = ( adgeo (i, j) + corga  (i, j) ) * .5
	    END IF
         END DO
      END DO

C     Compute gauge error using the average rain just computed.  
C     As before, the error of bias correction is currently 
C     approximated as:
C     e = 0.4 * ABS ( corga - gauge )
C     The actual value used is the larger of the sampling and bias.
C     egauge positive is taken to mean that the box has data.

      CALL SAMERR ( 24.,    LN, LT,  econst (6),  aconst (6), 
     +     'gaug', 1,  avg, gaugen, dum, egauge, iret )
      IF  ( iret .NE. 0 )  STOP
      DO  j = 1, LT
         DO  i = 1, LN
	    IF  ( egauge (i, j) .GT. 0. )  THEN
               IF  ( corga (i, j) .GT. 0. )  THEN
                  ebias = .4 * ABS ( corga (i, j) - gauge (i, j) )
               ELSE
                  ebias = 0.
               END IF
               egauge (i, j) = AMAX1 ( egauge (i, j), ebias )
	    END IF
         END DO
      END DO

C     Compute gauge-adjusted SSM/I sampling error using the average
C     rain just computed.  The same constants are used as for 
C     unajusted, and the number of samples adjusted for snow/ice are 
C     the same.
      
      IF  ( ssnam (1:l_ss) .EQ. 'none' )  THEN
         CALL SAMERR ( 24.,    LN, LT,  econst (1),  aconst (1), 
     +        'ssmi', 1,  avg, snadj,  dum, ess, iret )
         IF  ( iret .NE. 0 )  STOP
      ELSE
         CALL SAMERR ( 24.,    LN,  LT, econst,     aconst, 
     +        'ssto', 3,  avg, snadj,  ss, ess, iret )
         IF  ( iret .NE. 0 )  STOP
      END IF

C     Compute gauge-adjusted AGPI sampling error using the average
C     rain just computed.

      CALL SAMERR ( 24.,    LN, LT,  econst (5), aconst (5), 
     +     'agpi', 1,  avg, geon, dum,  egeo, iret )
      IF  ( iret .NE. 0 )  STOP
      DO  j = 1+ltskp, LT-ltskp
         DO  i = 1, LN
	    IF  ( smss (i, j) .LT. 0. )  THEN
               IF  ( egeo (i, j) .GT. 0. )  egeo (i, j) = 1.41 *
     +              egeo (i, j)
	    END IF
         END DO
      END DO

C     Make sure the error is only available where the estimate is
C     available.

      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( adssmi (i, j) .LT. 0. )  ess    (i, j) = RMISS
            IF  ( adgeo  (i, j) .LT. 0. )  egeo   (i, j) = RMISS
            IF  ( corga  (i, j) .LT. 0. )  egauge (i, j) = RMISS
         END DO
      END DO

C     Form the gauge-adjusted multi-satellite precip estimate (again).
      CALL WTCOMB  ( LN, LT, adssmi, ess, adgeo, egeo, sat, esat,
     +     wtss,   wtge,   iret )
C     
      IF  ( .NOT. mrg40ns )  THEN
         DO  j = 1, LT
            DO  i = 1, LN
               IF  ( geon (i, j) .GE. GENLIM .AND.
     +              egeo (i, j) .GT. 0. )  THEN
                  sat  (i, j) = adgeo (i, j)
                  esat (i, j) = egeo  (i, j)
                  wtss (i, j) = 0.
                  wtge (i, j) = 1.
               END IF
            END DO
         END DO
      END IF
      


C     Compute gauge-adjusted SSM/I, AGPI, and S quality index, and  
C     dump out fields.  Note the use of average rain, with which the 
C     errors were computed.
C     
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, avg,
     +     egeo,       qgeo,       iret )
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, avg,
     +     ess,        qss,        iret )
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, avg,
     +     egauge,     qgauge,     iret )
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, sat,
     +     esat,       qsat,       iret )
c     CALL DMPIMGF  ( adgeo,  fbas, 'gadge', .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( adssmi, fbas, 'gadss', .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( wtge,   fbas, 'gwtge', .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( wtss,   fbas, 'gwtss', .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( sat,    fbas, 'gs',    .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( egeo,   fbas, 'gegeo', .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( ess,    fbas, 'gess',  .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( egauge, fbas, 'gega',  .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( esat,   fbas, 'ges',   .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( qss,    fbas, 'gqss',  .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( qgeo,   fbas, 'gqgeo', .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( qgauge, fbas, 'gqga',  .FALSE., LN, LT, iret )
c     CALL DMPIMGF  ( qsat,   fbas, 'gqs',   .FALSE., LN, LT, iret )


C     Compute the sat-gauge combination.
      CALL WTCOMB  ( LN,  LT,  sat, esat, corga, egauge, sg, esg, 
     +     wts, wtg, iret )
      
C     Compute SG quality index and dump out fields.
      CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, sg,
     +     esg,        qsg,        iret )
c	CALL DMPIMGF  ( wtg,    fbas, 'wtg',  .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( wts,    fbas, 'wts',  .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( sg,     fbas, 'sg',   .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( esg,    fbas, 'esg',  .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( qsg,    fbas, 'qsg',  .FALSE., LN, LT, iret )

C	Jump-to point for avoiding gauge combination step if there is 
C	no gauge analysis.
C
c  900	CONTINUE
C
C	Jump around code involving computer if there is none.
C
c	CALL ST_LSTR ( cnam, l_cn, iret )
c	IF  ( cnam (1:l_cn) .EQ. 'none' )  GO TO 1000
C
C	Compute model error and quality index using its own rain 
C	estimate, for output, and dump fields.
C
c	CALL SAMERR ( 24.,    LN, LT,   econst (7), aconst (7), 
c     +		      'comp', 1,  comp, dum, dum,   ecomp, iret )
c	IF  ( iret .NE. 0 )  STOP
c	CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, comp,
c     +		     ecomp,      qcomp,      iret )
c	CALL DMPIMGF  ( comp,  fbas, 'c',   .FALSE., LN, LT, iret )
c	CALL DMPIMGF  ( ecomp, fbas, 'ec',  .FALSE., LN, LT, iret )
C
C	Immediately set a new average rain, which is the straight 
C	average for the SG step, with computer values in any blanks
C	(because the computer estimate is considered fill-in only).
C
c	DO  j = 1, LT
c	DO  i = 1, LN
c	    IF  ( avg (i, j) .LT. 0. )  avg (i, j) = comp (i, j)
c	END DO
c	END DO
c	CALL SAMERR ( 24.,    LN, LT,   econst (7), aconst (7), 
c     +		      'comp', 1,  avg, dum, dum,   ecomp, iret )
c	IF  ( iret .NE. 0 )  STOP
C
C	Make sure the error is only available where the estimate is
C	available.
C
c	DO  j = 1, LT
c	DO  i = 1, LN
c	    IF  ( comp (i, j) .LT. 0. )  ecomp (i, j) = RMISS
c	END DO
c	END DO
C
C	Adjust the model estimate to the bias of the sat-gauge.
C	Do this by smoothing each field on a ISMTH x ISMTH boxcar
C	(ISMTH+1 x ISMTH+1 if the sat-gauge is too sparse), computing the
C	ratio, and multiplying the ratio by the model field.
C	The model field is weighted by the esg field so that the
C	smoothing is done the same as for the sat-gauge.
C
cc	>> Comment out de-bias of C by SG using dumb logic. <<
cc	IF  ( ismth .NE. 0 )  THEN
c	IF  ( ismth .NE. 0 .AND. ismth .EQ. 0 )  THEN
c	    CALL SMWT  ( ismth,  smlim, comp, ecomp, esg, surf, LN, LT,
c     +			 .TRUE., sms,   iret )
c	    IF  ( iret .NE. 0 )  THEN
c		WRITE (*, *)  'Error: smoother size', ismth,
c     +			      ' out of bounds or even in SMWT'
c		STOP
c	    END IF
c	    CALL SMWT  ( ismth,  smlim, sg, esg, ecomp, surf, LN, LT,
c     +			 .TRUE., smg,   iret )
c	    DO  j = 1, LT
c	    DO  i = 1, LN
c		IF  ( smg (i, j) .GE. 0. .AND. 
c     +		      sg  (i, j) .GE. 0. )  THEN
c		    IF  ( sms (i, j) .GT. 0. )  THEN
c			srat (i, j) = smg  (i, j) / sms (i, j)
c			comp (i, j) = smg  (i, j) / sms (i, j) *
c     +				      comp (i, j)
c		      ELSE
c			srat (i, j) = 0.
c			comp (i, j) = smg (i, j)
c		    END IF
c		  ELSE
c		    srat (i, j) = RMISS
c		END IF
c	    END DO
c	    END DO
c	    CALL DMPIMGF  ( srat, fbas, 'mrat', .FALSE., LN, LT, iret )
c	END IF
C
C	Compute the sat-gauge-computer combination.
C
c	CALL WTCOMB  ( LN,   LT,  sg, esg, comp, ecomp, sgc, esgc, 
c     +		       wtsg, wtc, iret )
C
C	Compute SGC quality index and dump fields.  If there was
C	no gauge data the names only involve 'c' and 's'.
C
c	CALL E2QI  ( econst (6), aconst (6), 24., LN, LT, sgc,
c     +		     esgc,       qsgc,       iret )
c	CALL DMPIMGF  ( wtc,   fbas, 'wtc', .FALSE., LN, LT, iret )
c	IF  ( gnam (1:l_gn) .NE. 'none' )  THEN
c	    CALL DMPIMGF  ( wtsg, fbas, 'wtsg', .FALSE., LN, LT, iret )
c	    CALL DMPIMGF  ( sgc,  fbas, 'sgc',  .FALSE., LN, LT, iret )
c	    CALL DMPIMGF  ( esgc, fbas, 'esgc', .FALSE., LN, LT, iret )
c	    CALL DMPIMGF  ( qsgc, fbas, 'qsgc', .FALSE., LN, LT, iret )
c	  ELSE
c	    CALL DMPIMGF  ( wtsg, fbas, 'wts',  .FALSE., LN, LT, iret )
c	    CALL DMPIMGF  ( sgc,  fbas, 'sc',   .FALSE., LN, LT, iret )
c	    CALL DMPIMGF  ( esgc, fbas, 'esc',  .FALSE., LN, LT, iret )
c	    CALL DMPIMGF  ( qsgc, fbas, 'qsc',  .FALSE., LN, LT, iret )
c	END IF
C
c 1000	CONTINUE






c******************************************************************
c     Output Satelite gauge and error as well as adjusted multi-sat
      call String_Length(fbas,strlen)

c     Satellite/gauge precip (sg -> sg)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.sg'
      call Write_Binary_File(fname, sg, 144, 72)

c     Satellite/gauge error (esg -> esg)
      fname=fbas(1:strlen)//'oc.'//cyear//cmonth//'.esg'
      call Write_Binary_File(fname, esg, 144, 72)


      WRITE (*, *) 'Done'
      STOP
c     END


      end program Main_GPCPcdr_m4_fuse2






