      subroutine Read_GPCC (gnam, gfmt, gbnam, days, iyear, imonth,
     &     gauge, gaugen, gaugeb)
c     Subroutine to read and return GPCC data
c     This subroutine will read either the Full or the 
c     Monitoring product or the first guess product


      IMPLICIT NONE

      integer*4     i, ii, j, iunit, icount, ios
      integer*4     nskip, indat
      INTEGER*4     LN, LT, RMISS
      integer*4     iyear, imonth
      PARAMETER     ( LN = 144, LT = 72, RMISS = -99999. )
      logical       opn
      character*1   gfmt, cdummy
      character*256 gnam, gbnam
      REAL*4        gauge(LN, LT), gaugen(LN, LT), gaugeb(LN, LT)
      real*4        days



c     Read GPCC data
c     GPCC Full V7 and Mon V5 (used in V2.3) use NetCDF
      if (gfmt .EQ. 'M' .OR. gfmt .EQ. 'F') then
         call Read_GPCCv7_NetCDF(gnam, gfmt, iyear, imonth, 
     +        gauge, gaugen)

c     Read 1deg GPCC First guess data from NetCDF file and convert to 2.5deg
      else if (gfmt .EQ. 'G') then
         call Read_GPCC_FG_NetCDF(gnam, iyear, imonth,
     +        gauge, gaugen)
      else
         WRITE(*,*) 'Error in GPCPcdr_m2_Adjust_Gauge:'//
     &        'invalid data type flag (gfmt)'
         STOP
      endif


C     Convert monthly values to mm/d.
      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( gauge (i, j) .GE. 0. )  
     +           gauge (i, j) = gauge (i, j) / DAYS
         END DO
      END DO







c**********************************************
c Read Gauge bias error

      iunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         iunit = iunit + 1
         INQUIRE(iunit, OPENED = opn)
      END DO

      OPEN ( iunit, FILE=gbnam, ACCESS='SEQUENTIAL',  
     +     FORM='FORMATTED',  IOSTAT=ios)
      if ( ios .NE. 0 .AND. ios .NE. -1 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_FUSE2: '//
     &        'Failed to open input ASCII gauge bias file'
         STOP
      endif

      READ ( iunit, '(A1)', IOSTAT=ios) cdummy
      if ( ios .NE. 0 .AND. ios .NE. -1 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_FUSE2: '//
     &        'Failed to read input ASCII gauge bias file'
         STOP
      endif

      DO  j = 1, LT
         DO  i = 1, LN
            IF  ( i .LE. LN / 2 )  THEN
               ii = i + LN / 2
            ELSE
               ii = i - LN / 2
            END IF

            READ ( iunit, 250, IOSTAT=ios) indat
 250        FORMAT ( 9X, I5 )
            if ( ios .NE. 0 .AND. ios .NE. -1 ) then
               WRITE(*,*) 'Error in GPCPcdr_m4_FUSE2: '//
     &              'Failed to read input ASCII gauge bias file'
               STOP
            endif
            
            IF  ( indat .GT. 0 )  THEN
               gaugeb (ii, j) = indat * .001
            ELSE
               gaugeb (ii, j) = 1.
            END IF

         END DO
      END DO

      CLOSE (iunit, IOSTAT=ios)
      if ( ios .NE. 0 .AND. ios .NE. -1 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_FUSE2: '//
     &        'Failed to close input ASCII gauge bias file'
         STOP
      endif




      end subroutine Read_GPCC 

