	SUBROUTINE E2QI  ( ECONST, ACONST, FAC, LN, LT, r, e, q, iret )
C************************************************************************
C* E2QI  Convert absolute error to quality index			*
C*									*
C* Convert absolute error to quality index (QI).  QI is expressed as	*
C* equivalent gauges, which is an estimate of the number of gauges per	*
C* isolated 2.5x2.5 grid box that would give the same error as the	*
C* specified value).							*
C*									*
C* E2QI  ( ECONST, ACONST, FAC, LN, LT, r, e, q, iret )			*
C*									*
C* Input parameters:							*
C*   ECONST	REAL	Multiplicative constant				*
C*   ACONST	REAL	Additive constant (same units as r)		*
C*   FAC	REAL	Factor to scale from mm/hr to units of r	*
C*   LN		INT	X size						*
C*   LT		INT	Y size						*
C*   r		REAL*	Rainfall (size LN,LT)				*
C*   e		REAL*   Absolute error (same units as r; size LN,LT)	*
C*									*
C* Output parameters:							*
C*   q		REAL*	Quality Index (in equiv.gauges; size LN,LT)	*
C*   iret	INT	Return code: 0 = normal				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 3/96	Pull out of FUSEE.F			*
C* G.Huffman/SSAI	 9/98	Relabel HR as FAC for generality	*
C* D.Bolvin/SSAI	11/08	Explicitly type variables		*
C************************************************************************
	IMPLICIT	NONE
	INTEGER		i, j
C
C	Args are (ln,lt).
C
	REAL*4		ECONST,     ACONST,     FAC,
     +			r (LN, LT), e (LN, LT), q (LN, LT)
	INTEGER*4	LN, LT,     iret
C
	REAL*4		RMISS
	PARAMETER	( RMISS = -99999. )
C
	iret = 0
C
	DO  j = 1, LT
	DO  i = 1, LN
	    IF  ( e (i, j) .GT. 0. )  THEN
		q (i, j) =
     +		    ( ACONST + r (i, j) ) * ECONST * 
     +		    ( 1. * FAC + 10. * ( FAC * r (i, j) ) ** .5 ) / 
     +		    e (i, j) ** 2
	      ELSE
		q (i, j) = RMISS
	    END IF
	END DO
	END DO
	RETURN
	END
