	SUBROUTINE SMWT  ( ismth,  wtmin,  arrin,  arrwt1, arrwt2, surf, 
     +			   LN, LT, usewat, arrout, iret )
C************************************************************************
C* SMWT  Smooth a LNxLT field with weights				*
C*									*
C* Smooth a LNxLT field with weights.  The north and south boundaries	*
C* are filled with "missing", while the east and west are periodic.	*
C* The smoother must be square.  Weights are added up, and if less than	*
C* wtmin the search area is expanded by 1 each direction and repeated.	*
C* If there are still less than wtmin weights, the boxes are all	*
C* weighted evenly (unless there is no data or you have specified to	*
C* quit over at least 75% water-cover).					*
C*									*
C* SMWT  ( ismth,  smth, wtmin, arrin, arrwt1, arrwt2, surf, LN, LT,	*
C*         arrout, iret )						*
C*									*
C* Input parameters:							*
C*   ismth	INT	Dimension of smoothing box (in pixels)		*
C*   wtmin	REAL	Minimum weight for a point to be non-missing	*
C*   arrin	REAL*	Input data (size LN,LT)				*
C*   arrwt1	REAL*	Weight field 1 (size LN,LT)			*
C*   arrwt2	REAL*	Weight field 2 (size LN,LT)			*
C*   surf	REAL*	Percent sfc coverage by water (size LN,LT)	*
C*   LN		INT	X size						*
C*   LT		INT	Y size						*
C*   usewat	LOG	T|F = do|don't use scheme in water areas	*
C*									*
C* Output parameters:							*
C*   arrout	REAL*	Output smoothed data (size MXLNCL,MXLTCL)	*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = ismth out of bounds or even	*
C**									*
C* Log:									*
C* G.Huffman/USRA	 3/92						*
C* G.Huffman/USRA	10/92	Change to passing in dimensions		*
C* G.Huffman/USRA	 2/93	Add wtmin				*
C* G.Huffman/SSAI	10/94	Adapt SMOOTH.F				*
C* G.Huffman/SSAI	11/94	Change to percent land sfc scheme	*
C* G.Huffman/SSAI	 2/96	Change to weighting by number of gauges;*
C*				check two weight fields for coincidence	*
C* D.Bolvin/SSAI	11/08	LOGICAL*1; explicitly type variables	*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i,  ii,  j, jj,  k, is_2, itype (7)
	REAL*4		wt, xwt, reject
	INTEGER*1	itoil
C
C	Args are (ln,lt).
C
	INTEGER*4	MS_2, MXX, MXY, MXSM
	PARAMETER	( MXSM = 15,
     +			  MS_2 = MXSM / 2,
     +			  MXX  = 720,
     +			  MXY  = 360 )
C
	REAL*4		wtmin,
     +			arrin  (LN, LT), arrwt1 (LN, LT),
     +			arrwt2 (LN, LT),
     +			surf   (LN, LT), arrout (LN, LT)
	INTEGER*4	ismth, LN, LT,   iret
	LOGICAL*1	usewat
C
	REAL*4		aug    (MXX + 2 * MS_2, MXY + 2* MS_2),
     +			augwt  (MXX + 2 * MS_2, MXY + 2* MS_2, 2),
     +			augwt2 (144, 72)
C
	DO  i = 1, 7
	    itype (i) = 0
	END DO
C
C	Check that ismth is odd and in bounds.
C
	IF  ( ismth .LT. 3 .OR. MOD ( ismth, 2 ) .NE. 1 .OR.
     +	      ismth .GT. MXSM )  THEN
	    iret = -1
	    RETURN
	END IF
C
C	Set the minimum weight limit.
C
	IF  ( wtmin .LT. 0. )  THEN
	    reject = 0.
	  ELSE
	    reject = wtmin
	END IF
C
C	Load an augmented array of data so that indexing the convolution
C	is easy.  The "picture frame" is loaded all the way out to MS_2
C	(rather than is_2) to simplify indexing.  The fill is -.1
C	(missing) above and below, and periodic left and right.  The
C	weighting is number of gauges, so "missing" has zero weight.
C
	is_2 = ismth / 2
	DO  j = 1, LT
	DO  i = 1, LN
	    aug   (MS_2 + i, MS_2 + j) = arrin (i, j)
	    IF  ( arrwt1 (i, j) .GE. 0. )  THEN
		augwt (MS_2 + i, MS_2 + j, 1) = arrwt1 (i, j)
	      ELSE
		augwt (MS_2 + i, MS_2 + j, 1) = 0.
	    END IF
	    IF  ( arrwt2 (i, j) .GE. 0. )  THEN
		augwt (MS_2 + i, MS_2 + j, 2) = arrwt2 (i, j)
	      ELSE
		augwt (MS_2 + i, MS_2 + j, 2) = 0.
	    END IF
	END DO
	END DO
C
	DO  j = 1, MS_2
	DO  i = 1, LN + 2 * MS_2
	    aug   (i, j               ) = -.1
	    augwt (i, j,             1) =  .0
	    augwt (i, j,             2) =  .0
	    aug   (i, j + MS_2 + LT   ) = -.1
	    augwt (i, j + MS_2 + LT, 1) =  .0
	    augwt (i, j + MS_2 + LT, 2) =  .0
	END DO
	END DO
C
	DO  j = MS_2 + 1, MS_2 + LT
	DO  i = 1,        MS_2
	    aug   (i,             j   ) = aug   (i + LN  , j   )
	    augwt (i,             j, 1) = augwt (i + LN  , j, 1)
	    augwt (i,             j, 2) = augwt (i + LN  , j, 2)
	    aug   (i + LN + MS_2, j   ) = aug   (i + MS_2, j   )
	    augwt (i + LN + MS_2, j, 1) = augwt (i + MS_2, j, 1)
	    augwt (i + LN + MS_2, j, 2) = augwt (i + MS_2, j, 2)
	END DO
	END DO
C
C	Do the convolution, indexing on aug point and smoother center.
C
	DO  j = MS_2 + 1, MS_2 + LT
	DO  i = MS_2 + 1, MS_2 + LN
	    wt  = 0.
	    xwt = 0.
	    DO  jj = -is_2, is_2
	    DO  ii = -is_2, is_2
cc		IF  ( aug (ii + i, jj + j) .GE. 0. )  THEN
		IF  ( aug   (ii + i, jj + j   ) .GE. 0. .AND.
     +		      augwt (ii + i, jj + j, 1) .GT. 0. .AND.
     +		      augwt (ii + i, jj + j, 2) .GT. 0. )  THEN
cc		    wt  = wt  + augwt (ii + i, jj + j)
		    wt  = wt  + 1.
cc		    xwt = xwt + augwt (ii + i, jj + j) *
		    xwt = xwt + 
     +		 		aug   (ii + i, jj + j)
		END IF
	    END DO
	    END DO
	    IF  ( i .EQ. MS_2 + 53 .AND. j .EQ. MS_2 + 42 )  THEN
	        WRITE (*, 999) ((aug(ii+i,jj+j),ii=-is_2,is_2),jj=
     +		-is_2,is_2), ((augwt(ii+i,jj+j,1),ii=-is_2,is_2),jj=
     +		-is_2,is_2), ((augwt(ii+i,jj+j,2),ii=-is_2,is_2),jj=
     +		-is_2,is_2), xwt, wt
 999		FORMAT (5(1X, 5F7.0, /), 5(1X, 5F7.1, /), 
     +          5(1X, 5F7.0, /), 'xwt,wt=', 2E10.3 )
	    END IF
C
C	    If there are enough stations, save the result; otherwise,
C	    do a wider area for land or if "use water region" is specified.
C
	    IF  ( wt .GE. reject )  THEN
		augwt2 (i - MS_2, j - MS_2) = wt
		arrout (i - MS_2, j - MS_2) = xwt / wt
		itype (1) = itype (1) + 1
	      ELSE
		IF  ( surf (i - MS_2, j - MS_2) .LT. 99. .OR.
     +		      usewat )  THEN
		    wt  = 0.
		    xwt = 0.
		    DO  jj = -is_2 - 1, is_2 + 1
		    DO  ii = -is_2 - 1, is_2 + 1
cc			IF  ( aug (ii + i, jj + j) .GE. 0. )  THEN
			IF  ( aug   (ii + i, jj + j   ) .GE. 0. .AND.
     +			      augwt (ii + i, jj + j, 1) .GT. 0. .AND.
     +			      augwt (ii + i, jj + j, 2) .GT. 0. )  THEN
cc			    wt  = wt  + augwt (ii + i, jj + j)
			    wt  = wt  + 1.
cc			    xwt = xwt + augwt (ii + i, jj + j) *
			    xwt = xwt + 
     +		 			aug   (ii + i, jj + j)
			END IF
		    END DO
		    END DO
C
C		    If there are now enough stations code the weights as 
C	            negative; otherwise, do even weighting over land or
C		    coast surface type.
C
		    IF  ( wt .GE. reject )  THEN
			augwt2 (i - MS_2, j - MS_2) = -wt
			arrout (i - MS_2, j - MS_2) = xwt / wt
			itype  (2)                  = itype (2) + 1
		      ELSE
			wt  = 0.
			xwt = 0.
			DO  jj = -is_2 - 1, is_2 + 1
			DO  ii = -is_2 - 1, is_2 + 1
cc			    IF  ( aug (ii + i, jj + j) .GE. 0. )  THEN
		IF  ( aug   (ii + i, jj + j   ) .GE. 0. .AND.
     +		      augwt (ii + i, jj + j, 1) .GT. 0. .AND.
     +		      augwt (ii + i, jj + j, 2) .GT. 0. )  THEN
				wt  = wt  + 1.
				xwt = xwt + aug (ii + i, jj + j)
			    END IF
			END DO
			END DO
C
C			If there are now enough "stations", 
C			code the weights as hundredths; otherwise it's 
C			missing.
C
			IF  ( wt .GE. ( ismth + 2.) ** 2 / 2. )  THEN
			    augwt2 (i - MS_2, j - MS_2) = wt * .01
			    arrout (i - MS_2, j - MS_2) = xwt / wt
			    itype  (3)                  = itype (3) + 1
			  ELSE
			    IF  ( aug (i, j) .GE. 0. )  THEN
				itype (4) = itype (4) + 1
			      ELSE
				itype (5) = itype (5) + 1
			    END IF
			    augwt2 (i - MS_2, j - MS_2) = -wt * .01
			    arrout (i - MS_2, j - MS_2) = -.1
			END IF
		    END IF
C
C		  We would do even weighting, but the underlying
C		  surface has too much water and we don't want to do 
C		  weighting there, so call it missing.
C
		  ELSE
		    IF  ( aug (i, j) .GE. 0. )  THEN
			itype (6) = itype (6) + 1
		      ELSE
			itype (7) = itype (7) + 1
		    END IF
		    augwt2 (i - MS_2, j - MS_2) = -wt * .01
		    arrout (i - MS_2, j - MS_2) = -.1
		END IF
	    END IF
	END DO
	END DO
C
	WRITE (*, 10)  LT*LN, itype
   10	FORMAT ( I5, ' grid boxes:', /, 
     +           I5, ' were weighted on a NxN', /, 
     +           I5, ' were weighted on an N+1xN+1 (land/coast)', /, 
     +           I5, ' were even-weighted (land/coast)', /, 
     +           I5, ' had too little weight (land/coast)', /,
     +           I5, ' had no weight (land/coast)', /,
     +           I5, ' failed the NxN over water/ice', /,
     +           I5, ' had no weight (water/ice)' )
c	CALL DMPIMGF  ( augwt2, 'smwt', 'augwt', .FALSE., LN, LT, iret )
	iret = 0
	RETURN
	END
