	SUBROUTINE WTCOMB  ( LN, LT, a, qa, b, qb, c, qc, wta, wtb, 
     +			     iret )
C************************************************************************
C* WTCOMB  Combine two fields according to relative error		*
C*									*
C* Combine two fields whose relative errors are known into a new field	*
C* with a corresponding relative error.					*
C*									*
C* WTCOMB  ( LN, LT, a, qa, b, qb, c, qc, wta, wtb, iret )		*
C*									*
C* Input parameters:							*
C*   LN		INT	X size						*
C*   LT		INT	Y size						*
C*   a		REAL*	Input field a (size LN,LT)			*
C*   qa		REAL*	Corresponding relative error (size LN,LT)	*
C*   b		REAL*	Input field b (size LN,LT)			*
C*   qb		REAL*	Corresponding relative error (size LN,LT)	*
C*									*
C* Output parameters:							*
C*   c		REAL*	Output field (size LN,LT)			*
C*   qc		REAL*	Corresponding relative error (size LN,LT)	*
C*   wta	REAL*	Combination weighting for a (size LN,LT)	*
C*   wtb	REAL*	Combination weighting for b (size LN,LT)	*
C*   iret	INT	Return code: 0 = normal				*
C**									*
C* Log:									*
C* G.Huffman/SSAI	10/94	Pull out of FUSE.F			*
C* G.Huffman/SSAI	 3/96	Correct weird cases			*
C* D.Bolvin/SSAI	11/08	Explicitly type variables		*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i, j
	REAL*4		wta (LN, LT), wtb (LN, LT), qtot
C
	REAL*4		a   (LN, LT), qa  (LN, LT), 
     +			b   (LN, LT), qb  (LN, LT),
     +			c   (LN, LT), qc  (LN, LT)
	INTEGER*4	LN, LT, iret
C
	REAL*4		RMISS
	PARAMETER	( RMISS = -99999. )
C
	iret = 0
C
C	Compute individual fields' weights.  As necessary, the weighting 
C	is adjusted to get a total weighting of 1.
C
	DO  j = 1, LT
	DO  i = 1, LN
	    IF  ( qa (i, j) .GT. 0. .AND. qb (i, j) .GT. 0. )  THEN
		qtot = qa (i, j) ** 2 + qb (i, j) ** 2
		wtb (i, j) = qa (i, j) ** 2 / qtot
		wta (i, j) = qb (i, j) ** 2 / qtot
cc	      ELSE IF  ( qa (i, j) .EQ. 0. .AND.
cc     +			 qb (i, j) .EQ. 0. )  THEN
cc		wtb (i, j) = .5
cc		wta (i, j) = .5
cc	      ELSE IF  ( qa (i, j) .GE. 0. )  THEN
	      ELSE IF  ( qa (i, j) .GT. 0. )  THEN
		wtb (i, j) = 0.
		wta (i, j) = 1.
cc	      ELSE IF  ( qb (i, j) .GE. 0. )  THEN
	      ELSE IF  ( qb (i, j) .GT. 0. )  THEN
		wtb (i, j) = 1.
		wta (i, j) = 0.
	      ELSE
		wtb (i, j) = 0.
		wta (i, j) = 0.
	    END IF
	END DO
	END DO
C
C	Compute combined precip estimate and quality.  The formula for 
C	qc is standard theory for propagation of error.
C
	DO  j = 1, LT
	DO  i = 1, LN
cc	    IF  ( qb (i, j) .GE. 0. )  THEN
	    IF  ( b (i, j) .NE. RMISS .AND. wtb (i, j) .GT. 0. )  THEN
cc		IF  ( qa (i, j) .GE. 0. )  THEN
		IF  ( a   (i, j) .NE. RMISS .AND.
     +		      wta (i, j) .GT. 0. )  THEN
		    c  (i, j) = b  (i, j) * wtb (i, j) +
     +				a  (i, j) * wta (i, j)
		    qc (i, j) = qb (i, j) * SQRT ( wtb (i, j) )
		  ELSE
		    c  (i, j) = b  (i, j)
		    qc (i, j) = qb (i, j)
		END IF
cc	      ELSE IF  ( qa (i, j) .GE. 0. )  THEN
	      ELSE IF  ( a   (i, j) .NE. RMISS .AND. 
     +			 wta (i, j) .GT. 0. )  THEN
		c  (i, j) = a  (i, j)
		qc (i, j) = qa (i, j)
	      ELSE
		c  (i, j) = RMISS
		qc (i, j) = RMISS
	    END IF
	END DO
	END DO
C
	RETURN
	END
