	SUBROUTINE FILLSM  ( arrin, LN, LT, LTSKP, npass, arrout, iret )
C************************************************************************
C* FILLSM  Fill holes with a 3x3 smoother				*
C*									*
C* Fill holes in a LNxLT field by repetitively applying a 3x3 smoother	*
C* at the missing points.  The north and south boundaries are filled	*
C* with "missing" (-99999.), while the east and west are periodic.	*
C* Optionally, only a central band (size LT-2*LTSKP) is processed.  If	*
C* the number of passes, npass, is known, the smoother is applied that	*
C* many times; otherwise npass is 3 times the number of passes needed	*
C* to first fill  the holes.  In the smoother, missing values simply	*
C* give more weight to the good values contributing to a point.		*
C*									*
C* FILLSM  ( arrin, LN, LT, LTSKP, npass, arrout, iret )		*
C*									*
C* Input parameters:							*
C*   arrin	REAL*	Input data (size LN,LT)				*
C*   LN		INT	X size						*
C*   LT		INT	Y size						*
C*   LTSKP	INT	Number of rows to skip in processing		*
C*									*
C* Input and output parameters:						*
C*   npass	INT	Number of smoother passes (0 is legitimate); 	*
C*			-1 = calculate and return number		*
C*									*
C* Output parameters:							*
C*   arrout	REAL*	Output smoothed data (size MXLNCL,MXLTCL)	*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = ismth out of bounds or even	*
C**									*
C* Log:									*
C* G.Huffman/USRA	 2/93	Adapt SMOOTH.F				*
C* G.Huffman/SSAI	11/94	Make missing -99999.			*
C************************************************************************
	IMPLICIT	NONE
C
C	Args are (ln,lt).
C

C       removed param file and hardcoded RMISS
c	INCLUDE		'param_.5.h'
	real            RMISS
	parameter       ( RMISS  = -99999 )


	INTEGER		MXSM, MS_2, MXX, MXY
	PARAMETER	( MXSM = 3,
     +			  MS_2 = MXSM / 2,
     +			  MXX  = 720,
     +			  MXY  = 360 )


	INTEGER		LN, LT, LTSKP,  npass, iret
	REAL		arrin (LN, LT), arrout (LN, LT)

C
	REAL		wt, xwt, smth  (MXSM, MXSM),
     +			aug   (MXX + 2 * MS_2, MXY + 2* MS_2, 2),
     +			augwt (MXX + 2 * MS_2, MXY + 2* MS_2)
	INTEGER		i,  ii, j, jj, k, m1, m2, ismth, is_2, nfill, n
	LOGICAL		done
	DATA		ismth, smth / 3, 9*1. /
C
C	Check that ismth is odd and in bounds.
C
	IF  ( ismth .LT. 3 .OR. MOD ( ismth, 2 ) .NE. 1 .OR.
     +	      ismth .GT. MXSM )  THEN
	    iret = -1
	    RETURN
	END IF
C
C	Load 2 augmented arrays of data so that indexing the convolution
C	is easy.  The "picture frame" is loaded all the way out to MS_2
C	(rather than is_2) to simplify indexing.  The fill is RMISS
C	(missing) above and below, and periodic left and right.
C
	is_2 = ismth / 2
	DO  j = 1, LT
	DO  i = 1, LN
	    aug (MS_2 + i, MS_2 + j, 1) = arrin (i, j)
	    aug (MS_2 + i, MS_2 + j, 2) = arrin (i, j)
	END DO
	END DO
C
	DO  j = 1, MS_2
	DO  i = 1, LN + 2 * MS_2
	    aug (i, j            , 1) = RMISS
	    aug (i, j + MS_2 + LT, 1) = RMISS
	    aug (i, j            , 2) = RMISS
	    aug (i, j + MS_2 + LT, 2) = RMISS
	END DO
	END DO
C
	DO  j = MS_2 + 1, MS_2 + LT
	DO  i = 1,        MS_2
	    aug (i,             j, 1) = aug (i + LN  , j, 1)
	    aug (i + LN + MS_2, j, 1) = aug (i + MS_2, j, 1)
	END DO
	END DO
C
C	Section 1: If npass is .LE. 0, apply the smoother to originally
C	missing boxes until no boxes are missing.
C
	nfill = 0
	m1    = 1
	m2    = 2
	IF  ( npass .LT. 0 )  THEN
C
C	    Check that there are holes to fill.
C
	    DO  j = 1 + LTSKP, LT - LTSKP
	    DO  i = 1,         LN
		IF  ( arrin (i, j) .EQ. RMISS )  GO TO 50
	    END DO
	    END DO
	    done = .TRUE.
	    GO TO 55
   50	    CONTINUE
	    done = .FALSE.
   55	    CONTINUE
C
	    DO  WHILE ( .NOT. done )
		nfill = nfill + 1
C
C		Do the convolution, indexing on aug corner and smth center.
C
		DO  j = MS_2 + 1 + LTSKP, MS_2 + LT - LTSKP
		DO  i = MS_2 + 1,         MS_2 + LN
		    wt  = 0.
		    xwt = 0.
		    IF  ( arrin (i - MS_2, j - MS_2) .EQ. RMISS )  THEN
			DO  jj = -is_2, is_2
			DO  ii = -is_2, is_2
			    IF  ( aug (ii + i, jj + j, m1) .NE. RMISS )  
     +								    THEN
				wt  = wt  + 
     +				    smth (ii + is_2 + 1, jj + is_2 + 1)
				xwt = xwt + 
     +				    smth (ii + is_2 + 1, jj + is_2 + 1)*
     +		 		    aug  (ii + i,        jj + j, m1)
			    END IF
			END DO
			END DO
			augwt (i, j) = wt
			IF  ( wt .EQ. 0 )  THEN
			    aug (i, j, m2) = RMISS
			  ELSE
			    aug (i, j, m2) = xwt / wt
			END IF
		    END IF
		END DO
		END DO
C
C		Wrap up the smoothing by resetting the ping-pong indices
C		m1, m2 and filling the lateral boundaries.
C
		IF  ( m1 .EQ. 1 )  THEN
		    m1 = 2
		    m2 = 1
		  ELSE
		    m1 = 1
		    m2 = 2
		END IF
		DO  j = MS_2 + 1, MS_2 + LT
		DO  i = 1,        MS_2
		    aug (i,             j, m1) = aug (i + LN  , j, m1)
		    aug (i + LN + MS_2, j, m1) = aug (i + MS_2, j, m1)
		END DO
		END DO
C
C		Check that there are still holes to fill.
C
		DO  j = MS_2 + 1 + LTSKP, MS_2 + LT - LTSKP
		DO  i = MS_2 + 1,         MS_2 + LN
		    IF  ( aug (i, j, m1) .EQ. RMISS )  GO TO 150
		END DO
		END DO
		done = .TRUE.
		GO TO 155
  150		CONTINUE
		done = .FALSE.
  155		CONTINUE
	    END DO
C	    WRITE (*, *)  'It took', nfill, ' passes to fill holes'
	    npass = nfill * 3
	END IF
C
C	Done with optional code to find npass.  Now apply the smoother
C	a total of npass times (already done nfill times in the optional
C	part; nfill = 0 if npass started .GE. 0).  The ping-pong indices
C	m1, m2 are set such that m1 is the current field.
C
	IF  ( npass .GT. 0 )  THEN
	    DO  n = 1, npass - nfill
		DO  j = MS_2 + 1 + LTSKP, MS_2 + LT - LTSKP
		DO  i = MS_2 + 1,         MS_2 + LN
		    wt  = 0.
		    xwt = 0.
		    IF  ( arrin (i - MS_2, j - MS_2) .EQ. RMISS )  THEN
			DO  jj = -is_2, is_2
			DO  ii = -is_2, is_2
			    IF  ( aug (ii + i, jj + j, m1) .NE. RMISS )  
     +								    THEN
				wt  = wt  + 
     +				    smth (ii + is_2 + 1, jj + is_2 + 1)
				xwt = xwt + 
     +				    smth (ii + is_2 + 1, jj + is_2 + 1)*
     +		 		    aug  (ii + i,        jj + j, m1)
			    END IF
			END DO
			END DO
			augwt (i, j) = wt
			IF  ( wt .EQ. 0. )  THEN
			    aug (i, j, m2) = RMISS
			  ELSE
			    aug (i, j, m2) = xwt / wt
			END IF
		    END IF
		END DO
		END DO
C
C		Wrap up the smoothing by resetting the ping-pong indices
C		m1, m2 and filling the lateral boundaries.
C
		IF  ( m1 .EQ. 1 )  THEN
		    m1 = 2
		    m2 = 1
		  ELSE
		    m1 = 1
		    m2 = 2
		END IF
		DO  j = MS_2 + 1, MS_2 + LT
		DO  i = 1,        MS_2
		    aug (i,             j, m1) = aug (i + LN  , j, m1)
		    aug (i + LN + MS_2, j, m1) = aug (i + MS_2, j, m1)
		END DO
		END DO
	    END DO
	END IF
C
C	Even if npass = 0, we need to copy the "final" array to the output
C	array.
C
	DO  j = 1, LT
	DO  i = 1, LN
	    arrout (i, j) = aug (MS_2 + i, MS_2 + j, m1)
	END DO
	END DO

C
	iret = 0
	RETURN
	END
