      program Main_GPCPcdr_m4_XOVER
c     XOVER is the second part of the original monthly 
c     combination code.  This code combines the PMW with 
c     the TOVS/AIRS data.
c     The original code was a more generic piece of code
c     for merging any two datasets, but was used only
c     for PMW+TOVS in GPCP. Many options were thus removed
c     by Sapiano although large parts of the code remain
c     essetially unchanged from the original file (Xover.f)
c
c     original header form Xover.f:
C*  XOVER	Analyzes polar longitudal "cross-over" points for the  *
C*		two precipitation data fields and produces merged data *
C*		fields						       *
C*								       *
C* This program is used to analyze the zonal average northern and      *
C* southern hemisphere "cross-over" points between two precipitation   *
C* data fields.  The program also produces a averaged merged of the    *
C* two data field.  This is done to produce a global multisatellite    *
C* precipitation estimate.  The merge can be one of two options        *
C* (1) GPCP SC and TOVS or (2) AMSU and ATOVS.  Merge field (1) is     *
C* used as the base microwave field for creating the GPCP SG data set  *
C* durign the SSM/I period.  Merge field (2) is used as the base       *
C* microwave field for creating the GPCP SG data set during the        *
C* pre-SSM/I period.  A third output field, which contains only ATOVS  *
C* (no merging) can also be created.				       *
C*								       *
C* xover osatflag cmonth cbyear ceyear cflnh culnh cllnh culsh cllsh   *
C*	 cflsh cmask cclflag					       *
C*								       *
C* Input command-line arguments:				       *
C*								       *
C*   osatflag	CHAR	flag indicating source of other sat data       *
C*			(uncorrected TOVS = 'OTOVS',		       * 
C*			 adjusted TOVS = 'ATOVS',		       *
C*			 just TOVS = 'JTOVS')			       *
C*   cmonth	CHAR	month of data to be analyzed		       *
C*   cbyear	CHAR	begin year of data to be analyzed	       *
C*   ceyear	CHAR	end year of data to be analyzed		       *
C*   cflnh	CHAR	boundary of northern hemisphere "gauge"        *
C*			adjustment (degrees latitude)		       *
C*   culnh	CHAR	upper boundary of northern hemisphere fill     *
C*			(degrees latitude)			       *
C*   cllnh	CHAR	lower boundary of northern hemisphere fill     *
C*			(degrees latitude)			       *
C*   culsh	CHAR	upper boundary of southern hemisphere fill     *
C*			(degrees latitude)			       *
C*   cllsh	CHAR	lower boundary of southern hemisphere fill     *
C*			(degrees latitude)			       *
C*   cflsh	CHAR	boundary of southern hemisphere "gauge"        *
C*			adjustment (degrees latitude)		       *
C*   cmask	CHAR	flag indicating if ocean-only or land-only     *
C*			data is to be processed			       *
C*			(ocean-only data = 'OCEAN'		       *
C*			 land-only data = 'LAND'		       *
C*			 ocean and land data = 'NOMASK')	       *
C*   cclflag	CHAR	flag indicating if missing SSM/I or AMSU land  *
C*			data is to be replaced with TOVS or ATOVS      *
C*			(irrelevant for 'JTOVS' option)		       *

      IMPLICIT NONE




c	INCLUDE		"parm.h"
C
c	INTEGER*4	INITW_DISK, WRITE_DISK, EXIT_DISK, imonth, 
c     +			ibyear, ieyear, iyear, i, j, k, iolunit, iret, 
c     +			iunit, nin, itempyr, length
C
c	REAL*4		rulnh, rllnh, rulsh, rllsh, rflnh, rflsh, 
c     +			FILL/-9999.9/, FILLCHK1/-0.5/, FILLCHK2/0.5/, 
c     +			bias
c	REAL*4		gpcpdata(144,72), osatdata(144,72), 
c     +			accgpcp(144,72,2), accosat(144,72,2),
c     +			amaskd(144,72), afixdata(144,72), 
c     +			srcdata(144,72), gpcpzavg(72), osatzavg(72), 
c     +			accgzavg(72), accozavg(72), newfield(144,72), 
c     +			newzavg(72), accnew(144,72,2), accnzavg(72), 
c     +			temp1(144,72), temp2(144,72), temp3(144,72),
c     +			rgdata(144,72), rgzavg(72), accrg(144,72,2),
c     +			accrzavg(72), temp4(144,72)
C
c	CHARACTER*1	cyr1, cclflag
c	CHARACTER*2	cmonth, cyr2
c	CHARACTER*4	cbyear, ceyear
c	CHARACTER*5	culnh, cllnh, culsh, cllsh, cflnh, cflsh,
c     +			osatflag
c	CHARACTER*6	cmask
c	CHARACTER*8	fname
c	CHARACTER*256	osatfile



      integer*4     Ninputs, inunit, ios, tovsnameN
      integer*4     iyear, i, j, k, idum1, idum2
      integer*4     strlen, l_tp
      integer*4     imonth, ibyear, ieyear, testmon

      real*4        rulnh, rllnh, rulsh, rllsh, rflnh, rflsh
      real*4        amaskd(144,72),  osatdata(144,72),
     &     gpcpdata(144,72), afixdata(144,72), srcdata(144,72),
     &     rgdata(144,72), qrgdata(144,72), newfield(144,72)

      real*4        readdata(360,180), tmpdata(360,180)

      real*4        FILLCHK3/-0.1/, FILL/-9999.9/, 
     &     FILLCHK1/-0.5/, FILLCHK2/0.5/
      real*4        AMASKLIM/70.0/

      character*2   cmonth
      character*4   cyear
      character*256 fname, InterimPATH, tovsPATH, tovslist
      character*256 tovsname(100)
      logical       opn






c     ******************************************************
c     Setup and checks
      
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 5 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'//
     &        ' Incorrect number of input arguments (should be54)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, cyear)
      call getarg(2, cmonth)
      call getarg(3, InterimPATH)
      call getarg(4, tovsPATH)
      call getarg(5, tovslist)

c     Convert month and year to numbers
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth

c     Read percent surface water coverage data
      fname='GPCPcdr_m4/GPCPcdr_m4_XOVER/Ancillary/surfrac.2.5.le'
      call Read_Binary_File(fname,144,72,1,amaskd)

      



c*********************************************************
C     Read the begin and end years as well as the fill area band
C     limits from standard file. These vary by month

      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO
      
      fname='GPCPcdr_m4/GPCPcdr_m4_XOVER/TOVSprocParams.txt'
      open(inunit, file=fname, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'
     &        //'Failed to open TOVS processing settings file'
         STOP
      end if

c     skip header line
      read(inunit, *)

      do i=1,12
         read(inunit, 155, iostat=ios ) testmon,  
     &        rflnh, rulnh, rllnh, rulsh, rllsh, rflsh
         if ( testmon .EQ. imonth ) exit
         if ( ( ios .NE. 0 ) .OR. ( i .EQ. 12 ) ) then
            WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'
     &           //'Failed to read TOVS processing settings file'
            STOP
         end if
      end do
 
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'
     &        //'Failed to close TOVS processing settings file'
         STOP
      end if

 155  format(i2, x, f5.1, f5.1, f5.1, f6.1, f6.1, f6.1)



C     Check that the input northern and southern fill-area band limits
C     are valid. 
      ios=0
      IF  ((rulnh .LE. 0.0) .OR. (rllnh .LE. 0.0) .OR.
     &     (rflnh .LE. 0.0) .OR. (rulsh .GE. 0.0) .OR. 
     &     (rllsh .GE. 0.0) .OR. (rflsh .GE. 0.0)) ios=1 
      IF  ((rulnh .GE. rflnh) .OR.(rllnh .GE. rulnh) .OR.
     &     (rllsh .GE. rulsh) .OR.(rflsh .GE. rllsh)) ios=1
      IF  ((AMOD(rulnh,2.5).NE.0.0) .OR. (AMOD(rllnh,2.5).NE.0.0) .OR.
     &     (AMOD(rulsh,2.5).NE.0.0) .OR. (AMOD(rllsh,2.5).NE.0.0) .OR. 
     &     (AMOD(rflnh,2.5).NE.0.0) .OR. (AMOD(rflsh,2.5).NE.0.0)) ios=1
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'//
     &             'Boundary values failed checks'
         STOP
      end if





c*********************************************************
c     Read contents of TOVSlist
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO

      open(inunit, file=tovslist, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'
     &        //'Failed to open TOVS file listing file'
         STOP
      end if
      tovsnameN=0
      do tovsnameN=1,50
         read(inunit, '(A256)', iostat=ios ) tovsname(tovsnameN)
         if ( ios .NE. 0 ) exit
      end do
      tovsnameN=tovsnameN-1
      if ( tovsnameN .EQ. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'
     &        //'Failed to read TOVS file listing file'
         STOP
      end if
      close(inunit, iostat=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in Main_GPCPcdr_m4_XOVER:'
     &        //'Failed to close TOVS file listing file'
         STOP
      end if






C***********************************************************************
C     Ingest PMW data

      do strlen = 1,len_trim(InterimPATH)
         if (InterimPATH(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1

c     Read the PMW data from MWComb
      fname=InterimPATH(1:strlen)//'/MWComb/cg.'
     &     //cyear//cmonth//'.rfix'
      call Read_Binary_File(fname,144,72,1,gpcpdata)
      
c     Read the PMW number of images field
      fname=InterimPATH(1:strlen)//'/MWComb/cg.'
     &     //cyear//cmonth//'.Ifix'
      call Read_Binary_File(fname,144,72,1,afixdata)
      
c     Read the PMW data source field
      fname=InterimPATH(1:strlen)//'/MWComb/cg.'
     &     //cyear//cmonth//'.src'
      call Read_Binary_File(fname,144,72,1,srcdata)
      




      
C***********************************************************************
c     Read TOVS single month of TOVS data

c     get filename to read
      do l_tp = 1,len_trim(tovsPATH)
         if (tovsPATH(l_tp:l_tp) .eq. ' ') then
            continue
         end if
      end do
      l_tp=l_tp-1

      if ( (iyear - 1984) .GT. tovsnameN ) then
         write(*,*) "TOVS filename not in TOVS_Listing.txt "
     &        //"for that year - taking a guess at filename..."
         fname=tovsPATH(1:l_tp)//"monthlies.rain."//cyear//".am.pm.avg"
      else
c        Name is in TOVS_Listing.txt
         fname=tovsPATH(1:l_tp)//tovsname(iyear - 1984)
      end if

c     get unused unit number
      inunit = 0
      opn = .TRUE.
      DO WHILE (opn)
         inunit = inunit + 1
         INQUIRE(inunit, OPENED = opn)
      END DO

c     open file
      open(inunit, file=fname, IOSTAT=ios, status='old',
     &     form='unformatted',access='direct',recl=4*64802)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to open file'
         STOP
      end if

c     read records from file
      read(inunit,rec=imonth, IOSTAT=ios) idum1, readdata, idum2
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to read file'
         STOP
      end if

c     close file
      close(inunit, IOSTAT=ios)
      if ( ios .NE. 0 ) then
         WRITE(*,*) 'Error in '//fname//':'//'Failed to close file'
         STOP
      end if

C     Rearrange the TOVS data to match the GPCP coordinate system.  Also 
C     regrid the data from the 1 degree to the 2.5 degree resolution.
C     If the month of interest is not found, print an error message 
C     and terminate execution.
C     
      DO j = 1, 180
         DO k = 1, 360
            IF  (k .LE. 180) THEN
               tmpdata(k,j) = readdata(k + 180, 181 - j)
            ELSE
               tmpdata(k,j) = readdata(k - 180, 181 - j)
            END IF
         END DO
      END DO
      
      CALL G1TO2P5 (tmpdata, FILLCHK3, osatdata)
      


c      DO i = 1, 144
c         DO j = 1, 72
c            print *, i, j, osatdata(i,j)
c         end do
c      enddo

      


C***********************************************************************
C     Read the rain gauge data for use in comparing with the merged
C     data at the poles.
      
c     get corrected gauge data
      fname=InterimPATH(1:strlen)//'/Adjust_Guage/oc.'
     &     //cyear//cmonth//'.corg'
      call Read_Binary_File(fname,144,72,1,rgdata)
      
c     get gauge quality data
      fname=InterimPATH(1:strlen)//'/Adjust_Guage/oc.'
     &     //cyear//cmonth//'.qga'
      call Read_Binary_File(fname,144,72,1,qrgdata)
      
C     Screen out the "analysis" data to include only data that has
C     observations.
      DO i = 1, 144
         DO j = 1, 72
            IF  (qrgdata(i,j) .LT. FILLCHK1) THEN
               rgdata(i,j) = FILL
            END IF
         END DO
      END DO
      
      



c      print *,  osatdata
c, amaskd, afixdata, srcdata, 
c     +     rgdata, AMASKLIM, newfield)
      

C***********************************************************************
C     Average the two data fields in the user input northern and 
C     southern hemisphere band limits.
c     This routine also outputs the data - this is a little odd, 
c     but is retained from the original code for now...

      CALL SIMPAVG1 (rulnh, rllnh, rulsh, rllsh, rflnh, rflsh,
     +     imonth, iyear, InterimPATH, gpcpdata, 
     +     osatdata, amaskd, afixdata, srcdata, 
     +     rgdata, AMASKLIM, newfield)
      

      

      end program Main_GPCPcdr_m4_XOVER
