      SUBROUTINE SIMPAVG1 (rulnh, rllnh, rulsh, rllsh, rflnh, rflsh, 
     +     imonth, iyear, InterimPATH, gpcpdata, osatdata, 
     +     amaskd, afixdata, srcdata, rgdata, AMASKLIM,
     +     newfield)

c     Made some changes to this function
c     Reformatted to remove carriage returns and tabs
c     removed depenbdecy on parms.h and added extra inputs
c     changed output code

C***********************************************************************
C* SIMPAVG1          Average the GPCP SC and TOVS data in the transition    *
C*                zones                                                        *
C*                                                                       *
C* Called by:        XOVER                                                       *
C*                                                                          *
C* Calls:        CZONAVG, FCOLDLND, ST_INLN                               *
C*                                                                         *
C* SIMPAVG1 performs a simple average of the GPCP SC and TOVS data in  *
C* the transition zones.  If no GPCP SC data exists, ratio-adjusted    *
C* TOVS is used.                                                       *
C*                                                                       *
C* Input Parameters:                                                       *
C*                                                                       *
C*  rulnh        REAL        upper boundary of northern hemisphere fill     *
C*                        (degrees latitude)                               *
C*  rllnh        REAL        lower boundary of northern hemisphere fill     *
C*                        (degrees latitude)                               *
C*  rulsh        REAL        upper boundary of southern hemisphere fill     *
C*                        (degrees latitude)                               *
C*  rllsh        REAL        lower boundary of southern hemisphere fill     *
C*                        (degrees latitude)                               *
C*  rflnh        REAL        boundary of northern hemisphere "gauge"        *
C*                        adjustment (degrees latitude)                       *
C*  rflsh        REAL        boundary of southern hemisphere "gauge"        *
C*                        adjustment (degrees latitude)                       *
C*  imonth        INT        month number of data to be analyzed               *
C*  iyear        INT        year number of data to be analyzed               *
C*  cclflag        CHAR        flag indicating if missing SSM/I cold land     *
C*                        is to be replaced with TOVS                       *
C*  gpcpdata        REAL        array containing the month of GPCP SC data     *
C*  osatdata        REAL        array containing the month of TOVS data        *
C*  amaskd        REAL        array containing the percent surface water     *
C*                        coverage data                                       *
C*  afixdata        REAL        array containing the number of SSM/I images    *
C*  srcdata        REAL        array containing the source of the SSM/I data  *
C*  rgdata        REAL        array containing the month of gauge data       *
C*                                                                        *
C* Output Parameters:                                                       *
C*                                                                       *
C*  afixdata        REAL        array containing the number of images               *
C*                        (missing            =   -99999,                       *
C*                         SSM/I              =   <original Ifix number>, *
C*                         SSM/I / TOVS avg  =   <original Ifix number>, *
C*                         TOVS                   =   1,                       *
C*                         Ratio-Adj TOVS           =   1                       *
C*  srcdata        REAL        array containing the source of the data               *
C*                        (missing            =   -99999,                       *
C*                         SSM/I Chang (ocn) =   0,                        *
C*                         SSM/I Grody (lnd) =   1,                        *
C*                         SSM/I Combined           =   0 - 1,                        *
C*                         SSM/I / TOVS avg  =   SSM/I + 2 (2 - 3),      *
C*                         TOVS                   =   4,                       *
C*  newfield        REAL        array containing the month of averaged GPCP SC *
C*                         / TOVS data                                       *
C*                                                                        *
C**                                                                       *
C* Log:                                                                       *
C* D.Bolvin/SSAI         8/97        First Version                               *
C* D.Bolvin/SSAI        10/97        Add rssfix, ifix and src processing    *
C* D.Bolvin/SSAI         4/98        smooth-fill from indulnh and indllsh   *
C*                                ratios to 1 at indflnh and indflsh     *
C* D.Bolvin/SSAI         4/98        Add rain gauge data processing               *
C* D.Bolvin/SSAI        10/98        Northern ratios at Iceland set to 1    *
C* D.Bolvin/SSAI        10/98        Minor mods for production mode               *
C* D.Bolvin/SSAI         6/00        4-digit year file names                       *
C***********************************************************************
C
      IMPLICIT NONE
      
c     INCLUDE                "parm.h"

      INTEGER*4     i, j, indulnh, indllnh, indulsh, indllsh, 
     +     indflnh, indflsh, npass, iret, iyear, imonth, 
     +     lengthy, lengthm, itempyr, iunit, nin, strlen
     
      REAL*4        rulnh, rllnh, rulsh, rllsh, rflnh, rflsh, 
     +     FILLCHK/-0.5/, FILL/-9999.9/, ratio, rationh, 
     +     ratiosh, rationhg, ratioshg
      REAL*4        gpcpdata(144,72), newfield(144,72), 
     +     osatdata(144,72), amaskd(144,72), 
     +     smfield(144,40), afixdata(144,72), 
     +     srcdata(144,72), gpcpzavg(72), osatzavg(72), 
     +     newzavg(72), rtflino(144,72), rtflouto(144,72),
     +     rtflinl(144,72), rtfloutl(144,72), 
     +     rgdata(144,72), rgzavg(72), tovsdata(144,72)
      real*4        AMASKLIM     

      CHARACTER*2   cmonth
      CHARACTER*4   cyear
      CHARACTER*256 rfixfile, rssfile, afixfile, srcfile, gpcpfile, 
     +     osatfile, InterimPATH, fname


C***********************************************************************
C     Compute the indices of the northern and southern band limits 
C     indices based on the user input latitude limits.
C     
      indflnh = INT ((90.0 - rflnh) /2.5)
      indulnh = INT ((90.0 - rulnh) /2.5)
      indllnh = INT (((90.0 - rllnh) /2.5) + 1.0)
      indulsh = INT ((90.0 - rulsh) /2.5)
      indllsh = INT (((90.0 - rllsh) /2.5) + 1.0)
      indflsh = INT (((90.0 - rflsh) /2.5) + 1.0)
C     
c      WRITE (*,*) indulnh, indllnh
c      WRITE (*,*) indulsh, indllsh
C     
C***********************************************************************
C     Fill the new data field array with "missing" values.
C     
      DO i = 1, 144
         DO j = 1, 72
            newfield(i,j) = FILL
         END DO
      END DO
C     
C***********************************************************************
C     Fill the area at and below indllnh and at and above indulsh with
C     the GPCP SC data if it exists.  If no GPCP SC data exists, 
C     zonal average ratio-adjusted TOVS data will be used.  The ratio
C     adjustment and assignment is performed later.
C     
      DO i = 1, 144
         DO j = indllnh, indulsh
            IF  (gpcpdata(i,j) .GT. FILLCHK) THEN
               newfield(i,j) = gpcpdata(i,j)
            END IF
         END DO
      END DO
C     
C***********************************************************************
C     Fill the transition zones with the average of the GPCP SC and 
C     TOVS data.  Zonal average ratio-adjusted TOVS data will be used
C     to fill the gridboxes where GPCP SC data is missing.  The ratio
C     adjustment and assignment is performed later.
C     
      DO i = 1, 144
C     
         DO j = indulnh + 1, indllnh - 1
C     
            IF  (gpcpdata(i,j) .GT. FILLCHK) THEN
               newfield(i,j) = (gpcpdata(i,j) + osatdata(i,j)) 
     +              / 2.0
               srcdata(i,j) = srcdata(i,j) + 2.0
            END IF
C     
         END DO
C     
         
         DO j = indulsh + 1, indllsh - 1
C     
            IF  (gpcpdata(i,j) .GT. FILLCHK) THEN
               newfield(i,j) = (gpcpdata(i,j) + osatdata(i,j)) 
     +              / 2.0
               srcdata(i,j) = srcdata(i,j) + 2.0
            END IF
C     
         END DO
C     
      END DO
C     
C***********************************************************************
C     Compute the zonal averages of the new data field and the TOVS 
C     data field, accepting only coincident grid boxes in the 
C     averages.  First, the ratio of the zonal averages for each 
C     latitude will be used to adjust the bias of the TOVS data that 
C     will be used to fill in the areas where GPCP SC data is missing
C     (below indulnh and above indllsh).  Second, the ratios at 
C     (indulnh + 1) and at (indllsh - 1) will be used to ratio-adjust 
C     the TOVS data from 1 to indulnh and from indllsh to 72, 
C     respectively.
C     
      CALL CZONAVG (newfield, osatdata, newzavg, osatzavg)

c      DO j = 1, 72
c         print *, j, newzavg(j), osatzavg(j)
c      end do

C     
C***********************************************************************
C     Adjust the bias of the TOVS data, used to fill the missing 
C     GPCP SC data, by the ratio of the zonal averages of the new data
C     field and the TOVS data.
C     
      DO j = indulnh + 1, indllsh - 1
C     
         IF  (newzavg(j) .GT. FILLCHK) THEN
            ratio = newzavg(j) / osatzavg(j)
         ELSE
            WRITE (*,*) 'ERROR: ZONAL AVERAGE CONTAINS NO DATA; ',
     +           'CANNOT COMPUTE RATIO ADJUSTMENT; TERMINATING ',
     +           'EXECUTION; SUBROUTINE SIMPAVG1'
            STOP
         END IF
C     
         DO i = 1, 144
            IF  (newfield(i,j) .LT. FILLCHK) THEN
               newfield(i,j) = ratio * osatdata(i,j)
               afixdata(i,j) = 1.0
               srcdata(i,j) = 4.0
            END IF
         END DO
C     
      END DO
C     
C***********************************************************************
C     Adjust the polar TOVS data based on the zonal average 
C     GPCP SC / TOVS ratios at (indulnh + 1) and (indllsh - 1).
C     
      IF  (newzavg(indulnh + 1) .GT. FILLCHK) THEN
         rationh = newzavg(indulnh + 1) / osatzavg(indulnh + 1)
      ELSE
         WRITE (*,*) 'ERROR: ZONAL AVERAGE AT NORTHERN HEMISPHERE ',
     +        'BAND LIMIT IS FILLED; CANNOT COMPUTE RATIO ',
     +        'ADJUSTMENT; TERMINATING EXECUTION; ',
     +        'SUBROUTINE SIMPAVG1'
         STOP
      END IF
C     
      DO i = 1, 144
         DO j = 1, indulnh
            newfield(i,j) = rationh * osatdata(i,j)
            afixdata(i,j) = 1.0
            srcdata(i,j) = 4.0
         END DO
      END DO
C     
C     WRITE (*,*) indulnh + 1, rationh
C     
      IF  (newzavg(indllsh - 1) .GT. FILLCHK) THEN
         ratiosh = newzavg(indllsh - 1) / osatzavg(indllsh - 1)
      ELSE
         WRITE (*,*) 'ERROR: ZONAL AVERAGE AT SOUTHERN HEMISPHERE ',
     +        'BAND LIMIT IS FILLED; CANNOT COMPUTE RATIO ',
     +        'ADJUSTMENT; TERMINATING EXECUTION; ',
     +        'SUBROUTINE SIMPAVG1'
         STOP
      END IF
C     
      DO i = 1, 144
         DO j = indllsh, 72
            newfield(i,j) = ratiosh * osatdata(i,j)
            afixdata(i,j) = 1.0
            srcdata(i,j) = 4.0
         END DO
      END DO
C     
C     WRITE (*,*) indllsh - 1, ratiosh
C     
C***********************************************************************
C     Perform a smooth-fill of the SSM/I / TOVS ratio from indulnh + 1
C     to cgauge / TOVS ratio at indflnh.  Do the same thing in the
C     southern hemisphere.  The northern hemisphere gauge correction
C     to the TOVS data will be computed for the month based on the 
C     data.  The ratio will be capped at 2.  Because the southern
C     hemisphere gauges are sparse and strongly influenced by 
C     orography, the annual southern hemisphere average of 1.4 will
C     always be applied.  Create separate land and ocean ratio arrays
C     to be smooth-filled.  This allows treating land and ocean
C     separately since it is believed the gauge correction is 
C     artificially boosting ocean values.  The two latitude bands
C     covering Iceland are set to 1.0 to help moderate the ocean 
C     adjustment.  A land/ocean mask is used to determine which ratio
C     will be applied to the TOVS data.  To smooth the transition 
C     of ratios across the gauge/TOVS ratio and ratio = 1.0 zone, the 
C     gauge/TOVS ratio over ocean will be moved north by 1 latitude
C     band to indflnh - 1.  Note that the southern hemisphere is not 
C     affected by the different land/ocean ratio arrays.  
C     
      CALL CZONAVG (osatdata, rgdata, osatzavg, rgzavg)
C     
      IF  ((osatzavg(indflnh + 1) .GT. FILLCHK) .AND.
     +     (rgzavg(indflnh + 1) .GT. FILLCHK)) THEN
         rationhg = rgzavg(indflnh + 1) / osatzavg(indflnh + 1)
      ELSE
         WRITE (*,*) 'ERROR: ZONAL AVERAGE AT NORTHERN ',
     +        'HEMISPHERE GAUGE BAND LIMIT IS FILLED; ',
     +        'CANNOT COMPUTE RATIO ADJUSTMENT; ',
     +        'TERMINATING EXECUTION; SUBROUTINE SIMPAVG1'
         STOP
      END IF
C     
      IF  (rationhg .GT. 2.0) THEN
         rationhg = 2.0
         WRITE (*,*) 'WARNING: NORTHERN HEMISPHERE GAUGE-TOVS ',
     +        'RATIO > 2.0; SET TO 2.0; CONTINUING ',
     +        'EXECUTION; SUBROUTINE SIMPAVG1'
      END IF
C     
      ratioshg = 1.4
C     
      DO i = 1, 144
         DO j = 1, indflnh
            rtflinl(i,j) = rationhg
         END DO
         DO j = indflnh + 1, indulnh
            rtflinl(i,j) = -99999.0
         END DO
         DO j = 1, indflnh - 1 
            rtflino(i,j) = rationhg
         END DO
         DO j = indflnh, indulnh
            rtflino(i,j) = -99999.0
         END DO
         DO j = indulnh + 1, 36
            rtflinl(i,j) = rationh
            rtflino(i,j) = rationh
         END DO
         DO j = 37, indllsh - 1
            rtflinl(i,j) = ratiosh
            rtflino(i,j) = ratiosh
         END DO
         DO j = indllsh, indflsh - 1
            rtflinl(i,j) = -99999.0
            rtflino(i,j) = -99999.0
         END DO
         DO j = indflsh, 72
            rtflinl(i,j) = ratioshg
            rtflino(i,j) = ratioshg
         END DO
      END DO
C     
C***********************************************************************
C     The two latitude bands covering Iceland (j = 10,11) are set to
C     a ratio value of 1.0.
C     
      DO i = 1, 144
         DO j = 10, 11
            rtflino(i,j) = 1.0
         END DO
      END DO
C     
      npass = -1
      CALL FILLSM (rtflino, 144, 72, 0, npass, rtflouto, iret)
C     
      npass = -1
      CALL FILLSM (rtflinl, 144, 72, 0, npass, rtfloutl, iret)
C     
      DO i = 1, 144
         DO j = 1, indulnh
            IF  (amaskd(i,j) .GE. AMASKLIM) THEN
               newfield(i,j) = rtflouto(i,j) * osatdata(i,j)
            ELSE
               newfield(i,j) = rtfloutl(i,j) * osatdata(i,j)
            END IF
         END DO
         DO j = indllsh, 72
            IF  (amaskd(i,j) .GE. AMASKLIM) THEN
               newfield(i,j) = rtflouto(i,j) * osatdata(i,j)
            ELSE
               newfield(i,j) = rtfloutl(i,j) * osatdata(i,j)
            END IF
         END DO
      END DO
C
c        iunit = INITW_DISK ('input_ratfield_land', 20)
c        nin = WRITE_DISK (iunit, rtflinl, 41472)
c        iret = EXIT_DISK (iunit)
c        iunit = INITW_DISK ('output_ratfield_land', 21)
c        nin = WRITE_DISK (iunit, rtfloutl, 41472)
c        iret = EXIT_DISK (iunit)
C
c        iunit = INITW_DISK ('input_ratfield_ocean', 21)
c        nin = WRITE_DISK (iunit, rtflino, 41472)
c        iret = EXIT_DISK (iunit)
c        iunit = INITW_DISK ('output_ratfield_ocean', 22)
c        nin = WRITE_DISK (iunit, rtflouto, 41472)
c        iret = EXIT_DISK (iunit)
C
C***********************************************************************
C        Replace SSM/I cold land data with TOVS data, if requested.
C
c      IF  (cclflag .EQ. 'Y') THEN
c         CALL FCOLDLND (gpcpdata, osatdata, amaskd, afixdata, 
c     +        srcdata, newfield)
c      ELSE IF  (cclflag .NE. 'N') THEN
c         WRITE (*,*) 'ERROR: INVALID COLD LAND FLAG ', cclflag,
c     +        '; TERMINATING EXECUTION; SUBROUTINE ',
c     +        'SIMPAVG1'
c         STOP
c      END IF
C     




C***********************************************************************
C     Write output fields to file

      do strlen = 1,len_trim(InterimPATH)
         if (InterimPATH(strlen:strlen) .eq. ' ') then
            continue
         end if
      end do
      strlen=strlen-1

      write(cyear, '(I4.4)' ) iyear
      write(cmonth, '(I2.2)' ) imonth

c     output TOVS/AIRS data to file
      fname=InterimPATH(1:strlen)//'/XOVER/cgt.'
     &     //cyear//cmonth//'.tovs'
      call Write_Binary_File(fname,osatdata,144,72)

C     Write the rfix data to disk.
      rfixfile=InterimPATH(1:strlen)//'XOVER/cgt.'
     &     //cyear//cmonth//'.rfix'
      call Write_Binary_File(rfixfile, newfield, 144, 72)

C     Create the rssfix field and write to file
      DO i = 1, 144
         DO j = 1, 40
            smfield(i,j) = newfield(i,j + 16)
         END DO
      END DO

      rssfile=InterimPATH(1:strlen)//'XOVER/cgt.'
     &     //cyear//cmonth//'.rssfix'
      call Write_Binary_File(rssfile, smfield, 144, 40)

C     Write the Ifix data to disk.
      afixfile=InterimPATH(1:strlen)//'XOVER/cgt.'
     &     //cyear//cmonth//'.Ifix'
      call Write_Binary_File(afixfile, afixdata, 144, 72)

C     Write the src data to disk.
      srcfile=InterimPATH(1:strlen)//'XOVER/cgt.'
     &     //cyear//cmonth//'.src'
      call Write_Binary_File(srcfile, srcdata, 144, 72)



C***********************************************************************
C        Create the file names for the output fields.
C
c        CALL ST_INLN (iyear, cyear, lengthy, iret)
c        CALL ST_INLN (imonth, cmonth, lengthm, iret)



C
c        IF  (lengthm .EQ. 1)  THEN
c            rfixfile = 'cgt.'//cyear//'0'//cmonth(1:1)//'.rfix'
c            rssfile = 'cgt.'//cyear//'0'//cmonth(1:1)//'.rssfix'
c            afixfile = 'cgt.'//cyear//'0'//cmonth(1:1)//'.Ifix'
c            srcfile = 'cgt.'//cyear//'0'//cmonth(1:1)//'.src'
c        ELSE
c            rfixfile = 'cgt.'//cyear//cmonth//'.rfix'
c            rssfile = 'cgt.'//cyear//cmonth//'.rssfix'
c            afixfile = 'cgt.'//cyear//cmonth//'.Ifix'
c            srcfile = 'cgt.'//cyear//cmonth//'.src'
c        END IF
C
c        rfixfile = outspath//rfixfile
c        rssfile = outspath//rssfile
c        afixfile = outspath//afixfile
c        srcfile = outspath//srcfile
C
C***********************************************************************
C        Open the rfix file.
C
c        iunit = INITW_DISK (rfixfile, 256)
C
c        IF  (iunit .LE. 0) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO OPEN RFIX OUTPUT FILE; ',
c     +                        'TERMINATING EXECUTION; SUBROUTINE SIMPAVG1'
c                 STOP
c            END IF   
C
C***********************************************************************
C        Write the rfix data to disk.
C
c        nin = WRITE_DISK (iunit, newfield, 41472)
C
c        IF  (nin .NE. 41472) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO WRITE RFIX FIELD TO DISK; ',
c     +                        'CONTINUING EXECUTION; SUBROUTINE SIMPAVG1'
c             END IF
C
C***********************************************************************
C        Close the rfix field file.
C
c          iret = EXIT_DISK (iunit)
C
c        IF  (iret .NE. 1) THEN
c            WRITE (*,*) 'WARNING: UNABLE TO CLOSE RFIX FIELD ',
c     +                        'OUTPUT FILE; CONTINUING EXECUTION; ',
c     +                        'SUBROUTINE SIMPAVG1'
c        END IF
C
C
C***********************************************************************
C        Open the rssfix file.
C
c        iunit = INITW_DISK (rssfile, 256)
C
c        IF  (iunit .LE. 0) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO OPEN RSSFIX OUTPUT FILE; ',
c     +                        'TERMINATING EXECUTION; SUBROUTINE SIMPAVG1'
c                 STOP
c            END IF   
C
C***********************************************************************
C        Write the rssfix data to disk.
C
c        nin = WRITE_DISK (iunit, smfield, 23040)
C
c        IF  (nin .NE. 23040) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO WRITE RSSFIX FIELD TO DISK; ',
c     +                        'CONTINUING EXECUTION; SUBROUTINE SIMPAVG1'
c             END IF
C
C***********************************************************************
C        Close the rssfix field file.
C
c          iret = EXIT_DISK (iunit)
C
c        IF  (iret .NE. 1) THEN
c            WRITE (*,*) 'WARNING: UNABLE TO CLOSE RSSFIX FIELD ',
c     +                        'OUTPUT FILE; CONTINUING EXECUTION; ',
c     +                        'SUBROUTINE SIMPAVG1'
c        END IF
C
C***********************************************************************
C        Open the Ifix file.
C
c        iunit = INITW_DISK (afixfile, 256)
C
c        IF  (iunit .LE. 0) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO OPEN IFIX OUTPUT FILE; ',
c     +                        'TERMINATING EXECUTION; SUBROUTINE SIMPAVG1'
c                 STOP
c            END IF   
C
C***********************************************************************
C        Write the Ifix data to disk.
C
c        nin = WRITE_DISK (iunit, afixdata, 41472)
C
c        IF  (nin .NE. 41472) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO WRITE IFIX FIELD TO DISK; ',
c     +                        'CONTINUING EXECUTION; SUBROUTINE SIMPAVG1'
c             END IF
C
C***********************************************************************
C        Close the Ifix field file.
C
c          iret = EXIT_DISK (iunit)
C
c        IF  (iret .NE. 1) THEN
c            WRITE (*,*) 'WARNING: UNABLE TO CLOSE IFIX FIELD ',
c     +                        'OUTPUT FILE; CONTINUING EXECUTION; ',
c     +                        'SUBROUTINE SIMPAVG1'
c        END IF
C
C***********************************************************************
C        Open the src file.
C
c        iunit = INITW_DISK (srcfile, 256)
C
c        IF  (iunit .LE. 0) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO OPEN SRC OUTPUT FILE; ',
c     +                        'TERMINATING EXECUTION; SUBROUTINE SIMPAVG1'
c                 STOP
c            END IF   
C
C***********************************************************************
C        Write the src data to disk.
C
c        nin = WRITE_DISK (iunit, srcdata, 41472)
C
c        IF  (nin .NE. 41472) THEN
c            WRITE (*,*) 'ERROR: UNABLE TO WRITE SRC FIELD TO DISK; ',
c     +                        'CONTINUING EXECUTION; SUBROUTINE SIMPAVG1'
c             END IF
C
C***********************************************************************
C        Close the src field file.
C
c          iret = EXIT_DISK (iunit)
C
c        IF  (iret .NE. 1) THEN
c            WRITE (*,*) 'WARNING: UNABLE TO CLOSE SRCX FIELD ',
c     +                        'OUTPUT FILE; CONTINUING EXECUTION; ',
c     +                        'SUBROUTINE SIMPAVG1'
c        END IF
C





        RETURN
        END
