C@***h* GPCP_Monthly_Analysis/Main_GPCPcdr_m4_combMW.F
C       Global Precipitation Climatology Project (GPCP)
C
C NAME
C   Main_GPCPcdr_m4_combMW.F
C
C PURPOSE
C   This code is to combine microwave precipitation estimates based on 
C   cg_combo.f
C
C DESCRIPTION
C   This is a sample fortran program to merge the precipitation data
C   from 
C   SSMI and SSMIS.  There are other similar programs to merge the 
C   precipitation from other products — GPCC, TOVS/AIRS, IR.
C  
C AUTHOR
C   Robert Adler, Mathew Sapiano, Jian-Jian Wang
C   University of Maryland College Park
C
C COPYRIGHT
C   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE
C   PUBLIC
C   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
C   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
C   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
C   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
C   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
C   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE
C   TECHNICAL
C   SUPPORT TO USERS.
C
C HISTORY
C 3/2016 Initial Version prepared for NCEI
C
C@*****      

      program GPCPcdr_m4_combMW




c     ******************************************************
c     Declarations

      IMPLICIT      NONE


      REAL*4        cap, sum, sumf
      INTEGER*4     iilon, jjlat, i, ii, j, jj, k, kk  
      INTEGER*4     m, n, iostat, iln, ilt, ibl, irec 
      INTEGER*4     iret, l_out, n_pass, l_tf, l_n, iunit
      INTEGER*4     l_mon, landlim
      CHARACTER*256 inc, ing, ingn, ingf, ings 
      CHARACTER*256 ingsc, ingi, ingic, obase
      CHARACTER*256 inc5, tempfile, fname
      LOGICAL*1     is_c, is_gz, in_use

      REAL*4        c5(72, 36), c5n(72, 36)
      REAL*4        c5f(72, 36), c5fn(72, 36)
      REAL*4        c5zoom(144,72), c5fzoom(144, 72)

      REAL*4        ratio(144,72)
      INTEGER*1     snice(144,72,19)

      CHARACTER*1   c_y, c_m, ch_gin(4, 360)
      CHARACTER*5   dum1
      CHARACTER*7   dum2
      CHARACTER*256 ofile
      CHARACTER     temp
      INTEGER*4     iyyyy, iyy, imm,  n_cyr, n_cmo, n_dyr, n_dmo 
      INTEGER*4     n_off, in(10), ind
      REAL*4        cin(21), gin(360)
      REAL*4        src(144, 72), rain(144, 72), g(144, 72)
      REAL*4        gn(144, 72), gf(144, 72), gs(144, 72)
      REAL*4        gi(144, 72) 
      REAL*4        c(144, 72), cn(144, 72), cf(144, 72)
      REAL*4        cgn(144, 72), cgf(144, 72), fac

      REAL*4        sfc   (144, 72 )
      BYTE          sfmask (1440, 720)
      INTEGER*4     land   (144,  72 )

      REAL*4        RMISS
      INTEGER*4     LN, LT
      PARAMETER     (LN= 144, LT= 72, RMISS= -99999.)

      LOGICAL       exinc5

      integer*4     Ninputs
      character*2   cmonth
      character*4   cyear
      character*6   yyyymm

      CHARACTER*256 surtype, InputPATH, InterimPATH
      CHARACTER*1   SatNum
      INTEGER*4     mon_d (12), strlen, month, iyear, imonth
      DATA          surtype, mon_d / 
     +     'GPCPcdr_m4/GPCPcdr_m4_combMW/surtype.new.le',
     +     31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c     Need pi later on to replace sind functions with sin
      REAL          pi
      pi = 4*ATAN(1.0)







c     ******************************************************
c     Check the correct number of inputs were given
      Ninputs = iargc()
      if ( Ninputs .NE. 5 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_combMW:'//
     &        ' Incorrect number of input arguments (should be 5)'
         STOP
      endif

C     Get arguments and set the length of the month in hours.
      call getarg(1, cyear)
      call getarg(2, cmonth)
      call getarg(3, inc)
      call getarg(4, InputPATH)
      call getarg(5, InterimPATH)

c     Convert month and year to numbers
c     reconstruct the month variable used later on
      read(cyear, '(i10)' ) iyear
      read(cmonth, '(i10)' ) imonth
      yyyymm=cyear//cmonth
      read(yyyymm, '(i10)' ) month

c     Set Land limit
      landlim=10.0

c     Construct basis for output file
      call String_Length(InterimPATH,strlen)
      obase=InterimPATH(1:strlen)//'MWComb/cg.'//yyyymm//'.'

c     Set SatNum: this is either 1 (6am/pm sat) or 2 (8am/pm sat)
c     This effectively uses either files pr1, pf1 or pr2, pf2
c     pr2, pf2 are used during the time that teh 85GHz fails on F08
      IF (((iyear .EQ. 1990) .AND. (imonth .GE. 6)) .OR.
     &     (iyear .EQ. 1991)) THEN
         SatNum="2"
      ELSE
         SatNum="1"
      END IF

c     ******************************************************
c      Initialize arrays.
      DO  j = 1, LT
      DO  i = 1, LN
          c    (i, j) = RMISS
          g    (i, j) = RMISS
          cn   (i, j) = RMISS
          gn   (i, j) = RMISS
          cf   (i, j) = RMISS
          gf   (i, j) = RMISS
          gs   (i, j) = 0.
          gi   (i, j) = 0.
          land (i, j) = 0
c         Chang fix for 5x5
          c5zoom (i, j) = RMISS
      END DO
      END DO
      DO  j = 9, 28
      DO  i = 1, 72
          c5   (i, j) = 0.
          c5n  (i, j) = 0.
          c5f  (i, j) = 0.
          c5fn (i, j) = 0.
      END DO
      END DO



C     Set the length of the month in hours.
      iyyyy = month / 100
      imm   = MOD ( month, 100 )
      IF  ( iyyyy .LT. 2000 )  THEN
          iyy = iyyyy - 1900
        ELSE
          iyy = iyyyy - 2000
      END IF
      IF  ( imm .EQ. 2 .AND. MOD ( iyy, 4 ) .EQ. 0 )  THEN
          l_mon = 29
Chang fix for pentad month
c          c_fix = 29/30.
        ELSE
          l_mon = mon_d (imm)
Chang fix for pentad month
c          IF  ( imm .EQ. 8 )  THEN
c            c_fix = 31. / 35.
c            ELSE
c            c_fix = mon_d (imm) / 30.
c          END IF
      END IF
C
C      Find a free FORTRAN I/O iunit, check compression status, 
C      uncompress (unzip) if needed, open the surface type mask file,
C      read it in, and set 2.5x2.5 deg "land" array.
C
      iret  = 0
      in_use = .TRUE.
      iunit  = 2
      DO  WHILE ( in_use )
         iunit = iunit + 1
         INQUIRE ( iunit, OPENED=in_use )
      END DO

      OPEN  ( UNIT   = iunit,         FILE   = surtype,
     +     ACCESS = 'DIRECT',      RECL   = 259200*4, 
     +     FORM   = 'UNFORMATTED',
     +     STATUS = 'OLD',         IOSTAT = iostat )
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_combMW:'//
     &        'Failed to open input surface type file'
         STOP
      end if

      READ ( iunit, REC=1, IOSTAT=iostat )  sfmask
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_combMW:'//
     &        'Failed to read input surface type file'
         STOP
      end if

      CLOSE ( iunit , IOSTAT=iostat)
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_combMW:'//
     &        'Failed to close input surface type file'
         STOP
      end if



C     Do ocean, ice, land, and "other" (coast).

      DO  j = 1, 720
         DO  i = 1, 1440
            jj = 72 - ( j - 1 ) / 10
            ii = 1  + ( i - 1 ) / 10
            IF  ( sfmask (i, j) .EQ. 5 )  THEN
            ELSE IF  ( sfmask (i, j) .EQ. 3 .OR. 
     +              sfmask (i, j) .EQ. 4 )  THEN
            ELSE IF  ( sfmask (i, j) .EQ. 0 .OR.
     +              sfmask (i, j) .EQ. 1 )  THEN
               land (ii, jj) = land (ii, jj) + 10
            ELSE
               land (ii, jj) = land (ii, jj) + 1
            END IF
         END DO
      END DO

C
C      Read in the Chang data.  There are 2 formats.  In the "new"
C     (May 1995) format, each 102-character ASCII record has the
C      format:
C
C      1)  'ROW'      I3 
C      2)  ' COL'     I4 
C      3)  ' RAINAMT' I5 (mm/mo)
C      4)  ' CHISQ'   I6 
C      5)  ' NOBS'    I5 pixels
C      6)  ' P0'      I4 Prob(rain) in % 
C      7)  ' R0'      I5 avg rain rate (mm/hr) x100 
C      8)  ' FL'      I3 freezing level height in 100m 
C      9)  ' SR'      I4 sigma rain x100 
C      10) ' M1'      I5 obs 1st moment
C      11) ' M2'      I5 obs 2nd moment
C      12) ' M3'      I7 obs 3d moment
C
C      In the "old" (permanent?) format, there is a 155-character header, 
C      which starts as "Year:<yy> Month:<mm>", where <yy> and <mm> are 
C      2-digit year and month.  Succeeding data records are formatted 
C      ASCII:
C
C      1)  ROW     I2 
C      2)  COL     I4 
C      3)  CHISQ   F10.1 
C      4)  RAINAMT F9.2  monthly rain amount (mm/mo)
C      5)  P0      F6.2  Prob(rain) in fraction 
C      6)  R0      F6.2  lognormal rain rate const. (mm/hr)
C      7)  SR      F6.2  std. dev. of rain rate
C      8)  CUTOFF  F6.2  rain rate cutoff; 0.5 mm/hr
C      9)  SIGMA0  F6.2  std. dev. of non-raining Tb
C      10) T0      F8.2  avg. Tb of non-raining cases
C      11) M1      F8.2  1st moment of histogram
C      12) M2      F8.2  2nd moment of histogram
C      13) M3      F9.2  3d moment of histogram
C      14) NOBS    F8.0  number of samples in box
C      15) M1      F6.2  fitted 1st moment
C      16) M2      F8.2  fitted 2nd moment
C      17) M3      F10.2 fitted 3d moment
C      18) FL      F6.2  freezing level height (km)
C      . . . debugging output, if present . . .
C      19) RAINRT  F6.2  avg rain rate (mm/d)
C      20) MXSAMP  I5    max. number of samples in a histogram bin
C      21) FLAG2   I3    2 or 4; 64 for not converged
C      22) ULLAT   F6.1  upper left lat.
C      23) ULLON   F7.1  upper left lon.
C
C      In at least some of the "old" format files, the last data line 
C      is flagged with (j,i)=(99,999), and additional trailer lines 
C      follow.  My recollection is that the previous "old" format files 
C      didn't have that.
C
C      In both, the top edge of row 1 is at 65N and the left edge of 
C      col 1 is at the prime meridian.
C
C      Down in the READs, the number of samples is in pixels, and the 
C      factor to convert to number of equivalent images is 5.48, which 
C      converts low-res pixels to .5 deg boxes at the equator.  [Note 
C      that the number of independent samples in a 2.5x2.5 decreases, 
C      although not below 1, as you travel away from the equator.]  
C      The value is an approximate fit to data from 8709-8806, except 
C      8712-8801; it indicates overlapping pixels due to crowding at 
C      the scan edges.  The rain rate is immediately converted from
C      the native mm/mo to mm/d.
C
C      Check compression status, uncompress (unzip) if needed, open the 
C      file, and read it in.
C
C      Start by deciding which format is being used:  Read the first
C      line; if you get an error trying to read the year and month
C      numbers it is "new" format - rewind (because the first line has 
C      data).  Otherwise, it is "old" format - compare the year and 
C      month numbers to the requested year and month.  Note: n_cmo is 
C      I3 because sometimes it is "MM " sometimes it is " MM".  The 
C      system handles both correctly.  Note that the "old" format read 
C      can end either by hitting the EOF or by sensing (j,i)=(99,999).
C
C      If Chang estimates are set to 'none', set the array to fill
C      values and jump to the Grody processing.
C
c      IF  (inc(1:4) .EQ. 'none') THEN
c          DO i = 1, 144
c                DO j = 1, 72
c                c(i,j) = RMISS
c            END DO
c          END DO
c          GO TO 200
c      END IF
C
      OPEN  ( UNIT   = iunit,       FILE   = inc,
     +            FORM   = 'FORMATTED', STATUS = 'OLD', 
     +            IOSTAT = iostat )
      IF  ( iostat .NE. 0 )  THEN
          WRITE (*, *) 'Error', iostat, ' opening ', inc
          STOP
      END IF
C
      READ (iunit, 150, ERR=170) dum1, n_cyr, dum2, n_cmo
  150      FORMAT ( A5, I2, A7, I3 )
      IF  ( iyy .NE. n_cyr .OR. imm .NE. n_cmo )  THEN
            WRITE (*, 155) iyy, imm, n_cyr, n_cmo
  155          FORMAT ( 'Error: the specified YYMM=', 2I2, 
     +               ' doesn''t match the data file YYMM=', 2I2 )
          STOP
      END IF

c     Format statement was wrong in original code. Trying to read an 
c     original I6 field as I8, which then read start of following 
c     field (which was "-").  Assume sgi compiler interpreted number
c     anyway, but gfortran gave error.  Format below is consistent
c     with format used in METH
      DO  k = 1, ( 144*52 )
          READ (iunit, 165, END=210) j, i, ( cin (kk), kk = 1, 17 )
 165          FORMAT ( I2, I4, F10.1, F9.2, 5F6.2, 3F8.2, F9.2, F6.0, 
     +               F6.2, F8.2, F10.2, 2F6.2 )
c 165          FORMAT ( I2, I4, F10.1, F9.2, 5F6.2, 3F8.2, F9.2, F8.0, 
c     +               F6.2, F8.2, F10.2, 2F6.2 )

          IF  ( j .EQ. 99 .OR. i . EQ. 999 )  GO TO 210
          IF  ( cin (2) .GE. 0. )  THEN
              c  (i, j + 10) = cin (2) / l_mon
              cn (i, j + 10) = cin (12) / 5.48
              cf (i, j + 10) = cin (3)
          END IF
Chang fix for pentad months, early QC
cC          The tests on cin (4) (rain rate), cin (1) (chisq), and cin
cC          (5) (skewness) replicates quality control checks that the 
cC          Chang group had not yet made for 1995.
cC
c          IF  ( ( cin (4) .LT. 10. ) .AND.
c     +              ( cin (2) .LT. 25. .OR. 
c     +                ( cin (1) .LT. 5000. .AND. cin (15) .GE. 0. ) ) )  
c     +                                                THEN
c            c  (i, j + 10) = cin (2)  * c_fix / l_mon
c            cn (i, j + 10) = cin (12) * c_fix / 5.48
c            cf (i, j + 10) = cin (3)
c          END IF
      END DO
      GO TO 210
  170      CONTINUE
      REWIND ( iunit )
      DO  k = 1, ( 144*52 )
          READ (iunit, 205, END=210) j, i, ( in (kk), kk = 1, 10 )
  205       FORMAT ( 3X, I3, 4X, I4, 8X, I5, 6X, I6, 5X, I5, 3X, I4, 
     +               3X, I5, 3X, I3, 3X, I4, 3X, I5, 3X, I5, 3X, I7 )
          IF  ( in (1) .GE. 0. )  THEN
            c  (i, j + 10) = in (1) / l_mon
            cn (i, j + 10) = in (3) / 5.48
Chang fix for pentad months
c            c  (i, j + 10) = in (1) * c_fix / l_mon
c            cn (i, j + 10) = in (3) * c_fix / 5.48
            cf (i, j + 10) = in (4) * .01
          END IF
      END DO
  210      CONTINUE
      CLOSE (iunit)
      IF  ( ( is_c .OR. is_gz ) .AND.
     +        ( inc (1:l_n) .NE. tempfile (1:l_tf) ) ) 
     +          CALL SYSTEM ('rm ' // tempfile (1:l_tf) )
      WRITE (*, *) 'Done reading Chang data after', k, ' of', 144*52,
     +                 ' possible points.'
C
Chang fix for 5x5 - new
C      The "new" Chang 5x5-deg data come in two ('day' and 'nit') 
C      files using the "old" format documented above, except the data
C      coverage is 50N-S.  For now the names are automatically 
C      generated off inc.  Check to see if the 'day' file exists.  If 
C      not, bypass the 5-degree read code.  We allow the possibility 
C      that only one file has data in a box.  First, check compression 
C      status, uncompress (unzip) if needed, open the file, and read 
C      the first line (assuming "old" format.  Note: n_cmo is I3 
C      because sometimes it is "MM " sometimes it is " MM".  The 
C      system handles both correctly.  Note that the "old" format read 
C      can end either by hitting the EOF or by sensing (j,i)=(99,999).


      i    = INDEX ( inc, '_2.5' )
      inc5 = inc (1:i-1) // 'Day_5.0' // inc (i+4:256)
c     inc5 = inc (1:i-1) // '_5.0' // inc (i+4:l_n)

      i    = INDEX ( inc5, '/METH_Rain/' )
      inc5 = inc5 (1:i-1) // '/METH_Rain_5deg/' // inc5 (i+11:256)
C
      INQUIRE (FILE = inc5, EXIST = exinc5)
C
      IF  (exinc5) THEN

         call String_Length(inc5,l_tf)

c          WRITE (*, *) 'Open ', tempfile (1:l_tf)
c          WRITE (*, *) '(orig.file ', inc5 (1:l_n), ')'
          OPEN  ( UNIT   = iunit,       FILE   = inc5,
     +                FORM   = 'FORMATTED', STATUS = 'OLD', 
     +                IOSTAT = iostat )
          IF  ( iostat .NE. 0 )  THEN
              WRITE (*, *) 'Error', iostat, ' opening ', inc5(1:l_tf)
              STOP
          END IF
C
          READ (iunit, 150, IOSTAT=iostat) dum1, n_cyr, dum2, n_cmo
          IF  ( iostat .NE. 0 )  THEN
              WRITE (*, *) 'Error', iostat, ' reading ', inc5(1:l_tf)
              STOP
          END IF
          IF  ( iyy .NE. n_cyr .OR. imm .NE. n_cmo )  THEN
              WRITE (*, 155) iyy, imm, n_cyr, n_cmo
              STOP
          END IF

C          Day file read.  c5 and c5f are scaled by Nobs to prepare for
C          weighted averaging of the day and night values.
C
          DO  k = 1, ( 72*20 )
             READ (iunit, 165, END=1210) j, i, (cin (kk), kk = 1, 17)
             IF  ( j .EQ. 99 .OR. i . EQ. 999 )  GO TO 1210
             IF  ( cin (2) .GE. 0. )  THEN
                c5  (i, j + 8) = cin (2) * cin (12)
                c5n (i, j + 8) = cin (12)
                IF  ( cin (3) .GE. 0 )  THEN
                   c5f  (i, j + 8) = cin (3) * cin (12)
                   c5fn (i, j + 8) = cin (12)
                END IF
             END IF
          END DO
 1210     CONTINUE
          
          CLOSE (iunit, IOSTAT=iostat)
          IF  ( iostat .NE. 0 )  THEN
             WRITE (*, *) 'Error', iostat, ' closing ', inc5 (1:l_tf)
             STOP
          END IF

C          Now do "Nit" 5-deg data file, summing the Nobs-weighted 
C          monthly precip and fraction raining on the fly to prepare
C          for weighted averaging.
C
	    i    = INDEX ( inc5, 'Day' )
	    inc5 = inc5 (1:i-1) // 'Nit' // inc5 (i+3:256)

          OPEN  ( UNIT   = iunit,       FILE   = inc5,
     +                FORM   = 'FORMATTED', STATUS = 'OLD', 
     +                IOSTAT = iostat )
          IF  ( iostat .NE. 0 )  THEN
              WRITE (*, *) 'Error', iostat, ' opening ', inc5 (1:l_tf)
              STOP
          END IF
C
          READ (iunit, 150, IOSTAT=iostat) dum1, n_cyr, dum2, n_cmo
          IF  ( iostat .NE. 0 )  THEN
              WRITE (*, *) 'Error', iostat, ' reading ', inc5 (1:l_tf)
              STOP
          END IF
          IF  ( iyy .NE. n_cyr .OR. imm .NE. n_cmo )  THEN
              WRITE (*, 155) iyy, imm, n_cyr, n_cmo
              STOP
          END IF


          DO  k = 1, ( 72*20 )
             READ (iunit, 165, END=1310) j, i, (cin (kk), kk = 1, 17)
             IF  ( j .EQ. 99 .OR. i . EQ. 999 )  GO TO 1310
             IF  ( cin (2) .GE. 0. )  THEN
                c5  (i, j + 8) = c5  (i, j + 8) + cin (2) * cin (12)
                c5n (i, j + 8) = c5n (i, j + 8) + cin (12)
                IF  ( cin (3) .GE. 0 )  THEN
                   c5f  (i, j + 8) = c5f  (i, j + 8) + 
     +                  cin (3)*cin (12)
                   c5fn (i, j + 8) = c5fn (i, j + 8) + cin (12)
                END IF
             END IF
          END DO
 1310     CONTINUE
          
          CLOSE (iunit, IOSTAT=iostat)
          IF  ( iostat .NE. 0 )  THEN
             WRITE (*, *) 'Error', iostat, ' reading ', inc5 (1:l_tf)
             STOP
          END IF


C      Now compute the averages and do the scaling by length of month
C      for c5 and number of pixels per sample for c5n.  The polar caps
C      outside 50N-S are left out because they're never used.
C
          DO  j = 9, 28
          DO  i = 1, 72
              IF  ( c5n (i, j) .GT. 0. )  THEN
                c5  (i, j) = c5  (i, j) / ( l_mon * c5n (i, j) )
                c5n (i, j) = c5n (i, j) / 5.48
                IF  ( c5fn (i, j) .NE. 0 )  THEN
                    c5f (i, j) = c5f (i, j) / c5fn (i, j)
                  ELSE
                    c5f (i, j) = RMISS
                END IF
                ELSE
                c5  (i, j) = RMISS
                c5f (i, j) = RMISS
                c5n (i, j) = RMISS
              END IF
          END DO
          END DO
C
C          Do a 2x zoom using pixel replication.  The j offset and 
C          polar cap fill with RMISS accounts for 5x5 only in 50N-S.
C
          DO  j = 17, 56
          DO  i = 1, 144
              c5zoom  (i, j) = c5  ( (i+1)/2, (j+1)/2 )
              c5fzoom (i, j) = c5f ( (i+1)/2, (j+1)/2 )
          END DO
          END DO
          DO  j = 1, 16
          DO  i = 1, 144
              c5zoom  (i, j   ) = RMISS
              c5fzoom (i, j   ) = RMISS
              c5zoom  (i, 73-j) = RMISS
              c5fzoom (i, 73-j) = RMISS
          END DO
          END DO

C         Write the zoomed 5x5 day/night-averaged C rainrate in 
C         mm/d ('.rc5z').

c          call String_Length(obase,strlen)

c          fname=obase(1:strlen)//'rc5z'
c          call Write_Binary_File(fname,c5zoom,LN,LT)



c          CALL DMPIMGF  ( c5zoom, xbase, 'rc5z', .FALSE., LN, LT, 
c     +                      iret )
c          IF  ( iret .NE. 1 )  WRITE (*, *) 'Error', iret, 
c     +                                    ' writing rc5z'

C          In the region of 5x5 data, replace 5x5 values for 2.5x2.5 
C          that have avg fills (flagged by cf of -1) if the 5x5 is not 
C          missing.  The 5x5 is usually a fitted value.
C
          DO  j = 17, 56
          DO  i = 1,  144
              IF  ( cf     (i, j) .EQ. -1 .AND. 
     +                  c5zoom (i, j) .NE. RMISS )  THEN
                c  (i, j) = c5zoom  (i, j)
                cf (i, j) = c5fzoom (i, j)
              END IF
          END DO
          END DO
C
      ELSE
C
          WRITE (*,*) ' '
          WRITE (*,*) ' Warning: 5-degree file does not exist!'
          WRITE (*,*) ' Check rgfull file for holes.'
          WRITE (*,*) ' '      
C
      END IF

C
Chang fix for 5x5 - old
cC      Read in the 5x5 deg data file.  It is formatted ASCII, and only
cC      contains monthly precip.  The entire record of SSM/I months, 
cC      8707-9512, less 8712, is concatenated into one file with 
cC      'YyyMmm' separating months, where 'yy' is the year number and 
cC      'mm' is the month number.
cC
c      IF  ( month .LE. 199512 .AND. month .GE. 195001 ) THEN
c          OPEN  ( UNIT   = iunit, 
c     +                FILE   = 
c     +      '/usr/user0/huffman/data/chang/GPCP_SSMI_1295_5.0_v23',
c     +                ACCESS = 'SEQUENTIAL', FORM   = 'FORMATTED', 
c     +                ERR    = 1010,         STATUS = 'OLD',
c     +                IOSTAT = iostat )
c          GO TO 1020
c 1010              WRITE (*, *) 'Open error ', iostat, ' on ',
c     +            '/usr/user0/huffman/data/chang/GPCP_SSMI_1295_5.0_v23'
c              STOP
cC
cC          Compute how many lines to skip to get to the month of 
cC          interest.  There are 55 header lines and then each month 
cC          has 1 header line and 144 lines of 10 data values.  We have 
cC          to account for lacking 8712 by processing 1987 specially.
cC
c 1020          CONTINUE
c          IF  ( iyy .EQ. 87 )  THEN
c            n_off = 55 + ( imm - 7 ) * 145
c            ELSE
c            n_off = 55 + ( 5 + imm - 1 + ( iyy - 88 ) * 12 ) * 145
c          END IF
c          DO  j = 1, n_off
c            READ (iunit, 1030) c_dummy
c 1030            FORMAT ( A1 )
c          END DO
cC
cC          Read the next line and verify that it is the right header.  
cC          Then read the data.  Arrays c5in and c5 are equivalenced to 
cC          simplify indexing the read (width 10) and use (width 72).
cC
c          READ (iunit, 1040) c_y, n_dyr, c_m, n_dmo
c 1040          FORMAT ( 1X, A1, I2, A1, I2 )
c          IF  ( c_y .NE. 'Y' .OR. n_dyr .NE. iyy .OR.
c     +              c_m .NE. 'M' .OR. n_dmo .NE. imm )  THEN
c            WRITE (*, 1045) c_y, n_dyr, c_m, n_dmo, iyy, imm
c 1045            FORMAT ( 'Error: Chang 5x5 header = ', A1, I2, A1, I2, 
c     +                   ' .NE. Y', I2, 'M', I2 )
c            STOP
c          END IF
c          DO  j = 1, 144
c            READ (iunit, 1050) ( c5in (i, j), i = 1, 10 )
c 1050            FORMAT ( 10F8.1 )
c          END DO
c          CLOSE (iunit)
cC
cC          Do a 2x zoom using pixel replication and account for the 
cC          length-of-month problem.  The j offset and subsequent RMISS 
cC          fill in c5zoom account for c5 starting at 50N.
cC
c          DO  j = 1, 40
c          DO  i = 1, 144
c            c5zoom (i, j+16) = c5 ( (i+1)/2, (j+1)/2 ) * c_fix
c          END DO
c          END DO
c          DO  j = 1, 16
c          DO  i = 1, 144
c            c5zoom (i, j   ) = RMISS
c            c5zoom (i, 73-j) = RMISS
c          END DO
c          END DO
c        ELSE
c          DO  j = 1, 72
c          DO  i = 1, 144
c            c5zoom (i, j) = RMISS
c          END DO
c          END DO
c      END IF
cC
cC      Write the zoomed 5x5 repaired C rainrate in mm/mo ('.rc5z').
cC
c      CALL DMPIMGF  ( c5zoom, obase, 'rc5z', .FALSE., LN, LT, iret )
c      IF  ( iret .NE. 1 )  WRITE (*, *) 'Error', iret, 
c     +                                ' writing rc5z'
cC
cC      In the region of 5x5 data, replace 5x5 values for missing 2.5x2.5
cC      when the box's surface type is no more than 25% filled with coast.
cC
c      DO  j = 17, 56
c      DO  i = 1,  144
c          IF  ( c      (i, j) .EQ. RMISS .AND. 
c     +              c5zoom (i, j) .NE. RMISS .AND.
c     +              land   (i, j) .LE. 25 )  c (i, j) = c5zoom (i, j)
c      END DO
c      END DO
C
C      Do smooth-fills on fractional coverage and number of samples, 
C      then mask out values where c is missing, i.e., not fitted in
C      2.5 or 5 and not averaged in 2.5  This is necessary because
C      cn=cf=-1 to flag averaged boxes in 2.5.  More holes are created
C      at high latitudes by editing artifacts, including very high and
C      low values over open ocean and coasts.  The number of passes 
C      must be passed in as a variable because it is modified on 
C      return, and the fill should be the same boxes for both fields.
C
      n_pass = -1
      CALL FILLSM ( cn, LN, LT, LN, 0, n_pass, cn, iret )
      CALL FILLSM ( cf, LN, LT, LN, 0, n_pass, cf, iret )
      DO  j = 1, 72
      DO  i = 1, 144
          IF  ( c (i, j) .EQ. RMISS )  THEN
            cn (i, j) = RMISS
            cf (i, j) = RMISS
          END IF
      END DO
      END DO
Chang fix for pentad months, 5x5
cC
cC      Write separate, repaired files of C rainrate in mm/mo 
cC      ('.rcrep'), # equiv. images ('.Icfull'), C fractional coverage 
cC      ('.acrep').
cC
c      CALL DMPIMGF  ( c,  obase, 'rcrep', .FALSE., LN, LT, iret )
c      IF  ( iret .NE. 1 )  WRITE (*, *) 'Error', iret, 
c     +                                ' writing rcrep'
c      CALL DMPIMGF  ( cn, obase, 'Icrep', .FALSE., LN, LT, iret )
c      IF  ( iret .NE. 1 )  WRITE (*, *) 'Error', iret, 
c     +                                ' writing Icrep'
c      CALL DMPIMGF  ( cf, obase, 'acrep', .FALSE., LN, LT, iret )
c      IF  ( iret .NE. 1 )  WRITE (*, *) 'Error', iret, 
c     +                                ' writing acrep'
C
C      Write separate, unaltered files of C rainrate in mm/mo 
C      ('.rcfull'), # equiv. images ('.Icfull'), C fractional coverage 
C      ('.acfull').
C

      call String_Length(obase,strlen)

      fname=obase(1:strlen)//'rcfull'
      call Write_Binary_File(fname,c,LN,LT)

      fname=obase(1:strlen)//'Icfull'
      call Write_Binary_File(fname,cn,LN,LT)

      fname=obase(1:strlen)//'acfull'
      call Write_Binary_File(fname,cf,LN,LT)


C      07/25/2011 - Optional adjustment of the "rcfull" field through 
C      multiplication by a ratio field for three low sampling F08 months
C      (July 1987, January 1988, and December 1991).  This informal 
C      (and possibly temporary) option was motivated by GPCP V2.2 to 
C      adjust the low bias of the Chang V6 precipitation for these months
C      when compared to V4.  A unique ratio field was computed for each 
C      month using the current V6 and V4 estimates.  Currently, Si Gao
C      claims the V6 and V4 estimates are close, but we do not see the
C      same differences.  Investigation is ongoing. 
C
C      IF  ((iyyyy .EQ. 1987) .AND. (imm .EQ. 7)) THEN
C
C          OPEN (27, FILE = 'ANCILLARY_FILES/ratio_sm_198707.bin', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', RECL = 10368,
C     +              STATUS ='OLD', ERR = 880)
C               READ (27, REC = 1) ratio
C          CLOSE (27)
C
C          DO i = 1, 144
C                DO j = 1, 72
C                    IF  (c(i,j) .GE. 0.0) THEN
C                      IF  (ratio(i,j) .GE. 0.0) THEN
C                          c(i,j) = c(i,j) * ratio(i,j)
C                      END IF
C                END IF
C                END DO
C          END DO
C
C      END IF
C
C      GO to 881
C
C  880      WRITE (*,*) 'ERROR: rcfull ratio field not found!'
C        STOP
C
C  881      CONTINUE
C
C      IF  ((iyyyy .EQ. 1988) .AND. (imm .EQ. 1)) THEN
C
C          OPEN (28, FILE = 'ANCILLARY_FILES/ratio_sm_198801.bin', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', RECL = 10368,
C     +              STATUS ='OLD', ERR = 882)
C               READ (28, REC = 1) ratio
C          CLOSE (28)
C
C          DO i = 1, 144
C                DO j = 1, 72
C                    IF  (c(i,j) .GE. 0.0) THEN
C                      IF  (ratio(i,j) .GE. 0.0) THEN
C                          c(i,j) = c(i,j) * ratio(i,j)
C                      END IF
C                END IF
C                END DO
C          END DO
C
C      END IF
C
C      GO to 883
C
C  882      WRITE (*,*) 'ERROR: rcfull ratio field not found!'
C        STOP
C
C  883      CONTINUE
C
C      IF  ((iyyyy .EQ. 1991) .AND. (imm .EQ. 12)) THEN
C
C          OPEN (29, FILE = 'ANCILLARY_FILES/ratio_sm_199112.bin', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', RECL = 10368,
C     +              STATUS ='OLD', ERR = 884)
C               READ (29, REC = 1) ratio
C          CLOSE (29)
C
C          DO i = 1, 144
C                DO j = 1, 72
C                    IF  (c(i,j) .GE. 0.0) THEN
C                      IF  (ratio(i,j) .GE. 0.0) THEN
C                          c(i,j) = c(i,j) * ratio(i,j)
C                      END IF
C                END IF
C                END DO
C          END DO
C
C      END IF
C
C      GO to 885
C
C  884      WRITE (*,*) 'ERROR: rcfull ratio field not found!'
C        STOP
C
C  885      CONTINUE
C
C      Edit out exact zeros and values above 300 mm/mo (10 mm/d) outside 
C      of 50N-50S and wide-definition-coastal values above 500 mm (16 
C      mm/d) outside of 40N-40S (only need to implement 40-50).
C
      DO  j = 1, 16
      DO  i = 1, LN
          IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. 10. )  THEN
            c  (i, j) = RMISS
            cn (i, j) = RMISS
            cf (i, j) = RMISS
          END IF
      END DO
      END DO
      DO  j = 57, 72
      DO  i = 1,  LN
          IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. 10. )  THEN
            c  (i, j) = RMISS
            cn (i, j) = RMISS
            cf (i, j) = RMISS
          END IF
      END DO
      END DO
C
      DO  j = 17, 20
      DO  i = 1,  LN
          IF  ( land (i, j) .GT. 0   .AND. 
     +              c    (i, j) .GT. 16. )  THEN
            c  (i, j) = RMISS
            cn (i, j) = RMISS
            cf (i, j) = RMISS
          END IF
      END DO
      END DO
      DO  j = 53, 56
      DO  i = 1,  LN
          IF  ( land (i, j) .GT. 0   .AND. 
     +              c    (i, j) .GT. 16. )  THEN
            c  (i, j) = RMISS
            cn (i, j) = RMISS
            cf (i, j) = RMISS
          END IF
      END DO
      END DO
C
C      Edit out residual artifacts that are too embarrassing not to zap.
C      The relatively restrictive caps of 150 and 200 mm/mo (5 and 6.5
C      mm/d) are applied only in winter months with known problems.
C
C      North of Hokkaido, grid points (59,14), (59,17), (59,18), (60,17), 
C       (60,13), (61,15) variously light up in months 12-5.  Additional 
C      points are also checked, but that shouldn't matter.  The points
C      The Siberian coast of the Bering Sea (66,120, (68,12), (69,12),
C      (70,13) has artifacts in months 1, 3.
C      i=56,61, j=17,18 encompasses occasional artifacts in months 1-3.  
C      East of Newfoundland the block i=124,126 j=16,18 tends to light up in
C      months 1-4.  The block i=119,121, j=17,18 around the Canadian
C      Maritime lights up for 12-4.  The block i=122,127 j=11,15 has
C      spots in months 3-6.
C      The point (49,22) off the SW tip of Korea lit up in 9701.
C
      IF  ( imm .GT. 11 .OR. imm .LT. 6 )  THEN
          IF  ( imm .GT. 11 .OR. imm .LT. 3 )  THEN
            cap = 5.
            ELSE
            cap = 6.5
          END IF
          DO  i = 59,      61
          DO  j = 14+59-i, 18+59-i
            IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. cap )  THEN
                c  (i, j) = RMISS
                cn (i, j) = RMISS
                cf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
      IF  ( imm .GT. 11 .OR. imm .LT. 5 )  THEN
          IF  ( imm .GT. 11 .OR. imm .LT. 3 )  THEN
            cap = 5.
            ELSE
            cap = 6.5
          END IF
          DO  i = 65, 71
          DO  j = 11, 13
            IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. cap )  THEN
                c  (i, j) = RMISS
                cn (i, j) = RMISS
                cf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
      IF  ( imm .LT. 4 )  THEN
          DO  j = 17, 18
          DO  i = 56, 61
            IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. 6.5 )  THEN
                c  (i, j) = RMISS
                cn (i, j) = RMISS
                cf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
      IF  ( imm .LT. 5 )  THEN
          DO  j = 16,  18
          DO  i = 124, 126
            IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. 6.5 )  THEN
                c  (i, j) = RMISS
                cn (i, j) = RMISS
                cf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
      IF  ( imm .GT. 11 .OR. imm .LT. 5 )  THEN
          DO  j = 17,  18
          DO  i = 119, 121
            IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. 6.5 )  THEN
                c  (i, j) = RMISS
                cn (i, j) = RMISS
                cf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
      IF  ( imm .GE. 3 .AND. imm .LE. 6 )  THEN
          DO  j = 11,  15
          DO  i = 122, 127
            IF  ( c (i, j) .EQ. 0. .OR. c (i, j) .GT. 6.5 )  THEN
                c  (i, j) = RMISS
                cn (i, j) = RMISS
                cf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
      IF  ( imm .GT. 11 .OR. imm .LT. 3 )  THEN
          IF  ( c (49, 22) .EQ. 0. .OR. c (49, 22) .GT. 6.5 )  THEN
                c  (49, 22) = RMISS
                cn (49, 22) = RMISS
                cf (49, 22) = RMISS
            END IF
      END IF
C
C      One more embarrassing artifact: Exact zeros in areas of 
C      significant precip.  Inspecting the error curves, the +/- 1 RMS 
C      interval stops having a floor below 0 at about 30 mm/mo (1 mm/d)
C      of precip, so we assume that exact zero when the surrounding 
C      boxes average > 30 mm/mo (1 mm/d) *and* no surrounding boxes are 
C      zero is wrong and that it should be replaced with the 
C      surrounding average.  The j range is trimmmed, since Chang drops 
C      out around 65 deg lat and then we don't have to re-set indices.  
C      The tests on n (are hoped to) gracefully handle points at the 
C      edge of missing.  The case n=1 is just the point itself, which 
C      doesn't need any division.
C
      DO  j = 2, 71
      DO  i = 1, 144
          IF  ( c (i, j) .EQ. 0. )  THEN
            sum  = 0.
            sumf = 0.
            n    = 0
            DO  jj = j-1, j+1
            DO  ii = i-1, i+1
                IF  ( ii .GT. 144 )  THEN
                  iilon = ii - 144
                  ELSE IF  ( ii .LT. 1 )  THEN
                  iilon = ii + 144
                  ELSE
                  iilon = ii
                END IF
                IF  ( c (iilon, jj) .EQ. 0. )  THEN
                  GO TO 1100
                  ELSE IF  ( c (iilon, jj) .GT. 0. )  THEN
                  sum  = sum  + c  (iilon, jj)
                  sumf = sumf + cf (iilon, jj)
                  n    = n    + 1
                END IF
            END DO
            END DO
            IF  ( n .GT. 4 )  THEN
                IF  ( sum / ( n - 1. ) .GT. 1. )  THEN
                  c  (i, j) = sum  / ( n - 1. )
                  cf (i, j) = sumf / ( n - 1. )
                  WRITE (*, *) 'Note: Chang exact zero set to',
     +                             c (i, j), ' at i,j =', i, j
                END IF
              ELSE IF  ( n .GT. 1 )  THEN
                IF  ( sum / n .GT. 1. )  THEN
                  c  (i, j) = sum  / n
                  cf (i, j) = sumf / n
                  WRITE (*, *) 'Note: Chang exact zero set to',
     +                             c (i, j), ' at i,j =', i, j
                END IF
            END IF
          END IF
 1100          CONTINUE
      END DO
      END DO
C
 200  CONTINUE
C
C      Read in Grody data: precip, number of samples, frequency, 
C      fractional coverage by snow and ice.  The data are byte-swapped 
C      in unformatted direct-access files on the same grid as the 912, 
C      except stored S-N.
C      The number of samples is in the fraction of possible, where 
C      possible is defined as the number of days with an ascending 
C      orbit plus the number of days with a descending, divided by two 
C      times the number of days.  To convert to number of equivalent 
C      images, the factor is
C        (# of days) * 2 * 25 * SIND ( (j-.5)*180./72. )
C      where there can be 2 passes per day, 25 .5x.5 pixels per 2.5x2.5 
C      box at the equator, and the SIND accounts for shrinking box 
C      width as you move away from the equator.  The .5 offset accounts 
C      for box edges starting at the pole.  The fit to the same data 
C      period as above is pretty close.
C      Convert the native precip units of mm/mo to mm/d on input.
C      Note: preliminary investigation showed that BADLOC periods 
C      apparently do not reduce the NOAA frequency.  A query to Ralph 
C      Ferraro yielded the information that the maximum possible should 
C      be over .95 for all months other than the ones flagged below.
C
C
C      Start by checking compression status, uncompressing (unzip) if 
C      needed, and opening the data file.

      call String_Length(InputPATH, strlen)
      ing=InputPATH(1:strlen)//'Ferraro/PR'//SatNum//'.MON'

c     gfortran describes recordsin bytes; sgi/ifort use 4 byte words
      OPEN  ( UNIT   = iunit,    ACCESS = 'DIRECT', 
     +            FILE   = ing, FORM   = 'UNFORMATTED', 
     +            RECL   = 144*4,      IOSTAT = iostat,
     +            STATUS = 'OLD')
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &        'Failed to open Ferraro input file'
         STOP
      end if
      
C     Begin by computing how far to skip down in the file (which holds 
C     8701-present as contiguous blocks of records).

      n_dyr = month / 100 - 1987
      n_dmo = MOD ( month, 100 ) - 1
      n_off = ( n_dyr * 12 + n_dmo ) * 72

      DO  j = 1, LT
          READ (iunit, REC=j + n_off, IOSTAT=iostat)  
     +                                    (gin (i), i = 1, LN)
          if ( iostat .NE. 0 ) then
             WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &            'Failed to read Ferraro input file'
             STOP
          end if

c     This section does byte swapping for a BE machine (not needed of LE)
          DO  i = 1, LN
c            temp          = ch_gin (1, i)
c            ch_gin (1, i) = ch_gin (4, i)
c            ch_gin (4, i) = temp
c            temp          = ch_gin (2, i)
c            ch_gin (2, i) = ch_gin (3, i) 
c            ch_gin (3, i) = temp
            g (i, 73 - j) = gin (i) / l_mon
          END DO
      END DO

      CLOSE (iunit, IOSTAT=iostat)
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &        'Failed to read Ferraro input file'
         STOP
      end if

c      WRITE (*, *) 'Done reading NOAA precip data.'

C     The conversion constant for equiv. images depends on lat. and 
C     month.  The magic numbers account for known (at this time!)
C     cases where some totally missing days are not counted in the
C     NESDIS accounting of "possible" data.
C     
C     First, check compression status, uncompress (unzip) if needed, 
C     and open the data file.
      
      call String_Length(InputPATH, strlen)
      ingn=InputPATH(1:strlen)//'Ferraro/SSA.MON'
      
      OPEN  ( UNIT   = iunit,    ACCESS = 'DIRECT', 
     +            FILE   = ingn, FORM   = 'UNFORMATTED', 
     +            RECL   = 144*4,      IOSTAT = iostat,
     +            STATUS = 'OLD')
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &        'Failed to open Ferraro ssa input file'
         STOP
      end if

      DO  j = 1, LT
          IF  ( month .EQ. 198707 )  THEN
            fac = l_mon * 24. * 2.083 * 
     +                          SIN ( (j-.5)* pi /72. ) * .742
            ELSE IF  ( month .EQ. 198708 )  THEN
            fac = l_mon * 24. * 2.083 *  
     +                          SIN ( (j-.5)* pi /72. ) * .903
            ELSE IF  ( month .EQ. 198801 )  THEN
            fac = l_mon * 24. * 2.083 * 
     +                          SIN ( (j-.5)* pi /72. ) * .613
            ELSE IF  ( month .EQ. 199504 )  THEN
            fac = l_mon * 24. * 2.083 * 
     +                          SIN ( (j-.5)* pi /72. ) * .8
            ELSE IF  ( month .EQ. 199505 )  THEN
            fac = l_mon * 24. * 2.083 *
     +                          SIN ( (j-.5)* pi /72. ) * .548
            ELSE
            fac = l_mon * 24. * 2.083 *
     +                          SIN ( (j-.5)* pi /72. )
          END IF

          READ (iunit, REC=j + n_off, IOSTAT=iostat)  
     +                                    (gin (i), i = 1, 144)
          if ( iostat .NE. 0 ) then
             WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &            'Failed to read Ferraro ssa input file'
             STOP
          end if

c     This section does byte swapping for a BE machine (not needed of LE)
          DO  i = 1, LN
c            temp           = ch_gin (1, i)
c            ch_gin (1, i)  = ch_gin (4, i)
c            ch_gin (4, i)  = temp
c            temp           = ch_gin (2, i)
c            ch_gin (2, i)  = ch_gin (3, i)
c            ch_gin (3, i)  = temp
            gn (i, 73 - j) = gin (i) * fac
          END DO
      END DO
      CLOSE (iunit, IOSTAT=iostat)
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &        'Failed to close Ferraro ssa input file'
         STOP
      end if

C     Do NOAA fractional coverage.  First, check compression status, 
C     uncompress (unzip) if needed, and open the data file.

      call String_Length(InputPATH, strlen)
      ingf=InputPATH(1:strlen)//'Ferraro/PF'//SatNum//'.MON'

      OPEN  ( UNIT   = iunit,    ACCESS = 'DIRECT', 
     +            FILE   = ingf, FORM   = 'UNFORMATTED', 
     +            RECL   = 144*4,      IOSTAT = iostat,
     +            STATUS = 'OLD')
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &        'Failed to open Ferraro pf1 input file'
         STOP
      end if

      DO  j = 1, LT
          READ (iunit, REC=j + n_off, IOSTAT=iostat)  
     +                                    (gin (i), i = 1, LN)
          if ( iostat .NE. 0 ) then
             WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &            'Failed to read Ferraro pf1 input file'
             STOP
          end if

c     This section does byte swapping for a BE machine (not needed of LE)
          DO  i = 1, LN
c            temp           = ch_gin (1, i)
c            ch_gin (1, i)  = ch_gin (4, i)
c            ch_gin (4, i)  = temp
c            temp           = ch_gin (2, i)
c            ch_gin (2, i)  = ch_gin (3, i)
c            ch_gin (3, i)  = temp
            gf (i, 73 - j) = gin (i)
          END DO
      END DO

      CLOSE (iunit, IOSTAT=iostat)
      if ( iostat .NE. 0 ) then
         WRITE(*,*) 'Error in GPCPcdr_m4_CombMW:'//
     &        'Failed to close Ferraro pf1 input file'
         STOP
      end if



C     Now handle snow and ice separately.
      call String_Length(InputPATH, strlen)
      ings=InputPATH(1:strlen)//'Ferraro/SNW.MON'
      ingi=InputPATH(1:strlen)//'Ferraro/ICE.MON'

c     Set ingsc and ingic to "none"
      ingsc="none"
      ingic="none"

      CALL GET_S_I ( ings, ingsc, n_off, imm, 1.,  .01, gs, iret )
      CALL GET_S_I ( ingi, ingic, n_off, imm, .01, .01, gi, iret )

C     Write separate, unaltered files of G rainrate in mm/mo 
C     ('.rgfull'), G # equiv. images ('.Igfull'), G fractional 
C     coverage ('.agfull'), G snow cover ('.sgfull'), G ice cover 
C     ('.igfull').
      
      call String_Length(obase,strlen)
      
      fname=obase(1:strlen)//'rgfull'
      call Write_Binary_File(fname,g,LN,LT)

      fname=obase(1:strlen)//'Igfull'
      call Write_Binary_File(fname,gn,LN,LT)

c      fname=obase(1:strlen)//'agfull'
c      call Write_Binary_File(fname,gf,LN,LT)
c
c      fname=obase(1:strlen)//'sgfull'
c      call Write_Binary_File(fname,gs,LN,LT)

c      fname=obase(1:strlen)//'igfull'
c      call Write_Binary_File(fname,gs,LN,LT)


C      07/06/2011 - Adjustment of the "rgfull" field through 
C      multiplication by a ratio field.  This option was motivated by 
C      GPCP V2.2 to compute a simulated QCed field for July 1987 as 
C      D. Vila is unable to recompute an official QCed field due to a 
C      format incompatibility.  A ratio field was computed using the 
C      198708-198808 QCed F13/unQCed F13.  And since there was no QCed
C      NOAA estimates for July 1987, also read in the previous version
C      of the "rgfull" file so it can be corrected. 
C
C      IF  ((iyyyy .EQ. 1987) .AND. (imm .EQ. 7)) THEN
C
C          OPEN (20, FILE = 'ANCILLARY_FILES/ratio_sm_198707.bin', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', 
C     +              RECL = 10368, STATUS ='OLD', ERR = 990)
C               READ (20, REC = 1) ratio
C          CLOSE (20)
C
C          OPEN (21, FILE = 'ANCILLARY_FILES/v2.1_cg.198707.rgfull', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', RECL = 10368, 
C     +              STATUS ='OLD', ERR = 990)
C               READ (21, REC = 1) g
C          CLOSE (21)
C
C          DO i = 1, 144
C                DO j = 1, 72
C                    IF  (g(i,j) .GE. 0.0) THEN
C                      g(i,j) = g(i,j) * ratio(i,j)
C                END IF
C                END DO
C          END DO
C
C      END IF
C
C      07/06/2011 - Optional adjustment of the "rgfull" field through 
C      multiplication by a ratio field.  This informal (and possibly
C      temporary) option was motivated by GPCP V2.2 to correct 
C      D. Vila's 2009-2010 F17 estimates.  A ratio field was computed
C      using the 200901-200909 QCed F13/unQCed F17 to adjust the F17
C      (which was way too high) to the bias of the QCed F13.
C      Presumably this informal option will be disabled for 2009-2010
C      once D. Vila has reprocessed the 2009-present F17. 
C
C      IF  (iyyyy .GE. 2009) THEN
C
C          OPEN (22, FILE = 'ANCILLARY_FILES/ratio_sm_2009-2010.bin', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', RECL = 10368,
C     +              STATUS ='OLD', ERR = 990)
C               READ (22, REC = 1) ratio
C          CLOSE (22)
C
C          DO i = 1, 144
C                DO j = 1, 72
C                    IF  (g(i,j) .GE. 0.0) THEN
C                      IF  (ratio(i,j) .LE. 1.0) THEN
C                          g(i,j) = g(i,j) * ratio(i,j)
C                      END IF
C                END IF
C                END DO
C          END DO
C
C      END IF
C
C      GO to 991
C
C  990      WRITE (*,*) 'ERROR: rgfull ratio field not found!'
C        STOP
C
C  991      CONTINUE
C
C      07/11/2011 - During the 85GHz fallback period, the new QCed D.Vila
C      precipitation "developed" extremely large coastal artifacts that were
C      affecting the CG product.  An empirical correction was developed where
C      the precipitation was set to missing if values exceeded 12 mm/day in 
C      coastal areas. Note that the surfrac.2.5 file must now be used to
C      identify these areas
C
C      IF  ((iyyyy .EQ. 1991) .OR. 
C     +          ((iyyyy .EQ. 1990) .AND. (imm .GE. 6))) THEN
C
C          OPEN (25, FILE = 'ANCILLARY_FILES/surfrac.2.5', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', 
C     +              STATUS = 'OLD', RECL = 10368, ERR = 993)
C               READ (25, REC = 1) sfc
C          CLOSE (25)
C
C          DO i = 1, 144
C                DO j = 1, 72
C                    IF  ((sfc(i,j) .GE. 10.0) .AND. 
C     +                              (sfc(i,j) .LE. 99.5)) THEN
C                          IF  (g(i,j) .GT. 12.0) THEN
C                          g(i,j) = -9999.9
C                      END IF
C                END IF
C                END DO
C          END DO
C
C      END IF
C
C      GO TO 994 
C
C  993      WRITE (*,*) 'ERROR: percent surface water file not found!'
C        STOP
C
C  994      CONTINUE
C
C      07/12/2011 - During the 85GHz fallback period, the new QCed D.Vila
C      precipitation has no snow/ice screening so all the unknown values
C      were set to zero.  Clearly this is wrong and will artificially 
C      bring down the land averages so a snow/ice mask file was created 
C      using the previous version of NOAA data.  
C
C      IF  ((iyyyy .EQ. 1991) .OR. 
C     +          ((iyyyy .EQ. 1990) .AND. (imm .GE. 6))) THEN
C
C          OPEN (26, FILE = 'ANCILLARY_FILES/mask_rgfull_pr2.bin', 
C     +              FORM = 'UNFORMATTED', ACCESS = 'DIRECT', 
C     +              STATUS = 'OLD', RECL = 49248, ERR = 995)
C               READ (26, REC = 1) snice
C          CLOSE (26)
C
C          IF  (imm .EQ. 1) THEN
C                ind = 8
C          ELSE IF (imm .EQ. 2) THEN
C                ind = 9
C          ELSE IF (imm .EQ. 3) THEN
C                ind = 10
C          ELSE IF (imm .EQ. 4) THEN
C                ind = 11
C          ELSE IF (imm .EQ. 5) THEN
C                ind = 12
C          ELSE IF (imm .EQ. 6) THEN
C              IF  (iyyyy .EQ. 1990) THEN
C                ind = 1
C            ELSE
C                ind = 13
C            END IF
C          ELSE IF (imm .EQ. 7) THEN
C              IF  (iyyyy .EQ. 1990) THEN
C                ind = 2
C            ELSE
C                ind = 14
C            END IF
C          ELSE IF (imm .EQ. 8) THEN
C              IF  (iyyyy .EQ. 1990) THEN
C                ind = 3
C            ELSE
C                ind = 15
C            END IF
C          ELSE IF (imm .EQ. 9) THEN
C              IF  (iyyyy .EQ. 1990) THEN
C                ind = 4
C            ELSE
C                ind = 16
C            END IF
C          ELSE IF (imm .EQ. 10) THEN
C              IF  (iyyyy .EQ. 1990) THEN
C                ind = 5
C            ELSE
C                ind = 17
C            END IF
C          ELSE IF (imm .EQ. 11) THEN
C              IF  (iyyyy .EQ. 1990) THEN
C                ind = 6
C            ELSE
C                ind = 18
C            END IF
C          ELSE IF (imm .EQ. 12) THEN
C              IF  (iyyyy .EQ. 1990) THEN
C                ind = 7
C            ELSE
C                ind = 19
C            END IF
C          END IF
C
C          DO i = 1, 144
C                DO j = 1, 72
C                    IF  (snice(i,j,ind) .EQ. -1) THEN 
C                      g(i,j) = -9999.9
C                END IF
C                END DO
C          END DO
C
C      END IF
C
C      GO TO 996
C
C  995      WRITE (*,*) 'ERROR: snow/ice mask file not found!'
C        STOP
C
C  996      CONTINUE
C
C      When some snow or ice file exists, edit out land areas with at 
C      least 20% coverage and compute reduced population; otherwise,
C      reduce the population linearly with (snow + ice).
C
      DO  j = 1, LT
      DO  i = 1, LN
          gs (i, j) = AMAX1 ( gs (i, j), 0. )
          gi (i, j) = AMAX1 ( gi (i, j), 0. )
          IF  ( gs (i, j) + gi (i, j) .GT. .2 )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
            ELSE IF  ( gs (i, j) + gi (i, j) .GT. 0. )  THEN
            gn (i, j) =
     +                ( 1. - ( gs (i, j) + gi (i, j) ) * 5. ) 
     &              * gn (i, j)
          END IF
      END DO
      END DO
C
C      Edit out exact zeros and values above 300 mm/mo (10 mm/d) 
C      outside of 50N-50S (only need to implement to 60S) and wide-
C      definition-coastal values above 500 mm/mo (16 mm/d) outside of 
C      40N-40S (only need to implement 40-50). 
C
      DO  j = 1, 16
      DO  i = 1, LN
          IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 10. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
      DO  j = 57, 60
      DO  i = 1,  LN
          IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 10. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C
      DO  j = 1, 12
      DO  i = 1, LN
          IF  ( g (i, j) .GT. 10. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
      DO  j = 13, 20
      DO  i = 1,  LN
          IF  ( land (i, j) .LT. 950 .AND. 
     +              land (i, j) .GT. 0   .AND. 
     +              g    (i, j) .GT. 16. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
      DO  j = 52, 60
      DO  i = 1,  LN
          IF  ( land (i, j) .LT. 950 .AND. 
     +              land (i, j) .GT. 0   .AND. 
     +              g    (i, j) .GT. 16. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C
C      Edit out exact zeros and values above 200 mm/mo (6.5 mm/d) 
C      outside of 65N-65S (only need to implement above 65N here, code 
C      for the south is included as a comment), exact zeros and values 
C      above 300 mm/mo (10 mm/d) outside of 50N-50S (only need to 
C      implement to 60S here, but going all the way for generality; 
C      only need to implement to 65 in any case), and wide-definition-
C      coastal values above 500 mm/mo (16 mm/d) outside of 40N-40S 
C      (only need to implement 40-50). 
C
      DO  j = 1, 10
      DO  i = 1, LN
          IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 6.5 )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C      DO  j = 63, 72
C      DO  i = 1,  LN
C          IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 6.5 )  THEN
C            g  (i, j) = RMISS
C            gn (i, j) = RMISS
C            gf (i, j) = RMISS
C          END IF
C      END DO
C      END DO
C
      DO  j = 11, 16
      DO  i = 1,  LN
          IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 10. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
      DO  j = 57, 62
      DO  i = 1,  LN
          IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 10. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C
      DO  j = 17, 20
      DO  i = 1,  LN
          IF  ( land (i, j) .LT. 950 .AND. 
     +              land (i, j) .GT. 0   .AND. 
     +              g    (i, j) .GT. 16. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
      DO  j = 53, 56
      DO  i = 1,  LN
          IF  ( land (i, j) .LT. 950 .AND. 
     +              land (i, j) .GT. 0   .AND. 
     +              g    (i, j) .GT. 16. )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C
C      Edit out residual artifacts that are too embarrassing not to 
C      zap.  The relatively restrictive cap of 200 mm/mo (6.5 mm/d) is 
C      applied only in winter months with known problems.
C
C      Around Newfoundland the block i=121,125 j=17,18 has occasional
C      artifacts in months 2-3.  Hudson Bay (the block i=107,119, 
C      j=10,14) frequently has artifacts.  It is controlled in all 
C      months.  Sometimes the Sea of Okhotsk has contamination for 5 
C      boxes to the north of the line (56,20) to (63,13) over months 
C      12-3.
C
      IF  ( imm .LT. 4 )  THEN
          DO  j = 17,  18
          DO  i = 121, 125
            IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 6.5 )  THEN
                g  (i, j) = RMISS
                gn (i, j) = RMISS
                gf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
      DO  j = 10,  14
      DO  i = 107, 119
          IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 6.5 )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C
      IF  ( imm .GT. 11 .OR. imm .LT. 4 )  THEN
          DO  i = 56,      63
          DO  j = 15+56-i, 20+56-i
            IF  ( g (i, j) .EQ. 0. .OR. g (i, j) .GT. 6.5 )  THEN
                g  (i, j) = RMISS
                gn (i, j) = RMISS
                gf (i, j) = RMISS
            END IF
          END DO
          END DO
      END IF
C
C      Edit ocean areas north of 60N, ocean areas in the zone 50-60S 
C      (to leave all of South America, but not fill in for Chang 
C      fizzling out in the Southern Ocean), and all areas south of 60S 
C      (the southern edge of Chang's coverage in some areas and 
C      knocking out Antarctica).
C
      DO  j = 1, 12
      DO  i = 1, LN
          IF  ( land (i, j) .LE. 0 )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C
      DO  j = 57, 60
      DO  i = 1,  LN
          IF  ( land (i, j) .LE. 0 )  THEN
            g  (i, j) = RMISS
            gn (i, j) = RMISS
            gf (i, j) = RMISS
          END IF
      END DO
      END DO
C
      DO  j = 61, 72
      DO  i = 1,  LN
          g  (i, j) = RMISS
          gn (i, j) = RMISS
          gf (i, j) = RMISS
      END DO
      END DO
Chang fix for F11|F13 in Chang|NOAA
cC      Set C/G population ratios during the period of F11 C with F13 G
cC      in order to roughly equalize the populations for the comparison
cC      below.
cC
c      IF  ( month .GE. 199506 .AND. month .LE. 199512 )  THEN
c          IF  ( month .EQ. 199506 )  THEN
c            eq_pop = 1. / 1.3
c            ELSE IF  ( month .EQ. 199507 )  THEN
c            eq_pop = 1. / 2.0
c            ELSE IF  ( month .EQ. 199508 )  THEN
c            eq_pop = 1. / 1.9
c            ELSE IF  ( month .EQ. 199509 )  THEN
c            eq_pop = 1. / 1.2
c            ELSE IF  ( month .EQ. 199510 )  THEN
c            eq_pop = 1. / 1.1
c            ELSE IF  ( month .EQ. 199511 )  THEN
c            eq_pop = 1. / 2.1
c            ELSE
c            eq_pop = 1. / 1.2
c          END IF
c        ELSE
c          eq_pop = 1.
c      END IF
C
C      Do the combination; if Chang exists and has at least 75% of the 
C      samples of NOAA, use it (this covers Chang existing, but NOAA
C      not).  [The 75% limit essentially guarantees that the 
C      fluctuations in full-coverage number between Chang and NOAA as 
C      reviewed for 8708 won't trigger away from coastal areas.]  If 
C      Chang exists, but has less than 75%, do a weighted combination 
C      (should only be in coastal areas) and ascribe the higher number 
C      of samples to the result.  Otherwise, use NOAA if it exists, or 
C      call it missing.  If cf=-1 in a combination box, just use gf.
C
      DO  j = 1, LT
      DO  i = 1, LN
          IF  ( c (i, j) .GE. 0. )  THEN
            IF  ( cn (i, j) .GE. .75 * gn (i, j) )  THEN
Chang fix for F11|F13 in Chang|NOAA
c            IF  ( cn (i, j) .GE. .75 * gn (i, j) * eq_pop )  THEN
                 src  (i, j) = 0.
                 rain (i, j) = c  (i, j)
                 cgn  (i, j) = cn (i, j)
                 cgf  (i, j) = cf (i, j)
               ELSE
                src  (i, j) = cn (i, j) / gn (i, j)
Chang fix for F11|F13 in Chang|NOAA
c                src  (i, j) = cn (i, j) / ( gn (i, j) * eq_pop )
                rain (i, j) = ( c (i, j) * cn (i, j) + g  (i, j) * 
     +                          (gn (i, j) - cn (i, j)) ) / gn (i, j)
Chang fix for F11|F13 in Chang|NOAA
c     +                          (gn (i, j) * eq_pop - cn (i, j)) ) / 
c     +                          (gn (i, j) * eq_pop)
                cgn  (i, j) = gn (i, j)
Chang fix for F11|F13 in Chang|NOAA
c                cgn  (i, j) = gn (i, j) * eq_pop
                IF  ( cf (i, j) .GE. 0. )  THEN
                  cgf  (i, j) = ( cf (i, j) * cn (i, j) + 
     +                          gf (i, j) * 
     +                          ( gn (i, j) - cn (i, j)) ) / gn (i, j)
Chang fix for F11|F13 in Chang|NOAA
c     +                          ( gn (i, j) * eq_pop - cn (i, j)) ) / 
c     +                          ( gn (i, j) * eq_pop )
                  ELSE
                  cgf (i, j) = gf (i, j)
                END IF
            END IF
            ELSE IF  ( g (i, j) .GE. 0. )  THEN
            src  (i, j) = 1.
            rain (i, j) = g  (i, j)
            cgn  (i, j) = gn (i, j)
            cgf  (i, j) = gf (i, j)
            ELSE
            src  (i, j) = RMISS
            rain (i, j) = RMISS
            cgn  (i, j) = RMISS
            cgf  (i, j) = RMISS
          END IF
      END DO
      END DO
C
C      Write separate, edited files of Chang and Grody rainrate in 
C      mm/mo ('.rcfix', '.rgfix').

c      call String_Length(obase,strlen)
c      
c      fname=obase(1:strlen)//'rcfix'
c      call Write_Binary_File(fname,c,LN,LT)
c
c      fname=obase(1:strlen)//'rgfix'
c      call Write_Binary_File(fname,g,LN,LT)

C      Write separate files of CG source ('.src'), CG rainrate in 
C      mm/hr ('.rfix'), CG rainrate in the zone 50N-50S ('.rssfix'), 
C      CG number of equiv. images ('.Ifix'), and CG frequency in 
C      percent ('.afix').

      call String_Length(obase,strlen)

      fname=obase(1:strlen)//'src'
      call Write_Binary_File(fname,src,LN,LT)

      fname=obase(1:strlen)//'rfix'
      call Write_Binary_File(fname,rain,LN,LT)

      fname=obase(1:strlen)//'rssfix'
      call Write_Binary_File(fname,rain,LN,LT)

      fname=obase(1:strlen)//'Ifix'
      call Write_Binary_File(fname,cgn,LN,LT)

      fname=obase(1:strlen)//'afix'
      call Write_Binary_File(fname,cgf,LN,LT)


      STOP

      end program
