	SUBROUTINE COMPOPTS ( name, l_un, is_c, is_gz, unname, iret )
C************************************************************************
C* COMPOPTS  Create, if needed, an uncompressed file			*
C*									*
C* This subroutine creates, if needed, an uncompressed version of the	*
C* file NAME.  If NAME lacks a compression suffix ('.Z' or '.gz') and	*
C* it exists, then it is echoed back.  If NAME has explicit compression	*
C* suffixes, it is uncompressed into temporary space and the temporary	*
C* name is returned.  If NAME lacks a compression suffix, doesn't	*
C* exist, but a compressed version of does exist, the compressed	*
C* version is uncompressed into temporary space and the temporary name	*
C* is returned.								*
C*									*
C* COMPOPTS ( name, l_un, is_c, is_gz, unname, iret )			*
C*									*
C* Input parameters:							*
C*   name	CHAR*	File name					*
C*									*
C* Output parameters:							*
C*   l_un	INT	Length of the original uncompressed NAME	*
C*   is_c	LOG	T|F = has|lacks '.Z' suffix			*
C*   is_gz	LOG	T|F = has|lacks '.gz' suffix			*
C*   unname	CHAR*	Name of uncompressed dataset (NAME if is_c and	*
C*			is_gz are both .FALSE.)				*
C*   iret	INT	Return code: 0 = normal				*
C*				    -1 = couldn't find file name	*
C*				    -2 = '999' sent as file name	*
C**									*
C* Log:									*
C* G.Huffman/SSAI	 3/99	Pull out of GET_IRTB.F			*
C* G.Huffman/SSAI	 9/00	Check existance if have .Z or .gz	*
C* G.Huffman/SSAI	10/00	Separate return codes; init. output var	*
C* D.Bolvin/SSAI	 2/01	Set PID to 7 digits			*
C* G.Huffman/SSAI	 3/01	Extend strings to 256 char.		*
C* G.Huffman/SSAI	 6/01	Set PID to 10 digits (full I*4)		*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	ipid, l_n,   l_un, l_tf, iostat, iret
	LOGICAL*1	is_c, is_gz, is
	CHARACTER*256	tempfile,    name, in,   unname
	CHARACTER*256	cmdstr
	INTEGER*4	ntemp
	DATA		ntemp / 0 /
c	STATIC		ntemp
C
	INTEGER*4	getpid, i
C
C	Initialize output variables, check on compression status
C	and file existence, uncompress (gunzip) if needed, and
C	open the data file.
C
	in     = name
c	CALL ST_LSTR ( in, l_n, iostat )
c       get length of string excluding blanks (used to use ST_LSTR)
	l_n = len(in)
	do i=l_n,1,-1
	   if (in(i:i) .NE. ' ') then
	      l_n=i
	      exit
	   endif
	enddo


	l_un   = l_n
	is_c   = .FALSE.
	is_gz  = .FALSE.
	unname = in
	iret   = 0
	IF  ( in (1:l_n) .EQ. '999' )  THEN
	    WRITE (*, *) 'Error: ''999'' sent as file name to ',
     +			 'COMPOPTS'
	    iret = -2
	    RETURN
	  ELSE IF  ( in (l_n-1:l_n) .EQ. '.Z' )  THEN
	    INQUIRE ( FILE = in (1:l_n), EXIST = is )
	    IF  ( is )  THEN
		is_c  = .TRUE.
		is_gz = .FALSE.
		l_un  = l_n - 2
	      ELSE 
		WRITE (*, *) 'Error: can''t find ', in (1:l_n)
		iret = -1
		RETURN
	    END IF
	  ELSE IF  ( in (l_n-2:l_n) .EQ. '.gz' )  THEN
	    INQUIRE ( FILE = in (1:l_n), EXIST = is )
	    IF  ( is )  THEN
		is_c  = .FALSE.
		is_gz = .TRUE.
		l_un  = l_n - 3
	      ELSE 
		WRITE (*, *) 'Error: can''t find ', in (1:l_n)
		iret = -1
		RETURN
	    END IF
C
C	  If the filename doesn't explicitly denote compression
C	  or gzip'ing, check that it exists.  If not, check whether
C	  a compressed or gzip'ed version of that name does exist.
C	  If so, set up to uncompress (unzip) the file, otherwise
C	  it's an error.
C
	  ELSE
	    INQUIRE ( FILE = in, EXIST = is )
	    IF  ( is )  THEN
	        is_c  = .FALSE.
	        is_gz = .FALSE.
	        l_un  = l_n
	      ELSE 
		INQUIRE ( FILE = in (1:l_n) // '.Z', EXIST = is )
		IF  ( is )  THEN
		    in    = in (1:l_n) // '.Z'
		    is_c  = .TRUE.
		    is_gz = .FALSE.
		    l_un  = l_n
		    l_n   = l_n + 2
		  ELSE 
		    INQUIRE ( FILE = in (1:l_n) // '.gz', EXIST = is )
		    IF  ( is )  THEN
			in    = in (1:l_n) // '.gz'
			is_c  = .FALSE.
			is_gz = .TRUE.
			l_un  = l_n
			l_n   = l_n + 3
		      ELSE 
			WRITE (*, *) 'Error: can''t find ', in (1:l_n), 
     +				     ' or .Z or .gz versions'
			iret = -1
			RETURN
		    END IF
		END IF
	    END IF
	END IF
C
	IF  ( is_c .OR. is_gz )  THEN
	    ipid = getpid()
	    IF  ( ntemp .LT. 9999 )  THEN
		ntemp = ntemp + 1
	      ELSE
		ntemp = 0
	    END IF
	    WRITE ( tempfile, 10 ) ipid, ntemp
   10	    FORMAT ( '/usr/tmp/compopts.', I10.10, '.', I4.4 )

c	    CALL ST_LSTR ( tempfile, l_tf, iostat )
c           get length of string excluding blanks (used to use ST_LSTR)
	    l_tf = len(tempfile)
	    do i=l_tf,1,-1
	       if (tempfile(i:i) .NE. ' ') then
		  l_tf=i
		  exit
	       endif
	    enddo
	
            IF  ( is_c )  THEN
                cmdstr = 'uncompress -c ' // in (1:l_n) //  ' > ' //
     +			 tempfile (1:l_tf)
	      ELSE
                cmdstr = 'gunzip -c '     // in (1:l_n) //  ' > ' //
     +			 tempfile (1:l_tf)
            END IF
	    CALL SYSTEM ( cmdstr )
	    unname = tempfile
	END IF
cc	write (*, *) 'datafile =', datafile
cc	write (*, *) 'cmdstr =', cmdstr
C
	RETURN
	END
