	SUBROUTINE GET_S_I ( ings, ingsc, n_off, imm, sfac, scfac, gs, 
     +			     iret )
C************************************************************************
C* GET_S_I  Get NOAA snow or ice coverage fields			*
C*									*
C* Get either the NOAA snow or ice coverage fields, or use the NOAA	*
C* climatology if it is available.					*
C*									*
C* get_s_i ( ings, ingsc, n_off, imm, gs, iret )			*
C*									*
C* Input parameters:							*
C*   ings	CHAR*	NOAA snow or ice field name			*
C*   ingsc	CHAR*	NOAA snow or ice climatology field name		*
C*   n_off	INT	Record offset into snow/ice file for YYMM	*
C*   imm	INT	month number					*
C*   sfac	REAL	Multiplier to turn ings into fractional cover	*
C*   scfac	REAL	Multiplier to turn ingsc into fractional cover	*
C*                                                                      *
C* Output parameters:                                                   *
C*   gs		INT     Snow/ice coverage (size LN,LT)			*
C*   iret       INT     Return code: 0 = normal                         *
C**									*
C* Log:									*
C* G.Huffman/SSAI	 3/96	Split from CG_COMBO.F			*
C* G.Huffman/SSAI	 3/00	Update Grody->NOAA			*
C* G.Huffman/SSAI	 9/00	Upgrade arguments of REDUCE		*
C* D.Bolvin/SSAI	 5/09	Explicitly type declarations		*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i, j, k,      iostat,  irec, iav, jav,
     +			iret, l_ings, l_ingsc
	CHARACTER*256	ings, ingsc
	LOGICAL		actual
	REAL*4		sfac, scfac
C
	CHARACTER*1	ch_gin (4, 360)
	CHARACTER*5	dum1
	CHARACTER*7	dum2
	CHARACTER	temp
	INTEGER*4	imm, n_off
	REAL*4		gin   (360),      wt    (144, 72 ), 
     +			gs_in (360, 180), wt_in (360, 180),
     +			gs    (144, 72 ), gsc   (144, 72 )
        EQUIVALENCE	( ch_gin (1, 1), gin (1) )
C
	REAL*4		RMISS
	INTEGER*4	LN, LT
	PARAMETER	( LN    = 144,
     +			  LT    = 72,
     +			  RMISS = -99999. )
C

c	CALL ST_LSTR ( ings,  l_ings,  iret )
c	CALL ST_LSTR ( ingsc, l_ingsc, iret )
c       get length of string excluding blanks (used to use ST_LSTR)
	l_ings = len(ings)
	do i=l_ings,1,-1
	   if (ings(i:i) .NE. ' ') then
	      l_ings=i
	      exit
	   endif
	enddo
	l_ingsc = len(ingsc)
	do i=l_ingsc,1,-1
	   if (ingsc(i:i) .NE. ' ') then
	      l_ingsc=i
	      exit
	   endif
	enddo

C
C	Now handle the real data if the file is available, using the
C	appropriate scale factor to get fractional coverage.
C
	IF  ( ings (1:l_ings) .NE. 'none' )  THEN
c       gfortran describes recordsin bytes; sgi/ifort use 4 byte words
	    OPEN  ( UNIT = 14,   ACCESS = 'DIRECT', 
     +		    FILE = ings, FORM   = 'UNFORMATTED', 
     +		    RECL = 144*4,  IOSTAT = iostat,
     +		    ERR  = 296 )
	    GO TO 298
  296	    WRITE (*, *)  'Open error', iostat, ' on', ings
	    STOP
  298	    CONTINUE
	    DO  j = 1, LT
		READ (14, REC=j + n_off, ERR=310, IOSTAT=iostat)  
     +						(gin (i), i = 1, 144)
c     This section does byte swapping for a BE machine (not needed of LE)
		DO  i = 1, LN
c		    temp           = ch_gin (1, i)
c		    ch_gin (1, i)  = ch_gin (4, i)
c		    ch_gin (4, i)  = temp
c		    temp           = ch_gin (2, i)
c		    ch_gin (2, i)  = ch_gin (3, i) 
c		    ch_gin (3, i)  = temp
		    gs (i, 73 - j) = gin (i) * sfac
		END DO
	    END DO
	    CLOSE (14)
	    GO TO 315
  310	    WRITE (*, *) 'Error: Error', iostat, ' reading ', ings 
     +			 (1:l_ings)
	    STOP
  315	    CONTINUE
c	    WRITE (*, *) 'Done reading ', ings (1:l_ings)
C
C	    Check whether the actual file had any values for the 
C	    month of interest.
C
	    actual = .FALSE.
	    DO  j = 1, LT
	    DO  i = 1, LN
		IF  ( gs (i, j) .GE. 0. )  THEN
		    actual = .TRUE.
		    GO TO 320
		END IF
	    END DO
	    END DO
	    WRITE (*, *) 'Actual file ', ings (1:l_ings), 
     +			 ' was entirely missing'
  320	    CONTINUE
C
C	  There was no actual file, so it is automatically empty.
C
	  ELSE
	    actual = .FALSE.
	    WRITE (*, *) 'Actual file ', ings (1:l_ings), 
     +			 ' was input as ''none'''
	END IF
C
C	We assume that the climatology is a single file, organized 
C	by month on a 1x1 deg grid (so we have to compute a new offset).  
C	Use the specified factor to convert to fractional coverage.  Don't 
C	try to read it if it's missing or the actual file has data.
C

c	print *, ingsc


	IF  ( ingsc (1:l_ingsc) .NE. 'none' .AND. .NOT. actual )  THEN

c       gfortran describes recordsin bytes; sgi/ifort use 4 byte words
	    OPEN  ( UNIT = 15,    ACCESS = 'DIRECT', 
     +		    FILE = ingsc, FORM   = 'UNFORMATTED', 
     +		    RECL = 330*4,   IOSTAT = iostat,
     +		    ERR  = 330 )
	    GO TO 332
  330	        WRITE (*, *)  'Open error', iostat, ' on', 
     +			      ingsc (1:l_ingsc)
	        STOP
  332	    CONTINUE
	    n_off = ( imm - 1 ) * 180

	    DO  j = 1, 180
		READ (15, REC= j + n_off, ERR=335, IOSTAT=iostat)  
     +						(gin (i), i = 1, 360)
c     This section does byte swapping for a BE machine (not needed of LE)
		DO  i = 1, 360
c		    temp                = ch_gin (1, i)
c		    ch_gin (1, i)       = ch_gin (4, i)
c		    ch_gin (4, i)       = temp
c		    temp                = ch_gin (2, i)
c		    ch_gin (2, i)       = ch_gin (3, i)
c		    ch_gin (3, i)       = temp
		    gs_in  (i, 181 - j) = gin (i) * scfac
		END DO
	    END DO
	    CLOSE (15)
C
	    CALL REDUCE  ( 360,   180,   0.,  0.,  
     +			   2.5,   2.5,   360, 180, 144, 72, 
     +		           gs_in, wt_in, gsc, wt,  iav, jav, iret )
	    IF  ( iret .NE. 0 )  THEN
		WRITE (*, *) 'Error: Return code from REDUCE is', iret
		STOP
	    END IF
C
	    GO TO 340
  335	    WRITE (*, *) 'Error: Error', iostat, ' reading climo file ',
     +			 ingsc (1:l_ingsc)
	    STOP
  340	    CONTINUE
	    WRITE (*, *) 'Done reading NOAA climo data for month',
     +			 imm, ';'
	    WRITE (*, *) 'substituting for missing actual data'
C
C	    We've already determined that the actual file is 'none' or is 
C	    empty, and climo exists; substitute climo for actual.
C
	    DO  j = 1, LT
	    DO  i = 1, LN
		gs (i, j) = gsc (i, j)
	    END DO
	    END DO
	END IF
C
	RETURN
	END
