	SUBROUTINE REDUCE  ( ixn,  iyn,    x0,    y0,    dx,    dy,
     +			     LNIN, LTIN,   LNOUT, LTOUT, datin,
     +			     wtin, datout, wt,    ilomx, jlomx, iret )
C************************************************************************
C* REDUCE  Reduce a window of a field to a lower resolution		*
C*									*
C* Reduce an array to a lower resolution by averaging (and optionally	*
C* with weighting).  The user can specify operation on only a window	*
C* (or subimage) of the original.  The values (x0,y0)=(0,0) is the	*
C* upper left image corner.  The answer is returned in the first	*
C* ILOMXxJLOMX values of DATOUT and WT.					*
C*									*
C* REDUCE  ( ixn,   iyn,  x0, y0, dx, dy, LNIN, LTIN,  LNOUT, LTOUT,	*
C*	     datin, wtin, datout, wt, ilomx,    ilomx, iret )		*
C*									*
C* Input parameters:							*
C*   ixn	INT	X size of actual input *image*			*
C*   iyn	INT	Y size of actual input *image*			*
C*   x0		REAL	Left-most pixel edge X in input file (0=start	*
C*			of first grid box)				*
C*   y0		REAL	Top-most pixel edge Y in input file (0=start of	*
C*			first grid box)					*
C*   dx		REAL	Number of input X pixels avg'd to an output X	*
C*   dy		REAL	Number of input Y pixels avg'd to an output Y	*
C*   LNIN	INT	X size of input *array*				*
C*   LTIN	INT	Y size of input *array*				*
C*   LNOUT	INT	X size of output *array*			*
C*   LTOUT	INT	Y size of output *array*			*
C*   datin	REAL*	Input data (size IXNxIYN in LNINxLTIN array)	*
C*   wtin	REAL*	Input weight (size IXNxIYN in LNINxLTIN array)	*
C*									*
C* Output parameters:							*
C*   datout	REAL*	Output data (first ILOMXxJLOMX in LNOUTxLTOUT)	*
C*   wt		REAL*	Weighting (first ILOMXxJLOMX in LNOUTxLTOUT)	*
C*   ilomx	INT	X size of actual output *image*			*
C*   jlomx	INT	Y size of actual output *image*			*
C*   iret	INT	Return code: 0 = normal				*
C**									*
C* Log:									*
C* G.Huffman/USRA	10/92	Separate out core of PROGRAM AVG	*
C* G.Huffman/USRA	 2/93	Compute ilomx with MAX0 (n/a for jlomx)	*
C* G.Huffman/USRA	 3/93	Add input weight image			*
C* G.Huffman/SSAI	11/94	Move missing tests inside splitting test*
C* G.Huffman/SSAI	 7/96	Correct dims on wtin to LNIN,LTIN	*
C* G.Huffman/SSAI	 9/00	Add x0,y0, logicals on output; set	*
C*				ilomx, jlomx at the end			*
C* G.Huffman/SSAI	 1/02	Give byte size on all declarations;	*
C*				documentation				*
C************************************************************************
	IMPLICIT	NONE
	INTEGER*4	i,  j, k, ilo1, ilo2, ilomx, jlo1, jlo2, jlomx,
     +			ii, jj
	REAL*4		di, dj
	LOGICAL*1	i1, i2,   j1,   j2
	REAL*4		RMISS
	PARAMETER	( RMISS = -99999. )
C
C	Args are (ln,lt).
C
	INTEGER*4	ixn, iyn, LNIN, LTIN, LNOUT, LTOUT, iret
	REAL*4		datin  (LNIN,  LTIN ), wtin  (LNIN,  LTIN ),
     +			datout (LNOUT, LNOUT), wt    (LNOUT, LNOUT), 
     +			x0, y0, dx, dy
C
C	Zero the accumulation arrays, then accumulate input pixels
C	in output bins.  The various cases of splitting pixels among
C	bins must be handled explicitly.  The actual accumulations
C	only occur if the value is non-negative.
C
	DO  j = 1, LTOUT
	DO  i = 1, LNOUT
	    datout (i, j) = 0.
	    wt     (i, j) = 0.
	END DO
	END DO
C
	Do  j = 1, iyn
	    jlo1 = INT ( (j - y0 - 1.) * 1.000001 / dy ) + 1
	    jlo2 = INT ( (j - y0)      *  .999999 / dy ) + 1
	    IF  ( jlo1 .GE. 1 .AND. jlo1 .LE. LTOUT )  THEN
		j1 = .TRUE.
	      ELSE
		j1 = .FALSE.
	    END IF
	    IF  ( jlo2 .GE. 1 .AND. jlo2 .LE. LTOUT )  THEN
		j2 = .TRUE.
	      ELSE
		j2 = .FALSE.
	    END IF
C
C	    Test that there is no top-bottom split and that we're inside
C	    the output domain.
C
	    IF  ( jlo1 .EQ. jlo2 )  THEN
	        IF  ( j1 )  THEN
		    DO  i = 1, ixn
			ilo1 = INT ( (i - x0 - 1.) * 1.000001 / dx ) + 1
			ilo2 = INT ( (i - x0)      *  .999999 / dx ) + 1
			IF  ( ilo1 .GE. 1 .AND. ilo1 .LE. LNOUT )  THEN
			    i1 = .TRUE.
			  ELSE
			    i1 = .FALSE.
			END IF
			IF  ( ilo2 .GE. 1 .AND. ilo2 .LE. LNOUT )  THEN
			    i2 = .TRUE.
			  ELSE
			    i2 = .FALSE.
			END IF
C
C			Input pixel entirely in one output box.
C
			IF  ( ilo1 .EQ. ilo2 )  THEN
			    IF  ( i1 )  THEN
				IF  ( datin (i, j) .GE. 0. )  THEN
	datout (ilo1, jlo1) = datout (ilo1, jlo1) + datin  (i, j) 
     +						  * wtin   (i, j)
	wt     (ilo1, jlo1) = wt     (ilo1, jlo1) + wtin   (i, j)
				END IF
			    END IF
C
C			  Input pixel split into left, right parts.
C
			  ELSE
			    IF  ( datin (i, j) .GE. 0. )  THEN
				di = ilo1 * dx - i + x0 + 1.
				IF  ( i1 )  THEN
	datout (ilo1, jlo1) = datout (ilo1, jlo1) + datin  (i, j) 
     +						  * wtin   (i, j) * di
	wt     (ilo1, jlo1) = wt     (ilo1, jlo1) + wtin   (i, j) * di
				END IF
				IF  ( i2 )  THEN
	datout (ilo2, jlo1) = datout (ilo2, jlo1) + datin  (i, j) 
     +						  * wtin   (i, j)
     +						  * ( 1. - di )
	wt     (ilo2, jlo1) = wt     (ilo2, jlo1) + wtin   (i, j)
     +						  * ( 1. - di )
				END IF
			    END IF
			END IF
		    END DO
		END IF
C
C	      There *is* a top-bottom split.
C
	      ELSE
		dj = jlo1 * dy - j + y0 + 1.
		DO  i = 1, ixn
		    ilo1 = INT ( (i - x0 - 1.) * 1.000001 / dx ) + 1
		    ilo2 = INT ( (i - x0)      *  .999999 / dx ) + 1
		    IF  ( ilo1 .GE. 1 .AND. ilo1 .LE. LNOUT )  THEN
			i1 = .TRUE.
		      ELSE
			 i1 = .FALSE.
		    END IF
		    IF  ( ilo2 .GE. 1 .AND. ilo2 .LE. LNOUT )  THEN
			i2 = .TRUE.
		      ELSE
			i2 = .FALSE.
		    END IF
C
C		    Input pixel only split into top, bottom parts.
C
		    IF  ( ilo1 .EQ. ilo2 )  THEN
			IF  ( i1 )  THEN
			    IF  ( datin (i, j) .GE. 0. )  THEN
				IF  ( j1 )  THEN
	datout (ilo1, jlo1) = datout (ilo1, jlo1) + datin  (i, j) 
     +						  * wtin   (i, j) * dj
	wt     (ilo1, jlo1) = wt     (ilo1, jlo1) + wtin   (i, j) * dj
				END IF
				IF  ( j2 )  THEN
	datout (ilo1, jlo2) = datout (ilo1, jlo2) + datin  (i, j) 
     +						  * wtin   (i, j)
     +						  * ( 1. - dj )
	wt     (ilo1, jlo2) = wt     (ilo1, jlo2) + wtin   (i, j)
     +						  * ( 1. - dj )
				END IF
			    END IF
			END IF
C
C		      Input pixel split into 4 parts.
C
		      ELSE
			IF  ( datin (i, j) .GE. 0. )  THEN
			    di = ilo1 * dx - i + x0 + 1.
			    IF  ( i1 .AND. j1 )  THEN
	datout (ilo1, jlo1) = datout (ilo1, jlo1) + datin  (i, j) 
     +						  * wtin   (i, j)
     +						  * di * dj
	wt     (ilo1, jlo1) = wt     (ilo1, jlo1) + wtin   (i, j)
     +						  * di * dj
			    END IF
			    IF  ( i2 .AND. j1 )  THEN
	datout (ilo2, jlo1) = datout (ilo2, jlo1) + datin  (i, j) 
     +						  * wtin   (i, j)
     +						  * ( 1. - di ) * dj
	wt     (ilo2, jlo1) = wt     (ilo2, jlo1) + wtin   (i, j)
     +						  * ( 1. - di ) * dj
			    END IF
			    IF  ( i1 .AND. j2 )  THEN
	datout (ilo1, jlo2) = datout (ilo1, jlo2) + datin  (i, j) 
     +						  * wtin   (i, j)
     +						  * di * ( 1. - dj )
	wt     (ilo1, jlo2) = wt     (ilo1, jlo2) + wtin   (i, j)
     +						  * di * ( 1. - dj )
			    END IF
			    IF  ( i2 .AND. j2 )  THEN
	datout (ilo2, jlo2) = datout (ilo2, jlo2) + datin  (i, j) 
     +						  * wtin   (i, j)
     +						  * ( 1. - di )
     +						  * ( 1. - dj )
	wt     (ilo2, jlo2) = wt     (ilo2, jlo2) + wtin   (i, j)
     +						  * ( 1. - di )
     +						  * ( 1. - dj )
			    END IF
			END IF
		    END IF
		END DO
	    END IF
	END DO
C
C	Compute the output average.  Both ilo2 and jlo2 should be
C	at their maxima - i and j increment up, so ilomx and jlomx
C	are computed here.  Shift the result to the first
C	ilomx*jlomx entries in datout, wt.
C
	ilomx = MIN0 ( LNOUT, ilo2 )
	jlomx = MIN0 ( LTOUT, jlo2 )
	DO  j = 1, jlomx
	DO  i = 1, ilomx
	    k  = i + (j - 1) * ilomx
	    jj = 1 + INT ( ( k - 1 ) / LNOUT )
	    ii = k - ( jj - 1 ) * LNOUT
	    IF  ( wt (i, j) .EQ. 0. )  THEN
		datout (ii, jj) = RMISS
	      ELSE
		datout (ii, jj) = datout (i, j) / wt (i, j)
	    END IF
	    wt (ii, jj) = wt (i, j)
	END DO
	END DO
C
	iret = 0
	RETURN
	END
