clear all;

% Plot CDR files

% add path to directory where some useful functions are stored
path(path,'GPCPcdr_m5/GPCPcdr_m5_DiagPlots/');

% get path of directories from options file
% read options file, and match  find name of "OutputPATH", then extract it
lines=textread('OptionsM.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));
odir=[fdir '/DiagPlots/'];



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read all CDR files

vars={'precip';'precip_error'};
varnames={'GPCP CDR precipitation';'GPCP CDR precip random error'};

tmp=datevec(now); tmp(3)=1; tmp(4:6)=0;
alldates=datevec(datenum('01jan79'):datenum(tmp));
  alldates(:,3)=1; alldates=unique(datenum(alldates));

cdr_data=NaN*ones([144 72 length(alldates) length(vars)]);
for k=1:length(alldates), try,
  [yyyy mm trash]=datevec(alldates(k));

  tmp=ncload(sprintf('%sgpcp_cdr_v22rB1_y%04d_m%02d.nc',fdir,yyyy,mm));

  for i=1:length(vars), try,
    cdr_data(:,:,k,i)=tmp.(char(vars(i)));
  end; end;
end; end;


lat=[-90:2.5:87.5]'; lon=[0:2.5:357.5]';
cdr_data(find((cdr_data<-900)|(cdr_data>900)))=NaN;






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot each CDR field as a separate equal area averaged time series
xlabs=datenum(['01jan80';'01jan86';'01jan92';'01jan98';'01jan04';'01jan10';'01jan16']);
% get slims - make end now+1 month
xlims=datevec(now); xlims(3)=1; xlims(4:6)=0;
xlims=[datenum('01jan72') datenum(xlims+[0 1 0 0 0 0])];


% make weights and sum of weights for area averaging
siz=size(cdr_data);
w=repmat(abs(cos(lat))',[siz(1) 1 siz(3)]); wsum=nansum(nansum(w(:,:,1)));

% make mask for land/ocean
load land_mask_250km_from_1km; 
mask=repmat(land_mask_250km([73:end 1:72],:),[1 1 siz(3)]);

% Make anomaly version of int_data
cdra_data=NaN*ones(siz);
temp=datevec(alldates);
for i=1:12, idx=find(temp(:,2)==i);
  cdra_data(:,:,idx,:)=cdr_data(:,:,idx,:)-repmat(nanmean(cdr_data(:,:,idx,:),3),[1 1 length(idx) 1]);
end;

% Make tropics only version
cdrt_data=cdr_data; cdrt_data(:,find((lat<=-30)|(lat>30)),:,:)=NaN;


for i=1:length(vars),

  % Global, land, ocean Area Average
  datG=squeeze(cdr_data(:,:,:,i)); 
  datL=datG; datL(find(mask==0))=NaN; 
  datO=datG; datO(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datG)))=NaN;
  cdr_tsG=nansum(reshape(datG.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  cdr_tsG(find(cdr_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datL)))=NaN;
  cdr_tsL=nansum(reshape(datL.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  cdr_tsL(find(cdr_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datO)))=NaN;
  cdr_tsO=nansum(reshape(datO.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  cdr_tsO(find(cdr_tsO==0))=NaN;

  % Global, land, ocean Anomaly Area Average
  datGa=squeeze(cdra_data(:,:,:,i));
  datLa=datGa; datLa(find(mask==0))=NaN;
  datOa=datGa; datOa(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datGa)))=NaN;
  cdra_tsG=nansum(reshape(datGa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  cdra_tsG(find(cdra_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datLa)))=NaN;
  cdra_tsL=nansum(reshape(datLa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  cdra_tsL(find(cdra_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datOa)))=NaN;
  cdra_tsO=nansum(reshape(datOa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  cdra_tsO(find(cdra_tsO==0))=NaN;

  % Tropical, land, ocean Average
  datGt=squeeze(cdrt_data(:,:,:,i));
  datLt=datGt; datLt(find(mask==0))=NaN;
  datOt=datGt; datOt(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datGt)))=NaN;
  cdrt_tsG=nansum(reshape(datGt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  cdrt_tsG(find(cdrt_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datLt)))=NaN;
  cdrt_tsL=nansum(reshape(datLt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  cdrt_tsL(find(cdrt_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datOt)))=NaN;
  cdrt_tsO=nansum(reshape(datOt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  cdrt_tsO(find(cdrt_tsO==0))=NaN;

  clf;

  subplot 311; % Global Average
  plot(alldates,cdr_tsG,'-k',alldates,cdr_tsO,'-b',alldates,cdr_tsL,'-g','LineWidth',2);
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Area Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 312; % Anomaly
  plot(alldates,cdra_tsG,'-k',alldates,cdra_tsO,'-b',alldates,cdra_tsL,'-g','LineWidth',2);
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Anom Global Area Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 313; % Tropics
  plot(alldates,cdrt_tsG,'-k',alldates,cdrt_tsO,'-b',alldates,cdrt_tsL,'-g','LineWidth',2);
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Tropics Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  set(gcf,'PaperPosition',[0.25 2.5 12 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_CDR_Tseries_' char(vars(i))]);

  disp(char(vars(i)));
end;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot anomaly of each CDR field as a Hovmuller plot by lat


for i=1:length(vars),
  datGa=squeeze(cdra_data(:,:,:,i));
  datLa=datGa; datLa(find(mask==0))=NaN;
  datOa=datGa; datOa(find(mask~=0))=NaN;

  clf;

  subplot 311; 
  imagesc(alldates,lat,squeeze(nanmean(datGa,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Anomaly (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 312; 
  imagesc(alldates,lat,squeeze(nanmean(datOa,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Ocean Anomaly (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 313; 
  imagesc(alldates,lat,squeeze(nanmean(datLa,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Land Anomaly (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  set(gcf,'PaperPosition',[0.25 2.5 8 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_CDR_HovLat_' char(vars(i))]);

  disp(char(vars(i)));
end;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot anomaly of each CDR field as a Hovmuller plot by lat


for i=1:length(vars),
  datGa=squeeze(cdra_data(:,:,:,i));
  datLa=datGa; datLa(find(mask==0))=NaN;
  datOa=datGa; datOa(find(mask~=0))=NaN;

  clf;

  subplot 311; 
  imagesc(alldates,lon,squeeze(nanmean(datGa,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Anomaly (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 312; 
  imagesc(alldates,lon,squeeze(nanmean(datOa,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Ocean Anomaly (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 313; 
  imagesc(alldates,lon,squeeze(nanmean(datLa,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Land Anomaly (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  set(gcf,'PaperPosition',[0.25 2.5 8 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_CDR_HovLon_' char(vars(i))]);

  disp(char(vars(i)));
end;




