

clear all;

% Plot ICDR file diagnostics



% add path to directory where some useful functions are stored
path(path,'GPCPcdr_m5/GPCPcdr_m5_DiagPlots/');

% get path of directories from NRT options file
% read options file, and match  find name of "OutputPATH", then extract it
lines=textread('OptionsM_NRT.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));
odir=[fdir '/DiagPlots/'];


% get path of directories for CDR data from options file
lines=textread('OptionsM.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
cdrdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read all ICDR files as well as CDR files

vars={'precip';'precip_error'};
varnames={'GPCP ICDR precipitation';'GPCP ICDR precip random error'};

tmp=datevec(now); tmp(3)=1; tmp(4:6)=0;
alldates=datevec(datenum('01jan04'):datenum(tmp));
  alldates(:,3)=1; alldates=unique(datenum(alldates));

icdr_data=NaN*ones([144 72 length(alldates) length(vars)]);
cdr_data=icdr_data;
for k=1:length(alldates), try,
  [yyyy mm trash]=datevec(alldates(k));

  % read ICDR data
  tmp1=ncload(sprintf('%sgpcp_icdr_v22rB1_y%04d_m%02d.nc',fdir,yyyy,mm));
  for i=1:length(vars), try,
    icdr_data(:,:,k,i)=tmp1.(char(vars(i)));
  end; end;

  % read CDR data
  tmp2=ncload(sprintf('%sgpcp_cdr_v22rB1_y%04d_m%02d.nc',cdrdir,yyyy,mm));
  for i=1:length(vars), try,
    cdr_data(:,:,k,i)=tmp2.(char(vars(i)));
  end; end;

end; end;


lat=[-90:2.5:87.5]'; lon=[0:2.5:357.5]';
icdr_data(find((icdr_data<-900)|(icdr_data>900)))=NaN;
cdr_data(find((cdr_data<-900)|(cdr_data>900)))=NaN;









%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot each ICDR field as a separate equal area averaged time series
xlabs=datenum(['01jan04';'01jan06';'01jan08';'01jan10';'01jan12';'01jan14';'01jan16']);
% get slims - make end now+1 month
xlims=datevec(now); xlims(3)=1; xlims(4:6)=0;
xlims=[datenum('01jan02') datenum(xlims+[0 1 0 0 0 0])];


% make weights and sum of weights for area averaging
siz=size(icdr_data);
w=repmat(abs(cos(lat))',[siz(1) 1 siz(3)]); wsum=nansum(nansum(w(:,:,1)));

% make mask for land/ocean
load land_mask_250km_from_1km; 
mask=repmat(land_mask_250km([73:end 1:72],:),[1 1 siz(3)]);

% Make anomaly version of icdr_data
icdra_data=NaN*ones(siz);
temp=datevec(alldates);
for i=1:12, idx=find(temp(:,2)==i);
  icdra_data(:,:,idx,:)=icdr_data(:,:,idx,:)-repmat(nanmean(icdr_data(:,:,idx,:),3),[1 1 length(idx) 1]);
end;

% Make tropics only version
icdrt_data=icdr_data; icdrt_data(:,find((lat<=-30)|(lat>30)),:,:)=NaN;


for i=1:length(vars),

  % Global, land, ocean Area Average
  datG=squeeze(icdr_data(:,:,:,i)); 
  datL=datG; datL(find(mask==0))=NaN; 
  datO=datG; datO(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datG)))=NaN;
  icdr_tsG=nansum(reshape(datG.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  icdr_tsG(find(icdr_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datL)))=NaN;
  icdr_tsL=nansum(reshape(datL.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  icdr_tsL(find(icdr_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datO)))=NaN;
  icdr_tsO=nansum(reshape(datO.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  icdr_tsO(find(icdr_tsO==0))=NaN;

  % Global, land, ocean Anomaly Area Average
  datGa=squeeze(icdra_data(:,:,:,i));
  datLa=datGa; datLa(find(mask==0))=NaN;
  datOa=datGa; datOa(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datGa)))=NaN;
  icdra_tsG=nansum(reshape(datGa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  icdra_tsG(find(icdra_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datLa)))=NaN;
  icdra_tsL=nansum(reshape(datLa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  icdra_tsL(find(icdra_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datOa)))=NaN;
  icdra_tsO=nansum(reshape(datOa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  icdra_tsO(find(icdra_tsO==0))=NaN;

  % Tropical, land, ocean Average
  datGt=squeeze(icdrt_data(:,:,:,i));
  datLt=datGt; datLt(find(mask==0))=NaN;
  datOt=datGt; datOt(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datGt)))=NaN;
  icdrt_tsG=nansum(reshape(datGt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  icdrt_tsG(find(icdrt_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datLt)))=NaN;
  icdrt_tsL=nansum(reshape(datLt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  icdrt_tsL(find(icdrt_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datOt)))=NaN;
  icdrt_tsO=nansum(reshape(datOt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  icdrt_tsO(find(icdrt_tsO==0))=NaN;

  clf;

  subplot 311; % Global Average
  plot(alldates,icdr_tsG,'-k',alldates,icdr_tsO,'-b',alldates,icdr_tsL,'-g','LineWidth',2);
  axis tight;
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Area Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 312; % Anomaly
  plot(alldates,icdra_tsG,'-k',alldates,icdra_tsO,'-b',alldates,icdra_tsL,'-g','LineWidth',2);
  axis tight;
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Anom Global Area Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 313; % Tropics
  plot(alldates,icdrt_tsG,'-k',alldates,icdrt_tsO,'-b',alldates,icdrt_tsL,'-g','LineWidth',2);
  axis tight;
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Tropics Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  set(gcf,'PaperPosition',[0.25 2.5 12 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_ICDR_Tseries_' char(vars(i))]);

  disp(char(vars(i)));
end;








%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot anomaly of each ICDR field as a Hovmuller plot by lat


for i=1:length(vars),
  datGa=squeeze(icdra_data(:,:,:,i));
  datLa=datGa; datLa(find(mask==0))=NaN;
  datOa=datGa; datOa(find(mask~=0))=NaN;

  clf;

  subplot 311; 
  imagesc(alldates,lat,squeeze(nanmean(datGa,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Anomaly (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 312; 
  imagesc(alldates,lat,squeeze(nanmean(datOa,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Ocean Anomaly (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 313; 
  imagesc(alldates,lat,squeeze(nanmean(datLa,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Land Anomaly (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  set(gcf,'PaperPosition',[0.25 2.5 8 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_ICDR_HovLat_' char(vars(i))]);

  disp(char(vars(i)));
end;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot anomaly of each ICDR field as a Hovmuller plot by lat


for i=1:length(vars),
  datGa=squeeze(icdra_data(:,:,:,i));
  datLa=datGa; datLa(find(mask==0))=NaN;
  datOa=datGa; datOa(find(mask~=0))=NaN;

  clf;

  subplot 311; 
  imagesc(alldates,lon,squeeze(nanmean(datGa,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Anomaly (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 312; 
  imagesc(alldates,lon,squeeze(nanmean(datOa,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Ocean Anomaly (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 313; 
  imagesc(alldates,lon,squeeze(nanmean(datLa,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Land Anomaly (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  set(gcf,'PaperPosition',[0.25 2.5 8 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_ICDR_HovLon_' char(vars(i))]);

  disp(char(vars(i)));
end;






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mean difference from CDR by season

vars={'precip';'precip_error'};
varnames={'GPCP ICDR-CDR precipitation';'GPCP-CDR ICDR precip random error'};

% Make tropics only version
cdrt_data=cdr_data; cdrt_data(:,find((lat<=-30)|(lat>30)),:,:)=NaN;


for i=1:length(vars),

  % Global, land, ocean Area Average
  datG=squeeze(icdr_data(:,:,:,i)-cdr_data(:,:,:,i));
  datL=datG; datL(find(mask==0))=NaN;
  datO=datG; datO(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datG)))=NaN;
  diff_tsG=nansum(reshape(datG.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  diff_tsG(find(diff_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datL)))=NaN;
  diff_tsL=nansum(reshape(datL.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  diff_tsL(find(diff_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datO)))=NaN;
  diff_tsO=nansum(reshape(datO.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  diff_tsO(find(diff_tsO==0))=NaN;

  % Global, land, ocean RMSE
  diff_tsGr=sqrt(nanmean(reshape(datG.^2,prod(siz(1:2)),siz(3)))');
  diff_tsLr=sqrt(nanmean(reshape(datL.^2,prod(siz(1:2)),siz(3)))');
  diff_tsOr=sqrt(nanmean(reshape(datO.^2,prod(siz(1:2)),siz(3)))');

  % Tropical, land, ocean Average
  datGt=squeeze(icdrt_data(:,:,:,i)-cdrt_data(:,:,:,i));
  datLt=datGt; datLt(find(mask==0))=NaN;
  datOt=datGt; datOt(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datGt)))=NaN;  
  diff_tsGt=nansum(reshape(datGt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  diff_tsGt(find(diff_tsGt==0))=NaN;
  wsum=w; wsum(find(isnan(datLt)))=NaN;
  diff_tsLt=nansum(reshape(datLt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  diff_tsLt(find(diff_tsLt==0))=NaN;
wsum=w; wsum(find(isnan(datOt)))=NaN;
  diff_tsOt=nansum(reshape(datOt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  diff_tsOt(find(diff_tsOt==0))=NaN;




  clf;

  subplot 311; % Global Average
  plot(alldates,diff_tsG,'-k',alldates,diff_tsO,'-b',alldates,diff_tsL,'-g','LineWidth',2);
  axis tight;
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Area Diff','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 312; % RMSE
  plot(alldates,diff_tsGr,'-k',alldates,diff_tsOr,'-b',alldates,diff_tsLr,'-g','LineWidth',2);
  axis tight;
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global RMSE','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 313; % Tropics
  plot(alldates,diff_tsGt,'-k',alldates,diff_tsOt,'-b',alldates,diff_tsLt,'-g','LineWidth',2);
  axis tight;
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Tropics Diff','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  set(gcf,'PaperPosition',[0.25 2.5 12 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_ICDR_DiffTs_' char(vars(i))]);

  disp(char(vars(i)));
end;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hovmuller plot by lat of diff between cdd and icdr


for i=1:length(vars),
  datG=squeeze(icdr_data(:,:,:,i)-cdr_data(:,:,:,i));
  datL=datG; datL(find(mask==0))=NaN;
  datO=datG; datO(find(mask~=0))=NaN;

  clf;

  subplot 311; 
  imagesc(alldates,lat,squeeze(nanmean(datG,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Diff (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 312; 
  imagesc(alldates,lat,squeeze(nanmean(datO,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Ocean Diff (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 313; 
  imagesc(alldates,lat,squeeze(nanmean(datL,1)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Land Diff (Lat)','FontSize',20); title(varnames(i),'FontSize',20);

  set(gcf,'PaperPosition',[0.25 2.5 8 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_ICDR_DiffHovLat_' char(vars(i))]);

  disp(char(vars(i)));
end;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hovmuller plot by lon of diff between cdd and icdr


for i=1:length(vars),
  datG=squeeze(icdr_data(:,:,:,i)-cdr_data(:,:,:,i));
  datL=datG; datL(find(mask==0))=NaN;
  datO=datG; datO(find(mask~=0))=NaN;

  clf;

  subplot 311; 
  imagesc(alldates,lat,squeeze(nanmean(datG,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Diff (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 312; 
  imagesc(alldates,lat,squeeze(nanmean(datO,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Ocean Diff (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  subplot 313; 
  imagesc(alldates,lat,squeeze(nanmean(datL,2)));
  colorbar('h');
  set(gca,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Land Diff (Lon)','FontSize',20); title(varnames(i),'FontSize',20);

  set(gcf,'PaperPosition',[0.25 2.5 8 12]);
  print('-dpng','-painter',[char(odir) '/GPCPcdr_QCplots_ICDR_DiffHovLon_' char(vars(i))]);

  disp(char(vars(i)));
end;





