

% Plot Interim files
clear all;

% add path to directory where some useful functions are stored
path(path,'GPCPcdr_m5/GPCPcdr_m5_DiagPlots/');

% get path of directories from options file
% read options file, and match  find name of "OutputPATH", then extract it
lines=textread('OptionsM.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));
odir=[fdir '/DiagPlots/'];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read all CDR and ICDR files

vars={'pop';'pms';'pga';'psg';'eop';'ems';'ega';'esg';'nga';'pag';'eag';'pgp';'ngp';'pse';'nse';'pss';'nss';'psc';'ssc';'nsc';'pst';'est';'sst';'ptv'};
varnames={'OPI precipitation';'Multi-satellite precipitation';'Gauge precipitation';'Satellite/gauge precipitation';'OPI precipitation random error';'Multi-satellite precipitation random error';'Gauge precipitation random error';'Satellite/gauge precipitation random error';'Gauge precipitation number of gauges';'AGPI precipitation';'AGPI precipitation random error';'GPI precipitation';'GPI image number';'SSM/I emission precipitation';'SSM/I emission number of samples';'SSM/I scattering precipitation';'SSM/I scattering number of samples';'SSM/I composite precipitation';'SSM/I composite source';'SSM/I composite number of samples';'Merged SSMI/TOVS precipitation';'Merged SSMI/TOVS precipitation random error';'Merged SSMI/TOVS source';'TOVS precipitation'};



tmp=datevec(now); tmp(3)=1; tmp(4:6)=0;
alldates=datevec(datenum('01jan79'):datenum(tmp));
  alldates(:,3)=1; alldates=unique(datenum(alldates));

int_data=NaN*ones([144 72 length(alldates) length(vars)]);
for k=1:length(alldates), try,
  [yyyy mm trash]=datevec(alldates(k));

  tmp=ncload(sprintf('%sgpcp_Interim_v22rB1_y%04d_m%02d.nc',fdir,yyyy,mm));

  for i=1:length(vars), try,
    int_data(:,:,k,i)=tmp.(char(vars(i)));
  end; end;
end; end;


lat=[-90:2.5:87.5]'; lon=[0:2.5:357.5]';
int_data(find((int_data<-900)|(int_data>900)))=NaN;






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot each Interim field as a separate equal area averaged time series
xlabs=datenum(['01jan80';'01jan86';'01jan92';'01jan98';'01jan04';'01jan10';'01jan16']);
% get slims - make end now+1 month
xlims=datevec(now); xlims(3)=1; xlims(4:6)=0;
xlims=[datenum('01jan72') datenum(xlims+[0 1 0 0 0 0])];

% make weights and sum of weights for area averaging
siz=size(int_data);
w=repmat(abs(cos(lat))',[siz(1) 1 siz(3)]); wsum=nansum(nansum(w(:,:,1)));

% make mask for land/ocean
load land_mask_250km_from_1km; 
mask=repmat(land_mask_250km([73:end 1:72],:),[1 1 siz(3)]);

% Make anomaly version of int_data
inta_data=NaN*ones(siz);
temp=datevec(alldates);
for i=1:12, idx=find(temp(:,2)==i);
  inta_data(:,:,idx,:)=int_data(:,:,idx,:)-repmat(nanmean(int_data(:,:,idx,:),3),[1 1 length(idx) 1]);
end;

% Make tropics only version
intt_data=int_data; intt_data(:,find((lat<=-30)|(lat>30)),:,:)=NaN;


for i=1:length(vars),

  % Global, land, ocean Area Average
  datG=squeeze(int_data(:,:,:,i)); 
  datL=datG; datL(find(mask==0))=NaN; 
  datO=datG; datO(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datG)))=NaN;
  int_tsG=nansum(reshape(datG.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  int_tsG(find(int_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datL)))=NaN;
  int_tsL=nansum(reshape(datL.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  int_tsL(find(int_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datO)))=NaN;
  int_tsO=nansum(reshape(datO.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  int_tsO(find(int_tsO==0))=NaN;

  % Global, land, ocean Anomaly Area Average
  datGa=squeeze(inta_data(:,:,:,i));
  datLa=datGa; datLa(find(mask==0))=NaN;
  datOa=datGa; datOa(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datGa)))=NaN;
  inta_tsG=nansum(reshape(datGa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  inta_tsG(find(inta_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datLa)))=NaN;
  inta_tsL=nansum(reshape(datLa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  inta_tsL(find(inta_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datOa)))=NaN;
  inta_tsO=nansum(reshape(datOa.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  inta_tsO(find(inta_tsO==0))=NaN;

  % Tropical, land, ocean Average
  datGt=squeeze(intt_data(:,:,:,i));
  datLt=datGt; datLt(find(mask==0))=NaN;
  datOt=datGt; datOt(find(mask~=0))=NaN;
  wsum=w; wsum(find(isnan(datGt)))=NaN;
  intt_tsG=nansum(reshape(datGt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)'; 
  intt_tsG(find(intt_tsG==0))=NaN;
  wsum=w; wsum(find(isnan(datLt)))=NaN;
  intt_tsL=nansum(reshape(datLt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  intt_tsL(find(intt_tsL==0))=NaN;
  wsum=w; wsum(find(isnan(datOt)))=NaN;
  intt_tsO=nansum(reshape(datOt.*w,prod(siz(1:2)),siz(3)),1)'./nansum(reshape(wsum,prod(siz(1:2)),siz(3)),1)';
  intt_tsO(find(intt_tsO==0))=NaN;


  clf;

  subplot 311; % Global Average
  plot(alldates,int_tsG,'-k',alldates,int_tsO,'-b',alldates,int_tsL,'-g','LineWidth',2);
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Global Area Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 312; % Anomaly
  plot(alldates,inta_tsG,'-k',alldates,inta_tsO,'-b',alldates,inta_tsL,'-g','LineWidth',2);
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Anom Global Area Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  subplot 313; % Tropics
  plot(alldates,intt_tsG,'-k',alldates,intt_tsO,'-b',alldates,intt_tsL,'-g','LineWidth',2);
  set(gca,'XLim',xlims,'XTick',xlabs,'XTickLabel',datestr(xlabs,'yyyy'),'FontSize',20);
  ylabel('Tropics Ave','FontSize',20); title(varnames(i),'FontSize',20);
  legend({'Global';'Ocean';'Land'},'Location','West','FontSize',14);

  set(gcf,'PaperPosition',[0.25 2.5 12 12]);
  print('-dpng',[char(odir) '/GPCPcdr_QCplots_Interim_Tseries_' char(vars(i))]);

  disp(char(vars(i)));
end;










