function [] = GPCPcdr_m5_Products_ESPI();
% Matlab function to calculate and plot the ESPI using CDR data and 
% the most recent ICDR data
%
%
%
% Run from unix using:
% /usr/local/bin/matlab -nosplash -nodisplay -nodesktop -r "path(path,'GPCPcdr_m5/GPCPcdr_m5_Products/'); GPCPcdr_m5_Products_ESPI(); exit;"

% Writen by MRP Sapiano, Oct 2014
% Aug 2015 (Sapiano): Add code to produce plot of EI and LI



% add path to directory where some useful functions are stored
path(path,'GPCPcdr_m5/GPCPcdr_m5_Products/');



% read standard climatology for ESPI
fid=fopen('ESPI_Clim_GPCPv2p2_79to13_aug14.txt','rt');
  trash=fscanf(fid,'%s',4);
  clim=fscanf(fid,'%*f %*f %*d %f',inf); fclose(fid);
clim=reshape(clim,[144 72 12]);

% read normalization factors and normalize ESPI series
fid=fopen('ESPI_NormFacs_GPCPv2p2_79to13_aug14.txt','rt');
  trash=fscanf(fid,'%s',6);
  normfacs=fscanf(fid,'%f %f %f %f %f %f',inf); fclose(fid);
normfacs=reshape(normfacs,[6 12])';




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read Nino 3.4 from CPC:
[s w]=unix('curl -o GPCPcdr_m5/GPCPcdr_m5_Products/sstoi.indices  http://www.cpc.ncep.noaa.gov/data/indices/sstoi.indices');
clear nino; [nino.yr nino.mon nino.nino12 nino.nino12anom nino.nino3 nino.nino3anom nino.nino4 nino.nino4anom nino.nino34 nino.nino34anom]=textread('sstoi.indices','%n%n%n%n%n%n%n%n%n%n','headerlines',1);
nino.dates=datenum([nino.yr nino.mon ones(size(nino.yr))]);





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read data

% Calculate potential dates for the ESPI and initialize precip variable
alldates=datevec(datenum('01jan1979'):now);
alldates(:,3)=1; alldates=unique(datenum(alldates));
precip=NaN*ones([144 72 length(alldates)]);



%%%%%%%%%
% Read all available GPCP CDR data 

% Get CDR directory from OptionsM.h
lines=textread('OptionsM.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));

for k=1:length(alldates), clear data;
  fname=sprintf('%s/gpcp_cdr_v22rB1_y%s_m%s.nc',fdir,datestr(alldates(k),'yyyy'),datestr(alldates(k),'mm'));
  if ~isempty(dir(fname)),  % check file exists
    data=ncload(fname,'precip'); 
    if isfield(data,'precip'); % check that field exists
      data.precip(find(data.precip<-9900))=NaN;
      test=nanmean(nanmean(data.precip)); % test that precip is reasonable
      if (test>=0)&(test<=10),
        precip(:,:,k)=fliplr(data.precip);
      end;
    end;
  end;
end;



%%%%%%%%%
% Read ICDR data and backfill for up to last 12 months

% get months for which ICDR data is required
test=squeeze(sum(sum(isnan(precip))));
ksub=find(test~=0);
ksub=ksub(find(ksub>=(length(alldates)-11)));

% Get ICDR directory from OptionsM_NRT.h
lines=textread('OptionsM_NRT.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));

for k=ksub', clear data;
  fname=sprintf('%s/gpcp_icdr_v22rB1_y%s_m%s.nc',fdir,datestr(alldates(k),'yyyy'),datestr(alldates(k),'mm'));
  if ~isempty(dir(fname)),  % check file exists
    data=ncload(fname,'precip'); 
    if isfield(data,'precip'); % check that field exists
      data.precip(find(data.precip<-9900))=NaN;
      test=nanmean(nanmean(data.precip)); % test that precip is reasonable
      if (test>=0)&(test<=10),
        precip(:,:,k)=fliplr(data.precip);
        % print months that where ICDR was used
        disp(['ICDR used for ' datestr(alldates(k),'yyyy mm')]);
      end;
    end;
  end;
end;

% print months that where not read successfully 
test=squeeze(sum(sum(isnan(precip))));
disp(char({'Summary of missing CDR data:';'yyyy mm    Nmiss'; [datestr(alldates(find(test~=0)),'yyyy mm    ') num2str(test(find(test~=0)))]}));






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get ESPI for GPCP
% TRACKpac and TRACKindo find the largest precipitation anomalies in the search areas, 
% namely Ap+ and Ap-, and Amc+ and Amc-.  createENSO combines the indices to generate 
% the EI (Ap+ minus Amc-) and LI  (Amc+ minus Ap-). ESPI is then EI-LI.  EI, LI, and 
% ESPI are all normalized in the last step.

% set data and dates
data=precip; dates=alldates;
lat=-90:2.5:87.5; lon=0:2.5:357.5;

% Get anomalies from standard clim
for i=1:12, ii=datevec(dates); ii=find((ii(:,2)==i));
  data(:,:,ii)=data(:,:,ii)-repmat(clim(:,:,i),[1 1 length(ii)]);
end;

% Get equal area average for 10x50 deg box anomalies for Maritime Continent region (10 S/N, 90-150 E)
temp=[];
for ilat=1:5, for ilon=1:5, 
  idx=ilon+[36:56]; idy=ilat+[32:36];
  temp(ilat,ilon,:)=mapave(data(idx,idy,:),lat(idy),lat(idy)+2.5);
end; end;
temp=reshape(temp,25,size(temp,3));
% get max and min
[Amc_pos Amc_pos_ind]=max(temp,[],1); [Amc_neg Amc_neg_ind]=min(temp,[],1);

% Get equal area average for 10x50 deg box anomalies for Pacific region (10 S/N, 160-260 E)
temp=[];
for ilat=1:5, for ilon=1:21, 
  idx=ilon+[64:84]; idy=ilat+[32:36];
  temp(ilat,ilon,:)=mapave(data(idx,idy,:),lat(idy),lat(idy)+2.5);
end; end;
temp=reshape(temp,105,size(temp,3));
% get max and min
[Ap_pos Ap_pos_ind]=max(temp,[],1); [Ap_neg Ap_neg_ind]=min(temp,[],1);

% Now combine indices from MC and Pacific to get EI, LI and ESPI
ei=Ap_pos-Amc_neg; li=Amc_pos-Ap_neg; espi=ei-li;

% normalize ESPI series using standard normalization factors
gpcp_ei=[]; gpcp_li=[]; gpcp_espi=[];
for i=1:12, ii=datevec(dates); ii=find((ii(:,2)==i));
  gpcp_ei(ii)=(ei(ii)-normfacs(i,1))/normfacs(i,2);
  gpcp_li(ii)=(li(ii)-normfacs(i,3))/normfacs(i,4);
  gpcp_espi(ii)=(espi(ii)-normfacs(i,5))/normfacs(i,6);
end;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output data to file

% Get CDR directory from OptionsM.h
lines=textread('OptionsM.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));
fdir=[fdir '/ESPI/'];
if isempty(dir(fdir)), [s w]=unix(['mkdir ' fdir]); end;

data=datevec(alldates);
data=[data(:,1:2) gpcp_espi' gpcp_ei' gpcp_li'];
data(find(isnan(data)))=-9.9;

% output to file
fid=fopen([fdir 'ESPI.txt'],'wt');
  fprintf(fid,'YYYY MM     ESPI       EI       LI\n');
  fprintf(fid,'%4d %2d %8.5f %8.5f %8.5f\n',data');
fclose(fid);












%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% make plot of ESPI with most recent ENSO data overlaid


clf;

subplot 211;
xlims=datevec(now+60); xlims=datenum([1979 1 1; xlims(1) 1 1]);
xlabs=datevec(xlims(2)); xlabs=[1980:5:xlabs(1)]'; xlabs=datenum([xlabs ones(size(xlabs,1),2)]);
plot(alldates,gpcp_espi,'-b',nino.dates,nino.nino34anom,'-r','LineWidth',3);
set(gca,'Xtick',xlabs,'XtickLabel',datestr(xlabs,'mmmyy'),'XLim',alldates([1 end]),'YLim',[-3 4],'FontSize',26);
hold on; plot(get(gca,'Xlim'),[0 0],':k','LineWidth',3)

subplot 212;
xlims=datevec(now+60); xlims=datenum([xlims(1)-5 1 1; xlims(1) 1 1]);
xlabs=datevec(xlims(1):xlims(2)); xlabs(:,2:3)=1; xlabs=unique(datenum(xlabs));
h=plot(alldates,gpcp_espi,'-b',nino.dates,nino.nino34anom,'-r','LineWidth',4);
xlims=datevec(alldates(end)); xlims=[datenum([xlims(1)-4 1 1]) alldates(end)];
set(gca,'Xtick',xlabs,'XtickLabel',datestr(xlabs,'mmmyy'),'XLim',xlims,'YLim',[-3 4],'FontSize',26);
hold on; plot(get(gca,'Xlim'),[0 0],':k','LineWidth',2)
legend(h,{'ESPI';'Nino 3.4 Anoms'},'location','North','Orientation','horizontal','FontSize',36); legend boxoff;

set(gcf,'PaperPosition',[.25 2.5 20 12])
print('-dpng',[fdir 'ESPI_plot']);




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% make plot of ESPI with most recent ENSO data overlaid


clf;

subplot 211;
xlims=datevec(now+60); xlims=datenum([1979 1 1; xlims(1) 1 1]);
xlabs=datevec(xlims(2)); xlabs=[1980:5:xlabs(1)]'; xlabs=datenum([xlabs ones(size(xlabs,1),2)]);
plot(nino.dates,nino.nino34anom,'-k',alldates,gpcp_li,'-b',alldates,gpcp_ei,'-r','LineWidth',3);
set(gca,'Xtick',xlabs,'XtickLabel',datestr(xlabs,'mmmyy'),'XLim',alldates([1 end]),'YLim',[-4 4],'FontSize',26);
hold on; plot(get(gca,'Xlim'),[0 0],':k','LineWidth',3)

subplot 212;
xlims=datevec(now+60); xlims=datenum([xlims(1)-5 1 1; xlims(1) 1 1]);
xlabs=datevec(xlims(1):xlims(2)); xlabs(:,2:3)=1; xlabs=unique(datenum(xlabs));
h=plot(nino.dates,nino.nino34anom,'-k',alldates,gpcp_li,'-b',alldates,gpcp_ei,'-r','LineWidth',4);
xlims=datevec(alldates(end)); xlims=[datenum([xlims(1)-4 1 1]) alldates(end)];
set(gca,'Xtick',xlabs,'XtickLabel',datestr(xlabs,'mmmyy'),'XLim',xlims,'YLim',[-4 4],'FontSize',26);
hold on; plot(get(gca,'Xlim'),[0 0],':k','LineWidth',2)
legend(h,{'Nino 3.4 Anoms';'LI';'EI'},'location','North','Orientation','horizontal','FontSize',36); legend boxoff;

set(gcf,'PaperPosition',[.25 2.5 20 12])
print('-dpng',[fdir 'ESPI_plot_EI_LI']);






disp(['ESPI data and plots stored in ' fdir]);




