function [out] = mapave(data,latlo,lathi);
% mapavegrads -- Weighted mean of input data base on lat excluding NaNs
%  Operates on first dimension of input data, like nanmean
%  This version is adapted from grads and can handle uneven lats
%
%  data - The data to be averaged - should be lon x lat x time
%
%  latlo - the lower latitude of the grid-box in degrees
%   
%  lathi - the upper latitude of the grid-box in degrees
%   
% EXAMPLES:
%
% out=mapave(data,lat,lat+2.5);

%   Written by Matt Sapiano, May 2014

if nargin ~= 3,  help(mfilename), return; end;

a=size(data);
if (length(lathi)~=a(2))|(length(latlo)~=a(2)), disp('lat must be 1xn'); return; end;
if (sum(isnan(latlo))~=0)|(sum(isnan(lathi))~=0), disp('lat is filled with missings'); return; end;

% The aave function assumes that the world coordinates are longitude in the X dimension and latitude 
%  in the Y dimension, and does weighting in the latitude dimension by the delta of the sin of the
%  latitudes. Weighting is also performed appropriately for unequally spaced grids.

% get weights based on cos(lat) and make same size as data
%w=repmat(squeeze(sin(deg2rad(lathi))-sin(deg2rad(latlo))),[a(1) 1 a(3)]);
w=repmat(squeeze(sin((pi/180)*(lathi))-sin((pi/180)*(latlo))),[a(1) 1 a(3)]);
w(find(isnan(data)))=NaN;

% reshape data and w
data=reshape(data,a(1)*a(2),a(3));
w=reshape(w,a(1)*a(2),a(3));

% calculation is very simple:
out=squeeze(nansum(data.*w)./nansum(w));



