
clear all;

% code to calibrate input datasets for the ESPI
cd /Users/msapiano/Documents/Work/misc/GPCP_CDR/ENSO
path(path,'/Users/msapiano/Documents/Work/matlab_functions');
load coast; coast_long=[long; NaN; long+360]; coast_lat=[lat; NaN; lat]; clear coast
load col_schemes;



lat=-90:2.5:87.5; lon=0:2.5:357.5;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read in GPCP V2.2
% data comes from: ftp://precip.gsfc.nasa.gov/pub/gpcp-v2.2/psg
fdir='/Users/msapiano/Documents/Work/misc/GPCP_CDR/ENSO/GPCPv2p2/';
gpcp=[]; gpcp_dates=[];
for kyr=1979:2014, for mm=1:12, ii=(kyr-1978)*12-12+mm;
  fid=fopen(sprintf('%sgpcp_v2.2_psg.%04d',fdir,kyr),'r','ieee-be');
    fseek(fid,576+(mm-1)*144*72*4,-1); a=fread(fid,144*72,'single'); fclose(fid);
  a=reshape(a,144,72); a(find(a<-9000))=NaN; 
  gpcp(:,:,ii)=fliplr(a); gpcp_dates(ii,:)=[kyr mm 1];
end; end;
gpcp_dates=datenum(gpcp_dates);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read in GPCP ICDR
fdir='/Users/msapiano/Documents/Work/misc/GPCP_CDR/ENSO/ICDR/';
icdr_dates=datevec(datenum('01jan04'):now); icdr_dates(:,3)=1; 
  icdr_dates=datevec(unique(datenum(icdr_dates)));
icdr=NaN*ones([144 72 size(icdr_dates,1)]);
for i=1:length(icdr_dates),
  fname=sprintf('%sgpcp_icdr_v22rB1_y%04d_m%02d.nc',fdir,icdr_dates(i,1),icdr_dates(i,2));
  if ~isempty(dir(fname)), a=ncload(fname); a=a.precip; 
     a(find(a<0))=NaN; a(find(a>500))=NaN; icdr(:,:,i)=fliplr(a); end;
end;
icdr_dates=datenum(icdr_dates);


test=[datevec(icdr_dates) squeeze(sum(sum(isnan(icdr)))) squeeze(sum(sum((icdr==0))))]
test=[test(:,[1:3 7 8]) test(:,7)+test(:,8)];
test(find(squeeze(sum(sum(isnan(icdr))))>0),:)






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Revised calculation of the ESPI for the CDR and ICDR


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculate monthly mean climatology for anomalies and output to binary file
% Calculate based on GPCP V2.2 for 35 years: 1979-2013

gpcp_clim=[];
for i=1:12,
  ii=datevec(gpcp_dates); ii=find((ii(:,2)==i)&(ii(:,1)>=1979)&(ii(:,1)<=2013));
  gpcp_clim(:,:,i)=nanmean(gpcp(:,:,ii),3);
end;

% Concatenate lon, lat, month, gpcp_clim
tmp=[repmat([0:2.5:357.5]',72*12,1) repmat(sort(repmat([-90:2.5:87.5]',144,1)),12,1) sort(repmat([1:12]',144*72,1)) reshape(gpcp_clim,144*72*12,1)];

% output to file
fid=fopen('ESPI_Clim_GPCPv2p2_79to13_aug14.txt','wt');
  fprintf(fid,'   Lon   Lat mon Climatol\n');
  fprintf(fid,'%6.1f %5.1f %3d %8.5f\n',tmp');
fclose(fid);








%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get un-normalized ESPI for GPCP ICDR need to do this in order to get normalization factors
% TRACKpac and TRACKindo find the largest precipitation anomalies in the search areas, 
% namely Ap+ and Ap-, and Amc+ and Amc-.  createENSO combines the indices to generate 
% the EI (Ap+ minus Amc-) and LI  (Amc+ minus Ap-). ESPI is then EI-LI.  EI, LI, and 
% ESPI are all normalized in the last step.

% set data and dates
data=gpcp; dates=gpcp_dates;

% Make anomalies from standard clim
fid=fopen('ESPI_Clim_GPCPv2p2_79to13_aug14.txt','rt');
  trash=fscanf(fid,'%s',4);
  clim=fscanf(fid,'%*f %*f %*d %f',inf); fclose(fid);
clim=reshape(clim,[144 72 12]);
for i=1:12,
  ii=datevec(dates); ii=find((ii(:,2)==i));
  data(:,:,ii)=data(:,:,ii)-repmat(clim(:,:,i),[1 1 length(ii)]);
end;

% Get equal area average for 10x50 deg box anomalies for Maritime Continent region (10 S/N, 90-150 E)
temp=[];
for ilat=1:5, for ilon=1:5, 
  idx=ilon+[36:56]; idy=ilat+[32:36];
  temp(ilat,ilon,:)=mapave(data(idx,idy,:),lat(idy),lat(idy)+2.5);
end; end;
temp=reshape(temp,25,size(temp,3));
% get max and min
[Amc_pos Amc_pos_ind]=max(temp,[],1); [Amc_neg Amc_neg_ind]=min(temp,[],1);

% Get equal area average for 10x50 deg box anomalies for Pacific region (10 S/N, 160-260 E)
temp=[];
for ilat=1:5, for ilon=1:21, 
  idx=ilon+[64:84]; idy=ilat+[32:36];
  temp(ilat,ilon,:)=mapave(data(idx,idy,:),lat(idy),lat(idy)+2.5);
end; end;
temp=reshape(temp,105,size(temp,3));
% get max and min
[Ap_pos Ap_pos_ind]=max(temp,[],1); [Ap_neg Ap_neg_ind]=min(temp,[],1);

% Now combine indices from MC and Pacific to get EI, LI and ESPI
ei=Ap_pos-Amc_neg; li=Amc_pos-Ap_neg; espi=ei-li;

% get normalization factors and output to file
normfacs=[];
for i=1:12,
  ii=datevec(dates); ii=find((ii(:,2)==i)&(ii(:,1)>=1979)&(ii(:,1)<=2013));
  normfacs(i,:)=[nanmean(ei(ii)) nanstd(ei(ii)) nanmean(li(ii)) nanstd(li(ii)) nanmean(espi(ii)) nanstd(espi(ii))];
end;

% output to file
fid=fopen('ESPI_NormFacs_GPCPv2p2_79to13_aug14.txt','wt');
  fprintf(fid,' EImean   EIstd    LImean   LIstd    ESPImean ESPIstd\n');
  fprintf(fid,'%8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n',normfacs');
fclose(fid);











