

clear all;

% code to fit the gamma distribution for SPI to GPCP CDR/ICDR
cd /home/msapiano/GPCP_CDR/GPCPv2p2_CDR_may12/monthly/GPCPcdr_m5/GPCPcdr_m5_Products
path(path,'/home/msapiano/matlab_functions');



% lat=-90:2.5:87.5; lon=0:2.5:357.5;









%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read data

% Calculate potential dates for the ESPI and initialize precip variable
alldates=datevec(datenum('01jan1979'):now);
alldates(:,3)=1; alldates=unique(datenum(alldates));
precip=NaN*ones([144 72 length(alldates)]);





%%%%%%%%%
% Read all available GPCP CDR data 

% Get CDR directory from OptionsM.h
lines=textread('../../OptionsM.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));

for k=1:length(alldates), clear data;
  fname=sprintf('%s/gpcp_cdr_v22rB1_y%s_m%s.nc',fdir,datestr(alldates(k),'yyyy'),datestr(alldates(k),'mm'));
  if ~isempty(dir(fname)),  % check file exists
    data=ncload(fname,'precip'); 
    if isfield(data,'precip'); % check that field exists
      data.precip(find(data.precip<-9900))=NaN;
      test=nanmean(nanmean(data.precip)); % test that precip is reasonable
      if (test>=0)&(test<=10),
        precip(:,:,k)=data.precip;
      end;
    end;
  end;
end;


% print months that where not read successfully 
test=squeeze(sum(sum(isnan(precip))));
disp(char({'Summary of missing CDR data:';'yyyy mmm   Nmiss'; [datestr(alldates(find(test~=0)),'yyyy mm    ') num2str(test(find(test~=0)))]}));

% rename data
gpcp_dates=alldates; gpcp=precip;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calculation of SPI requires a fitted gamma distribution at each grid box
% calculate alpha and beta using GPCP V2.2 for 35 years: 1979-2013
% output to a file
idx=find((gpcp_dates>=datenum('01jan79'))&(gpcp_dates<=datenum('01dec13')));
data=gpcp(:,:,idx); dates=gpcp_dates(idx);

% MoM method - not used!
% Calculate gamma distribution using maximum likelihood technique from Thom (1966)
%data1=data; data1(find(data1==0))=NaN;
%a=log(nanmean(data1,3))-nansum(log(data1),3)./nansum(~isnan(data1),3);
%alpha=(1./(4*a)).*(1+sqrt(1+4*a./3));
%beta=nanmean(data1,3)./alpha;



% Lloyd-hughes and Saunders (2002) preferred an iterative ML process 
% use Matlab function to do this:



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%for each month:
alpha=[]; beta=[]; warning off; 
for i=1:12, ii=datevec(dates); ii=find(ii(:,2)==i);
  for i1=1:144, for i2=1:72,
    b=gamfit(squeeze(data(i1,i2,ii)));
    alpha(i1,i2,i)=b(1); beta(i1,i2,i)=b(2);
end; end; end; warning on;

% Concatenate lon, lat, month, Alpha, Beta
tmp=[repmat([0:2.5:357.5]',72*12,1) repmat(sort(repmat([-90:2.5:87.5]',144,1)),12,1) sort(repmat([1:12]',144*72,1)) reshape(alpha,144*72*12,1) reshape(beta,144*72*12,1)];

% output to file
fid=fopen('SPI_Monthly_AlphaBeta_GPCPv2p2_79to13_aug14.txt','wt');
  fprintf(fid,'   Lon   Lat mon  alpha    beta\n');
  fprintf(fid,'%6.1f %5.1f %3d %8.5f %8.5f\n',tmp');
fclose(fid);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3 month means:
alpha=[]; beta=[]; warning off; 
for i=1:12, ii=datevec(dates); mons=mod([i i-1 i-2],12); mons(find(mons==0))=12;
  data1=nanmean(cat(4,data(:,:,find(ii(:,2)==mons(1))),data(:,:,find(ii(:,2)==mons(2))),data(:,:,find(ii(:,2)==mons(3)))),4);
  for i1=1:144, for i2=1:72,
    b=gamfit(data1(i1,i2,:));
    alpha(i1,i2,i)=b(1); beta(i1,i2,i)=b(2);
end; end; end; warning on;

% Concatenate lon, lat, month, Alpha, Beta
tmp=[repmat([0:2.5:357.5]',72*12,1) repmat(sort(repmat([-90:2.5:87.5]',144,1)),12,1) sort(repmat([1:12]',144*72,1)) reshape(alpha,144*72*12,1) reshape(beta,144*72*12,1)];

% output to file
fid=fopen('SPI_3Monthly_AlphaBeta_GPCPv2p2_79to13_aug14.txt','wt');
  fprintf(fid,'   Lon  Lat End3Mon alpha  beta\n');
  fprintf(fid,'%6.1f %5.1f %3d %8.5f %8.5f\n',tmp');
fclose(fid);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 6 month means:
alpha=[]; beta=[]; warning off; 
for i=1:12, ii=datevec(dates); 
  mons=mod(i-[0:5],12); mons(find(mons==0))=12;
  data1=[]; for j=1:6, data1=cat(4,data1,data(:,:,find(ii(:,2)==mons(j)))); end;
  data1=nanmean(data1,4);
  for i1=1:144, for i2=1:72,
    b=gamfit(data1(i1,i2,:));
    alpha(i1,i2,i)=b(1); beta(i1,i2,i)=b(2);
end; end; end; warning on;

% Concatenate lon, lat, month, Alpha, Beta
tmp=[repmat([0:2.5:357.5]',72*12,1) repmat(sort(repmat([-90:2.5:87.5]',144,1)),12,1) sort(repmat([1:12]',144*72,1)) reshape(alpha,144*72*12,1) reshape(beta,144*72*12,1)];

% output to file
fid=fopen('SPI_6Monthly_AlphaBeta_GPCPv2p2_79to13_aug14.txt','wt');
  fprintf(fid,'   Lon  Lat End6Mon alpha  beta\n');
  fprintf(fid,'%6.1f %5.1f %3d %8.5f %8.5f\n',tmp');
fclose(fid);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Annual:
alpha=[]; beta=[]; warning off; 
yyyy=datevec(dates); yyyy=yyyy(:,1); yyyy_un=unique(yyyy); data1=[];
for i=1:length(yyyy_un), data1(:,:,i)=nanmean(data(:,:,find(yyyy==yyyy_un(i))),3); end;
for i1=1:144, for i2=1:72,
    b=gamfit(data1(i1,i2,:));
    alpha(i1,i2)=b(1); beta(i1,i2)=b(2);
end; end; warning on;
% Concatenate lon, lat, Alpha, Beta
tmp=[repmat([0:2.5:357.5]',72,1) sort(repmat([-90:2.5:87.5]',144,1)) reshape(alpha,144*72,1) reshape(beta,144*72,1)];

% output to file
fid=fopen('SPI_Annual_AlphaBeta_GPCPv2p2_79to13_aug14.txt','wt');
  fprintf(fid,'   Lon   Lat  alpha    beta\n');
  fprintf(fid,'%6.1f %5.1f %8.5f %8.5f\n',tmp');
fclose(fid);







