clear all;

% Read all CDR, ICDR and Interim NC files and summarize output
% Code is written so that it returns info for all vars with dim>=2
% If a file has an extra (erroneous) 2d variable, this will be printed
% If a variable is missing for a month, it will set to missing 


% add path to directory where some useful functions are stored
path(path,'GPCPcdr_m5/GPCPcdr_m5_TestOutput/');

% get path of directories from options file
% read options file, and match  find name of "OutputPATH", then extract it
lines=textread('OptionsM.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));

% read NRT options file, and match  find name of "OutputPATH", then extract it
lines=textread('OptionsM_NRT.h','%s','delimiter','\n','whitespace','');
idx=strmatch('#define OutputPATH',lines);
fdir_NRT=char(regexprep(lines(idx,:),{'#define OutputPATH',' ','"'},''));





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% initialize output file:
oname='GPCP_FullTest.txt';
fid=fopen(oname,'wt');
fprintf(fid,'Results of full test of GPCP CDR, ICDR and Interim files\n');
fprintf(fid,'Processed: %s\n',datestr(now,0));




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read all CDR files

tmp=datevec(now); tmp(3)=1; tmp(4:6)=0;
alldates=datevec(datenum('01jan79'):datenum(tmp));
  alldates(:,3)=1; alldates=unique(datenum(alldates));

Table=[]; VarList='';
for k=1:length(alldates), try,
  [yyyy mm trash]=datevec(alldates(k));
  Table(k,1:2)=[yyyy mm];

  % read all data in file
  data=ncload(sprintf('%sgpcp_cdr_v22rB1_y%04d_m%02d.nc',fdir,yyyy,mm));
  vars=fieldnames(data);

  % scroll through vars and deal only with fields that are 2D
  for var=vars',
    if sum(size(data.(char(var)))>1)>=2,
      if isempty(strmatch(var,VarList)),
        VarList={char(VarList);char(var)};
      end;
    end;
  end;
  VarList=cellstr(char(VarList));

  % read each possible var and output
  for i=2:length(VarList),
    try, miss=data.atts.(char(VarList(i))).missing_value;
      fail; miss=-9999; end;
    Table(k,2+2*(i-2)+1)=sum(sum((data.(char(VarList(i)))==miss)));
    Table(k,2+2*(i-2)+2)=sum(sum((data.(char(VarList(i)))==0)));
  end;

end; end;

% add to output file
fprintf(fid,'\n\nGPCP CDR: \n');
fprintf(fid,'Variables:   %s \n',char(VarList(2:end))');
fprintf(fid,['YYYY MM   ' repmat(' Nmiss   N==0 ',1,length(VarList)-1) '\n']);
fprintf(fid,['%04d %02d   '  repmat('%6d %6d ',1,length(VarList)-1),'\n'],Table');




% Make summary of potential empty/corrupted files
% find variables with any missing data (usually there is none)
idx1=Table(:,3:2:end)~=0;
% find variables with more than average numbers of zero data
idx2=Table(:,4:2:end)>2000;

cdr_summary=sprintf('Variables:   %s',char(VarList(2:end))');
cdr_summary=char({cdr_summary; sprintf(['YYYY MM   ' repmat(' Nmiss   N==0 ',1,length(VarList)-1)])});
for i=find((sum(idx1,2)>0)|(sum(idx2,2)>0))',
  cdr_summary=char({cdr_summary; sprintf(['%04d %02d   '  repmat('%6d %6d ',1,length(VarList)-1)],Table(i,:)')});
end;










%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read all ICDR files

tmp=datevec(now); tmp(3)=1; tmp(4:6)=0;
alldates=datevec(datenum('01jan04'):datenum(tmp));
  alldates(:,3)=1; alldates=unique(datenum(alldates));

Table=[]; VarList='';
for k=1:length(alldates), try,
  [yyyy mm trash]=datevec(alldates(k));
  Table(k,1:2)=[yyyy mm];

  % read all data in file
  data=ncload(sprintf('%sgpcp_icdr_v22rB1_y%04d_m%02d.nc',fdir_NRT,yyyy,mm));
  vars=fieldnames(data);

  % scroll through vars and deal only with fields that are 2D
  for var=vars',
    if sum(size(data.(char(var)))>1)>=2,
      if isempty(strmatch(var,VarList)),
        VarList={char(VarList);char(var)};
      end;
    end;
  end;
  VarList=cellstr(char(VarList));

  % read each possible var and output
  for i=2:length(VarList),
    try, miss=data.atts.(char(VarList(i))).missing_value;
      fail; miss=-9999; end;
    Table(k,2+2*(i-2)+1)=sum(sum((data.(char(VarList(i)))==miss)));
    Table(k,2+2*(i-2)+2)=sum(sum((data.(char(VarList(i)))==0)));
  end;

end; end;

% add to output file
fprintf(fid,'\n\nGPCP ICDR: \n');
fprintf(fid,'Variables:   %s \n',char(VarList(2:end))');
fprintf(fid,['YYYY MM   ' repmat(' Nmiss   N==0 ',1,length(VarList)-1) '\n']);
fprintf(fid,['%04d %02d   '  repmat('%6d %6d ',1,length(VarList)-1),'\n'],Table');



% Make summary of potential empty/corrupted files
% find variables with any missing data (usually there is none)
idx1=Table(:,3:2:end)~=0;
% find variables with more than average numbers of zero data
idx2=Table(:,4:2:end)>2000;

icdr_summary=sprintf('Variables:   %s',char(VarList(2:end))');
icdr_summary=char({icdr_summary; sprintf(['YYYY MM   ' repmat(' Nmiss   N==0 ',1,length(VarList)-1)])});
for i=find((sum(idx1,2)>0)|(sum(idx2,2)>0))',
  icdr_summary=char({icdr_summary; sprintf(['%04d %02d   '  repmat('%6d %6d ',1,length(VarList)-1)],Table(i,:)')});
end;











%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% read all Interim files

tmp=datevec(now); tmp(3)=1; tmp(4:6)=0;
alldates=datevec(datenum('01jan79'):datenum(tmp));
  alldates(:,3)=1; alldates=unique(datenum(alldates));

Table=[]; VarList='';
for k=1:length(alldates), try,
  [yyyy mm trash]=datevec(alldates(k));
  Table(k,1:2)=[yyyy mm];

  % read all data in file
  data=ncload(sprintf('%sgpcp_Interim_v22rB1_y%04d_m%02d.nc',fdir,yyyy,mm));
  vars=fieldnames(data); vars_idx=zeros(length(vars),1);

  % scroll through vars and deal only with fields that are 2D
  for j=1:length(vars), var=vars(j);
    if sum(size(data.(char(var)))>1)>=2, vars_idx(j)=1;
      if isempty(strmatch(var,VarList)), 
        VarList={char(VarList);char(var)};
      end;
    end;
  end;
  VarList=cellstr(char(VarList));

  % read each possible var and output
  for i=find(vars_idx)',
    idx=strmatch(vars(i),VarList);
    try, miss=data.atts.(char(VarList(idx))).missing_value;
      fail; miss=-9999; end;
    Table(k,2+2*(idx-2)+1)=sum(sum((data.(char(VarList(idx)))==miss)));
    Table(k,2+2*(idx-2)+2)=sum(sum((data.(char(VarList(idx)))==0)));
  end;

end; end;


% Make summary of potential empty/corrupted files
% find variables with any missing data (usually there is none)
idx1=Table(:,3:2:end)~=0;
% find variables with more than average numbers of zero data
idx2=Table(:,4:2:end)>2000;
idx=idx1|idx2;

% add to output file
tmp=char(VarList(2:end))';
fprintf(fid,'\n\nGPCP Interim files: \n');
fprintf(fid,['         ' repmat('%c ',1,size(VarList,1)-1) '\n'],tmp(1,:));
fprintf(fid,['         ' repmat('%c ',1,size(VarList,1)-1) '\n'],tmp(2,:));
fprintf(fid,['YYYY MM  ' repmat('%c ',1,size(VarList,1)-1) '\n'],tmp(3,:));
fprintf(fid,['%04d %02d  ' repmat('%1d ',1,length(VarList)-1) '\n'],[Table(:,1:2) idx]');



% make summary - include a row if sum>4 as 3 or 4 cols are always ==1
int_summary=char({sprintf(['         ' repmat(' %c ',1,length(VarList))],tmp(1,:));sprintf(['         ' repmat(' %c ',1,length(VarList))],tmp(2,:));sprintf(['YYYY MM  ' repmat(' %c ',1,length(VarList))],tmp(3,:))});
for i=find(sum(idx,2)>4)',
  int_summary=char({int_summary; sprintf(['%04d %02d  ' repmat(' %1d ',1,length(VarList)-1)],Table(i,1:2),idx(i,:)')});
end;





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Close output file
fclose(fid);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Output summary stats to screen
disp(char({'ICDR Summary of possible corrupted years:';'';icdr_summary;'';'';'CDR Summary of possible corrupted years:';'';cdr_summary;'';'';'Interim file Summary of possible corrupted years:';'';int_summary;'';'';'Suggest reprocessing any potentially corrupt months until all year are OK';'';'Full output saved in:';[pwd '/' oname]}));















