/** GPCP_Monthly_Analysis/Main_GPCPcdr_m5_WriteNC.c
 *  Global Precipitation Climatology Project (GPCP)
 *
 * NAME
 *   Main_GPCPcdr_m5_WriteNC.c
 *
 * PURPOSE
 *   This routine writes the monthly GPCP CDR NetCDF file.
 *
 * DESCRIPTION
 *   This is a sample C program to writes the monthly GPCP CDR NetCDF file.
 *   There are other similar programs to writes the monthly GPCP ICDR NetCDF
 *   file.
 *
 * AUTHOR
 *   Robert Adler, Mathew Sapiano, Jian-Jian Wang
 *   University of Maryland College Park
 *
 * COPYRIGHT
 *   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
 *   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
 *   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
 *   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
 *   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
 *   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
 *   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
 *   SUPPORT TO USERS.
 *
 * HISTORY
 * 3/2016 Initial Version prepared for NCEI
 *
 **/
#include <stdio.h>
#include <time.h>
#include <sys/utsname.h>

#include "../../OptionsM.h"

int main(int argc, char *argv[])
{
  int    yyyy;
  int    mm;
  char   fname_cdr[256];
  char   VersNum[6];
  char  *CompName;

  // Establish time variables and structure fo robtaining local time
  time_t current_time_t;
  struct tm *current_time;
  char   current_time_str[256];

  // establish structure for getting unix system name
  struct utsname unameData;

  /* Get input arguments */
  yyyy = atoi(argv[1]);
  mm = atoi(argv[2]);

  /* Set version number - must be 6 digits
   * V22 reflects GPCP V2.2, rXX is release number and can change more often */
  sscanf(argv[3],"%6s",&VersNum);

  /* Obtain current time as seconds elapsed since the Epoch and convert to text */
  current_time_t = time(NULL);
  current_time = localtime(&current_time_t);
  sprintf(current_time_str,"%04d-%02d-%02dT%02d:%02d:%02dZ", 
	  1900+current_time->tm_year, 1+current_time->tm_mon, current_time->tm_mday, 
	  current_time->tm_hour, current_time->tm_min, current_time->tm_sec);

  /* Get Computer name */
  if ( uname(&unameData) == -1) {
    fprintf(stderr,"Can't get system name\n");
    CompName="unknown";
  }
  else {
    CompName=unameData.nodename;
  }

  /* Set output name (CDR or ICDR) */
  sprintf(fname_cdr,"gpcp_cdr_%s_y%04d_m%02d.nc",VersNum,yyyy,mm);

  /* Call function to create primary CDR file*/ 
  Write_CDR_nc(OutputPATH,fname_cdr,VersNum,CompName,&current_time_str,yyyy,mm);

}
