#!/bin/sh
#@***h* configure_monthly.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   configure_monthly.sh
#
# PURPOSE
#   To build GPCP V2.3 CDR
#
# DESCRIPTION
#   Configure script to build GPCP V2.3 CDR
#
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 3/2016 Initial Version prepared for NCEI
#
#@*****

pdir=`pwd`

##################################################
# establish paths that need to exist
InputPATH=$(echo `grep "InputPATH" OptionsM.h` | awk -F\"  '{print $2}')
InterimPATH=$(echo `grep "InterimPATH" OptionsM.h` | awk -F\"  '{print $2}')
OutputPATH=$(echo `grep "OutputPATH" OptionsM.h` | awk -F\"  '{print $2}')

if [ ! -d $InputPATH ]; then
  echo "Warning: Input path is undefined; please get input data!"
  echo "reluctantly continuing..."
fi
if [ ! -d $InterimPATH ]; then
  echo "Interim path is undefined - creating"
  mkdir $InterimPATH
fi
if [ ! -d $OutputPATH ]; then
  echo "Output path is undefined - creating"
  mkdir $OutputPATH
fi
if [ ! -d $OutputPATH/bin ]; then
  echo "Output path for binary files is undefined - creating"
  mkdir $OutputPATH/bin
fi
if [ ! -d $OutputPATH/DiagPlots ]; then
  echo "Output path for diagnostic plots is undefined - creating"
  mkdir $OutputPATH/DiagPlots
fi



# Establish locations for interim files
if [ ! -d "$InterimPATH/Adjust_Guage" ]; then
  echo "Adjust_Gauge path is undefined - creating"
  mkdir "$InterimPATH/Adjust_Guage"
fi
if [ ! -d "$InterimPATH/AIRS_V5_1DD" ]; then
  echo "AIRS_V5_1DD path is undefined - creating"
  mkdir "$InterimPATH/AIRS_V5_1DD"
fi
if [ ! -d "$InterimPATH/AIRS_V6_1DD" ]; then
  echo "AIRS_V6_1DD path is undefined - creating"
  mkdir "$InterimPATH/AIRS_V6_1DD"
fi
if [ ! -d "$InterimPATH/AIRS_tmp" ]; then
  echo "AIRS_tmp path is undefined - creating"
  mkdir "$InterimPATH/AIRS_tmp"
fi
if [ ! -d "$InterimPATH/AIRS_TOVS" ]; then
  echo "AIRS_TOVS path is undefined - creating"
  mkdir "$InterimPATH/AIRS_TOVS"
fi
if [ ! -d "$InterimPATH/CORR2" ]; then
  echo "CORR2 path is undefined - creating"
  mkdir "$InterimPATH/CORR2"
fi
if [ ! -d "$InterimPATH/CORR_ADJ" ]; then
  echo "CORR_ADJ path is undefined - creating"
  mkdir "$InterimPATH/CORR_ADJ"
fi
if [ ! -d "$InterimPATH/FUSE2" ]; then
  echo "FUSE2 path is undefined - creating"
  mkdir "$InterimPATH/FUSE2"
fi
if [ ! -d "$InterimPATH/METH_Hist" ]; then
  echo "METH_Hist path is undefined - creating"
  mkdir "$InterimPATH/METH_Hist"
fi
if [ ! -d "$InterimPATH/METH_Hist_5deg" ]; then
  echo "METH_Hist_5deg path is undefined - creating"
  mkdir "$InterimPATH/METH_Hist_5deg"
fi
if [ ! -d "$InterimPATH/METH_Rain" ]; then
  echo "METH_Rain path is undefined - creating"
  mkdir "$InterimPATH/METH_Rain"
fi
if [ ! -d "$InterimPATH/METH_Rain_5deg" ]; then
  echo "METH_Rain_5deg path is undefined - creating"
  mkdir "$InterimPATH/METH_Rain_5deg"
fi
if [ ! -d "$InterimPATH/MWComb" ]; then
  echo "MWComb path is undefined - creating"
  mkdir "$InterimPATH/MWComb"
fi
if [ ! -d "$InterimPATH/OPI_le_rssfix" ]; then
  echo "OPI_le_rssfix path is undefined - creating"
  mkdir "$InterimPATH/OPI_le_rssfix"
fi
if [ ! -d "$InterimPATH/XOVER" ]; then
  echo "XOVER path is undefined - creating"
  mkdir "$InterimPATH/XOVER"
fi
if [ ! -d "$InterimPATH/GPROF2004V" ]; then
  echo "GPROF2004V path is undefined - creating"
  mkdir "$InterimPATH/GPROF2004V"
fi





##################################################
# establish paths for NRT data

InterimPATHrt=$(echo `grep "InterimPATH" OptionsM_NRT.h` | awk -F\"  '{print $2}')
OutputPATHrt=$(echo `grep "OutputPATH" OptionsM_NRT.h` | awk -F\"  '{print $2}')
GPCCPATHrt=$(echo `grep "GPCCPATH" OptionsM_NRT.h` | awk -F\"  '{print $2}')


if [ ! -d $InterimPATHrt ]; then
  echo "Interim path is undefined - creating"
  mkdir $InterimPATHrt
fi
if [ ! -d $OutputPATHrt ]; then
  echo "Output path is undefined - creating"
  mkdir $OutputPATHrt
fi
if [ ! -d $OutputPATHrt/bin ]; then
  echo "Output path for binary files is undefined - creating"
  mkdir $OutputPATHrt/bin
fi
if [ ! -d $OutputPATHrt/DiagPlots ]; then
  echo "Output path for diagnostic plots is undefined - creating"
  mkdir $OutputPATHrt/DiagPlots
fi

if [ ! -d $GPCCPATHrt ]; then
  echo "GPCC First Guess path is undefined - creating"
  mkdir $GPCCPATHrt
fi

# Establish locations for NRT interim files
if [ ! -d "$InterimPATHrt/Adjust_Guage" ]; then
  echo "Adjust_Gauge path is undefined - creating"
  mkdir "$InterimPATHrt/Adjust_Guage"
fi
if [ ! -d "$InterimPATHrt/CORR2" ]; then
  echo "CORR2 path is undefined - creating"
  mkdir "$InterimPATHrt/CORR2"
fi
if [ ! -d "$InterimPATHrt/CORR_ADJ" ]; then
  echo "CORR_ADJ path is undefined - creating"
  mkdir "$InterimPATHrt/CORR_ADJ"
fi
if [ ! -d "$InterimPATHrt/FUSE2" ]; then
  echo "FUSE2 path is undefined - creating"
  mkdir "$InterimPATHrt/FUSE2"
fi
if [ ! -d "$InterimPATHrt/GPCC_FG_2p5" ]; then
  echo "GPCC_FG_2p5 path is undefined - creating"
  mkdir "$InterimPATHrt/GPCC_FG_2p5"
fi
if [ ! -d "$InterimPATHrt/MWComb" ]; then
  echo "MWComb path is undefined - creating"
  mkdir "$InterimPATHrt/MWComb"
fi
if [ ! -d "$InterimPATHrt/OPI_le_rssfix" ]; then
  echo "OPI_le_rssfix path is undefined - creating"
  mkdir "$InterimPATHrt/OPI_le_rssfix"
fi
if [ ! -d "$InterimPATHrt/XOVER" ]; then
  echo "XOVER path is undefined - creating"
  mkdir "$InterimPATHrt/XOVER"
fi







##################################################
# Make L1 file checking codes
cd ${pdir}/GPCPcdr_m1/
rm GPCPcdr_m1_ChkL1files_Ferraro
gcc GPCPcdr_m1_ChkL1files_Ferraro.c -lm -o GPCPcdr_m1_ChkL1files_Ferraro



##################################################
# Make METH 2.5deg version 

# hist and rain rate are separate codes
cd ${pdir}/GPCPcdr_m2/GPCPcdr_m2_SSMI_Ocean/GPCPcdr_m2_SSMI_Ocean_Hist/
rm *.o
make -f Makefile
cd ${pdir}/GPCPcdr_m2/GPCPcdr_m2_SSMI_Ocean/GPCPcdr_m2_SSMI_Ocean_Rain/
rm *.o
make -f Makefile

# Make METH 5deg version
# hist and rain rate are separate codes
cd ${pdir}/GPCPcdr_m2/GPCPcdr_m2_SSMI_Ocean/GPCPcdr_m2_SSMI_Ocean_Hist_5deg/
rm *.o
make -f Makefile
cd ${pdir}/GPCPcdr_m2/GPCPcdr_m2_SSMI_Ocean/GPCPcdr_m2_SSMI_Ocean_Rain_5deg/
rm *.o
make -f Makefile


##################################################
# Make Gauge adjustment (wind-loss correction) code
cd ${pdir}/GPCPcdr_m2/GPCPcdr_m2_Adjust_Gauge/
rm *.o
make -f Makefile


##################################################
# TOVS/AIRS data

# Make code to grid AIRS data
cd ${pdir}/GPCPcdr_m2/GPCPcdr_m2_Grid_AIRS/
rm *.o
make -f Makefile

# Make code to get AIRS/TOVS data
cd ${pdir}/GPCPcdr_m2/GPCPcdr_m2_Get_AIRS_TOVS/
rm *.o
make -f Makefile


#############################################
# Precipitation Merge

# Make CombMW
cd ${pdir}/GPCPcdr_m4/GPCPcdr_m4_combMW/
rm *.o
make -f Makefile

# Make XOVER
cd ${pdir}/GPCPcdr_m4/GPCPcdr_m4_XOVER/
rm *.o
make -f Makefile

# Make CORR2
cd ${pdir}/GPCPcdr_m4/GPCPcdr_m4_CORR2/
rm *.o
make -f Makefile

# Make FUSE2
cd ${pdir}/GPCPcdr_m4/GPCPcdr_m4_FUSE2
rm *.o
make -f Makefile

# Make NetCDF4 output code
cd ${pdir}/GPCPcdr_m5/GPCPcdr_m5_WriteNC
rm *.o
make -f Makefile

# Make NetCDF4 output code for Interim products
cd ${pdir}/GPCPcdr_m5/GPCPcdr_m5_WriteNC_Interim
rm *.o
make -f Makefile

# Make NetCDF4 output code for NRT products
cd ${pdir}/GPCPcdr_m5/GPCPcdr_m5_WriteNC_NRT
rm *.o
make -f Makefile




##################################################
# return to present directory and exit
cd ${pdir}/
exit



