#!/bin/sh
#@***h* GPCP_Monthly_Analysis/GPCPcdr_ECPm_Process1month.sh
#       Global Precipitation Climatology Project (GPCP)
#
# NAME
#   GPCPcdr_ECPm_Process1month.sh
#
# PURPOSE
#   To make diagnostic plots for monthly GPCP product
#
# DESCRIPTION
# process_ICDR.sh 2014 06 
# or
# process_ICDR.sh 2014 06 2
#
# If the third argument is missing, diagnostics are produced
# if $3==1, GPCP CDR and ICDR are processed + diagnostics
# if $3==2, Data are downloaded + CDR/ICDR are processed + diagnostics
#
# AUTHOR
#   Robert Adler, Mathew Sapiano, Jian-Jian Wang
#   University of Maryland College Park
#
# COPYRIGHT
#   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
#   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE
#   FURNISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
#   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
#   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
#   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
#   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL
#   SUPPORT TO USERS.
#
# HISTORY
# 3/2016 Initial Version prepared for NCEI
#
#@*****

if [ $# -eq 3 ]; then
  flag=$3
else
  flag=0
fi

# get year and month for CDR - 2 month before current
yyyy=`date -d "$2/01/$1 - 2 months" '+%Y'`
mm=`date -d "$2/01/$1 - 2 months" '+%m'`
yyyym1=`date -d "$2/01/$1 - 1 month" '+%Y'`
mmm1=`date -d "$2/01/$1 - 1 month" '+%m'`


# download data:
if [ $flag -eq 2 ]; then 
  cd GPCPcdr_ftp
  echo "-----+++++==========+++++-----"
  echo "Downloading GPCP only for $yyyy/$mm"
  ./get_GPCPcdr_1month.sh $yyyy $mm gpcc
  echo "-----+++++==========+++++-----"
  echo "Downloading all data for $1/$2"
  ./get_GPCPcdr_1month.sh $1 $2
  cd ../
fi


# process CDR and ICDR data
if [ $flag -ge 1 ]; then
  echo "-----+++++==========+++++-----"
  echo "Processing CDR and ICDR..."
  ./GPCPcdr_ECPm_Process1month.sh $1 $2 0 0 0 0 0 0 0 0 0 0
  ./GPCPcdr_ECPm_Process1month.sh $1 $2 1 1 0 0 0 0 0 0 0 0
  ./GPCPcdr_ECPm_NRT_Process1month.sh $1 $2 1 1 0 0 0 0 0 0 0 0
  ./GPCPcdr_ECPm_NRT_Process1month.sh $1 $2 0 0 1 1 0 0 0 0 0 0
  ./GPCPcdr_ECPm_NRT_Process1month.sh $1 $2 1 0 1 0 1 1 1 1 1 1
  ./GPCPcdr_ECPm_Process1month.sh $1 $2 0 0 0 0 0 0 0 0 0 1
  ./GPCPcdr_ECPm_Process1month.sh $yyyy $mm 1 0 1 0 1 1 1 1 1 1
  ./GPCPcdr_ECPm_Process1month.sh $yyyy $mm 0 0 0 0 0 0 0 0 0 1
fi

# Make plots for website
echo "-----+++++==========+++++-----"
echo "Making diagnostic plots..."
./GPCPcdr_ECPm_DiagPlots_AnomMap.sh $1 $2 icdr
./GPCPcdr_ECPm_DiagPlots_AnomMapPerc.sh $1 $2 icdr
./GPCPcdr_ECPm_DiagPlots_Map.sh $1 $2 icdr
#./GPCPcdr_ECPm_DiagPlots.sh
./GPCPcdr_ECPm_DiagPlots_AnomMap.sh $yyyy $mm cdr
./GPCPcdr_ECPm_DiagPlots_AnomMapPerc.sh $yyyy $mm cdr
./GPCPcdr_ECPm_DiagPlots_Map.sh $yyyy $mm cdr
./GPCPcdr_ECPm_m5_Products.sh $yyyy $mm
./GPCPcdr_ECPm_m5_Products.sh $yyyym1 $mmm1
./GPCPcdr_ECPm_m5_Products.sh $1 $2



# get output locations:
OutputPATH=$(echo `grep "OutputPATH" OptionsM.h` | awk -F\"  '{print $2}')
NRTOutputPATH=$(echo `grep "OutputPATH" OptionsM_NRT.h` | awk -F\"  '{print $2}')

# set location of remote server:
wwwbase1="eagle1.umd.edu:/data1/www/GPCP_CDR"
wwwbase2="eagle1.umd.edu:/data1/www/GPCP_ICDR"

# create or clear temporary directories
if [ ! -d "${OutputPATH}/tmp_ICDRdata/" ]; then
  mkdir ${OutputPATH}/tmp_ICDRdata/
fi
if [ ! -d "${OutputPATH}/tmp_ICDRplots/" ]; then
  mkdir ${OutputPATH}/tmp_ICDRplots/
fi
`rm -f ${OutputPATH}/tmp_ICDRplots/*.png`
`rm -f ${OutputPATH}/tmp_ICDRdata/*.nc`
`rm -f ${OutputPATH}/tmp_ICDRdata/*.txt`

# copy all required data into temporary location
`cp -f ${NRTOutputPATH}/DiagPlots/*$1$2.png ${OutputPATH}/tmp_ICDRplots/`
`cp -f ${OutputPATH}/DiagPlots/*$yyyy$mm.png ${OutputPATH}/tmp_ICDRplots/`
`cp -f ${OutputPATH}/Anoms/*$1$2.png ${OutputPATH}/tmp_ICDRplots/`
`cp -f ${OutputPATH}/SPI/*$1$2.png ${OutputPATH}/tmp_ICDRplots/`
`cp -f ${OutputPATH}/SPI/*$yyyy$mm.png ${OutputPATH}/tmp_ICDRplots/`
`cp -f ${OutputPATH}/ESPI/*.png ${OutputPATH}/tmp_ICDRplots/`
`cp -f ${OutputPATH}/ESPI/*.png ${OutputPATH}/tmp_ICDRplots/`
`cp -f ${NRTOutputPATH}/*_y$1_m$2* ${OutputPATH}/tmp_ICDRdata/`
`cp -f ${OutputPATH}/ESPI/ESPI.txt ${OutputPATH}/tmp_ICDRdata/`




# manually copy data:
echo "-----+++++==========+++++-----"
echo "Please manually copy data using the following commands:"
echo ""
echo "cd ${OutputPATH}"
echo "scp -r *$yyyy* $wwwbase1/Monthly_Data"
echo ""
echo "cd ${OutputPATH}1DD"
echo "scp -r $yyyym1 $wwwbase1/Daily_Data"
echo ""
echo "cd ${OutputPATH}tmp_ICDRdata"
echo "scp -r * $wwwbase2/Data"
echo ""
echo "cd ${OutputPATH}tmp_ICDRplots"
echo "scp -r * $wwwbase2/images/"
